Package: cmstatr
Type: Package
Title: Statistical Methods for Composite Material Data
Version: 0.10.0
Date: 2024-11-18
Depends: R (>= 3.3)
Authors@R: c(
    person("Stefan", "Kloppenborg", email = "stefan@kloppenborg.ca",
      comment = c(ORCID = "0000-0002-1908-5214"),
      role = c("aut", "cre")),
    person("Billy", "Cheng", email = "bcheng@comtekadvanced.com",
      role = c("ctb")),
    person("Ally", "Fraser", email = "ally.fraser25@gmail.com",
      role = c("ctb")),
    person("Jeffrey", "Borlik",
      role = c("ctb")),
    person("Brice", "Langston", email = "tblangst@gmail.com",
      role = c("ctb")),
    person("Comtek Advanced Structures, Ltd.",
      role = c("fnd"))
  )
Description: An implementation of the statistical methods commonly
  used for advanced composite materials in aerospace applications.
  This package focuses on calculating basis values (lower tolerance
  bounds) for material strength properties, as well as performing the
  associated diagnostic tests. This package provides functions for
  calculating basis values assuming several different distributions,
  as well as providing functions for non-parametric methods of computing
  basis values. Functions are also provided for testing the hypothesis
  that there is no difference between strength and modulus data from an
  alternate sample and that from a "qualification" or "baseline" sample.
  For a discussion of these statistical methods and their use, see the
  Composite Materials Handbook, Volume 1 (2012, ISBN: 978-0-7680-7811-4).
  Additional details about this package are available in the paper by
  Kloppenborg (2020, <doi:10.21105/joss.02265>).
URL: https://www.cmstatr.net/, https://github.com/cmstatr/cmstatr
BugReports: https://github.com/cmstatr/cmstatr/issues
License: AGPL-3
Encoding: UTF-8
LazyData: true
Imports: dplyr, generics, ggplot2, kSamples, MASS, purrr, rlang, stats,
        tibble, tidyr
Suggests: knitr, lintr, rmarkdown, spelling, testthat, vdiffr
RoxygenNote: 7.3.2
VignetteBuilder: knitr
Language: en-US
Config/testthat/parallel: true
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2024-11-19 04:04:50 UTC; stefan
Author: Stefan Kloppenborg [aut, cre] (<https://orcid.org/0000-0002-1908-5214>),
  Billy Cheng [ctb],
  Ally Fraser [ctb],
  Jeffrey Borlik [ctb],
  Brice Langston [ctb],
  Comtek Advanced Structures, Ltd. [fnd]
Maintainer: Stefan Kloppenborg <stefan@kloppenborg.ca>
Repository: CRAN
Date/Publication: 2024-11-19 07:50:08 UTC
Built: R 4.6.0; ; 2026-04-29 14:06:25 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(augment,mnr)
S3method(condition_summary,basis)
S3method(condition_summary,data.frame)
S3method(glance,adk)
S3method(glance,anderson_darling)
S3method(glance,basis)
S3method(glance,equiv_change_mean)
S3method(glance,equiv_mean_extremum)
S3method(glance,levene)
S3method(glance,mnr)
S3method(print,adk)
S3method(print,anderson_darling)
S3method(print,basis)
S3method(print,equiv_change_mean)
S3method(print,equiv_mean_extremum)
S3method(print,levene)
S3method(print,mnr)
export(ad_ksample)
export(anderson_darling_lognormal)
export(anderson_darling_normal)
export(anderson_darling_weibull)
export(augment)
export(basis_anova)
export(basis_hk_ext)
export(basis_lognormal)
export(basis_nonpara_large_sample)
export(basis_normal)
export(basis_pooled_cv)
export(basis_pooled_sd)
export(basis_weibull)
export(calc_cv_star)
export(condition_summary)
export(cv)
export(equiv_change_mean)
export(equiv_mean_extremum)
export(geom_jitter_failure_mode)
export(glance)
export(hk_ext_z)
export(hk_ext_z_j_opt)
export(k_equiv)
export(k_factor_normal)
export(levene_test)
export(maximum_normed_residual)
export(nested_data_plot)
export(nonpara_binomial_rank)
export(normalize_group_mean)
export(normalize_ply_thickness)
export(separate_failure_modes)
export(stat_esf)
export(stat_normal_surv_func)
export(tidy)
export(transform_mod_cv)
export(transform_mod_cv_ad)
importFrom(MASS,fitdistr)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(dplyr,ungroup)
importFrom(generics,augment)
importFrom(generics,glance)
importFrom(generics,tidy)
importFrom(ggplot2,GeomPoint)
importFrom(ggplot2,Stat)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,expansion)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggproto)
importFrom(ggplot2,labs)
importFrom(ggplot2,layer)
importFrom(ggplot2,position_jitter)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(kSamples,ad.test)
importFrom(purrr,map_dfr)
importFrom(rlang,.data)
importFrom(rlang,abort)
importFrom(rlang,call2)
importFrom(rlang,call_args)
importFrom(rlang,call_args_names)
importFrom(rlang,call_name)
importFrom(rlang,enquo)
importFrom(rlang,enquos)
importFrom(rlang,ensym)
importFrom(rlang,eval_tidy)
importFrom(rlang,exec)
importFrom(rlang,fn_fmls_names)
importFrom(rlang,inform)
importFrom(rlang,list2)
importFrom(rlang,quo_get_expr)
importFrom(rlang,quo_text)
importFrom(rlang,warn)
importFrom(stats,dbeta)
importFrom(stats,dnorm)
importFrom(stats,integrate)
importFrom(stats,median)
importFrom(stats,pbeta)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,pnorm)
importFrom(stats,pweibull)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,qweibull)
importFrom(stats,sd)
importFrom(stats,uniroot)
importFrom(stats,var.test)
importFrom(tibble,tibble)
importFrom(tidyr,separate_rows)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# cmstatr <img src="man/figures/logo.png" align="right" alt="" width="120" />

<!-- badges: start -->

[![R-CMD-check](https://github.com/cmstatr/cmstatr/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/cmstatr/cmstatr/actions/workflows/R-CMD-check.yaml)
[![`Codecov` test
coverage](https://codecov.io/gh/cmstatr/cmstatr/branch/master/graph/badge.svg)](https://app.codecov.io/gh/cmstatr/cmstatr?branch=master)
[![DOI](https://joss.theoj.org/papers/10.21105/joss.02265/status.svg)](https://doi.org/10.21105/joss.02265)
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/cmstatr)](https://cran.r-project.org/package=cmstatr)
[![](https://cranlogs.r-pkg.org/badges/cmstatr)](https://cran.r-project.org/package=cmstatr)
<!-- badges: end -->

# What It Does

The `cmstatr` package provides functions for performing statistical
analysis of composite material data. The statistical methods implemented
are those described in [CMH-17-1G](https://www.cmh17.org/). This package
focuses on calculating basis values (lower tolerance bounds) for
material strength properties, as well as performing the associated
diagnostic tests. Functions are also provided for testing for
equivalency between alternate samples and the “qualification” or
“baseline” samples.

Additional details about the package are available in the paper by
Kloppenborg (2020, <https://doi.org/10.21105/joss.02265>).

There is a companion package `cmstatrExt` which provides statistical
methods that are not included in CMH-17, but which may be of use to
practitioners. For more information, please see the [`cmstatrExt`
Website](https://cmstatrext.cmstatr.net).

# Installation

To install `cmstatr` from CRAN, simply run:

``` r
install.packages("cmstatr")
```

If you want the latest development version, you can install it from
`github` using `devtools`. This will also install the dependencies
required to build the vignettes. Optionally, change the value of the
argument `ref` to install `cmstatr` from a different branch of the
repository.

``` r
install.packages(c("devtools", "rmarkdown", "dplyr", "tidyr"))
devtools::install_github("cmstatr/cmstatr", build_vignettes = TRUE,
                         ref = "master",
                         build_opts = c("--no-resave-data", "--no-manual"))
```

# Usage

To compute a B-Basis value from an example data set packaged with
`cmstatr` you can do the following:

``` r
library(dplyr)
library(cmstatr)

carbon.fabric.2 %>%
  filter(test == "FC") %>%
  filter(condition == "RTD") %>%
  basis_normal(strength, batch)
#> 
#> Call:
#> basis_normal(data = ., x = strength, batch = batch)
#> 
#> Distribution:  Normal    ( n = 18 )
#> B-Basis:   ( p = 0.9 , conf = 0.95 )
#> 76.88082
```

For more examples of usage of the `cmstatr` package, see the tutorial
vignette, which can be [viewed
online](https://www.cmstatr.net/articles/cmstatr_Tutorial.html), or can
be loaded as follows, once the package is installed:

``` r
vignette("cmstatr_Tutorial")
```

There is also a vignette showing some examples of the types of graphs
that are typically produced when analyzing composite materials. You can
view this [vignette
online](https://www.cmstatr.net/articles/cmstatr_Graphing.html), or you
can load this vignette with:

``` r
vignette("cmstatr_Graphing")
```

# Philosophical Notes

This package expects
[`tidy data`](https://doi.org/10.18637/jss.v059.i10). That is,
individual observations should be in rows and variables in columns.

Where possible, this package uses general solutions. Look-up tables are
avoided wherever possible.

# Issues

If you’ve found a bug, please open an issue in this repository and
describe the bug. Please include a [reproducible
example](https://reprex.tidyverse.org/) of the bug. If you’re able to
fix the bug, you can do so by submitting a pull request.

If your bug is related to a particular data set, sharing that data set
will help to fix the bug. If you cannot share the data set, please strip
any identifying information and optionally scale the data by an
unspecified factor so that the bug can be reproduced and diagnosed.

# Contributing

Contributions to `cmstatr` are always welcomed. For small changes
(fixing typos or improving the documentation), go ahead and submit a
pull request. For more significant changes, such as new features, please
discuss the proposed change in an issue first.

## Contribution Guidelines

- Please create a git branch for each pull request (PR)
- Before submitting a pull request, please make sure that `R CMD check`
  passes with no errors, warnings or notes
- New and modified code should follow the style guide enforced by the
  [`lintr`](https://cran.r-project.org/package=lintr) package
- Document all exported functions using
  [`roxygen2`](https://cran.r-project.org/package=roxygen2)
- Write tests using
  [`testthat`](https://cran.r-project.org/package=testthat). If your
  contribution fixes a bug, then the test(s) that you add should fail
  before your bug-fix patch is applied and should pass after the code is
  patched.
- For changes that affect the user, add a bullet at the top of `NEWS.md`
  below the current development version

## Development

Testing is performed using `testthat`. Edition 3 of that package is used
and parallel processing enabled. If you wish to use more than two CPUs,
set the environment variable `TESTTHAT_CPUS` to the number of CPUs that
you want to use. One way of doing this is to create the file `.Rprofile`
with the following contents. This file is ignored both by `git` and also
in `.Rbuildingore`.

``` r
Sys.setenv(TESTTHAT_CPUS = 8)
```

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 6
)

# If any of the required packages are unavailable,
# don't re-run the code
# nolint start
required <- c("dplyr", "ggplot2", "tidyr", "cmstatr")
if (!all(unlist(lapply(required, function(pkg) {
    requireNamespace(pkg, quietly = TRUE)}
  )))) {
  knitr::opts_chunk$set(eval = FALSE)
}
# nolint end

## ----message=FALSE------------------------------------------------------------
library(dplyr)
library(ggplot2)
library(tidyr)
library(cmstatr)

## -----------------------------------------------------------------------------
dat <- carbon.fabric.2 %>%
  filter(test == "WT") %>%
  mutate(condition = ordered(condition, c("CTD", "RTD", "ETW", "ETW2")))

dat %>%
  head(10)

## -----------------------------------------------------------------------------
b_basis_pooled <- dat %>%
  basis_pooled_cv(strength, condition, batch,
                  override = c("between_group_variability",
                               "normalized_variance_equal"))

b_basis_pooled

## -----------------------------------------------------------------------------
b_basis_pooled$basis

## -----------------------------------------------------------------------------
dat %>%
  ggplot(aes(x = batch, y = strength)) +
  geom_boxplot() +
  geom_jitter(width = 0.25) +
  geom_hline(aes(yintercept = value),
             data = b_basis_pooled$basis %>% rename(condition = group),
             color = "blue") +
  facet_grid(. ~ condition) +
  theme_bw() +
  ggtitle("Batch Plot")

## -----------------------------------------------------------------------------
dat %>%
  ggplot(aes(x = batch, y = strength)) +
  geom_boxplot() +
  geom_jitter_failure_mode(aes(color = failure_mode, shape = failure_mode), width = 0.25) +
  geom_hline(aes(yintercept = value),
             data = b_basis_pooled$basis %>% rename(condition = group),
             color = "blue") +
  facet_grid(. ~ condition) +
  theme_bw() +
  ggtitle("Batch Plot with Failure Modes")

## -----------------------------------------------------------------------------
dat %>%
  ggplot(aes(x = strength, color = condition)) +
  stat_ecdf(geom = "point") +
  coord_flip() +
  theme_bw() +
  ggtitle("Quantile Plot")

## -----------------------------------------------------------------------------
dat %>%
  ggplot(aes(x = strength, color = condition)) +
  stat_normal_surv_func() +
  stat_esf() +
  theme_bw() +
  ggtitle("Normal Survival Function Plot")

## -----------------------------------------------------------------------------
dat %>%
  group_by(condition) %>%
  mutate(norm.score = scale(strength)) %>%
  ggplot(aes(x = norm.score, y = strength, colour = condition)) +
  geom_point() +
  ggtitle("Normal Scores Plot") +
  theme_bw()

## -----------------------------------------------------------------------------
dat %>%
  ggplot(aes(sample = strength, colour = condition)) +
  geom_qq() +
  geom_qq_line() +
  ggtitle("Q-Q Plot") +
  theme_bw()

## -----------------------------------------------------------------------------
b_basis_fcn <- tribble(
  ~condition, ~fcn, ~args,
  "CTD", "basis_normal", list(override = c("between_batch_variability")),
  "RTD", "basis_normal", list(override = c("between_batch_variability")),
  "ETW", "basis_hk_ext", NULL,
  "ETW2", "basis_normal", list(override = c("between_batch_variability"))
)

a_basis_fcn <- tribble(
  ~condition, ~fcn, ~args,
  "CTD", "basis_normal", list(override = c("between_batch_variability")),
  "RTD", "basis_normal", list(override = c("between_batch_variability")),
  "ETW", "basis_hk_ext", list(method = "woodward-frawley"),
  "ETW2", "basis_normal", list(override = c("between_batch_variability"))
)

## -----------------------------------------------------------------------------
single_point_fcn <- function(group_x, group_batch, cond, basis_fcn, p) {
  fcn <- basis_fcn$fcn[basis_fcn$condition == cond[1]]
  extra_args <- basis_fcn$args[basis_fcn$condition == cond[1]]

  args <- c(
    list(x = group_x, batch = group_batch, p = p),
    unlist(extra_args))
  basis <- do.call(fcn, args)
  basis$basis
}

single_point_results <- dat %>%
  group_by(condition) %>%
  summarise(single_point_b_basis = single_point_fcn(
              strength, batch, condition, b_basis_fcn, 0.90),
            single_point_a_basis = single_point_fcn(
              strength, batch, condition, a_basis_fcn, 0.99),
            minimum = min(strength),
            mean = mean(strength)) %>%
  mutate(condition = ordered(condition, c("CTD", "RTD", "ETW", "ETW2")))

single_point_results

## -----------------------------------------------------------------------------
a_basis_pooled <- dat %>%
  basis_pooled_cv(strength, condition, batch, p = 0.99,
                  override = c("between_group_variability",
                               "normalized_variance_equal"))

a_basis_pooled

## -----------------------------------------------------------------------------
a_basis_pooled$basis

## -----------------------------------------------------------------------------
a_basis_pooled$basis %>%
  rename(condition = group,
         b_basis_pooled = value)

## -----------------------------------------------------------------------------
a_basis_pooled_results <- a_basis_pooled$basis %>%
  rename(condition = group,
         a_basis_pooled = value) %>%
  mutate(condition = ordered(condition, c("CTD", "RTD", "ETW", "ETW2")))

a_basis_pooled_results

## -----------------------------------------------------------------------------
b_basis_pooled_results <- b_basis_pooled$basis %>%
  rename(condition = group,
         b_basis_pooled = value) %>%
  mutate(condition = ordered(condition, c("CTD", "RTD", "ETW", "ETW2")))

b_basis_pooled_results

## -----------------------------------------------------------------------------
single_point_results %>%
  inner_join(b_basis_pooled_results, by = "condition") %>%
  inner_join(a_basis_pooled_results, by = "condition")

## -----------------------------------------------------------------------------
single_point_results %>%
  inner_join(b_basis_pooled_results, by = "condition") %>%
  inner_join(a_basis_pooled_results, by = "condition") %>%
  pivot_longer(cols = single_point_b_basis:a_basis_pooled)

## -----------------------------------------------------------------------------
single_point_results %>%
  inner_join(b_basis_pooled_results, by = "condition") %>%
  inner_join(a_basis_pooled_results, by = "condition") %>%
  pivot_longer(cols = single_point_b_basis:a_basis_pooled) %>%
  ggplot(aes(x = condition, y = value)) +
  geom_boxplot(aes(y = strength), data = dat) +
  geom_point(aes(shape = name, color = name)) +
  ggtitle("Property Graph") +
  theme_bw()

## -----------------------------------------------------------------------------
carbon.fabric.2 %>%
  mutate(panel = as.character(panel)) %>%
  filter(test == "WT") %>%
  nested_data_plot(strength,
                   groups = c(batch, panel))

## -----------------------------------------------------------------------------
carbon.fabric.2 %>%
  mutate(panel = as.character(panel)) %>%
  filter(test == "WT" & condition == "RTD") %>%
  nested_data_plot(strength,
                   groups = c(batch, panel),
                   fill = batch,
                   color = panel)

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

# If any of the required packages are unavailable,
# don't re-run the code
# nolint start
required <- c("dplyr", "ggplot2", "tidyr", "cmstatr", "purrr")
if (!all(unlist(lapply(required, function(pkg) {
    requireNamespace(pkg, quietly = TRUE)}
  )))) {
  knitr::opts_chunk$set(eval = FALSE)
}
#nolint end

## ----message=FALSE------------------------------------------------------------
library(cmstatr)
library(dplyr)
library(ggplot2)
library(tidyr)
library(purrr)

## -----------------------------------------------------------------------------
carbon.fabric.2 %>%
  head(10)

## -----------------------------------------------------------------------------
norm_data <- carbon.fabric.2 %>%
  filter(test == "WT" | test == "FC") %>%
  mutate(strength.norm = normalize_ply_thickness(strength,
                                                 thickness / nplies,
                                                 0.0079))

norm_data %>%
  head(10)

## -----------------------------------------------------------------------------
norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  anderson_darling_normal(strength.norm)

## ----include=FALSE------------------------------------------------------------
# Verify that the AD test always provides the same conclusion
# If this assertion fails, the Vignette needs to be re-written
if (0.05 >= (norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  anderson_darling_normal(strength.norm))$osl) {
  stop("Unexpected vale for Anderson-Darling test")
  }

## -----------------------------------------------------------------------------
norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  basis_normal(strength.norm)

## -----------------------------------------------------------------------------
norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  basis_normal(strength.norm, 
               override = c("outliers_within_batch",
                            "between_batch_variability"))

## -----------------------------------------------------------------------------
b_basis_wt_rtd <- norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  basis_normal(strength.norm, batch)

## -----------------------------------------------------------------------------
b_basis_wt_rtd$diagnostic_obj$between_batch_variability

## -----------------------------------------------------------------------------
norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  ad_ksample(strength.norm, batch)

## -----------------------------------------------------------------------------
norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  group_by(batch) %>%
  ggplot(aes(x = strength.norm, color = batch)) +
  stat_normal_surv_func() +
  stat_esf() +
  ggtitle("Distribution of Data For Each Batch")

## -----------------------------------------------------------------------------
norm_data %>%
  filter(test == "FC") %>%
  group_by(condition, batch) %>%
  nest() %>%
  mutate(mnr = map(data,
                   ~maximum_normed_residual(data = .x, x = strength.norm)),
         tidied = map(mnr, glance)) %>%
  select(-c(mnr, data)) %>%  # remove unneeded columns
  unnest(tidied)

## ----include=FALSE------------------------------------------------------------
if ((norm_data %>%
  filter(test == "FC") %>%
  group_by(condition, batch) %>%
  summarise(
    n_outliers = maximum_normed_residual(x = strength.norm)$n_outliers
    ) %>%
  ungroup() %>%
  summarise(n_outliers = sum(n_outliers)))[[1]] != 0) {
  stop("Unexpected number of outliers")
  }

## -----------------------------------------------------------------------------
norm_data %>%
  filter(test == "FC") %>%
  group_by(condition) %>%
  nest() %>%
  mutate(adk = map(data, ~ad_ksample(data = .x,
                                     x = strength.norm,
                                     groups = batch)),
         tidied = map(adk, glance)) %>%
  select(-c(data, adk)) %>%  # remove unneeded columns
  unnest(tidied)

## ----include=FALSE------------------------------------------------------------
if (!all(!(norm_data %>%
  filter(test == "FC") %>%
  group_by(condition) %>%
  summarise(different_dist =
           ad_ksample(x = strength.norm, groups = batch)$reject_same_dist
  ))$different_dist)) {
  stop("Unexpected ADK result")
  }

## -----------------------------------------------------------------------------
norm_data %>%
  filter(test == "FC") %>%
  group_by(condition) %>%
  nest() %>%
  mutate(mnr = map(data, ~maximum_normed_residual(data = .x,
                                                  x = strength.norm)),
         tidied = map(mnr, glance)) %>%
  select(-c(mnr, data)) %>%  # remove unneeded columns
  unnest(tidied)

## ----include=FALSE------------------------------------------------------------
if ((norm_data %>%
  filter(test == "FC") %>%
  group_by(condition) %>%
  summarise(
    n_outliers = maximum_normed_residual(x = strength.norm)$n_outliers
    ) %>%
  ungroup() %>%
  summarise(n_outliers = sum(n_outliers)))[[1]] != 0) {
  stop("Unexpected number of outliers")
  }

## -----------------------------------------------------------------------------
norm_data %>%
  filter(test == "FC") %>%
  condition_summary(strength.norm, condition, "RTD")

## -----------------------------------------------------------------------------
norm_data %>%
  filter(test == "FC") %>%
  levene_test(strength.norm, condition)

## ----include=FALSE------------------------------------------------------------
if (!(norm_data %>%
  filter(test == "FC") %>%
  levene_test(strength.norm, condition))$reject_equal_variance) {
  stop("Unexpected result from Levene's test")
  }

## -----------------------------------------------------------------------------
norm_data %>%
  filter(test == "FC") %>%
  mutate(
    strength_norm_group = normalize_group_mean(strength.norm, condition)) %>%
  levene_test(strength_norm_group, condition)

## ----include=FALSE------------------------------------------------------------
if ((norm_data %>%
  filter(test == "FC") %>%
  mutate(
    strength_norm_group = normalize_group_mean(strength.norm, condition)) %>%
  levene_test(strength_norm_group, condition))$reject_equal_variance) {
  stop("Unexpected value from Levene's test")
  }

## -----------------------------------------------------------------------------
norm_data %>%
  filter(test == "FC") %>%
  mutate(
    strength_norm_group = normalize_group_mean(strength.norm, condition)) %>%
  anderson_darling_normal(strength_norm_group)

## ----include=FALSE------------------------------------------------------------
if ((norm_data %>%
  filter(test == "FC") %>%
  mutate(
    strength_norm_group = normalize_group_mean(strength.norm, condition)) %>%
  anderson_darling_normal(strength_norm_group))$osl <= 0.05) {
  stop("Unexpected value from AD test")
  }

## -----------------------------------------------------------------------------
norm_data %>%
  filter(test == "FC") %>%
  basis_pooled_cv(strength.norm, condition, batch)

## -----------------------------------------------------------------------------
basis_res <- norm_data %>%
  mutate(condition = ordered(condition,
                             c("CTD", "RTD", "ETD", "ETW", "ETW2"))) %>%
  filter(test == "FC") %>%
  basis_pooled_cv(strength.norm, condition, batch)
basis_res

## -----------------------------------------------------------------------------
basis_res %>%
  condition_summary("RTD")

## -----------------------------------------------------------------------------
carbon.fabric.2 %>%
  filter(test == "FC" & condition == "RTD") %>%
  equiv_mean_extremum(strength, n_sample = 5, alpha = 0.01)

# This function attempts to catch common errors, such as the
# user passing a vector into df and not specifying x (as would happen
# if the user called a function from inside summarize without using
# the named argument x).
#
# @param df a data.frame or NULL
# @param x the vector
# @param c the call
# @param arg_name the name of the vector argument being targeted
#
#' @importFrom rlang enquo eval_tidy abort quo_text call_name
#' @importFrom rlang call_args call_args_names
verify_tidy_input <- function(df, x, c, arg_name) {
  if (is.vector(df)) {
    cname <- call_name(c)
    cargs <- call_args(c)
    cargnames <- call_args_names(c)
    abort(
      paste0(
        "Argument `", cargnames[[1]],
        "` should be either a data.frame or NULL.\n",
        "  Did you mean: `",
        cname, "(", arg_name, " = ", quo_text(cargs[[1]]), ")` ?\n"
      )
    )
  }
}

suppressMessages(library(rlang))

# A mock function that uses process_tidy_vector_input
mock_fcn <- function(data = NULL, x) {
  verify_tidy_input(
    df = data,
    x = x,
    c = match.call(),
    arg_name = "x")

  vec <- eval_tidy(enquo(x), data)
  return(vec)
}

test_that("A vector is returned", {
  res <- mock_fcn(carbon.fabric, strength)
  expect_true(is.vector(res))

  res <- mock_fcn(carbon.fabric, "strength")
  expect_true(is.vector(res))

  res <- mock_fcn(NULL, c(1, 2, 3))
  expect_true(is.vector(res))
})

test_that("Correct warning messages are given", {
  # Should give the name of the data argument in mock_fcn
  expect_error(
    mock_fcn(c(1, 2, 3)),
    regexp = ".*`data`"
  )

  expect_error(
    mock_fcn(data = c(1, 2, 3)),
    regexp = ".*`data`"
  )

  expect_error(
    mock_fcn(c(1, 2, 3)),
    regexp = ".*mock_fcn\\(x = c\\(1, 2, 3\\)\\)"
  )

  expect_error(
    mock_fcn(carbon.fabric$strength),
    regexp = ".*mock_fcn\\(x = carbon\\.fabric\\$strength\\)"
  )
})

# A non-exported function used to format text in `print` methods.
format_row <- function(content, justify, width, ...) {
  # content must be a list, justify and width must be vectors
  res <- character(0L)
  for (i in seq_along(content)) {
    res <- c(res,
      format(
        format(content[[i]], ...),
        justify = justify[i],
        width = width[i],
        ...)
    )
  }
  res <- c(res, "\n")
  return(res)
}

# A non-exported function used to format text in `print` methods.
format_row_equal <- function(content, justify, column_width, ...) {
  # content must be a list with the content:
  # (var_name1, var1, var_name2, var2, etc)
  # the length of content will be twice the length of the other vectors
  res <- character(0L)
  for (i in seq_along(justify)) {
    res <- c(res,
      format(
        paste0(content[[2 * i - 1]],
               " = ",
               format(content[[2 * i]], ...)
               ),
        justify = justify[i],
        width = column_width[i],
        ...)
    )
  }
  res <- c(res, "\n")
  return(res)
}

#' Levene's Test for Equality of Variance
#'
#' @description
#' This function performs the Levene's test for equality of variance.
#'
#' @param data a data.frame
#' @param x the variable in the data.frame or a vector on which to perform the
#'          Levene's test (usually strength)
#' @param groups a variable in the data.frame that defines the groups
#' @param alpha the significance level (default 0.05)
#' @param modcv a logical value indicating whether the modified CV approach
#'              should be used.
#'
#' @return
#' Returns an object of class \code{adk}. This object has the following fields:
#' \item{\code{call}}{the expression used to call this function}
#' \item{\code{data}}{the original data supplied by the user}
#' \item{\code{groups}}{a vector of the groups used in the computation}
#' \item{\code{alpha}}{the value of alpha specified}
#' \item{\code{modcv}}{a logical value indicating whether the modified
#'                     CV approach was used.}
#' \item{\code{n}}{the total number of observations}
#' \item{\code{k}}{the number of groups}
#' \item{\code{f}}{the value of the F test statistic}
#' \item{\code{p}}{the computed p-value}
#' \item{\code{reject_equal_variance}}{a boolean value indicating whether the
#'   null hypothesis that all samples have the same variance is rejected}
#' \item{\code{modcv_transformed_data}}{the data after the modified CV
#'                                      transformation}
#' @details
#' This function performs the Levene's test for equality of variance. The
#' data is transformed as follows:
#'
#' \deqn{w_{ij} = \left| x_{ij} - m_i \right|}{wij = | xij - mi |}
#'
#' Where \eqn{m_i}{mi} is median of the \eqn{ith} group. An F-Test is then
#' performed on the transformed data.
#'
#' When \code{modcv=TRUE}, the data from each group is first transformed
#' according to the modified coefficient of variation (CV) rules before
#' performing Levene's test.
#'
#' @references
#' “Composite Materials Handbook, Volume 1. Polymer Matrix Composites
#' Guideline for Characterization of Structural Materials,” SAE International,
#' CMH-17-1G, Mar. 2012.
#'
#' @examples
#' library(dplyr)
#'
#' carbon.fabric.2 %>%
#'   filter(test == "FC") %>%
#'   levene_test(strength, condition)
#' ##
#' ## Call:
#' ## levene_test(data = ., x = strength, groups = condition)
#' ##
#' ## n = 91          k = 5
#' ## F = 3.883818    p-value = 0.00600518
#' ## Conclusion: Samples have unequal variance ( alpha = 0.05 )
#'
#' @importFrom rlang enquo eval_tidy
#' @importFrom stats var.test median pf
#'
#' @seealso \code{\link{calc_cv_star}}
#' @seealso \code{\link{transform_mod_cv}}
#'
#' @export
levene_test <- function(data = NULL, x, groups, alpha = 0.05, modcv = FALSE) {
  res <- list()
  class(res) <- "levene"

  res$call <- match.call()

  verify_tidy_input(
    df = data,
    x = x,
    c = match.call(),
    arg_name = "x")
  data_vector <- eval_tidy(enquo(x), data)

  verify_tidy_input(
    df = data,
    x = groups,
    c = match.call(),
    arg_name = "groups")
  group_vector <- eval_tidy(enquo(groups), data)

  res$data <- data_vector
  res$groups <- group_vector
  res$alpha <- alpha

  if (modcv == TRUE) {
    res$modcv <- TRUE
    res$modcv_transformed_data <- transform_mod_cv_grouped(
      data_vector, group_vector
    )
    data_vector <- res$modcv_transformed_data
  } else {
    res$modcv <- FALSE
    res$modcv_transformed_data <- NULL
  }

  transformed_groups <- lapply(levels(as.factor(group_vector)), function(lvl) {
    group_data <- data_vector[group_vector == lvl]
    w <- abs(group_data - median(group_data))
    return(w)
  })

  n <- length(data_vector)
  k <- length(transformed_groups)

  grand_mean <- mean(unlist(transformed_groups))

  f_stat_numerator <- sum(vapply(transformed_groups, function(group_data) {
    ni <- length(group_data)
    res <- ni * (mean(group_data) - grand_mean) ^ 2 / (k - 1)
    return(res)
  }, FUN.VALUE = numeric(1L)))

  f_stat_denomenator <- sum(vapply(transformed_groups, function(group_data) {
    group_data_minus_group_mean <- group_data - mean(group_data)
    res <- sum((group_data_minus_group_mean) ^ 2) / (n - k)
    return(res)
  }, FUN.VALUE = numeric(1L)))

  res$n <- n
  res$k <- k
  res$f <- f_stat_numerator / f_stat_denomenator

  res$p <- pf(res$f, df1 = k - 1, df2 = n - k, lower.tail = FALSE)
  res$reject_equal_variance <- res$p <= alpha

  return(res)
}


#' Glance at a \code{levene} object
#'
#' @description
#' Glance accepts an object of type \code{levene} and returns a
#' \code{\link[tibble:tibble]{tibble::tibble}} with
#' one row of summaries.
#'
#' Glance does not do any calculations: it just gathers the results in a
#' tibble.
#'
#' @param x a \code{levene} object returned from \code{\link{levene_test}}
#' @param ... Additional arguments. Not used. Included only to match generic
#'            signature.
#'
#'
#' @return
#' A one-row \code{\link[tibble:tibble]{tibble::tibble}} with the following
#' columns:
#'
#' \item{\code{alpha}}{the value of alpha specified}
#' \item{\code{modcv}}{a logical value indicating whether the modified
#'                     CV approach was used.}
#' \item{\code{n}}{the total number of observations}
#' \item{\code{k}}{the number of groups}
#' \item{\code{f}}{the value of the F test statistic}
#' \item{\code{p}}{the computed p-value}
#' \item{\code{reject_equal_variance}}{a boolean value indicating whether the
#'       null hypothesis that all samples have the same variance is rejected}
#'
#'
#' @seealso
#' \code{\link{levene_test}}
#'
#' @examples
#' df <- data.frame(
#'   groups = c(rep("A", 5), rep("B", 6)),
#'   strength = c(rnorm(5, 100, 6), rnorm(6, 105, 7))
#' )
#' levene_result <- levene_test(df, strength, groups)
#' glance(levene_result)
#'
#' ## # A tibble: 1 x 7
#' ##   alpha modcv     n     k      f     p reject_equal_variance
#' ##   <dbl> <lgl> <int> <int>  <dbl> <dbl> <lgl>
#' ## 1  0.05 FALSE    11     2 0.0191 0.893 FALSE
#'
#' @method glance levene
#' @importFrom tibble tibble
#'
#' @export
glance.levene <- function(x, ...) {  # nolint
  with(
    x,
    tibble::tibble(
      alpha = alpha,
      modcv = modcv,
      n = n,
      k = k,
      f = f,
      p = p,
      reject_equal_variance = reject_equal_variance
    )
  )
}


#' @export
print.levene <- function(x, ...) {
  cat("\nCall:\n",
      paste(deparse(x$call), sep = "\n", collapse = "\n"), "\n\n", sep = "")

  justify <- c("left", "left")
  width <- c(16L, 16L)

  cat(format_row_equal(list("n", x$n, "k", x$k),
                       justify, width, ...))

  if (x$modcv == TRUE) {
    cat("Modified CV Approach Used", "\n")
  }

  cat(format_row_equal(list("F", x$f, "p-value", x$p),
                       justify, width, ...))
  if (x$reject_equal_variance) {
    cat("Conclusion: Samples have unequal variance ( alpha =",
        x$alpha, ")\n\n")
  } else {
    cat("Conclusion: Samples have equal variances ( alpha =",
        x$alpha, ")\n\n")
  }
}

#' Calculate k factor for basis values (\eqn{kB}, \eqn{kA}) with normal
#' distribution
#'
#' @description
#' The factors returned by this function are used when calculating basis
#' values (one-sided confidence bounds) when the data are normally
#' distributed. The basis value will
#' be equal to \eqn{\bar{x} - k s}{x_bar - k s},
#' where \eqn{\bar{x}}{x_bar} is the sample mean,
#' \eqn{s} is the sample standard deviation and \eqn{k} is the result
#' of this function.
#' This function is internally used by \code{\link{basis_normal}} when
#' computing basis values.
#'
#' @param n the number of observations (i.e. coupons)
#' @param p the desired content of the tolerance bound.
#'          Should be 0.90 for B-Basis and 0.99 for A-Basis
#' @param conf confidence level. Should be 0.95 for both A- and B-Basis
#'
#' @details
#' This function calculates the k factors used when determining A- and
#' B-Basis values for normally distributed data. To get \eqn{kB}, set
#' the content of the tolerance bound to \code{p = 0.90} and
#' the confidence level to \code{conf = 0.95}. To get \eqn{kA}, set
#' \code{p = 0.99} and \code{conf = 0.95}. While other tolerance bound
#' contents and confidence levels may be computed, they are infrequently
#' needed in practice.
#'
#' The k-factor is calculated using equation 2.2.3 of
#' Krishnamoorthy and Mathew (2008).
#'
#' This function has been validated against the \eqn{kB} tables in
#' CMH-17-1G for each value of \eqn{n} from \eqn{n = 2} to \eqn{n = 95}.
#' It has been validated against the \eqn{kA} tables in CMH-17-1G for each
#' value of \eqn{n} from \eqn{n = 2} to \eqn{n = 75}. Larger values of \eqn{n}
#' also match the tables in CMH-17-1G, but R
#' emits warnings that "full precision may not have been achieved." When
#' validating the results of this function against the tables in CMH-17-1G,
#' the maximum allowable difference between the two is 0.002. The tables in
#' CMH-17-1G give values to three decimal places.
#'
#' For more information about tolerance bounds in general, see
#' Meeker, et. al. (2017).
#'
#' @return the calculated factor
#'
#' @references
#' K. Krishnamoorthy and T. Mathew, Statistical Tolerance Regions: Theory,
#' Applications, and Computation. Hoboken: John Wiley & Sons, 2008.
#'
#' W. Meeker, G. Hahn, and L. Escobar, Statistical Intervals: A Guide
#' for Practitioners and Researchers, Second Edition.
#' Hoboken: John Wiley & Sons, 2017.
#'
#' “Composite Materials Handbook, Volume 1. Polymer Matrix Composites
#' Guideline for Characterization of Structural Materials,” SAE International,
#' CMH-17-1G, Mar. 2012.
#'
#' @seealso
#' \code{\link{basis_normal}}
#'
#' @examples
#' kb <- k_factor_normal(n = 10, p = 0.9, conf = 0.95)
#' print(kb)
#'
#' ## [1] 2.35464
#'
#' # This can be used to caclulate the B-Basis if
#' # the sample mean and sample standard deviation
#' # is known, and data is assumed to be normally
#' # distributed
#'
#' sample_mean <- 90
#' sample_sd <- 5.2
#' print("B-Basis:")
#' print(sample_mean - sample_sd * kb)
#'
#' ## [1] B-Basis:
#' ## [1] 77.75587
#'
#' @importFrom stats qnorm qt
#'
#' @export
k_factor_normal <- function(n, p = 0.90, conf = 0.95) {
  z <- qnorm(p)
  t <- qt(conf, df = n - 1, ncp = z * sqrt(n))
  return(t / sqrt(n))
}

#' Calculate basis values
#'
#' @description
#' Calculate the basis value for a given data set. There are various functions
#' to calculate the basis values for different distributions.
#' The basis value is the lower one-sided tolerance bound of a certain
#' proportion of the population. For more information on tolerance bounds,
#' see Meeker, et. al. (2017).
#' For B-Basis, set the content of tolerance bound to \eqn{p=0.90} and
#' the confidence level to \eqn{conf=0.95}; for A-Basis, set \eqn{p=0.99} and
#' \eqn{conf=0.95}. While other tolerance bound
#' contents and confidence levels may be computed, they are infrequently
#' needed in practice.
#'
#' These functions also perform some automated diagnostic
#' tests of the data prior to calculating the basis values. These diagnostic
#' tests can be overridden if needed.
#'
#' @param data a data.frame
#' @param x the variable in the data.frame for which to find the basis value
#' @param batch the variable in the data.frame that contains the batches.
#' @param groups the variable in the data.frame representing the groups
#' @param p the content of the tolerance bound. Should be 0.90 for B-Basis
#'          and 0.99 for A-Basis
#' @param conf confidence level Should be 0.95 for both A- and B-Basis
#' @param override a list of names of diagnostic tests to override,
#'                 if desired.
#' @param modcv a logical value indicating whether the modified CV approach
#'              should be used. Only applicable to pooling methods.
#' @param method the method for Hanson--Koopmans nonparametric basis values.
#'               should be "optimum-order" for B-Basis and "woodward-frawley"
#'               for A-Basis.
#'
#' @details
#' \code{data} is an optional argument. If \code{data} is given, it should
#' be a
#' \code{data.frame} (or similar object). When \code{data} is specified, the
#' value of \code{x} is expected to be a variable within \code{data}. If
#' \code{data} is not specified, \code{x} must be a vector.
#'
#' When \code{modcv=TRUE} is set, which is only applicable to the
#' pooling methods,
#' the data is first modified according to the modified coefficient
#' of variation (CV)
#' rules. This modified data is then used when both calculating the
#' basis values and
#' also when performing the diagnostic tests. The modified CV approach
#' is a way of
#' adding extra variance to datasets with unexpectedly low variance.
#'
#' \code{basis_normal} calculate the basis value by subtracting \eqn{k} times
#' the standard deviation from the mean. \eqn{k} is given by
#' the function \code{\link{k_factor_normal}}. The equations in
#' Krishnamoorthy and Mathew (2008) are used.
#' \code{basis_normal} also
#' performs a diagnostic test for outliers (using
#' \code{\link{maximum_normed_residual}})
#' and a diagnostic test for normality (using
#' \code{\link{anderson_darling_normal}}).
#' If the argument \code{batch} is given, this function also performs
#' a diagnostic test for outliers within
#' each batch (using \code{\link{maximum_normed_residual}})
#' and a diagnostic test for between batch variability (using
#' \code{\link{ad_ksample}}). The argument \code{batch} is only used
#' for these diagnostic tests.
#'
#' \code{basis_lognormal} calculates the basis value in the same way
#' that \code{basis_normal} does, except that the natural logarithm of the
#' data is taken.
#'
#' \code{basis_lognormal} function also performs
#' a diagnostic test for outliers (using
#' \code{\link{maximum_normed_residual}})
#' and a diagnostic test for normality (using
#' \code{\link{anderson_darling_lognormal}}).
#' If the argument \code{batch} is given, this function also performs
#' a diagnostic test for outliers within
#' each batch (using \code{\link{maximum_normed_residual}})
#' and a diagnostic test for between batch variability (using
#' \code{\link{ad_ksample}}). The argument \code{batch} is only used
#' for these diagnostic tests.
#'
#' \code{basis_weibull} calculates the basis value for data distributed
#' according to a Weibull distribution. The confidence level for the
#' content requested is calculated using the conditional method, as
#' described in Lawless (1982) Section 4.1.2b. This has good agreement
#' with tables published in CMH-17-1G. Results differ between this function
#' and STAT17 by approximately 0.5\%.
#'
#' \code{basis_weibull} function also performs
#' a diagnostic test for outliers (using
#' \code{\link{maximum_normed_residual}})
#' and a diagnostic test for normality (using
#' \code{\link{anderson_darling_weibull}}).
#' If the argument \code{batch} is given, this function also performs
#' a diagnostic test for outliers within
#' each batch (using \code{\link{maximum_normed_residual}})
#' and a diagnostic test for between batch variability (using
#' \code{\link{ad_ksample}}). The argument \code{batch} is only used
#' for these diagnostic tests.
#'
#' \code{basis_hk_ext} calculates the basis value using the Extended
#' Hanson--Koopmans method, as described in CMH-17-1G and Vangel (1994).
#' For nonparametric distributions, this function should be used for samples
#' up to n=28 for B-Basis and up to \eqn{n=299} for A-Basis.
#' This method uses a pair of order statistics to determine the basis value.
#' CMH-17-1G suggests that for A-Basis, the first and last order statistic
#' is used: this is called the "woodward-frawley" method in this package,
#' after the paper in which this approach is described (as referenced
#' by Vangel (1994)). For B-Basis, another approach is used whereby the
#' first and \code{j-th} order statistic are used to calculate the basis value.
#' In this approach, the \code{j-th} order statistic is selected to minimize
#' the difference between the tolerance limit (assuming that the order
#' statistics are equal to the expected values from a standard normal
#' distribution) and the population quantile for a standard normal
#' distribution. This approach is described in Vangel (1994). This second
#' method (for use when calculating B-Basis values) is called
#' "optimum-order" in this package.
#' The results of \code{basis_hk_ext} have been
#' verified against example results from the program STAT-17. Agreement is
#' typically well within 0.2\%, however, for a few sample sizes, the
#' agreement can be as poor as 1\% with the result of this function
#' being more conservative than STAT-17.
#'
#' \code{basis_hk_ext} also performs
#' a diagnostic test for outliers (using
#' \code{\link{maximum_normed_residual}})
#' and performs a pair of tests that the sample size and method selected
#' follow the guidance described above.
#' If the argument \code{batch} is given, this function also performs
#' a diagnostic test for outliers within
#' each batch (using \code{\link{maximum_normed_residual}})
#' and a diagnostic test for between batch variability (using
#' \code{\link{ad_ksample}}). The argument \code{batch} is only used
#' for these diagnostic tests.
#'
#' \code{basis_nonpara_large_sample} calculates the basis value
#' using the large sample method described in CMH-17-1G. This method uses
#' a sum of binomials to determine the rank of the ordered statistic
#' corresponding with the desired tolerance limit (basis value). Results
#' of this function have been verified against results of the STAT-17
#' program.
#'
#' \code{basis_nonpara_large_sample} also performs
#' a diagnostic test for outliers (using
#' \code{\link{maximum_normed_residual}})
#' and performs a test that the sample size is sufficiently large.
#' If the argument \code{batch} is given, this function also performs
#' a diagnostic test for outliers within
#' each batch (using \code{\link{maximum_normed_residual}})
#' and a diagnostic test for between batch variability (using
#' \code{\link{ad_ksample}}). The argument \code{batch} is only used
#' for these diagnostic tests.
#'
#' \code{basis_anova} calculates basis values using the ANOVA method.
#' \code{x} specifies the data (normally strength) and \code{groups}
#' indicates the group corresponding to each observation. This method is
#' described in CMH-17-1G. This function automatically performs a diagnostic
#' test for outliers within each group
#' (using \code{\link{maximum_normed_residual}}) and a test for between
#' group variability (using \code{\link{ad_ksample}}) as well as checking
#' that the data contains at least 5 groups.
#' This function has been verified against the results of the STAT-17 program.
#'
#' \code{basis_pooled_sd} calculates basis values by pooling the data from
#' several groups together. \code{x} specifies the data (normally strength)
#' and \code{group} indicates the group corresponding to each observation.
#' This method is described in CMH-17-1G and matches the pooling method
#' implemented in ASAP 2008.
#'
#' \code{basis_pooled_cv} calculates basis values by pooling the data from
#' several groups together. \code{x} specifies the data (normally strength)
#' and \code{group} indicates the group corresponding to each observation.
#' This method is described in CMH-17-1G.
#'
#' \code{basis_pooled_sd} and \code{basis_pooled_cv} both automatically
#' perform a number of diagnostic tests. Using
#' \code{\link{maximum_normed_residual}}, they check that there are no
#' outliers within each group and batch (provided that \code{batch} is
#' specified). They check the between batch variability using
#' \code{\link{ad_ksample}}. They check that there are no outliers within
#' each group (pooling all batches) using
#' \code{\link{maximum_normed_residual}}. They check for the normality
#' of the pooled data using \code{\link{anderson_darling_normal}}.
#' \code{basis_pooled_sd} checks for equality of variance of all
#' data using \code{\link{levene_test}} and \code{basis_pooled_cv}
#' checks for equality of variances of all data after transforming it
#' using \code{\link{normalize_group_mean}}
#' using \code{\link{levene_test}}.
#'
#' The object returned by these functions includes the named vector
#' \code{diagnostic_results}. This contains all of the diagnostic tests
#' performed. The name of each element of the vector corresponds with the
#' name of the diagnostic test. The contents of each element will be
#' "P" if the diagnostic test passed, "F" if the diagnostic test failed,
#' "O" if the diagnostic test was overridden and \code{NA} if the
#' diagnostic test was skipped (typically because an optional
#' argument was not supplied).
#'
#' @return an object of class \code{basis}
#' This object has the following fields:
#' \item{\code{call}}{the expression used to call this function}
#' \item{\code{distribution}}{the distribution used (normal, etc.)}
#' \item{\code{p}}{the value of \eqn{p} supplied}
#' \item{\code{conf}}{the value of \eqn{conf} supplied}
#' \item{\code{modcv}}{a logical value indicating whether the modified
#'                     CV approach was used. Only applicable to pooling
#'                     methods.}
#' \item{\code{data}}{a copy of the data used in the calculation}
#' \item{\code{groups}}{a copy of the groups variable.
#'                      Only used for pooling and ANOVA methods.}
#' \item{\code{batch}}{a copy of the batch data used for diagnostic tests}
#' \item{\code{modcv_transformed_data}}{the data after the modified CV
#'                                      transformation}
#' \item{\code{override}}{a vector of the names of diagnostic tests that
#'                        were overridden. \code{NULL} if none were
#'                        overridden}
#' \item{\code{diagnostic_results}}{a named character vector containing the
#'                                  results of all the diagnostic tests. See
#'                                  the Details section for additional
#'                                  information}
#' \item{\code{diagnostic_failures}}{a vector containing any diagnostic tests
#'                                   that produced failures}
#' \item{\code{n}}{the number of observations}
#' \item{\code{r}}{the number of groups, if a pooling method was used.
#'                 Otherwise it is NULL.}
#' \item{\code{basis}}{the basis value computed. This is a number
#'                     except when pooling methods are used, in
#'                     which case it is a data.frame.}
#'
#' @seealso \code{\link{hk_ext_z_j_opt}}
#' @seealso \code{\link{k_factor_normal}}
#' @seealso \code{\link{transform_mod_cv}}
#' @seealso \code{\link{maximum_normed_residual}}
#' @seealso \code{\link{anderson_darling_normal}}
#' @seealso \code{\link{anderson_darling_lognormal}}
#' @seealso \code{\link{anderson_darling_weibull}}
#' @seealso \code{\link{ad_ksample}}
#' @seealso \code{\link{normalize_group_mean}}
#'
#' @references
#' J. F. Lawless, Statistical Models and Methods for Lifetime Data.
#' New York: John Wiley & Sons, 1982.
#'
#' “Composite Materials Handbook, Volume 1. Polymer Matrix Composites
#' Guideline for Characterization of Structural Materials,” SAE International,
#' CMH-17-1G, Mar. 2012.
#'
#' M. Vangel, “One-Sided Nonparametric Tolerance Limits,”
#' Communications in Statistics - Simulation and Computation,
#' vol. 23, no. 4. pp. 1137–1154, 1994.
#'
#' K. Krishnamoorthy and T. Mathew, Statistical Tolerance Regions: Theory,
#' Applications, and Computation. Hoboken: John Wiley & Sons, 2008.
#'
#' W. Meeker, G. Hahn, and L. Escobar, Statistical Intervals: A Guide
#' for Practitioners and Researchers, Second Edition.
#' Hoboken: John Wiley & Sons, 2017.
#'
#' @examples
#' library(dplyr)
#'
#' # A single-point basis value can be calculated as follows
#' # in this example, three failed diagnostic tests are
#' # overridden.
#'
#' carbon.fabric %>%
#'   filter(test == "FC") %>%
#'   filter(condition == "RTD") %>%
#'   basis_normal(strength, batch,
#'                override = c("outliers",
#'                             "outliers_within_batch",
#'                             "anderson_darling_normal"))
#'
#' ## Call:
#' ## basis_normal(data = ., x = strength, batch = batch,
#' ##     override = c("outliers", "outliers_within_batch",
#' ##    "anderson_darling_normal"))
#' ##
#' ## Distribution:  Normal 	( n = 18 )
#' ## The following diagnostic tests were overridden:
#' ##     `outliers`,
#' ##     `outliers_within_batch`,
#' ##     `anderson_darling_normal`
#' ## B-Basis:   ( p = 0.9 , conf = 0.95 )
#' ## 76.94656
#'
#' # A set of pooled basis values can also be calculated
#' # using the pooled standard deviation method, as follows.
#' # In this example, one failed diagnostic test is overridden.
#' carbon.fabric %>%
#'   filter(test == "WT") %>%
#'   basis_pooled_sd(strength, condition, batch,
#'                   override = c("outliers_within_batch"))
#'
#' ## Call:
#' ## basis_pooled_sd(data = ., x = strength, groups = condition,
#' ##                 batch = batch, override = c("outliers_within_batch"))
#' ##
#' ## Distribution:  Normal - Pooled Standard Deviation 	( n = 54, r = 3 )
#' ## The following diagnostic tests were overridden:
#' ##     `outliers_within_batch`
#' ## B-Basis:   ( p = 0.9 , conf = 0.95 )
#' ## CTD  127.6914
#' ## ETW  125.0698
#' ## RTD  132.1457
#'
#' @name basis
NULL

new_basis <- function(
  call,
  distribution,
  modcv,
  p,
  conf,
  override,
  data,
  groups,
  batch
) {
  res <- list()
  class(res) <- "basis"

  res$call <- call
  res$distribution <- distribution
  res$modcv <- modcv
  res$p <- p
  res$conf <- conf
  res$data <- data
  res$groups <- groups
  res$batch <- batch
  res$modcv_transformed_data <- NA
  res$override <- override
  res$diagnostic_results <- character(0L)
  res$diagnostic_failures <- character(0L)
  res$n <- length(res$data)
  res$r <- NA
  if (!is.null(groups) & !all(is.na(groups))) {
    res$r <- length(levels(as.factor(groups)))
  }
  res$basis <- NA

  return(res)
}


#' Glance at a basis object
#'
#' @description
#' Glance accepts an object of type basis and returns a
#' \code{\link[tibble:tibble]{tibble::tibble}} with
#' one row of summaries for each basis value.
#'
#' Glance does not do any calculations: it just gathers the results in a
#' tibble.
#'
#' @param x a basis object
#' @param include_diagnostics a logical value indicating whether to include
#'                            columns for diagnostic tests. Default FALSE.
#' @param ... Additional arguments. Not used. Included only to match generic
#'            signature.
#'
#'
#' @return
#' A \code{\link[tibble:tibble]{tibble::tibble}} with the following
#' columns:
#'
#' \item{\code{p}}{the the content of the tolerance bound.
#'                 Normally 0.90 or 0.99}
#' \item{\code{conf}}{the confidence level. Normally 0.95}
#' \item{\code{distribution}}{a string representing the distribution assumed
#'        when calculating the basis value}
#' \item{\code{modcv}}{a logical value indicating whether the modified
#'                     CV approach was used. Only applicable to pooling
#'                     methods.}
#' \item{\code{n}}{the sample size}
#' \item{\code{r}}{the number of groups used in the calculation. This will
#'        be \code{NA} for single-point basis values}
#' \item{\code{basis}}{the basis value}
#'
#' @details
#' For the pooled basis methods (\code{basis_pooled_cv} and
#' \code{basis_pooled_sd}), the \code{\link[tibble:tibble]{tibble::tibble}}
#' returned by \code{glance} will have one row for each group included in
#' the pooling. For all other basis methods, the resulting \code{tibble}
#' will have a single row.
#'
#' If \code{include_diagnostics=TRUE}, there will be additional columns
#' corresponding with the diagnostic tests performed. These column(s) will
#' be of type character and will contain a "P" if the diagnostic test
#' passed, a "F" if the diagnostic test failed, an "O" if the diagnostic
#' test was overridden or \code{NA} if the test was not run (typically
#' because an optional argument was not passed to the function that
#' computed the basis value).
#'
#'
#' @seealso
#' \code{\link{basis}}
#'
#' @examples
#' set.seed(10)
#' x <- rnorm(20, 100, 5)
#' b <- basis_normal(x = x)
#' glance(b)
#'
#' ## # A tibble: 1 x 7
#' ##       p  conf distribution modcv     n r     basis
#' ##   <dbl> <dbl> <chr>        <lgl> <int> <lgl> <dbl>
#' ## 1   0.9  0.95 Normal       FALSE    20 NA     92.0
#'
#'
#' glance(b, include_diagnostics = TRUE)
#'
#' ## # A tibble: 1 x 11
#' ##        p  conf distribution modcv     n r     basis outliers_within…
#' ##    <dbl> <dbl> <chr>        <lgl> <int> <lgl> <dbl> <chr>
#' ##  1   0.9  0.95 Normal       FALSE    20 NA     92.0 NA
#' ## # … with 3 more variables: between_batch_variability <chr>,
#' ## #   outliers <chr>, anderson_darling_normal <chr>
#'
#' @method glance basis
#' @importFrom tibble tibble
#'
#' @export
glance.basis <- function(x, include_diagnostics = FALSE, ...) {  # nolint
  res <- tibble::tibble(
    p = x$p,
    conf = x$conf,
    distribution = x$distribution,
    modcv = x$modcv,
    n = x$n,
    r = x$r,
    basis = x$basis
  )

  if (include_diagnostics) {
    for (dn in names(x$diagnostic_results)) {
      res[[dn]] <- x$diagnostic_results[[dn]]
    }
  }

  res
}

format_diagnostic_helper <- function(heading, vec) {
  if (!is.null(vec) & length(vec) > 0) {
    return(paste0(
      heading, "\n",
      "    `",
      paste(vec, collapse = "`,\n    `"),
      "`\n"
    ))
  }
  return("")
}

#' @export
print.basis <- function(x, ...) {
  cat("\nCall:\n",
      paste(deparse(x$call), sep = "\n", collapse = "\n"), "\n\n", sep = "")

  cat("Distribution: ", x$distribution, "\t")

  cat("( n =", x$n)
  if (!is.null(x$r) & !all(is.na(x$r))) {
    cat(", r =", x$r)
  }
  cat(" )\n")

  if (x$modcv == TRUE) {
    cat("Modified CV Approach Used", "\n")
  }

  cat(format_diagnostic_helper(
    "The following diagnostic tests failed:",
    x$diagnostic_failures)
  )
  cat(format_diagnostic_helper(
    "The following diagnostic tests were overridden:",
    x$override)
  )

  if (x$conf == 0.95 & x$p == 0.9) {
    cat("B-Basis: ", " ( p =", x$p, ", conf =", x$conf, ")\n")
  }
  else if (x$conf == 0.95 & x$p == 0.99) {
    cat("A-Basis: ", " ( p =", x$p, ", conf =", x$conf, ")\n")
  }
  else {
    cat("Basis: ", " ( p =", x$p, ", conf =", x$conf, ")\n")
  }

  if (is.numeric(x$basis)) {
    cat(x$basis, "\n")
  } else if (is.data.frame(x$basis)) {
    col_width <- max(nchar(as.character(x$basis[["group"]]))) + 2
    for (j in seq(along.with = x$basis$group)) {
      cat(format(x$basis[["group"]][j], width = col_width))
      cat(x$basis[["value"]][j], "\n")
    }
  } else {
    stop("`basis` is an unexpected data type")  # nocov
  }

  cat("\n")
}

single_point_rules <- list(
  outliers_within_batch = function(x, batch, ...) {
    group_mnr <- vapply(unique(batch), function(b) {
      x_group <- x[batch == b]
      mnr <- maximum_normed_residual(x = x_group)
      mnr$n_outliers == 0
    }, FUN.VALUE = logical(1L))
    ifelse(all(group_mnr), "",
           paste0("Maximum normed residual test detected ",
                  "outliers within one or more batch"))
  },
  between_batch_variability = function(x, batch, ...) {
    adk <- ad_ksample(x = x, groups = batch, alpha = 0.025)
    ifelse(!adk$reject_same_dist, "",
          paste0("Anderson-Darling k-Sample test indicates that ",
                 "batches are drawn from different distributions"))
  },
  outliers = function(x, ...) {
    mnr <- maximum_normed_residual(x = x)
    ifelse(mnr$n_outliers == 0, "",
           paste0("Maximum normed residual test detected outliers ",
                  "within data"))
  }
)

basis_normal_rules <- single_point_rules
basis_normal_rules[["anderson_darling_normal"]] <-
  function(x, ...) {
    ad <- anderson_darling_normal(x = x)
    ifelse(!ad$reject_distribution, "",
           paste0("Anderson-Darling test rejects hypothesis that data ",
                  "is drawn from a normal distribution"))
  }

#' @rdname basis
#' @importFrom rlang enquo eval_tidy
#' @importFrom stats sd
#' @export
basis_normal <- function(data = NULL, x, batch = NULL, p = 0.90, conf = 0.95,
                         override = c()) {
  verify_tidy_input(
    df = data,
    x = x,
    c = match.call(),
    arg_name = "x")

  verify_tidy_input(
    df = data,
    x = batch,
    c = match.call(),
    arg_name = "batch")

  res <- new_basis(
    call = match.call(),
    distribution = "Normal",
    modcv = FALSE,
    p = p,
    conf = conf,
    override = override,
    data = eval_tidy(enquo(x), data),
    groups = NA,
    batch = eval_tidy(enquo(batch), data)
  )

  res$diagnostic_results <- perform_checks(
    basis_normal_rules, x = res$data, batch = res$batch, override = override
  )
  res$diagnostic_failures <- get_check_failure_names(res$diagnostic_results)

  k <- k_factor_normal(n = res$n, p = p, conf = conf)

  cv <- sd(res$data) / mean(res$data)

  res$basis <- mean(res$data) * (1 - k * cv)

  return(res)
}

basis_lognormal_rules <- single_point_rules
basis_lognormal_rules[["anderson_darling_lognormal"]] <-
  function(x, ...) {
    ad <- anderson_darling_lognormal(x = x)
    ifelse(!ad$reject_distribution, "",
           paste0("Anderson-Darling test rejects hypothesis that data ",
                  "is drawn from a log-normal distribution"))
  }

#' @rdname basis
#' @importFrom rlang enquo eval_tidy
#' @importFrom stats sd
#' @export
basis_lognormal <- function(data = NULL, x, batch = NULL, p = 0.90,
                            conf = 0.95, override = c()) {
  verify_tidy_input(
    df = data,
    x = x,
    c = match.call(),
    arg_name = "x")

  verify_tidy_input(
    df = data,
    x = batch,
    c = match.call(),
    arg_name = "batch")

  res <- new_basis(
    call = match.call(),
    distribution = "Lognormal",
    modcv = FALSE,
    p = p,
    conf = conf,
    override = override,
    data = eval_tidy(enquo(x), data),
    groups = NA,
    batch = eval_tidy(enquo(batch), data)
  )

  res$diagnostic_results <- perform_checks(
    basis_lognormal_rules, x = res$data, batch = res$batch, override = override
  )
  res$diagnostic_failures <- get_check_failure_names(res$diagnostic_results)

  k <- k_factor_normal(n = res$n, p = p, conf = conf)
  res$basis <- exp(mean(log(res$data)) - k * sd(log(res$data)))

  return(res)
}

basis_weibull_rules <- single_point_rules
basis_weibull_rules[["anderson_darling_weibull"]] <-
  function(x, ...) {
    ad <- anderson_darling_weibull(x = x)
    ifelse(!ad$reject_distribution, "",
           paste0("Anderson-Darling test rejects hypothesis that data ",
                  "is drawn from a Weibull distribution"))
  }

#' @rdname basis
#' @importFrom rlang enquo eval_tidy
#' @importFrom stats qweibull integrate pchisq uniroot
#' @importFrom MASS fitdistr
#'
#' @export
basis_weibull <- function(data = NULL, x, batch = NULL, p = 0.90,
                          conf = 0.95, override = c()) {
  verify_tidy_input(
    df = data,
    x = x,
    c = match.call(),
    arg_name = "x")

  verify_tidy_input(
    df = data,
    x = batch,
    c = match.call(),
    arg_name = "batch")

  res <- new_basis(
    call = match.call(),
    distribution = "Weibull",
    modcv = FALSE,
    p = p,
    conf = conf,
    override = override,
    data = eval_tidy(enquo(x), data),
    groups = NA,
    batch = eval_tidy(enquo(batch), data)
  )

  res$diagnostic_results <- perform_checks(
    basis_weibull_rules, x = res$data, batch = res$batch, override = override
  )
  res$diagnostic_failures <- get_check_failure_names(res$diagnostic_results)

  dist <- fitdistr(res$data, "weibull")

  alpha_hat <- dist$estimate[["scale"]]
  beta_hat <- dist$estimate[["shape"]]

  # The data must be transformed to fit an extreme value distribution
  data_evd <- log(res$data)
  u_hat <- log(alpha_hat)
  b_hat <- 1 / beta_hat

  # Next, find the ancillary statistic for the data
  a <- (data_evd - u_hat) / b_hat

  k_integrand <- function(z) {
    return(
      z ^ (res$n - 2) * exp((z - 1) * sum(a)) /
        ((1 / res$n) * sum(exp(a * z))) ^ res$n
    )
  }

  k_inv <- integrate(Vectorize(k_integrand), lower = 0, upper = Inf)
  k <- 1 / k_inv$value

  incomplete_gamma <- function(ki, xi) {
    pchisq(xi * 2, ki * 2)
  }

  h2 <- function(z) {
    return(
      k * z ^ (res$n - 2) * exp((z - 1) * sum(a)) /
        ((1 / res$n) * sum(exp(a * z))) ^ res$n
    )
  }

  wp <- log(-log(p))

  pr_zp_integrand <- function(z, t) {
    h2(z) * incomplete_gamma(res$n, exp(wp + t * z) * sum(exp(a * z)))
  }

  pr_fcn <- function(t) {
    int_res <- integrate(Vectorize(function(z) pr_zp_integrand(z, t)), 0, Inf)
    return(int_res$value - conf)
  }

  res_root <- uniroot(pr_fcn, c(0, 10), extendInt = "yes")

  res$basis <- exp(u_hat - res_root$root * b_hat)

  return(res)
}

pooled_rules <- list(
  outliers_within_batch = function(x, groups, batch, ...) {
    group_batch_mnr <- vapply(unique(groups), function(g) {
      batch_mnr <- vapply(unique(batch), function(b) {
        x_group <- x[batch == b & groups == g]
        if (length(x_group) == 0) {
          return(TRUE)
        }
        mnr <- maximum_normed_residual(x = x_group)
        return(mnr$n_outliers == 0)
      }, FUN.VALUE = logical(1L))
      all(batch_mnr)
    }, FUN.VALUE = logical(1L))
    ifelse(all(group_batch_mnr), "",
           paste0("Maximum normed residual test detected ",
                  "outliers within one or more batch and group"))
  },
  between_group_variability = function(x_ad, groups, batch, ...) {
    group_adk <- vapply(unique(groups), function(g) {
      x_group <- x_ad[groups == g]
      batch_group <- batch[groups == g]
      adk <- ad_ksample(x = x_group, groups = batch_group)
      !adk$reject_same_dist
    }, FUN.VALUE = logical(1L))
    ifelse(all(group_adk), "",
           paste0("Anderson-Darling k-Sample test indicates that ",
                  "batches are drawn from different distributions"))
  },
  outliers_within_group = function(x, groups, ...) {
    group_mnr <- vapply(unique(groups), function(g) {
      x_group <- x[groups == g]
      mnr <- maximum_normed_residual(x = x_group)
      return(mnr$n_outliers == 0)
    }, FUN.VALUE = logical(1L))
    ifelse(all(group_mnr), "",
           paste0("Maximum normed residual test detected ",
                  "outliers within one or more group"))
  },
  pooled_data_normal = function(x_ad, groups, ...) {
    norm_x <- normalize_group_mean(x = x_ad, group = groups)
    ad <- anderson_darling_normal(x = norm_x)
    ifelse(!ad$reject_distribution, "",
           paste0("Anderson-Darling test rejects hypothesis that pooled ",
                  "data is drawn from a normal distribution"))
  }
)

pooled_rules_cv <- pooled_rules
pooled_rules_cv[["normalized_variance_equal"]] <- function(x, groups, ...) {
  norm_x <- normalize_group_mean(x = x, group = groups)
  lev <- levene_test(x = norm_x, groups = groups)
  return(ifelse(!lev$reject_equal_variance, "",
                paste0("Levene's test rejected the hypothesis that the ",
                       "variance of all groups are equal")))
}

#' @rdname basis
#' @importFrom rlang enquo eval_tidy
#' @export
basis_pooled_cv <- function(data = NULL, x, groups, batch = NULL,
                            p = 0.90, conf = 0.95, modcv = FALSE,
                            override = c()) {
  verify_tidy_input(
    df = data,
    x = x,
    c = match.call(),
    arg_name = "x")

  verify_tidy_input(
    df = data,
    x = groups,
    c = match.call(),
    arg_name = "groups")

  verify_tidy_input(
    df = data,
    x = batch,
    c = match.call(),
    arg_name = "batch")

  res <- new_basis(
    call = match.call(),
    distribution = "Normal - Pooled CV",
    modcv = modcv,
    p = p,
    conf = conf,
    override = override,
    data = eval_tidy(enquo(x), data),
    groups = eval_tidy(enquo(groups), data),
    batch = eval_tidy(enquo(batch), data)
  )

  if (modcv == TRUE) {
    res$modcv <- TRUE
    res$modcv_transformed_data <- transform_mod_cv_grouped(res$data, res$groups)
    data_to_use <- res$modcv_transformed_data
    x_ad <- transform_mod_cv_ad(res$data, res$groups, res$batch)
  } else {
    res$modcv <- FALSE
    data_to_use <- res$data
    x_ad <- data_to_use
  }

  res$diagnostic_results <- perform_checks(
    pooled_rules_cv, x = data_to_use, x_ad = x_ad,
    groups = res$groups, batch = res$batch, override = override
  )
  res$diagnostic_failures <- get_check_failure_names(res$diagnostic_results)

  norm_data <- normalize_group_mean(data_to_use, res$groups)

  pooled_sd <- sqrt(sum((norm_data - 1) ^ 2) / (res$n - res$r))

  basis <- vapply(levels(as.factor(res$groups)), function(g) {
    nj <- length(data_to_use[res$groups == g])
    z <- qnorm(p)
    suppressWarnings(
      kj <- qt(conf, df = res$n - res$r, ncp = z * sqrt(nj)) / sqrt(nj)
    )
    xj_bar <- mean(data_to_use[res$groups == g])
    xj_bar * (1 - kj * pooled_sd)
  }, FUN.VALUE = numeric(1L))

  res$basis <- data.frame(group = names(basis), value = basis)

  return(res)
}

pooled_rules_sd <- pooled_rules
pooled_rules_sd[["pooled_variance_equal"]] <- function(x, groups, ...) {

  lev <- levene_test(x = x, groups = groups)
  return(ifelse(!lev$reject_equal_variance, "",
                paste0("Levene's test rejected the hypothesis that the ",
                       "variance of all conditions are equal")))
}

#' @rdname basis
#' @importFrom rlang enquo eval_tidy
#' @export
basis_pooled_sd <- function(data = NULL, x, groups, batch = NULL,
                            p = 0.90, conf = 0.95, modcv = FALSE,
                            override = c()) {
  verify_tidy_input(
    df = data,
    x = x,
    c = match.call(),
    arg_name = "x")

  verify_tidy_input(
    df = data,
    x = groups,
    c = match.call(),
    arg_name = "groups")

  verify_tidy_input(
    df = data,
    x = batch,
    c = match.call(),
    arg_name = "batch")

  res <- new_basis(
    call = match.call(),
    distribution = "Normal - Pooled Standard Deviation",
    modcv = modcv,
    p = p,
    conf = conf,
    override = override,
    data = eval_tidy(enquo(x), data),
    groups = eval_tidy(enquo(groups), data),
    batch = eval_tidy(enquo(batch), data)
  )

  if (modcv == TRUE) {
    res$modcv <- TRUE
    res$modcv_transformed_data <- transform_mod_cv_grouped(res$data, res$groups)
    data_to_use <- res$modcv_transformed_data
    x_ad <- transform_mod_cv_ad(res$data, res$groups, res$batch)
  } else {
    res$modcv <- FALSE
    data_to_use <- res$data
    x_ad <- data_to_use
  }

  res$diagnostic_results <- perform_checks(
    pooled_rules_sd, x = data_to_use, x_ad = x_ad,
    groups = res$groups, batch = res$batch, override = override
  )
  res$diagnostic_failures <- get_check_failure_names(res$diagnostic_results)

  pooled_sd <- sqrt(
    sum(
      vapply(levels(as.factor(res$groups)), function(g) {
        xj_bar <- mean(data_to_use[res$groups == g])
        sum((data_to_use[res$groups == g] - xj_bar) ^ 2)
      }, FUN.VALUE = numeric(1L))
    ) / (res$n - res$r))

  basis <- vapply(levels(as.factor(res$groups)), function(g) {
    nj <- length(data_to_use[res$groups == g])
    z <- qnorm(p)
    suppressWarnings(
      kj <- qt(conf, df = res$n - res$r, ncp = z * sqrt(nj)) / sqrt(nj)
    )
    xj_bar <- mean(data_to_use[res$groups == g])
    xj_bar - kj * pooled_sd
  }, FUN.VALUE = numeric(1L))

  res$basis <- data.frame(group = names(basis), value = basis)

  return(res)
}

#' @importFrom stats pbeta dbeta
hk_ext_h <- function(z, n, i, j, p) {
  if (!(1 <= i && i < j && j <= n)) {
    # This function is called internally, so i, j and n should always be valid
    stop("Error: The condition 1 <= i < j <= n must be true.")  # nocov
  }

  # for z >= 1
  qb <- pbeta(1 - p, j, n - j + 1)
  int <- integrate(function(t) {
      pbeta(((1 - p) / t) ^ (1 / z), i, j - i) * dbeta(t, j, n - j + 1)
    },
    lower = 1 - p, upper = 1)
  if (int$message != "OK") {
    warning(int$message)  # nocov
  }
  qb + int$value
}

#' Calculate values related to Extended Hanson--Koopmans tolerance bounds
#'
#' @description
#' Calculates values related to Extended Hanson--Koopmans tolerance bounds
#' as described by Vangel (1994).
#'
#' @param n the sample size
#' @param i the first order statistic (1 <= i < j)
#' @param j the second order statistic (i < j <= n)
#' @param p the content of the tolerance bound (normally 0.90 or 0.99)
#' @param conf the confidence level (normally 0.95)
#'
#' @return
#' For \code{hk_ext_z}, the return value is a numeric value representing
#' the parameter z (denoted as k in CMH-17-1G).
#'
#' For \code{hk_ext_z_j_opt}, the return value is named list containing
#' \code{z} and \code{k}. The former is the value of z, as defined by
#' Vangel (1994), and the latter is the corresponding order statistic.
#'
#' @details
#' Hanson (1964) presents a nonparametric method for determining
#' tolerance bounds based on consecutive order statistics.
#' Vangel (1994) extends this method using non-consecutive order statistics.
#'
#' The extended Hanson--Koopmans method calculates a tolerance bound
#' (basis value) based on two order statistics and a weighting value
#' \code{z}. The value of \code{z} is based on the sample size, which
#' order statistics are selected, the desired content of the tolerance
#' bond and the desired confidence level.
#'
#' The function \code{hk_ext_z} calculates the weighting variable \code{z}
#' based on selected order statistics \code{i} and \code{j}. Based on this
#' value \code{z}, the tolerance bound can be calculated as:
#'
#' \deqn{S = z X_{(i)} + (1 - z) X_{(j)}}{S = z X(i) + (1 - z) X(j)}
#'
#' Where \eqn{X_{(i)}}{X(i)} and \eqn{X_{(j)}}{X(j)} are the \code{i-th}
#' and \code{j-th} ordered observation.
#'
#' The function \code{hk_ext_z_j_opt} determines the value of \code{j} and
#' the corresponding value of \code{z}, assuming \code{i=1}. The value
#' of \code{j} is selected such that the computed tolerance limit is
#' nearest to the desired population quantile for a standard normal
#' distribution when the order statistics are equal to the expected
#' value of the order statistics for the standard normal distribution.
#'
#' @references
#' M. Vangel, “One-Sided Nonparametric Tolerance Limits,”
#' Communications in Statistics - Simulation and Computation,
#' vol. 23, no. 4. pp. 1137–1154, 1994.
#'
#' D. L. Hanson and L. H. Koopmans,
#' “Tolerance Limits for the Class of Distributions with Increasing
#' Hazard Rates,” The Annals of Mathematical Statistics,
#' vol. 35, no. 4. pp. 1561–1570, 1964.
#'
#' @examples
#' # The factors from Table 1 of Vangel (1994) can be recreated
#' # using the hk_ext_z function. For the sample size n=21,
#' # the median is the 11th ordered observation. The factor
#' # required for calculating the tolerance bound with a content
#' # of 0.9 and a confidence level of 0.95 based on the median
#' # and first ordered observation can be calculated as follows.
#' hk_ext_z(n = 21, i = 1, j = 11, p = 0.9, conf = 0.95)
#'
#' ## [1] 1.204806
#'
#' # The hk_ext_z_j_opt function can be used to refine this value
#' # of z by finding an optimum value of j, rather than simply
#' # using the median. Here, we find that the optimal observation
#' # to use is the 10th, not the 11th (which is the median).
#' hk_ext_z_j_opt(n = 21, p = 0.9, conf = 0.95)
#'
#' ## $z
#' ## [1] 1.217717
#' ##
#' ## $j
#' ## [1] 10
#'
#' @seealso \code{\link{basis_hk_ext}}
#'
#' @name hk_ext
#'
#' @rdname hk_ext
#' @export
hk_ext_z <- function(n, i, j, p, conf) {
  res <- uniroot(
    function(z) {
      hk_ext_h(z, n, i, j, p) - conf
    },
    lower = 1, upper = 10,
    extendInt = "upX")
  z <- res$root
  z
}

#' @rdname hk_ext
#' @export
hk_ext_z_j_opt <- function(n, p, conf) {
  i <- 1  # i is always 1

  # an approximation of the expected value of the order statistic
  # for a standard normal distribution. This approximation is
  # known to be incorrect, especially in the tails of the
  # distribution. However, it appears to be close enough
  # for the purposes of determining which value of j is optimum
  # per Vangel's approach.
  expected_order_statistic <- function(i, n) {
    qnorm((i - 0.5) / n)
  }

  # Try all the allowable values of j to find the value of T
  # that is closest to the population quantile for a standard
  # normal distribution
  j <- (i + 1):n
  z_vals <- vapply(j, function(ji) {
    hk_ext_z(n, i, ji, p, conf)
  }, FUN.VALUE = numeric(1L))

  err_vals <- vapply(seq(along.with = j), function(index) {
    ji <- j[index]
    zi <- z_vals[index]
    e1 <- expected_order_statistic(i, n)
    e2 <- expected_order_statistic(ji, n)
    abs(zi * e1 + (1 - zi) * e2 - qnorm(p))
  }, FUN.VALUE = numeric(1L))

  list(
    z = z_vals[err_vals == min(err_vals)],
    j = j[err_vals == min(err_vals)]
  )
}

basis_hk_ext_rules <- single_point_rules
basis_hk_ext_rules[["correct_method_used"]] <-
  function(method, p, conf, ...) {
    if (p == 0.90 & conf == 0.95) {
      # B-Basis
      return(ifelse(method == "optimum-order", "",
                    paste0("For B-Basis, the optimum order method ",
                           "should be used")))
    } else if (p == 0.99 & conf == 0.95) {
      # A-Basis
      return(ifelse(method == "woodward-frawley", "",
                    paste0("For A-Basis, the Woodward-Frawley method ",
                           "should be used")))
    } else {
      return("")
    }
  }
basis_hk_ext_rules[["sample_size"]] <-
  function(n, p, conf, ...) {
    if (p == 0.90 & conf == 0.95) {
      # B-Basis
      return(ifelse(n <= 28, "",
                    paste0("For B-Basis, Hanson-Koopmans should only be ",
                           "used for samples of 28 or fewer observations")))
    } else if (p == 0.99 & conf == 0.95) {
      # A-Basis
      return(ifelse(n <= 299, "",
                    paste0("For A-Basis, Hanson-Koopmans should only be ",
                           "used for samples of 299 or fewer observations")))
    } else {
      return("")
    }
  }

#' @rdname basis
#' @importFrom rlang enquo eval_tidy
#'
#' @export
basis_hk_ext <- function(data = NULL, x, batch = NULL, p = 0.90, conf = 0.95,
                       method = c("optimum-order", "woodward-frawley"),
                       override = c()) {
  method <- match.arg(method)

  verify_tidy_input(
    df = data,
    x = x,
    c = match.call(),
    arg_name = "x")

  verify_tidy_input(
    df = data,
    x = batch,
    c = match.call(),
    arg_name = "batch")

  res <- new_basis(
    call = match.call(),
    distribution = paste0(
      "Nonparametric (Extended Hanson-Koopmans, ",
      ifelse(method == "optimum-order", "optimum two-order-statistic method",
             "Woodward-Frawley method"),
      ")"),
    modcv = FALSE,
    p = p,
    conf = conf,
    override = override,
    data = eval_tidy(enquo(x), data),
    groups = NA,
    batch = eval_tidy(enquo(batch), data)
  )

  res$diagnostic_results <- perform_checks(
    basis_hk_ext_rules, x = res$data, batch = res$batch, n = res$n,
    p = res$p, conf = res$conf, method = method, override = override
  )
  res$diagnostic_failures <- get_check_failure_names(res$diagnostic_results)

  if (method == "optimum-order") {
    zj <- hk_ext_z_j_opt(res$n, p, conf)
    z <- zj$z
    j <- zj$j
  } else if (method == "woodward-frawley") {
    j <- res$n
    z <- hk_ext_z(res$n, 1, j, p, conf)
  } else {
    stop("Invalid value for method.")  # nocov
  }

  x_ordered <- sort(res$data)
  res$basis <- x_ordered[j] * (x_ordered[1] / x_ordered[j]) ^ z

  return(res)
}

#' Rank for distribution-free tolerance bound
#'
#' @description
#' Calculates the rank order for finding distribution-free tolerance
#' bounds for large samples. This function should only be used for
#' computing B-Basis for samples larger than 28 or A-Basis for samples
#' larger than 298. This function is used by
#' \code{\link{basis_nonpara_large_sample}}.
#'
#' @param n the sample size
#' @param p the desired content for the tolerance bound
#' @param conf the confidence level for the desired tolerance bound
#'
#' @return
#' The rank corresponding with the desired tolerance bound
#'
#' @details
#' This function uses the sum of binomial terms to determine the rank
#' of the ordered statistic that corresponds with the desired tolerance
#' limit. This approach does not assume any particular distribution. This
#' approach is described by Guenther (1969) and by CMH-17-1G.
#'
#' The results of this function have been verified against the tables in
#' CMH-17-1G and agreement was found for all sample sizes published in
#' CMH-17-1G for both A- and B-Basis, as well as the sample sizes
#' \code{n+1} and \code{n-1}, where
#' \code{n} is the sample size published in CMH-17-1G.
#'
#' The tables in CMH-17-1G purportedly list the smallest sample sizes
#' for which a particular rank can be used. That is, for a sample size
#' one less than the \code{n} published in the table, the next lowest rank
#' would be used. In some cases, the results of this function disagree by a
#' rank of one for sample sizes one less than the \code{n} published in the
#' table. This indicates a disagreement in that sample size at which
#' the rank should change. This is likely due to numerical
#' differences in this function and the procedure used to generate the tables.
#' However, the disagreement is limited to sample 6500 for A-Basis; no
#' discrepancies have been identified for B-Basis. Since these sample sizes are
#' uncommon for composite materials
#' testing, and the difference between subsequent order statistics will be
#' very small for samples this large, this difference will have no practical
#' effect on computed tolerance bounds.
#'
#' @references
#' W. Guenther, “Determination of Sample Size for Distribution-Free
#' Tolerance Limits,” Jan. 1969.
#' Available online: \url{https://www.duo.uio.no/handle/10852/48686}
#'
#' “Composite Materials Handbook, Volume 1. Polymer Matrix Composites
#' Guideline for Characterization of Structural Materials,” SAE International,
#' CMH-17-1G, Mar. 2012.
#'
#' @seealso \code{\link{basis_nonpara_large_sample}}
#'
#' @examples
#' nonpara_binomial_rank(n = 1693, p = 0.99, conf = 0.95)
#' ## [1] 11
#'
#' # The above example indicates that for a sample of 1693 observations,
#' # the A-Basis is best approximated as the 11th ordered observation.
#' # In the example below, the same ordered observation would also be used
#' # for a sample of size 1702.
#'
#' nonpara_binomial_rank(n = 1702, p = 0.99, conf = 0.95)
#' ## [1] 11
#'
#' @export
nonpara_binomial_rank <- function(n, p, conf) {
  p_orig <- p
  p <- 1 - p

  e_fcn <- function(r) {
    sum(vapply(r:n, function(w) {
      exp(lchoose(n, w) + w * log(p) + (n - w) * log(1 - p))
    }, FUN.VALUE = numeric(1L)))
  }

  r1 <- 1
  e1 <- e_fcn(r1)

  if (e1 < conf) {
    stop(paste0(
      "Sample size ", n, " is too small to compute a non-parametric ",
      "tolerance limit for p=", p_orig, " and conf=", conf))
  }

  r2 <- n
  e2 <- e_fcn(r2)

  if (e2 > conf) {
    stop(paste0(
      "No rank found for n=", n, ", p=", p_orig, " conf=", conf))
  }

  for (i in 1:n) {
    # use a for loop just to give it a limit to prevent infinite loope
    if (abs(r2 - r1) == 1) {
      break
    }
    rm <- round((r1 + r2) / 2, digits = 0)
    em <- e_fcn(rm)

    # nolint start
    # We know that the following holds, and we want this to continue to hold:
    # E1 > conf
    # E2 < conf
    # nolint end
    if (em > conf) {
      r1 <- rm
      e1 <- em
    } else {
      r2 <- rm
      e2 <- em
    }
  }
  r1
}

nonpara_large_sample_rules <- single_point_rules
nonpara_large_sample_rules[["sample_size"]] <-
  function(n, p, conf, ...) {
    if (p == 0.90 & conf == 0.95) {
      # B-Basis
      return(ifelse(n >= 28, "",
                    paste0("This method should only be used for ",
                           "B-Basis for sample sizes larger than 28")))
    } else if (p == 0.99 & conf == 0.95) {
      # A-Basis
      return(ifelse(n >= 299, "",
                    paste0("This method should only be used for ",
                           "A-Basis for sample sizes larger than 299")))
    } else {
      return(TRUE)
    }
  }

#' @rdname basis
#' @importFrom rlang enquo eval_tidy
#'
#' @export
basis_nonpara_large_sample <- function(data = NULL, x, batch = NULL,
                                       p = 0.90, conf = 0.95,
                                       override = c()) {
  verify_tidy_input(
    df = data,
    x = x,
    c = match.call(),
    arg_name = "x")

  verify_tidy_input(
    df = data,
    x = batch,
    c = match.call(),
    arg_name = "batch")

  res <- new_basis(
    call = match.call(),
    distribution = "Nonparametric (large sample)",
    modcv = FALSE,
    p = p,
    conf = conf,
    override = override,
    data = eval_tidy(enquo(x), data),
    groups = NA,
    batch = eval_tidy(enquo(batch), data)
  )

  res$diagnostic_results <- perform_checks(
    nonpara_large_sample_rules, x = res$data, batch = res$batch, n = res$n,
    p = res$p, conf = res$conf, override = override
  )
  res$diagnostic_failures <- get_check_failure_names(res$diagnostic_results)

  x_ordered <- sort(res$data)
  r <- nonpara_binomial_rank(res$n, p, conf)
  res$basis <- x_ordered[r]

  return(res)
}

anova_rules <- list(
  outliers_within_group = function(x, groups, ...) {
    group_mnr <- vapply(unique(groups), function(b) {
      x_group <- x[groups == b]
      mnr <- maximum_normed_residual(x = x_group)
      mnr$n_outliers == 0
    }, FUN.VALUE = logical(1L))
    ifelse(all(group_mnr), "",
           paste0("Maximum normed residual test detected ",
                  "outliers within one or more batch"))
  },
  equality_of_variance = function(x, groups, ...) {
    lt <- levene_test(x = x, groups = groups)
    ifelse(!lt$reject_equal_variance, "",
           paste0("Levene's test rejected the hypothesis that the ",
                  "variance of all groups is equal"))
  },
  number_of_groups = function(r, ...) {
    ifelse(r >= 5, "",
           "ANOVA should only be used for 5 or more groups")
  }
)

#' @rdname basis
#' @importFrom rlang enquo eval_tidy
#' @export
basis_anova <- function(data = NULL, x, groups, p = 0.90, conf = 0.95,
                        override = c()) {
  verify_tidy_input(
    df = data,
    x = x,
    c = match.call(),
    arg_name = "x")

  verify_tidy_input(
    df = data,
    x = groups,
    c = match.call(),
    arg_name = "groups")

  res <- new_basis(
    call = match.call(),
    distribution = "ANOVA",
    modcv = FALSE,
    p = p,
    conf = conf,
    override = override,
    data = eval_tidy(enquo(x), data),
    groups = eval_tidy(enquo(groups), data),
    batch = NA
  )

  if (res$r < 2) {
    stop("ANOVA cannot be computed with fewer than 2 groups")
  }

  res$diagnostic_results <- perform_checks(
    rules = anova_rules, x = res$data, groups = res$groups,
    r = res$r, override = override
  )
  res$diagnostic_failures <- get_check_failure_names(res$diagnostic_results)

  grand_mean <- mean(res$data)

  ssb <- sum(vapply(
    levels(as.factor(res$groups)),
    function(g) {
      group_data <- res$data[res$groups == g]
      length(group_data) * mean(group_data) ^ 2
    },
    FUN.VALUE = numeric(1L)
  )) - res$n * grand_mean ^ 2

  sst <- sum(vapply(
    res$data,
    function(xi) {
      xi ^ 2
    },
    FUN.VALUE = numeric(1L)
  )) - res$n * grand_mean ^ 2

  sse <- sst - ssb

  msb <- ssb / (res$r - 1)
  mse <- sse / (res$n - res$r)

  n_star <- sum(vapply(
    levels(as.factor(res$groups)),
    function(g) {
      group_data <- res$data[res$group == g]
      length(group_data) ^ 2 / res$n
    },
    FUN.VALUE = numeric(1L)
  ))

  effective_batch <- (res$n - n_star) / (res$r - 1)

  pop_sd <- sqrt(
    msb / effective_batch + (effective_batch - 1) / effective_batch * mse
  )

  u <- msb / mse

  k0 <- k_factor_normal(res$n, p, conf)
  k1 <- k_factor_normal(res$r, p, conf)

  w <- sqrt(u / (u + effective_batch - 1))

  tol_factor <- (k0 - k1 / sqrt(effective_batch) + (k1 - k0) * w) /
    (1 - 1 / sqrt(effective_batch))

  res$basis <- grand_mean - tol_factor * pop_sd

  return(res)
}

suppressMessages(library(dplyr))


# data from CMH-17-1G Section 8.3.11.1.1
cmh_17_8_3_11_1_1 <- tribble(
  ~batch, ~strength, ~condition,
  "A", 118.3774604, "CTD", "A", 84.9581364, "RTD", "A", 83.7436035, "ETD",
  "A", 123.6035612, "CTD", "A", 92.4891822, "RTD", "A", 84.3831677, "ETD",
  "A", 115.2238092, "CTD", "A", 96.8212659, "RTD", "A", 94.8030433, "ETD",
  "A", 112.6379744, "CTD", "A", 109.030325, "RTD", "A", 94.3931537, "ETD",
  "A", 116.5564277, "CTD", "A", 97.8212659, "RTD", "A", 101.702222, "ETD",
  "A", 123.1649896, "CTD", "A", 100.921519, "RTD", "A", 86.5372121, "ETD",
  "B", 128.5589027, "CTD", "A", 103.699444, "RTD", "A", 92.3772684, "ETD",
  "B", 113.1462103, "CTD", "B", 93.7908212, "RTD", "B", 89.2084024, "ETD",
  "B", 121.4248107, "CTD", "B", 107.526709, "RTD", "B", 100.686001, "ETD",
  "B", 134.3241906, "CTD", "B", 94.5769704, "RTD", "B", 81.0444192, "ETD",
  "B", 129.6405117, "CTD", "B", 93.8831373, "RTD", "B", 91.3398070, "ETD",
  "B", 117.9818658, "CTD", "B", 98.2296605, "RTD", "B", 93.1441939, "ETD",
  "C", 115.4505226, "CTD", "B", 111.346590, "RTD", "B", 85.8204168, "ETD",
  "C", 120.0369467, "CTD", "B", 100.817538, "RTD", "C", 94.8966273, "ETD",
  "C", 117.1631088, "CTD", "C", 100.382203, "RTD", "C", 95.8068520, "ETD",
  "C", 112.9302797, "CTD", "C", 91.5037811, "RTD", "C", 86.7842252, "ETD",
  "C", 117.9114501, "CTD", "C", 100.083233, "RTD", "C", 94.4011973, "ETD",
  "C", 120.1900159, "CTD", "C", 95.6393615, "RTD", "C", 96.7231171, "ETD",
  "C", 110.7295966, "CTD", "C", 109.304779, "RTD", "C", 89.9010384, "ETD",
  "C", 100.078562, "RTD", "C", 99.1205847, "RTD", "C", 89.3672306, "ETD",
  "A", 106.357525, "ETW", "A", 99.0239966, "ETW2",
  "A", 105.898733, "ETW", "A", 103.341238, "ETW2",
  "A", 88.4640082, "ETW", "A", 100.302130, "ETW2",
  "A", 103.901744, "ETW", "A", 98.4634133, "ETW2",
  "A", 80.2058219, "ETW", "A", 92.2647280, "ETW2",
  "A", 109.199597, "ETW", "A", 103.487693, "ETW2",
  "A", 61.0139431, "ETW", "A", 113.734763, "ETW2",
  "B", 99.3207107, "ETW", "B", 108.172659, "ETW2",
  "B", 115.861770, "ETW", "B", 108.426732, "ETW2",
  "B", 82.6133082, "ETW", "B", 116.260375, "ETW2",
  "B", 85.3690411, "ETW", "B", 121.049610, "ETW2",
  "B", 115.801622, "ETW", "B", 111.223082, "ETW2",
  "B", 44.3217741, "ETW", "B", 104.574843, "ETW2",
  "B", 117.328077, "ETW", "B", 103.222552, "ETW2",
  "B", 88.6782903, "ETW", "C", 99.3918538, "ETW2",
  "C", 107.676986, "ETW", "C", 87.3421658, "ETW2",
  "C", 108.960241, "ETW", "C", 102.730741, "ETW2",
  "C", 116.122640, "ETW", "C", 96.3694916, "ETW2",
  "C", 80.2334815, "ETW", "C", 99.5946088, "ETW2",
  "C", 106.145570, "ETW", "C", 97.0712407, "ETW2",
  "C", 104.667866, "ETW",
  "C", 104.234953, "ETW"
)

test_that("expected diagnostic failures are noted for pooling methods", {
  # This test follows CMH-17-1G Section
  # This section in CMH-17-1G shows the removal of one condition
  # before running Levene's test on the pooled data, so this test
  # will be skipped in this test.
  expect_warning(
    expect_warning(
      expect_warning(
        expect_warning(
          expect_warning(
            res <- basis_pooled_sd(cmh_17_8_3_11_1_1, strength, condition,
                                   batch),
            "outliers_within_batch"
          ),
          "between_group_variability"
        ),
        "outliers_within_group"
      ),
      "pooled_data_normal"
    ),
    "pooled_variance_equal"
  )

  expect_length(res$diagnostic_obj$outliers_within_batch, 5)
  expect_length(res$diagnostic_obj$outliers_within_batch$RTD, 3)
  expect_equal(class(res$diagnostic_obj$outliers_within_batch$CTD$A), "mnr")
  expect_equal(res$diagnostic_obj$outliers_within_batch$CTD$A$n_outliers, 0)

  expect_length(res$diagnostic_obj$between_group_variability, 5)
  expect_equal(class(res$diagnostic_obj$between_group_variability$RTD), "adk")
  expect_false(
    res$diagnostic_obj$between_group_variability$RTD$reject_same_dist)

  expect_length(res$diagnostic_obj$outliers_within_group, 5)
  expect_equal(class(res$diagnostic_obj$outliers_within_group$CTD), "mnr")
  expect_equal(res$diagnostic_obj$outliers_within_group$CTD$n_outliers, 0)

  expect_equal(class(res$diagnostic_obj$pooled_data_normal), "anderson_darling")
  expect_true(res$diagnostic_obj$pooled_data_normal$reject_distribution)

  expect_equal(class(res$diagnostic_obj$pooled_variance_equal), "levene")
  expect_true(res$diagnostic_obj$pooled_variance_equal$reject_equal_variance)

  expect_warning(
    expect_warning(
      expect_warning(
        expect_warning(
          res <- cmh_17_8_3_11_1_1 %>%
            filter(condition != "ETW2") %>%
            basis_pooled_sd(strength, condition, batch),
          "outliers_within_batch"
        ),
        "outliers_within_group"
      ),
      "pooled_data_normal"
    ),
    "pooled_variance_equal"
  )

  # removing both ETW and ETW2 should remove all diagnostic failures
  res <- cmh_17_8_3_11_1_1 %>%
    filter(condition != "ETW2" & condition != "ETW") %>%
    basis_pooled_sd(strength, condition, batch)

  expect_equal(res$basis$value[res$basis$group == "CTD"], 108.70,
               tolerance = 0.02)
  expect_equal(res$basis$value[res$basis$group == "RTD"], 88.52,
               tolerance = 0.02)
  expect_equal(res$basis$value[res$basis$group == "ETD"], 80.68,
               tolerance = 0.02)

  expect_warning(
    expect_warning(
      expect_warning(
        expect_warning(
          expect_warning(
            res <- basis_pooled_cv(cmh_17_8_3_11_1_1, strength,
                                   condition, batch),
            "outliers_within_batch"
          ),
          "between_group_variability"
        ),
        "outliers_within_group"
      ),
      "pooled_data_normal"
    ),
    "normalized_variance_equal"
  )

  expect_warning(
    expect_warning(
      expect_warning(
        expect_warning(
          res <- cmh_17_8_3_11_1_1 %>%
            filter(condition != "ETW2") %>%
            basis_pooled_cv(strength, condition, batch),
          "outliers_within_batch"
        ),
        "outliers_within_group"
      ),
      "pooled_data_normal"
    ),
    "normalized_variance_equal"
  )

  # removing both ETW and ETW2 should remove all diagnostic failures
  res <- cmh_17_8_3_11_1_1 %>%
    filter(condition != "ETW2" & condition != "ETW") %>%
    basis_pooled_cv(strength, condition, batch)
})

# data from CMH-17-1G Section 8.3.11.1.2
poolable_data <- tribble(
  ~batch, ~strength, ~condition,
  1, 79.04517, "CTD",
  1, 102.6014, "CTD",
  1, 97.79372, "CTD",
  1, 92.86423, "CTD",
  1, 117.218,  "CTD",
  1, 108.7168, "CTD",
  1, 112.2773, "CTD",
  1, 114.0129, "CTD",
  2, 106.8452, "CTD",
  2, 112.3911, "CTD",
  2, 115.5658, "CTD",
  2, 87.40657, "CTD",
  2, 102.2785, "CTD",
  2, 110.6073, "CTD",
  3, 105.2762, "CTD",
  3, 110.8924, "CTD",
  3, 108.7638, "CTD",
  3, 110.9833, "CTD",
  3, 101.3417, "CTD",
  3, 100.0251, "CTD",
  1, 103.2006, "RTD",
  1, 105.1034, "RTD",
  1, 105.1893, "RTD",
  1, 100.4189, "RTD",
  2, 85.32319, "RTD",
  2, 92.69923, "RTD",
  2, 98.45242, "RTD",
  2, 104.1014, "RTD",
  2, 91.51841, "RTD",
  2, 101.3746, "RTD",
  2, 101.5828, "RTD",
  2, 99.57384, "RTD",
  2, 88.84826, "RTD",
  3, 92.18703, "RTD",
  3, 101.8234, "RTD",
  3, 97.68909, "RTD",
  3, 101.5172, "RTD",
  3, 100.0481, "RTD",
  3, 102.0544, "RTD",
  1, 63.22764, "ETW",
  1, 70.84454, "ETW",
  1, 66.43223, "ETW",
  1, 75.37771, "ETW",
  1, 72.43773, "ETW",
  1, 68.43073, "ETW",
  1, 69.72524, "ETW",
  2, 66.20343, "ETW",
  2, 60.51251, "ETW",
  2, 65.69334, "ETW",
  2, 62.73595, "ETW",
  2, 59.00798, "ETW",
  2, 62.37761, "ETW",
  3, 64.3947,  "ETW",
  3, 72.8491,  "ETW",
  3, 66.56226, "ETW",
  3, 66.56779, "ETW",
  3, 66.00123, "ETW",
  3, 59.62108, "ETW",
  3, 60.61167, "ETW",
  3, 57.65487, "ETW",
  3, 66.51241, "ETW",
  3, 64.89347, "ETW",
  3, 57.73054, "ETW",
  3, 68.94086, "ETW",
  3, 61.63177, "ETW",
  1, 54.09806, "ETW2",
  1, 58.87615, "ETW2",
  1, 61.60167, "ETW2",
  1, 60.23973, "ETW2",
  1, 61.4808,  "ETW2",
  1, 64.55832, "ETW2",
  2, 57.76131, "ETW2",
  2, 49.91463, "ETW2",
  2, 61.49271, "ETW2",
  2, 57.7281,  "ETW2",
  2, 62.11653, "ETW2",
  2, 62.69353, "ETW2",
  3, 61.38523, "ETW2",
  3, 60.39053, "ETW2",
  3, 59.17616, "ETW2",
  3, 60.17616, "ETW2",
  3, 46.47396, "ETW2",
  3, 51.16616, "ETW2"
)

test_that("Pooled SD results match ASAP results", {
  # This data fails the anderson-darling test for normality for the
  # transformed data
  expect_warning(
    expect_message(
      expect_message(
        res_b <- basis_pooled_sd(poolable_data, strength, condition,
                                 override = c("pooled_variance_equal")),
        "outliers_within_batch"
      ),
      "between_group_variability"
    ),
    "pooled_data_normal"
  )

  expect_equal(res_b$basis$value[res_b$basis$group == "CTD"],
               93.64, tolerance = 0.01)
  expect_equal(res_b$basis$value[res_b$basis$group == "RTD"],
               87.30, tolerance = 0.01)
  expect_equal(res_b$basis$value[res_b$basis$group == "ETW"],
               54.33, tolerance = 0.01)
  expect_equal(res_b$basis$value[res_b$basis$group == "ETW2"],
               47.12, tolerance = 0.01)

  expect_equal(res_b$n, 83)
  expect_equal(res_b$r, 4)
  expect_output(print(res_b), "b-basis", ignore.case = TRUE)
  expect_output(print(res_b), "pooled standard deviation", ignore.case = TRUE)
  expect_output(print(res_b), "CTD.*93\\.6", ignore.case = TRUE)
  expect_output(print(res_b), "RTD.*87\\.29", ignore.case = TRUE)
  expect_output(print(res_b), "ETW.*54\\.3", ignore.case = TRUE)
  expect_output(print(res_b), "ETW2.*47\\.07", ignore.case = TRUE)

  res_a <- basis_pooled_sd(poolable_data, strength, condition,
                           p = 0.99, conf = 0.95,
                           override = c("pooled_data_normal",
                                        "pooled_variance_equal",
                                        "outliers_within_batch",
                                        "between_group_variability"))
  expect_equal(res_a$basis$value[res_a$basis$group == "CTD"],
               86.19, tolerance = 0.01)
  expect_equal(res_a$basis$value[res_a$basis$group == "RTD"],
               79.86, tolerance = 0.01)
  expect_equal(res_a$basis$value[res_a$basis$group == "ETW"],
               46.84, tolerance = 0.01)
  expect_equal(res_a$basis$value[res_a$basis$group == "ETW2"],
               39.69, tolerance = 0.01)
})

test_that("Pooled CV results match CMH17STATS", {
  # This data fails the anderson-darling test for normality for the
  # transformed data
  expect_warning(
    expect_message(
      expect_message(
        res_b <- basis_pooled_cv(poolable_data, strength, condition),
        "outliers_within_batch"
      ),
      "between_group_variability"
    ),
    "pooled_data_normal"
  )

  expect_equal(res_b$basis$value[res_b$basis$group == "CTD"],
               90.89, tolerance = 0.01)
  expect_equal(res_b$basis$value[res_b$basis$group == "RTD"],
               85.37, tolerance = 0.01)
  expect_equal(res_b$basis$value[res_b$basis$group == "ETW"],
               56.79, tolerance = 0.01)
  expect_equal(res_b$basis$value[res_b$basis$group == "ETW2"],
               50.55, tolerance = 0.01)

  expect_equal(res_b$n, 83)
  expect_equal(res_b$r, 4)
  expect_output(print(res_b), "b-basis", ignore.case = TRUE)
  expect_output(print(res_b), "pooled CV", ignore.case = TRUE)
  expect_output(print(res_b), "CTD.*90\\.8", ignore.case = TRUE)
  expect_output(print(res_b), "RTD.*85\\.3", ignore.case = TRUE)
  expect_output(print(res_b), "ETW.*56\\.7", ignore.case = TRUE)
  expect_output(print(res_b), "ETW2.*50\\.5", ignore.case = TRUE)

  expect_equal(class(res_b$diagnostic_obj$normalized_variance_equal), "levene")
  expect_false(
    res_b$diagnostic_obj$normalized_variance_equal$reject_equal_variance)

  res_a <- basis_pooled_cv(poolable_data, strength, condition,
                           p = 0.99, conf = 0.95,
                           override = c("pooled_data_normal",
                                        "outliers_within_batch",
                                        "between_group_variability"))
  expect_equal(res_a$basis$value[res_a$basis$group == "CTD"],
               81.62, tolerance = 0.01)
  expect_equal(res_a$basis$value[res_a$basis$group == "RTD"],
               76.67, tolerance = 0.01)
  expect_equal(res_a$basis$value[res_a$basis$group == "ETW"],
               50.98, tolerance = 0.01)
  expect_equal(res_a$basis$value[res_a$basis$group == "ETW2"],
               45.40, tolerance = 0.01)
  expect_output(print(res_a), "a-basis", ignore.case = TRUE)
})

test_that("Pooled data matches CMH17-STATS with mod CV, SD pooling", {
  # pooled SD modified CV results
  # pooled data fails Levene's test after mod CV transform
  # based on `poolable_data` dataset with ETW2 removed

  data <- filter(poolable_data, condition != "ETW2")

  res_b <- basis_pooled_sd(data, strength, condition, modcv = TRUE,
                           override = c("pooled_variance_equal",
                                        "outliers_within_batch",
                                        "between_group_variability"))

  expect_equal(res_b$basis$value[res_b$basis$group == "CTD"],
               92.25, tolerance = 0.01)
  expect_equal(res_b$basis$value[res_b$basis$group == "RTD"],
               85.91, tolerance = 0.01)
  expect_equal(res_b$basis$value[res_b$basis$group == "ETW"],
               52.97, tolerance = 0.01)

  expect_output(print(res_b), "Modified CV")

  res_a <- basis_pooled_sd(data, strength, condition,
                           p = 0.99, conf = 0.95, modcv = TRUE,
                           override = c("pooled_variance_equal",
                                        "outliers_within_batch",
                                        "between_group_variability"))
  expect_equal(res_a$basis$value[res_a$basis$group == "CTD"],
               83.81, tolerance = 0.01)
  expect_equal(res_a$basis$value[res_a$basis$group == "RTD"],
               77.48, tolerance = 0.01)
  expect_equal(res_a$basis$value[res_a$basis$group == "ETW"],
               44.47, tolerance = 0.01)
})

test_that("Pooled data matches CMH17-STATS with mod CV, CV pooling", {
  # pooled CV modified CV results
  # pooled data passes Levene's test after mod CV transform
  # based on `poolable_data` dataset with ETW2 removed

  data <- filter(poolable_data, condition != "ETW2")

  res_b <- basis_pooled_cv(data, strength, condition, modcv = TRUE,
                           override = c("outliers_within_batch",
                                        "between_group_variability"))

  expect_equal(res_b$basis$value[res_b$basis$group == "CTD"],
               90.31, tolerance = 0.01)
  expect_equal(res_b$basis$value[res_b$basis$group == "RTD"],
               84.83, tolerance = 0.01)
  expect_equal(res_b$basis$value[res_b$basis$group == "ETW"],
               56.43, tolerance = 0.01)

  expect_output(print(res_b), "Modified CV")

  res_a <- basis_pooled_cv(data, strength, condition,
                           p = 0.99, conf = 0.95, modcv = TRUE,
                           override = c("outliers_within_batch",
                                        "between_group_variability"))
  expect_equal(res_a$basis$value[res_a$basis$group == "CTD"],
               80.57, tolerance = 0.01)
  expect_equal(res_a$basis$value[res_a$basis$group == "RTD"],
               75.69, tolerance = 0.01)
  expect_equal(res_a$basis$value[res_a$basis$group == "ETW"],
               50.33, tolerance = 0.01)
})

test_that("glance for pooled methods works", {
  res <- carbon.fabric %>%
    filter(test == "WT") %>%
    basis_pooled_sd(strength, condition, batch,
                    override = c("outliers_within_batch")) %>%
    glance(TRUE)

  # 3 conditions should produce 3 basis values and hence 3 rows
  expect_equal(nrow(res), 3)
})

test_that("pooled methods process override='all'", {

  res <- basis_pooled_sd(poolable_data, strength, condition, modcv = TRUE,
                         override = "all")
  expect_equal(res$override,
               c("outliers_within_batch",
                 "between_group_variability",
                 "outliers_within_group",
                 "pooled_data_normal",
                 "pooled_variance_equal"))
  expect_length(res$diagnostic_failures, 0)

  res <- basis_pooled_cv(poolable_data, strength, condition, modcv = TRUE,
                         override = "all")
  expect_equal(res$override,
               c("outliers_within_batch",
                 "between_group_variability",
                 "outliers_within_group",
                 "pooled_data_normal",
                 "normalized_variance_equal"))
  expect_length(res$diagnostic_failures, 0)
})

suppressMessages(library(dplyr))

test_that("(normal) basis value equals mean when sd = 0", {
  expect_equal(
    (data.frame(x = rep(100, 10)) %>%
      basis_normal(x, p = 0.9, conf = 0.95,
                   override = c("anderson_darling_normal",
                                "outliers_within_batch",
                                "between_batch_variability")))$basis,
    100
  )
})

test_that("(normal) basis value approx equals percentile for large samples", {
  m <- 100
  s <- 5
  set.seed(100)  # make sure that this doesn't fail by pure chance
  q <- qnorm(0.10, m, s, lower.tail = TRUE)
  basis <- (data.frame(x = rnorm(50, m, s)) %>%
    basis_normal(x, p = 0.90, conf = 0.95,
                 override = c("outliers_within_batch",
                              "between_batch_variability")))$basis
  expect_lt(abs(basis - q), 0.1)
})

test_that("printing of basis objects works as expected", {
  set.seed(100)
  x <- c(runif(25))

  expect_output(
    print(basis_normal(x = x, p = 0.9, conf = 0.95,
                       override = c("outliers_within_batch",
                                    "between_batch_variability"))),
    "B-Basis"
  )

  expect_output(
    print(basis_normal(x = x, p = 0.99, conf = 0.95,
                       override = c("outliers_within_batch",
                                    "between_batch_variability"))),
    "A-Basis"
  )

  expect_output(
    print(basis_normal(x = x, p = 0.9, conf = 0.9,
                       override = c("outliers_within_batch",
                                    "between_batch_variability"))),
    "[^AB-]Basis"
  )

  expect_error(
    print.basis(x)  # give it the wrong type
  )
})

test_that("normal basis value matches STAT17/ASAP result", {
  data <- c(
    137.4438,
    139.5395,
    150.89,
    141.4474,
    141.8203,
    151.8821,
    143.9245,
    132.9732,
    136.6419,
    138.1723,
    148.7668,
    143.283,
    143.5429,
    141.7023,
    137.4732,
    152.338,
    144.1589,
    128.5218
  )

  res <- basis_normal(x = data, p = 0.9, conf = 0.95,
                      override = c("outliers_within_batch",
                                   "between_batch_variability"))
  expect_equal(res$basis, 129.287, tolerance = 0.0005)
  expect_output(print(res), "b-basis.*129\\.2", ignore.case = TRUE)
  expect_output(print(res), "normal", ignore.case = TRUE)

  res <- basis_normal(x = data, p = 0.99, conf = 0.95,
                      override = c("outliers_within_batch",
                                   "between_batch_variability"))
  expect_equal(res$basis, 120.336, tolerance = 0.0005)
  expect_output(print(res), "a-basis.*120\\.3", ignore.case = TRUE)
  expect_output(print(res), "normal", ignore.case = TRUE)

  expect_match(res$distribution, "normal", ignore.case = TRUE)
})

test_that("normal basis values produce expected diagnostic failures", {
  set.seed(100)
  x <- c(runif(25), runif(25, max = 2), 200)
  batch <- c(rep("A", 25), rep("B", 26))

  expect_warning(
    expect_warning(
      expect_warning(
        expect_warning(
          res <- basis_normal(x = x, batch = batch),
          "outliers_within_batch"
        ),
        "between_batch_variability"
      ),
      "outliers"
    ),
    "anderson_darling_normal"
  )

  # Check that res$... contains the correct value
  expect_equal(res$batch, batch)
  expect_equal(res$diagnostic_failures,
               c("outliers_within_batch",
                 "between_batch_variability",
                 "outliers",
                 "anderson_darling_normal"))
  expect_length(res$override, 0)

  expect_output(print(res),
                regexp = paste("failed.+",
                               "outliers_within_batch",
                               "between_batch_variability",
                               "outliers",
                               "anderson_darling_normal",
                               sep = ".+"))
  output <- capture_output(print(res))
  expect_false(grepl("overridden", output, ignore.case = TRUE))

  expect_equal(class(res$diagnostic_obj$outliers), "mnr")
  expect_gt(res$diagnostic_obj$outliers$n_outliers, 0)

  expect_length(res$diagnostic_obj$outliers_within_batch, 2)  # two batches
  expect_equal(class(res$diagnostic_obj$outliers_within_batch$A), "mnr")
  expect_equal(class(res$diagnostic_obj$outliers_within_batch$B), "mnr")
  expect_equal(res$diagnostic_obj$outliers_within_batch$A$n_outliers, 0)
  expect_gt(res$diagnostic_obj$outliers_within_batch$B$n_outliers, 0)

  expect_equal(class(res$diagnostic_obj$between_batch_variability), "adk")
  expect_true(res$diagnostic_obj$between_batch_variability$reject_same_dist)

  expect_equal(class(res$diagnostic_obj$anderson_darling_normal),
               "anderson_darling")
  expect_true(res$diagnostic_obj$anderson_darling_normal$reject_distribution)

  # overriding the diagnostics should eliminate the warnings
  res <- basis_normal(x = x, batch = batch,
                      override = c("outliers_within_batch",
                                   "between_batch_variability",
                                   "outliers",
                                   "anderson_darling_normal"))

  expect_equal(res$override,
               c("outliers_within_batch",
                 "between_batch_variability",
                 "outliers",
                 "anderson_darling_normal"))
  expect_length(res$diagnostic_failures, 0)

  expect_output(print(res),
                regexp = paste("overridden.+",
                               "outliers_within_batch",
                               "between_batch_variability",
                               "outliers",
                               "anderson_darling_normal",
                               sep = ".+"))
  output <- capture_output(print(res))
  expect_false(grepl("failed", output, ignore.case = TRUE))

  # overriding the diagnostics using "all" should do the same thing
  res <- basis_normal(x = x, batch = batch,
                      override = "all")
  expect_equal(res$override,
               c("outliers_within_batch",
                 "between_batch_variability",
                 "outliers",
                 "anderson_darling_normal"))
  expect_length(res$diagnostic_failures, 0)

  # call basis_normal without batch
  expect_warning(
    expect_warning(
      expect_message(
        expect_message(
          res <- basis_normal(x = x),
          "outliers_within_batch"
        ),
        "between_batch_variability"
      ),
      "outliers"
    ),
    "anderson_darling_normal"
  )

  # Check that res$... contains the correct value
  expect_equal(res$diagnostic_failures,
               c("outliers",
                 "anderson_darling_normal"))
  expect_length(res$override, 0)

  # overriding the diagnostics should eliminate the warnings
  res <- basis_normal(x = x,
                      override = c("outliers",
                                   "anderson_darling_normal",
                                   "outliers_within_batch",
                                   "between_batch_variability"))

  expect_equal(res$override,
               c("outliers",
                 "anderson_darling_normal",
                 "outliers_within_batch",
                 "between_batch_variability"))
  expect_length(res$diagnostic_failures, 0)
})

test_that("log-normal basis value matches STAT17 result", {
  data <- c(
    137.4438,
    139.5395,
    150.89,
    141.4474,
    141.8203,
    151.8821,
    143.9245,
    132.9732,
    136.6419,
    138.1723,
    148.7668,
    143.283,
    143.5429,
    141.7023,
    137.4732,
    152.338,
    144.1589,
    128.5218
  )

  res <- basis_lognormal(x = data, p = 0.9, conf = 0.95, override = "all")
  expect_equal(res$basis, 129.664, tolerance = 0.0005)
  expect_output(print(res), "b-basis.*129.6", ignore.case = TRUE)
  expect_output(print(res), "normal", ignore.case = TRUE)
  expect_output(print(res), "log", ignore.case = TRUE)

  res <- basis_lognormal(x = data, p = 0.99, conf = 0.95, override = "all")
  expect_equal(res$basis, 121.710, tolerance = 0.0005)
  expect_output(print(res), "a-basis.*121.7", ignore.case = TRUE)
  expect_output(print(res), "normal", ignore.case = TRUE)
  expect_output(print(res), "log", ignore.case = TRUE)

  expect_match(res$distribution, "log", ignore.case = TRUE)
  expect_match(res$distribution, "normal", ignore.case = TRUE)
})

test_that("lognormal basis values produce expected diagnostic failures", {
  set.seed(100)
  x <- c(runif(25), runif(25, max = 2), 200)
  batch <- c(rep("A", 25), rep("B", 26))

  expect_warning(
    expect_warning(
      expect_warning(
        expect_warning(
          res <- basis_lognormal(x = x, batch = batch),
          "outliers_within_batch"
        ),
        "between_batch_variability"
      ),
      "outliers"
    ),
    "anderson_darling_lognormal"
  )

  # Check that res$... contains the correct value
  expect_equal(res$batch, batch)
  expect_equal(res$diagnostic_failures,
               c("outliers_within_batch",
                 "between_batch_variability",
                 "outliers",
                 "anderson_darling_lognormal"))
  expect_length(res$override, 0)

  expect_equal(class(res$diagnostic_obj$anderson_darling_lognormal),
               "anderson_darling")
  expect_true(res$diagnostic_obj$anderson_darling_lognormal$reject_distribution)

  # overriding the diagnostics should eliminate the warnings
  res <- basis_lognormal(x = x, batch = batch,
                      override = c("outliers_within_batch",
                                   "between_batch_variability",
                                   "outliers",
                                   "anderson_darling_lognormal"))

  expect_equal(res$override,
               c("outliers_within_batch",
                 "between_batch_variability",
                 "outliers",
                 "anderson_darling_lognormal"))
  expect_length(res$diagnostic_failures, 0)

  # overriding the diagnostics with "all" should do the same thing
  res <- basis_lognormal(x = x, batch = batch,
                         override = "all")

  expect_equal(res$override,
               c("outliers_within_batch",
                 "between_batch_variability",
                 "outliers",
                 "anderson_darling_lognormal"))
  expect_length(res$diagnostic_failures, 0)

  # call basis_normal without batch
  expect_warning(
    expect_warning(
      expect_message(
        expect_message(
          res <- basis_lognormal(x = x),
          "outliers_within_batch"
        ),
        "between_batch_variability"
      ),
      "outliers"
    ),
    "anderson_darling_lognormal"
  )

  # Check that res$... contains the correct value
  expect_equal(res$diagnostic_failures,
               c("outliers",
                 "anderson_darling_lognormal"))
  expect_length(res$override, 0)

  # overriding the diagnostics should eliminate the warnings
  res <- basis_lognormal(x = x,
                      override = c("outliers",
                                   "anderson_darling_lognormal",
                                   "outliers_within_batch",
                                   "between_batch_variability"))

  expect_equal(res$override,
               c("outliers",
                 "anderson_darling_lognormal",
                 "outliers_within_batch",
                 "between_batch_variability"))
  expect_length(res$diagnostic_failures, 0)
})

test_that("Weibull basis value matches STAT17 result", {
  data <- c(
    137.4438,
    139.5395,
    150.89,
    141.4474,
    141.8203,
    151.8821,
    143.9245,
    132.9732,
    136.6419,
    138.1723,
    148.7668,
    143.283,
    143.5429,
    141.7023,
    137.4732,
    152.338,
    144.1589,
    128.5218
  )

  # stat17 B-Basis: 125.441
  # stat17 A-Basis: 109.150

  res <- basis_weibull(x = data, p = 0.9, conf = 0.95,
                       override = c("outliers_within_batch",
                                    "between_batch_variability"))
  expect_equal(res$basis, 125.441, tolerance = 0.3)
  expect_output(print(res), "b-basis.*125", ignore.case = TRUE)
  expect_output(print(res), "weibull", ignore.case = TRUE)

  res <- basis_weibull(x = data, p = 0.99, conf = 0.95,
                       override = c("outliers_within_batch",
                                    "between_batch_variability"))
  expect_equal(res$basis, 109.150, tolerance = 0.6)
  expect_output(print(res), "a-basis.*109", ignore.case = TRUE)
  expect_output(print(res), "weibull", ignore.case = TRUE)

  expect_match(res$distribution, "weibull", ignore.case = TRUE)
})

test_that("weibull basis values produce expected diagnostic failures", {
  set.seed(100)
  x <- c(rnorm(10, 100, 2), rnorm(10, 103, 2), 120)
  batch <- c(rep("A", 10), rep("B", 11))

  expect_warning(
    expect_warning(
      expect_warning(
        expect_warning(
          res <- basis_weibull(x = x, batch = batch),
          "outliers_within_batch"
        ),
        "between_batch_variability"
      ),
      "outliers"
    ),
    "anderson_darling_weibull"
  )

  # Check that res$... contains the correct value
  expect_equal(res$batch, batch)
  expect_equal(res$diagnostic_failures,
               c("outliers_within_batch",
                 "between_batch_variability",
                 "outliers",
                 "anderson_darling_weibull"))
  expect_length(res$override, 0)

  expect_equal(class(res$diagnostic_obj$anderson_darling_weibull),
               "anderson_darling")
  expect_true(res$diagnostic_obj$anderson_darling_weibull$reject_distribution)

  # overriding the diagnostics should eliminate the warnings
  res <- basis_weibull(x = x, batch = batch,
                         override = c("outliers_within_batch",
                                      "between_batch_variability",
                                      "outliers",
                                      "anderson_darling_weibull"))

  expect_equal(res$override,
               c("outliers_within_batch",
                 "between_batch_variability",
                 "outliers",
                 "anderson_darling_weibull"))
  expect_length(res$diagnostic_failures, 0)

  # overriding the diagnostics with "all" should do the same thing
  res <- basis_weibull(x = x, batch = batch,
                       override = "all")

  expect_equal(res$override,
               c("outliers_within_batch",
                 "between_batch_variability",
                 "outliers",
                 "anderson_darling_weibull"))
  expect_length(res$diagnostic_failures, 0)

  # call basis_normal without batch
  expect_warning(
    expect_warning(
      expect_message(
        expect_message(
          res <- basis_weibull(x = x),
          "outliers_within_batch"
        ),
        "between_batch_variability"
      ),
      "outliers"
    ),
    "anderson_darling_weibull"
  )

  # Check that res$... contains the correct value
  expect_equal(res$diagnostic_failures,
               c("outliers",
                 "anderson_darling_weibull"))
  expect_length(res$override, 0)

  # overriding the diagnostics should eliminate the warnings
  res <- basis_weibull(x = x,
                         override = c("outliers",
                                      "anderson_darling_weibull",
                                      "outliers_within_batch",
                                      "between_batch_variability"))

  expect_equal(res$override,
               c("outliers",
                 "anderson_darling_weibull",
                 "outliers_within_batch",
                 "between_batch_variability"))
  expect_length(res$diagnostic_failures, 0)
})

test_that("Non-parametric (small sample) basis value matches STAT17 result", {
  data <- c(
    137.4438,
    139.5395,
    150.89,
    141.4474,
    141.8203,
    151.8821,
    143.9245,
    132.9732,
    136.6419,
    138.1723,
    148.7668,
    143.283,
    143.5429,
    141.7023,
    137.4732,
    152.338,
    144.1589,
    128.5218
  )

  res <- basis_hk_ext(x = data, p = 0.9, conf = 0.95,
                      method = "optimum-order",
                      override = c("outliers_within_batch",
                                   "between_batch_variability"))
  expect_equal(res$basis, 124.156, tolerance = 0.002)
  expect_output(print(res), "b-basis.*124", ignore.case = TRUE)
  expect_output(print(res), "nonparametric", ignore.case = TRUE)
  expect_match(res$distribution, "nonparametric.*optimum", ignore.case = TRUE)

  res <- basis_hk_ext(x = data, p = 0.99, conf = 0.95,
                      method = "woodward-frawley",
                      override = c("outliers_within_batch",
                                   "between_batch_variability"))
  expect_equal(res$basis, 99.651, tolerance = 0.002)
  expect_output(print(res), "a-basis.*99", ignore.case = TRUE)
  expect_output(print(res), "nonparametric", ignore.case = TRUE)
  expect_match(res$distribution,
               "nonparametric.*Woodward-Frawley", ignore.case = TRUE)

  expect_error(basis_hk_ext(x = data, method = "something invalid",
                            override = c("outliers_within_batch",
                                         "between_batch_variability")))
})

test_that("non-para (small) basis values produce expected diag failures", {
  set.seed(100)
  x_small <- c(rnorm(10, 100, 2), rnorm(10, 103, 2), 120)
  batch_small <- c(rep("A", 10), rep("B", 11))
  x_large <- c(rnorm(200, 100, 2), rnorm(100, 103, 2), 120)
  batch_large <- c(rep("A", 200), rep("B", 101))

  # woodward-frawley is only for A-Basis. Should fail if we calculate B-Basis
  expect_warning(
    expect_warning(
      expect_warning(
        expect_warning(
          expect_warning(
            res <- basis_hk_ext(
              x = x_large, batch = batch_large, method = "woodward-frawley"),
            "outliers_within_batch"
          ),
          "between_batch_variability"
        ),
        "outliers"
      ),
      "correct_method_used"
    ),
    "sample_size"
  )

  # Check that res$... contains the correct value
  expect_equal(res$batch, batch_large)
  expect_equal(res$diagnostic_failures,
               c("outliers_within_batch",
                 "between_batch_variability",
                 "outliers",
                 "correct_method_used",
                 "sample_size"))
  expect_length(res$override, 0)

  expect_type(res$diagnostic_obj$correct_method_used, "logical")
  expect_false(res$diagnostic_obj$correct_method_used)
  expect_type(res$diagnostic_obj$sample_size, "logical")
  expect_false(res$diagnostic_obj$sample_size)

  # overriding the diagnostics should eliminate the warnings
  res <- basis_hk_ext(x = x_large, batch = batch_large,
                      method = "woodward-frawley",
                      override = c("outliers_within_batch",
                                   "between_batch_variability",
                                   "outliers",
                                   "correct_method_used",
                                   "sample_size"))

  expect_equal(res$override,
               c("outliers_within_batch",
                 "between_batch_variability",
                 "outliers",
                 "correct_method_used",
                 "sample_size"))
  expect_length(res$diagnostic_failures, 0)

  # overriding the diagnostics with "all" should do the same thing
  res <- basis_hk_ext(x = x_large, batch = batch_large,
                      method = "woodward-frawley",
                      override = "all")
  expect_equal(res$override,
               c("outliers_within_batch",
                 "between_batch_variability",
                 "outliers",
                 "correct_method_used",
                 "sample_size"))
  expect_length(res$diagnostic_failures, 0)

  # optimum-order is only for B-Basis. Should fail if we calculate A-Basis
  expect_warning(
    expect_warning(
      expect_warning(
        expect_warning(
          expect_warning(
            res <- basis_hk_ext(
              x = x_large, batch = batch_large, method = "optimum-order",
              p = 0.99, conf = 0.95),
            "outliers_within_batch"
          ),
          "between_batch_variability"
        ),
        "outliers"
      ),
      "correct_method_used"
    ),
    "sample_size"
  )

  # call basis_normal without batch
  expect_warning(
    expect_message(
      expect_message(
        res <- basis_hk_ext(x = x_small, method = "optimum-order"),
        "outliers_within_batch"
      ),
      "between_batch_variability"
    ),
    "outliers"
  )

  # Check that res$... contains the correct value
  expect_equal(res$diagnostic_failures,
               c("outliers"))
  expect_length(res$override, 0)

  # overriding the diagnostics should eliminate the warnings
  res <- basis_hk_ext(x = x_small, method = "optimum-order",
                      override = c("outliers",
                                   "outliers_within_batch",
                                   "between_batch_variability"))

  expect_equal(res$override,
               c("outliers",
                 "outliers_within_batch",
                 "between_batch_variability"))
  expect_length(res$diagnostic_failures, 0)
})

test_that("Non-parametric (large sample) basis value matches STAT17 result", {
  data <- c(
    137.3603, 135.6665, 136.6914, 154.7919, 159.2037, 137.3277, 128.821,
    138.6304, 138.9004, 147.4598, 148.6622, 144.4948, 131.0851, 149.0203,
    131.8232, 146.4471, 123.8124, 126.3105, 140.7609, 134.4875, 128.7508,
    117.1854, 129.3088, 141.6789, 138.4073, 136.0295, 128.4164, 141.7733,
    134.455,  122.7383, 136.9171, 136.9232, 138.8402, 152.8294, 135.0633,
    121.052,  131.035,  138.3248, 131.1379, 147.3771, 130.0681, 132.7467,
    137.1444, 141.662,  146.9363, 160.7448, 138.5511, 129.1628, 140.2939,
    144.8167, 156.5918, 132.0099, 129.3551, 136.6066, 134.5095, 128.2081,
    144.0896, 141.8029, 130.0149, 140.8813, 137.7864
  )

  res <- basis_nonpara_large_sample(x = data, p = 0.9, conf = 0.95,
                                    override = "all")
  expect_equal(res$basis, 122.738297, tolerance = 0.005)
  expect_output(print(res), "b-basis.*122", ignore.case = TRUE)
  expect_output(print(res), "nonparametric", ignore.case = TRUE)
  expect_match(res$distribution, "nonparametric.*large", ignore.case = TRUE)
})

test_that("non-para (large) basis values produce expected diag failures", {
  set.seed(100)
  x_small <- c(rnorm(13, 100, 2), rnorm(13, 103, 2), 120)
  batch_small <- c(rep("A", 13), rep("B", 14))
  x_large <- c(rnorm(200, 100, 2), rnorm(100, 103, 2), 120)
  batch_large <- c(rep("A", 200), rep("B", 101))

  expect_warning(
    expect_warning(
      expect_warning(
        res <- basis_nonpara_large_sample(
          x = x_large, batch = batch_large),
        "outliers_within_batch"
      ),
      "between_batch_variability"
    ),
    "outliers"
  )

  # Check that res$... contains the correct value
  expect_equal(res$batch, batch_large)
  expect_equal(res$diagnostic_failures,
               c("outliers_within_batch",
                 "between_batch_variability",
                 "outliers"))
  expect_length(res$override, 0)

  expect_type(res$diagnostic_obj$sample_size, "logical")
  expect_true(res$diagnostic_obj$sample_size)

  # overriding the diagnostics should eliminate the warnings
  res <- basis_nonpara_large_sample(x = x_large, batch = batch_large,
                      override = c("outliers_within_batch",
                                   "between_batch_variability",
                                   "outliers"))

  expect_equal(res$override,
               c("outliers_within_batch",
                 "between_batch_variability",
                 "outliers"))
  expect_length(res$diagnostic_failures, 0)

  # overriding the diagnostics with "all" should do the same thing
  res <- basis_nonpara_large_sample(x = x_large, batch = batch_large,
                                    override = "all")
  expect_equal(res$override,
               c("outliers_within_batch",
                 "between_batch_variability",
                 "outliers",
                 "sample_size"))
  expect_length(res$diagnostic_failures, 0)

  expect_warning(
    expect_warning(
      expect_warning(
        res <- basis_nonpara_large_sample(
          x = x_large, batch = batch_large,
          p = 0.99, conf = 0.95),
        "outliers_within_batch"
      ),
      "between_batch_variability"
    ),
    "outliers"
  )

  # call basis_normal without batch
  expect_warning(
    expect_message(
      expect_message(
        res <- basis_nonpara_large_sample(x = x_large),
        "outliers_within_batch"
      ),
      "between_batch_variability"
    ),
    "outliers"
  )

  # Check that res$... contains the correct value
  expect_equal(res$diagnostic_failures,
               c("outliers"))
  expect_length(res$override, 0)

  # overriding the diagnostics should eliminate the warnings
  res <- basis_nonpara_large_sample(x = x_large,
                      override = c("outliers",
                                   "outliers_within_batch",
                                   "between_batch_variability"))

  expect_equal(res$override,
               c("outliers", "outliers_within_batch",
                 "between_batch_variability"))
  expect_length(res$diagnostic_failures, 0)
})

test_that("Hanson-Koopman results match STAT17 for several values of n", {
  data <- c(
    139.6734,
    143.0032,
    130.4757,
    144.8327,
    138.7818,
    136.7693,
    148.636,
    131.0095,
    131.4933,
    142.8856,
    158.0198,
    145.2271,
    137.5991,
    139.8298,
    140.8557,
    137.6148,
    131.3614,
    152.7795,
    145.8792,
    152.9207,
    160.0989,
    145.192,
    128.6383,
    141.5992,
    122.5297,
    159.8209,
    151.672,
    159.0156
  )

  res <- basis_hk_ext(x = head(data, 28), p = 0.9, conf = 0.95,
                      method = "optimum-order", override = "all")
  expect_equal(res$basis, 122.36798, tolerance = 0.001)

  res <- basis_hk_ext(x = head(data, 27), p = 0.9, conf = 0.95,
                      method = "optimum-order", override = "all")
  expect_equal(res$basis, 121.96939, tolerance = 0.001)

  res <- basis_hk_ext(x = head(data, 26), p = 0.9, conf = 0.95,
                      method = "optimum-order", override = "all")
  expect_equal(res$basis, 121.57073, tolerance = 0.001)

  res <- basis_hk_ext(x = head(data, 23), p = 0.9, conf = 0.95,
                      method = "optimum-order", override = "all")
  expect_equal(res$basis, 127.11286, tolerance = 0.001)

  res <- basis_hk_ext(x = head(data, 22), p = 0.9, conf = 0.95,
                      method = "optimum-order", override = "all")
  expect_equal(res$basis, 128.82397, tolerance = 0.001)

  res <- basis_hk_ext(x = head(data, 21), p = 0.9, conf = 0.95,
                      method = "optimum-order", override = "all")
  expect_equal(res$basis, 128.52107, tolerance = 0.001)

  res <- basis_hk_ext(x = head(data, 20), p = 0.9, conf = 0.95,
                      method = "optimum-order", override = "all")
  expect_equal(res$basis, 128.20999, tolerance = 0.001)

  res <- basis_hk_ext(x = head(data, 19), p = 0.9, conf = 0.95,
                      method = "optimum-order", override = "all")
  expect_equal(res$basis, 127.74060, tolerance = 0.002)

  res <- basis_hk_ext(x = head(data, 18), p = 0.9, conf = 0.95,
                      method = "optimum-order", override = "all")
  expect_equal(res$basis, 127.36697, tolerance = 0.001)

  res <- basis_hk_ext(x = head(data, 17), p = 0.9, conf = 0.95,
                      method = "optimum-order", override = "all")
  expect_equal(res$basis, 127.02732, tolerance = 0.001)

  res <- basis_hk_ext(x = head(data, 16), p = 0.9, conf = 0.95,
                      method = "optimum-order", override = "all")
  expect_equal(res$basis, 126.23545, tolerance = 0.002)

  res <- basis_hk_ext(x = head(data, 15), p = 0.9, conf = 0.95,
                      method = "optimum-order", override = "all")
  expect_equal(res$basis, 125.68740, tolerance = 0.001)

  res <- basis_hk_ext(x = head(data, 14), p = 0.9, conf = 0.95,
                      method = "optimum-order", override = "all")
  expect_equal(res$basis, 125.17500, tolerance = 0.001)

  res <- basis_hk_ext(x = head(data, 13), p = 0.9, conf = 0.95,
                      method = "optimum-order", override = "all")
  expect_equal(res$basis, 124.07851, tolerance = 0.002)

  res <- basis_hk_ext(x = head(data, 12), p = 0.9, conf = 0.95,
                      method = "optimum-order", override = "all")
  expect_equal(res$basis, 121.17418, tolerance = 0.001)

  res <- basis_hk_ext(x = head(data, 11), p = 0.9, conf = 0.95,
                      method = "optimum-order", override = "all")
  expect_equal(res$basis, 120.26382, tolerance = 0.001)

  res <- basis_hk_ext(x = head(data, 10), p = 0.9, conf = 0.95,
                      method = "optimum-order", override = "all")
  expect_equal(res$basis, 120.75149, tolerance = 0.002)

  res <- basis_hk_ext(x = head(data, 9), p = 0.9, conf = 0.95,
                      method = "optimum-order", override = "all")
  expect_equal(res$basis, 119.80108, tolerance = 0.001)
})

cmh_17_1g_8_3_11_1_1_etw2 <- tribble(
  ~batch, ~strength,
  1, 99.0239966,
  1, 103.341238,
  1, 100.30213,
  1, 98.4634133,
  1, 92.264728,
  1, 103.487693,
  1, 113.734763,
  2, 108.172659,
  2, 108.426732,
  2, 116.260375,
  2, 121.04961,
  2, 111.223082,
  2, 104.574843,
  2, 103.222552,
  3, 99.3918538,
  3, 87.3421658,
  3, 102.730741,
  3, 96.3694916,
  3, 99.5946088,
  3, 97.0712407
)

test_that("ANOVA results match STAT17 for sample data", {
  # Sample data from CMH-17-1G Section 8.3.11.2.2

  res <- cmh_17_1g_8_3_11_1_1_etw2 %>%
    basis_anova(strength, batch,
                override = c("equality_of_variance",
                             "number_of_groups"))

  expect_equal(res$basis, 63.2, tolerance = 0.05)
  expect_output(print(res), "b-basis.*63\\.2", ignore.case = TRUE)
  expect_output(print(res), "ANOVA", ignore.case = TRUE)
  expect_match(res$distribution, "ANOVA", ignore.case = TRUE)

  res <- cmh_17_1g_8_3_11_1_1_etw2 %>%
    basis_anova(strength, batch, p = 0.99, conf = 0.95,
                override = c("equality_of_variance",
                             "number_of_groups"))

  expect_equal(res$basis, 34.6, tolerance = 0.05)
  expect_output(print(res), "a-basis.*34\\.", ignore.case = TRUE)
  expect_output(print(res), "ANOVA", ignore.case = TRUE)
  expect_match(res$distribution, "ANOVA", ignore.case = TRUE)
})

test_that("ANOVA produces an error when there is only one group", {
  strength <- rep(10, 1)
  batch <- rep(10, 1)

  expect_error(
    basis_anova(x = strength, group = batch),
    "fewer than 2"
  )
})

test_that("anova basis values produce expected diagnostic failures", {
  set.seed(100)
  x <- c(rnorm(30, 100, 1), rnorm(30, 100, 10), 80)
  batch <- c(rep("A", 30), rep("B", 30), "A")

  expect_warning(
    expect_warning(
      expect_warning(
        res <- basis_anova(x = x, group = batch),
        "outliers_within_group"
      ),
      "equality_of_variance"
    ),
    "number_of_groups"
  )

  # Check that res$... contains the correct value
  expect_equal(res$group, batch)
  expect_equal(res$diagnostic_failures,
               c("outliers_within_group",
                 "equality_of_variance",
                 "number_of_groups"))
  expect_length(res$override, 0)

  expect_length(res$diagnostic_obj$outliers_within_group, 2)
  expect_equal(class(res$diagnostic_obj$outliers_within_group$A), "mnr")
  expect_equal(res$diagnostic_obj$outliers_within_group$A$n_outliers, 2)
  expect_equal(class(res$diagnostic_obj$outliers_within_group$B), "mnr")
  expect_equal(res$diagnostic_obj$outliers_within_group$B$n_outliers, 0)

  expect_equal(class(res$diagnostic_obj$equality_of_variance), "levene")
  expect_true(res$diagnostic_obj$equality_of_variance$reject_equal_variance)

  expect_false(res$diagnostic_obj$number_of_groups)

  # overriding the diagnostics should eliminate the warnings
  res <- basis_anova(x = x, group = batch,
                     override = c("outliers_within_group",
                                  "equality_of_variance",
                                  "number_of_groups"))

  expect_equal(res$override,
               c("outliers_within_group",
                 "equality_of_variance",
                 "number_of_groups"))
  expect_length(res$diagnostic_failures, 0)

  # overriding the diagnostics with "all" should do the same thing
  res <- basis_anova(x = x, group = batch,
                     override = "all")
  expect_equal(res$override,
               c("outliers_within_group",
                 "equality_of_variance",
                 "number_of_groups"))
  expect_length(res$diagnostic_failures, 0)
})

test_that("ANOVA method matches STAT17 when between-batch var. is small", {
  data <- tribble(
    ~x, ~batch,
    105.04953017290813, 1,
    105.74515635546253, 1,
    99.7549396676824, 1,
    107.44219439303261, 1,
    100.17657481474124, 1,
    106.601810738431, 1,
    101.15202811896768, 2,
    90.63466521331704, 2,
    106.93692070778634, 2,
    116.14555531325212, 2,
    100.20555336225114, 2,
    103.89002397699194, 2,
    110.50367678215923, 3,
    95.34690617376182, 3,
    105.03624331633935, 3,
    105.83852344481843, 3,
    105.8785931848096, 3,
    103.97623814685818, 3,
    94.92344509669459, 4,
    89.35739844589054, 4,
    110.45073142288507, 4,
    108.32807015574465, 4,
    104.35498641239826, 4,
    109.39785860273314, 4,
    102.88966425996772, 5,
    105.08208381529616, 5,
    109.82310733067601, 5,
    108.64289487358796, 5,
    99.87084985403291, 5,
    96.7651412720645, 5
  )
  res <- basis_anova(data, x, batch, override = "all")
  expect_equal(res$basis, 93.2, tolerance = 0.05)
})

test_that("glance.basis produces expected value", {
  # Sample data from CMH-17-1G Section 8.3.11.2.2

  res <- cmh_17_1g_8_3_11_1_1_etw2 %>%
    basis_anova(strength, batch,
                override = c("number_of_groups"))

  glance_res <- glance(res)

  expect_equal(glance_res[["p"]][1], 0.9)
  expect_equal(glance_res[["conf"]][1], 0.95)
  expect_equal(glance_res[["distribution"]][1], "ANOVA")
  expect_equal(glance_res[["n"]][1], nrow(cmh_17_1g_8_3_11_1_1_etw2))
  expect_equal(glance_res[["r"]][1], 3)
  expect_equal(glance_res[["basis"]][1], 63.2, tolerance = 0.05)

  glance_res_2 <- glance(res, TRUE)

  for (gn in names(glance_res)) {
    expect_equal(glance_res[[gn]], glance_res_2[[gn]])
  }

  expect_equal(glance_res_2[["outliers_within_group"]], "P")
  expect_equal(glance_res_2[["equality_of_variance"]], "P")
  expect_equal(glance_res_2[["number_of_groups"]], "O")

  expect_warning({
    glance_res_3 <- cmh_17_1g_8_3_11_1_1_etw2 %>%
      basis_anova(strength, batch) %>%
      glance(TRUE)
  })

  expect_equal(glance_res_3[["outliers_within_group"]], "P")
  expect_equal(glance_res_3[["equality_of_variance"]], "P")
  expect_equal(glance_res_3[["number_of_groups"]], "F")
})

#' Normalizes strength values to ply thickness
#'
#' @description
#' This function takes a vector of strength values and a
#' vector of measured thicknesses, and a nominal thickness
#' and returns the normalized strength.
#'
#' @param strength the strength to be normalized. Either a vector or a numeric
#' @param measured_thk the measured thickness of the samples. Must be the same
#' length as strength
#' @param nom_thk the nominal thickness. Must be a single numeric value.
#'
#' @return
#' The normalized strength values
#'
#' @details
#' It is often necessary to normalize strength values so that variation in
#' specimen thickness does not unnecessarily increase variation in strength.
#' See CMH-17-1G, or other references, for information about the cases where
#' normalization is appropriate.
#'
#' Either cured ply thickness or laminate thickness may be used for
#' `measured_thk` and `nom_thk`, as long as the same decision
#' made for both values.
#'
#' The formula applied is:
#' \deqn{normalized\,value = test\,value \frac{t_{measured}}{t_{nominal}}}{
#' normalized value = test value * t_measured / t_nominal}
#'
#' If you need to normalize based on fiber volume fraction (or another method),
#' you will first need to calculate the nominal cured ply thickness (or laminate
#' thickness). Those calculations are outside the scope of this documentation.
#'
#' @references
#' “Composite Materials Handbook, Volume 1. Polymer Matrix Composites
#' Guideline for Characterization of Structural Materials,” SAE International,
#' CMH-17-1G, Mar. 2012.
#'
#' @examples
#' library(dplyr)
#'
#' carbon.fabric.2 %>%
#' select(thickness, strength) %>%
#'   mutate(normalized_strength = normalize_ply_thickness(strength,
#'                                                        thickness,
#'                                                        0.105)) %>%
#'   head(10)
#'
#' ##    thickness strength normalized_strength
#' ## 1      0.112  142.817            152.3381
#' ## 2      0.113  135.901            146.2554
#' ## 3      0.113  132.511            142.6071
#' ## 4      0.112  135.586            144.6251
#' ## 5      0.113  125.145            134.6799
#' ## 6      0.113  135.203            145.5042
#' ## 7      0.113  128.547            138.3411
#' ## 8      0.113  127.709            137.4392
#' ## 9      0.113  127.074            136.7558
#' ## 10     0.114  126.879            137.7543
#'
#'
#' @importFrom dplyr select mutate
#'
#' @export
normalize_ply_thickness <- function(strength, measured_thk, nom_thk) {
  if (length(strength) != length(measured_thk)) {
    stop("strength and measured_thk must be the same length")
  }
  if (length(nom_thk) != 1) {
    stop("nom_thk must be a single numeric value (not a vector)")
  }

  return(strength * measured_thk / nom_thk)
}


#' Normalize values to group means
#'
#' @description
#' This function computes the mean of each group, then divides each
#' observation by its corresponding group mean. This is commonly done
#' when pooling data across environments.
#'
#' @param x the variable containing the data to normalized
#' @param group the variable containing the groups
#'
#' @return
#' Returns a vector of normalized values
#'
#' @details
#' Computes the mean for each group, then divides each value by the mean for
#' the corresponding group.
#'
#' @references
#' “Composite Materials Handbook, Volume 1. Polymer Matrix Composites
#' Guideline for Characterization of Structural Materials,” SAE International,
#' CMH-17-1G, Mar. 2012.
#'
#' @examples
#' library(dplyr)
#' carbon.fabric.2 %>%
#' filter(test == "WT") %>%
#'   select(condition, strength) %>%
#'   mutate(condition_norm = normalize_group_mean(strength, condition)) %>%
#'   head(10)
#'
#' ##    condition strength condition_norm
#' ## 1        CTD  142.817      1.0542187
#' ## 2        CTD  135.901      1.0031675
#' ## 3        CTD  132.511      0.9781438
#' ## 4        CTD  135.586      1.0008423
#' ## 5        CTD  125.145      0.9237709
#' ## 6        CTD  135.203      0.9980151
#' ## 7        CTD  128.547      0.9488832
#' ## 8        CTD  127.709      0.9426974
#' ## 9        CTD  127.074      0.9380101
#' ## 10       CTD  126.879      0.9365706
#'
#' @importFrom rlang enquo eval_tidy
#'
#' @export
normalize_group_mean <- function(x, group) {
  if (length(x) != length(group)) {
    stop("The length of x and groups must be equal")
  }

  if (length(x) == 0) {
    return(numeric(0))
  }

  group_means <- vapply(group, function(g) {
    cur_group <- x[group == g]
    group_mean <- mean(cur_group)
    return(group_mean)
  },
  USE.NAMES = FALSE,
  FUN.VALUE = numeric(1L))

  return(x / group_means)
}

#' Calculate the modified CV from the CV
#'
#' @description
#' This function calculates the modified coefficient of variation (CV)
#' based on a (unmodified) CV.
#' The modified CV is calculated based on the rules in CMH-17-1G. Those
#' rules are:
#'
#' - For CV < 4\\%, CV* = 6\\%
#' - For 4\\% <= CV < 8\\%, CV* = CV / 2 + 4\\%
#' - For CV > 8\\%, CV* = CV
#'
#' @param cv The CV to modify
#'
#' @return
#' The value of the modified CV
#'
#' @seealso
#' [cv()]
#'
#' @references
#' "Composite Materials Handbook, Volume 1. Polymer Matrix Composites
#' Guideline for Characterization of Structural Materials,"
#' SAE International, CMH-17-1G, Mar. 2012.
#'
#' @examples
#' # The modified CV for values of CV smaller than 4% is 6%
#' calc_cv_star(0.01)
#' ## [1] 0.06
#'
#' # The modified CV for values of CV larger than 8% is unchanged
#' calc_cv_star(0.09)
#' ## [1] 0.09
#'
#' @export
calc_cv_star <- Vectorize(
  function(cv) {
    if (cv < 0.04) {
      cv <- 0.06
    } else if (cv < 0.08) {
      cv <- cv / 2 + 0.04
    } else {
      cv <- cv
    }
    return(cv)
  }
)

#' Transforms data according to the modified CV rule
#'
#' @description
#' Transforms data according to the modified coefficient of variation (CV)
#' rule. This is used to add additional variance to datasets with
#' unexpectedly low variance, which is sometimes encountered during
#' testing of new materials over short periods of time.
#'
#' Two versions of this transformation are implemented. The first version,
#' `transform_mod_cv()`, transforms the data in a single group (with
#' no other structure) according to the modified CV rules.
#'
#' The second
#' version, `transform_mod_cv_ad()`, transforms data that is structured
#' according to both condition and batch, as is commonly done for
#' the Anderson--Darling k-Sample and Anderson-Darling tests when pooling
#' across environments.
#'
#' @details
#' The modified CV transformation takes the general form:
#'
#' \deqn{\frac{S_i^*}{S_i} (x_{ij} - \bar{x_i}) + \bar{x_i}}{
#'   Si*/Si (xij-x_bar_i) + x_bar_i
#' }
#'
#' Where \eqn{S_i^*}{Si*} is the modified standard deviation
#' (mod CV times mean) for
#' the \eqn{ith} group; \eqn{S_i}{Si} is the standard deviation
#' for the \eqn{ith} group, \eqn{\bar{x_i}}{x_bar_i} is
#' the group mean and \eqn{x_{ij}}{xij} is the observation.
#'
#' `transform_mod_cv()` takes a vector
#' containing the observations and transforms the data.
#' The equation above is used, and all observations
#' are considered to be from the same group.
#'
#' `transform_mod_cv_ad()` takes a vector containing the observations
#' plus a vector containing the corresponding conditions and a vector
#' containing the batches. This function first calculates the modified
#' CV value from the data from each condition (independently). Then,
#' within each condition, the transformation
#' above is applied to produce the transformed data \eqn{x'}.
#' This transformed data is further transformed using the following
#' equation.
#'
#' \deqn{x_{ij}'' = C (x'_{ij} - \bar{x_i}) + \bar{x_i}}{
#'   x_ij'' = C (x'_ij - x_bar_i) + x_bar_i}
#'
#' Where:
#'
#' \deqn{C = \sqrt{\frac{SSE^*}{SSE'}}}{C = sqrt(SSE* / SSE')}
#'
#' \deqn{SSE^* = (n-1) (CV^* \bar{x})^2 - \sum(n_i(\bar{x_i}-\bar{x})^2)}{
#'   SSE* = (n-1) (CV* x_bar)^2 - sum(n_i(x_bar_i-x_bar)^2)}
#'
#' \deqn{SSE' = \sum(x'_{ij} - \bar{x_i})^2}{SSE' = sum(x'_ij - x_bar_i)^2}
#'
#'
#' @param x a vector of data to transform
#' @param condition a vector indicating the condition to which each
#'        observation belongs
#' @param batch a vector indicating the batch to which each observation
#'        belongs
#'
#' @return
#' A vector of transformed data
#'
#' @examples
#' # Transform data according to the modified CV transformation
#' # and report the original and modified CV for each condition
#'
#' library(dplyr)
#' carbon.fabric %>%
#' filter(test == "FT") %>%
#'   group_by(condition) %>%
#'   mutate(trans_strength = transform_mod_cv(strength)) %>%
#'   head(10)
#'
#' ## # A tibble: 10 x 6
#' ## # Groups:   condition [1]
#' ##    id         test  condition batch strength trans_strength
#' ##    <chr>      <chr> <chr>     <int>    <dbl>          <dbl>
#' ##  1 FT-RTD-1-1 FT    RTD           1     126.           126.
#' ##  2 FT-RTD-1-2 FT    RTD           1     139.           141.
#' ##  3 FT-RTD-1-3 FT    RTD           1     116.           115.
#' ##  4 FT-RTD-1-4 FT    RTD           1     132.           133.
#' ##  5 FT-RTD-1-5 FT    RTD           1     129.           129.
#' ##  6 FT-RTD-1-6 FT    RTD           1     130.           130.
#' ##  7 FT-RTD-2-1 FT    RTD           2     131.           131.
#' ##  8 FT-RTD-2-2 FT    RTD           2     124.           124.
#' ##  9 FT-RTD-2-3 FT    RTD           2     125.           125.
#' ## 10 FT-RTD-2-4 FT    RTD           2     120.           119.
#'
#' # The CV of this transformed data can be computed to verify
#' # that the resulting CV follows the rules for modified CV
#'
#' carbon.fabric %>%
#'   filter(test == "FT") %>%
#'   group_by(condition) %>%
#'   mutate(trans_strength = transform_mod_cv(strength)) %>%
#'   summarize(cv = sd(strength) / mean(strength),
#'             mod_cv = sd(trans_strength) / mean(trans_strength))
#'
#' ## # A tibble: 3 x 3
#' ##   condition     cv mod_cv
#' ##   <chr>      <dbl>  <dbl>
#' ## 1 CTD       0.0423 0.0612
#' ## 2 ETW       0.0369 0.0600
#' ## 3 RTD       0.0621 0.0711
#'
#' @seealso [calc_cv_star()]
#' @seealso [cv()]
#'
#' @name transform_mod_cv
NULL


transform_mod_cv_2_within_condition <- function(x, batch, cv_star) {  # nolint
  if (length(x) != length(batch)) {
    stop("x and batches must be the same length")
  }

  x_prime <- transform_mod_cv_grouped(x, batch)
  n <- length(x)
  x_bar <- mean(x)

  sse_prime <- sum(vapply(seq(along.with = x), function(i) {
    x_prime_i <- x_prime[i]
    x_bar_i <- mean(x[batch == batch[i]])
    (x_prime_i - x_bar_i) ^ 2
  }, FUN.VALUE = numeric(1L)))

  sse_star <- (n - 1) * (cv_star * x_bar) ^ 2 -
    sum(vapply(unique(batch), function(gi) {
      n_i <- sum(batch == gi)
      x_bar_i <- mean(x[batch == gi])
      n_i * (x_bar_i - x_bar) ^ 2
    }, FUN.VALUE = numeric(1L)))

  c_prime <- sqrt(sse_star / sse_prime)

  res <- vapply(seq(along.with = x), function(i) {
    x_bar_i <- mean(x[batch == batch[i]])
    c_prime * (x_prime[i] - x_bar_i) + x_bar_i
  }, FUN.VALUE = numeric(1L))

  res
}

#' @rdname transform_mod_cv
#' @export
transform_mod_cv_ad <- function(x, condition, batch) {
  if (is.null(batch)) {
    batch <- rep("A", length(x))
  }

  if (is.null(condition)) {
    condition <- rep("A", length(x))
  }

  if (length(x) != length(batch)) {
    stop("x and batches must be the same length")
  }

  if (length(x) != length(condition)) {
    stop("x and conditions must be the same length")
  }

  res <- numeric(0)

  for (ci in unique(condition)) {
    x_condition <- x[condition == ci]
    cv_star <- calc_cv_star(sd(x_condition) / mean(x_condition))
    res[condition == ci] <- transform_mod_cv_2_within_condition(
      x_condition, batch[condition == ci], cv_star
    )
  }

  res
}

transform_mod_cv_grouped <- function(x, group) {
  if (length(x) != length(group)) {
    stop("x and groups must be the same length")
  }

  res <- vapply(seq(along.with = x), function(i) {
    xi <- x[i]
    cur_group <- x[group == group[i]]
    s <- sd(cur_group)
    x_bar <- mean(cur_group)
    cv <- s / x_bar
    cv_star <- calc_cv_star(cv)
    s_star <- cv_star * x_bar

    s_star / s * (xi - x_bar) + x_bar
  }, FUN.VALUE = numeric(1L))

  res
}

#' @rdname transform_mod_cv
#' @export
transform_mod_cv <- function(x) {
  group <- rep("A", length(x))
  transform_mod_cv_grouped(x, group)
}

suppressMessages(library(dplyr))

# In this test case, the "steps" refer to the flowchart in CMH-17-1G
# Figure 8.3.1.1(a)

test_that("carbon.fabric.2 MNR test matches CMH17-STATS (step 7)", {
  # Test for outliers within each batch and condition

  conditions <- unique(carbon.fabric.2$condition)
  batches <- unique(carbon.fabric.2$batch)

  sapply(conditions, function(c) {
    n_outliers <- sum(
      sapply(batches, function(b) {
        dat <- carbon.fabric.2 %>%
          filter(test == "FC") %>%
          filter(condition == c) %>%
          filter(batch == b)

        mnr <- maximum_normed_residual(data = dat, strength)
        glance(mnr)$n_outliers
      })
    )

    expected_outliers <- case_when(c == "ETW2" ~ 1,
                                   TRUE ~ 0)

    expect_equal(n_outliers, expected_outliers)
  })
})

test_that("carbon.fabric.2 ADK matches CMH17-STATS (step 10)", {
  # Test for between-batch variability within each condition

  conditions <- unique(carbon.fabric.2$condition)

  sapply(conditions, function(c) {
    dat <- carbon.fabric.2 %>%
      filter(test == "FC") %>%
      filter(condition == c)
    adk <- ad_ksample(dat, strength, batch)
    ad <- glance(adk)$ad
    k <- glance(adk)$k
    reject_same_dist <- glance(adk)$reject_same_dist

    adk_expected <- case_when(c == "CTD" ~ 0.351,
                              c == "RTD" ~ 0.448,
                              c == "ETD" ~ 0.453,
                              c == "ETW" ~ 1.627,
                              c == "ETW2" ~ 0.784,
                              TRUE ~ 0)
    # There is a difference between the way that the test statistic is defined
    # in cmstatr and CMH17-STATS and a corresponding difference in the
    # critical value. See documentation for ad_ksample()
    adk_expected <- adk_expected * (k - 1)

    expect_equal(ad, adk_expected, tolerance = 0.005)
    expect_false(reject_same_dist)

  })
})

test_that("carbon.fabric.2 MNR test matches CMH17-STATS (step 18)", {
  # Test for outliers within each condition

  conditions <- unique(carbon.fabric.2$condition)

  sapply(conditions, function(c) {
    dat <- carbon.fabric.2 %>%
      filter(test == "FC") %>%
      filter(condition == c)
    mnr <- maximum_normed_residual(dat, strength)
    n_outliers <- glance(mnr)$n_outliers
    expect_equal(n_outliers, 0)
  })
})

test_that("carbon.fabric.2 Normality of pooled norm data matches (step 23)", {
  # Check normality of pooled set of normalized data using Anderson Darling
  # test
  res <- carbon.fabric.2 %>%
    filter(test == "FC") %>%
    mutate(norm_strength = normalize_group_mean(strength, condition)) %>%
    anderson_darling_normal(norm_strength)

  expect_equal(res$osl, 0.5765, tolerance = 1e-4)
})

test_that("carbon.fabric.2 equality of variances matches (step 28)", {
  # Test for equality of variances among condition groups
  res <- carbon.fabric.2 %>%
    filter(test == "FC") %>%
    levene_test(strength, condition)

  expect_equal(res$f, 3.884, tolerance = 0.001)
})

test_that("carbon.fabric.2 pooled sd basis values match (step 30)", {
  res <- carbon.fabric.2 %>%
    filter(test == "FC") %>%
    basis_pooled_sd(strength, condition,
                    override = c("pooled_variance_equal",
                                 "outliers_within_batch",
                                 "between_group_variability"))

  res$basis %>%
    mutate(expected = case_when(group == "CTD" ~ 89.22,
                                group == "RTD" ~ 80.06,
                                group == "ETW" ~ 48.53,
                                group == "ETW2" ~ 42.27,
                                group == "ETD" ~ 66.56,
                                TRUE ~ 0)) %>%
    mutate(expect_equal(value, expected, tolerance = 0.005))
})

test_that("carbon.fabric.2 equality of normalized variance (step 31)", {
  # Test for equality of normalized variances among condition groups
  res <- carbon.fabric.2 %>%
    filter(test == "FC") %>%
    mutate(norm_strength = normalize_group_mean(strength, condition)) %>%
    levene_test(norm_strength, condition)

  expect_equal(res$f, 1.065, tolerance = 0.0005)
})

test_that("carbon.fabric.2 pooled CV basis values match (step 39-41)", {
  res <- carbon.fabric.2 %>%
    filter(test == "FC") %>%
    basis_pooled_cv(strength, condition, batch,
                    override = c("outliers_within_batch"))

  res$basis %>%
    mutate(expected = case_when(group == "CTD" ~ 86.91,
                                group == "RTD" ~ 78.83,
                                group == "ETW" ~ 51.04,
                                group == "ETW2" ~ 45.51,
                                group == "ETD" ~ 66.93,
                                TRUE ~ 0)) %>%
    mutate(expect_equal(value, expected, tolerance = 0.005))
})

test_that("carbon.fabric.2 ADK matches CMH17-STATS - modCV (step 10)", {
  # Test for between-batch variability within each condition (modified CV)

  conditions <- unique(carbon.fabric.2$condition)

  sapply(conditions, function(c) {
    dat <- carbon.fabric.2 %>%
      filter(test == "FC") %>%
      mutate(trans_strength = transform_mod_cv_ad(
        strength, condition, batch)) %>%
      filter(condition == c)
    adk <- ad_ksample(dat, trans_strength, batch)
    ad <- glance(adk)$ad
    k <- glance(adk)$k
    reject_same_dist <- glance(adk)$reject_same_dist

    adk_expected <- case_when(c == "CTD" ~ 0.351,
                              c == "RTD" ~ 0.401,
                              c == "ETD" ~ 0.495,
                              c == "ETW" ~ 1.051,
                              c == "ETW2" ~ 0.670,
                              TRUE ~ 0)

    # There is a difference between the way that the test statistic is defined
    # in cmstatr and CMH17-STATS and a corresponding difference in the
    # critical value. See documentation for ad_ksample()
    adk_expected <- adk_expected * (k - 1)

    expect_equal(ad, adk_expected, tolerance = 0.005)
    expect_false(reject_same_dist)
  })
})

test_that("carbon.fabric.2 Normality of pooled norm data - modCV (step 23)", {
  # Check normality of pooled set of normalized data using Anderson Darling
  # test (modified CV)
  res <- carbon.fabric.2 %>%
    filter(test == "FC") %>%
    mutate(trans_strength = transform_mod_cv_ad(strength, condition, batch)) %>%
    mutate(norm_strength = normalize_group_mean(trans_strength, condition)) %>%
    anderson_darling_normal(norm_strength)

  expect_equal(res$osl, 0.6036, tolerance = 1e-4)
})

test_that("carbon.fabric.2 equality of variances - modCV (step 28)", {
  # Test for equality of variances among condition groups
  res <- carbon.fabric.2 %>%
    filter(test == "FC") %>%
    group_by(condition) %>%
    mutate(trans_strength = transform_mod_cv(strength)) %>%
    ungroup() %>%
    levene_test(trans_strength, condition)

  expect_equal(res$f, 2.862, tolerance = 0.075)

  # Test again, using the modcv argument
  res <- carbon.fabric.2 %>%
    filter(test == "FC") %>%
    levene_test(strength, condition, modcv = TRUE)

  expect_equal(res$f, 2.862, tolerance = 0.075)
})

test_that("carbon.fabric.2 pooled sd basis values - modCV (step 30)", {
  res <- carbon.fabric.2 %>%
    filter(test == "FC") %>%
    group_by(condition) %>%
    mutate(trans_strength = transform_mod_cv(strength)) %>%
    ungroup() %>%
    basis_pooled_sd(trans_strength, condition,
                    override = c("pooled_variance_equal",
                                 "outliers_within_batch",
                                 "between_group_variability"))

  res$basis %>%
    mutate(expected = case_when(group == "CTD" ~ 88.64,
                                group == "RTD" ~ 79.48,
                                group == "ETW" ~ 47.95,
                                group == "ETW2" ~ 41.68,
                                group == "ETD" ~ 65.97,
                                TRUE ~ 0)) %>%
    mutate(expect_equal(value, expected, tolerance = 0.005))

  # Test again, using the modcv argument
  res <- carbon.fabric.2 %>%
    filter(test == "FC") %>%
    basis_pooled_sd(strength, condition, modcv = TRUE,
                    override = c("pooled_variance_equal",
                                 "outliers_within_batch",
                                 "between_group_variability"))

  res$basis %>%
    mutate(expected = case_when(group == "CTD" ~ 88.64,
                                group == "RTD" ~ 79.48,
                                group == "ETW" ~ 47.95,
                                group == "ETW2" ~ 41.68,
                                group == "ETD" ~ 65.97,
                                TRUE ~ 0)) %>%
    mutate(expect_equal(value, expected, tolerance = 0.005))
})

test_that("carbon.fabric.2 equality of norm variance - modCV (step 31)", {
  # Test for equality of normalized variacnes among condition groups
  res <- carbon.fabric.2 %>%
    filter(test == "FC") %>%
    group_by(condition) %>%
    mutate(trans_strength = transform_mod_cv(strength)) %>%
    ungroup() %>%
    mutate(norm_strength = normalize_group_mean(trans_strength, condition)) %>%
    levene_test(norm_strength, condition)

  expect_equal(res$f, 0.226, tolerance = 0.05 / 0.226)

  # Test again, using the modcv argument
  res <- carbon.fabric.2 %>%
    filter(test == "FC") %>%
    mutate(norm_strength = normalize_group_mean(strength, condition)) %>%
    levene_test(norm_strength, condition, modcv = TRUE)

  expect_equal(res$f, 0.226, tolerance = 0.05 / 0.226)
})

test_that("carbon.fabric.2 pooled CV basis values - modCV (step 39-41)", {
  res <- carbon.fabric.2 %>%
    filter(test == "FC") %>%
    group_by(condition) %>%
    mutate(trans_strength = transform_mod_cv(strength)) %>%
    ungroup() %>%
    basis_pooled_cv(trans_strength, condition,
                    override = c("outliers_within_batch",
                                 "between_group_variability"))

  res$basis %>%
    mutate(expected = case_when(group == "CTD" ~ 85.98,
                                group == "RTD" ~ 77.99,
                                group == "ETW" ~ 50.50,
                                group == "ETW2" ~ 45.02,
                                group == "ETD" ~ 66.22,
                                TRUE ~ 0)) %>%
    mutate(expect_equal(value, expected, tolerance = 0.005))

  # Test again, using the modcv argument
  res <- carbon.fabric.2 %>%
    filter(test == "FC") %>%
    basis_pooled_cv(strength, condition, modcv = TRUE,
                    override = c("outliers_within_batch",
                                 "between_group_variability"))

  res$basis %>%
    mutate(expected = case_when(group == "CTD" ~ 85.98,
                                group == "RTD" ~ 77.99,
                                group == "ETW" ~ 50.50,
                                group == "ETW2" ~ 45.02,
                                group == "ETD" ~ 66.22,
                                TRUE ~ 0)) %>%
    mutate(expect_equal(value, expected, tolerance = 0.005))
})

test_dat <- tribble(
  ~cond, ~batch, ~strength,
  1, "A", 97.89583,
  1, "A", 109.76291,
  1, "A", 107.66701,
  1, "A", 112.69444,
  1, "A", 99.71637,
  1, "A", 108.68871,
  1, "B", 106.37708,
  1, "B", 110.96946,
  1, "B", 102.78598,
  1, "D", 100.83316,
  1, "D", 105.66699,
  1, "D", 103.14165,
  1, "D", 105.36496,
  1, "D", 101.76552,
  1, "D", 103.81940,
  1, "E", 105.59004,
  1, "E", 107.74373,
  1, "E", 108.88401,
  1, "E", 103.72913,
  1, "E", 107.07630,
  1, "E", 104.94092,
  1, "C", 103.99478,
  1, "C", 104.57191,
  1, "C", 104.65751,
  1, "C", 106.26021,
  1, "C", 108.80221,
  1, "C", 104.45781,
  1, "C", 105.71117,
  1, "C", 104.68779,
  1, "C", 96.90000,
  2, "A", 108.89755,
  2, "A", 112.26424,
  2, "A", 114.14414,
  2, "A", 109.56092,
  2, "A", 112.81857,
  2, "A", 110.04231,
  2, "B", 105.92060,
  2, "B", 104.76159,
  2, "B", 105.39842,
  2, "D", 97.83402,
  2, "D", 103.60583,
  2, "D", 101.81152,
  2, "D", 100.55827,
  2, "D", 104.10650,
  2, "D", 103.56471,
  2, "E", 105.18761,
  2, "E", 103.81970,
  2, "E", 104.31939,
  2, "E", 105.21502,
  2, "E", 109.61211,
  2, "E", 108.43762,
  2, "C", 104.08681,
  2, "C", 109.60930,
  2, "C", 107.17149,
  2, "C", 107.10460,
  2, "C", 102.88207,
  2, "C", 106.91041,
  2, "C", 104.40203,
  2, "C", 105.43848,
  2, "C", 109.50752,
  3, "B", 93.22953,
  3, "B", 97.09608,
  3, "B", 97.05682,
  3, "A", 102.57570,
  3, "A", 98.16441,
  3, "A", 97.67216,
  3, "A", 103.10081,
  3, "A", 106.05421,
  3, "A", 100.43507,
  3, "B", 101.64746,
  3, "B", 103.24892,
  3, "B", 101.74833,
  3, "D", 93.48767,
  3, "D", 89.77051,
  3, "D", 95.24123,
  3, "D", 96.81138,
  3, "D", 86.88577,
  3, "D", 89.46870,
  3, "E", 96.80020,
  3, "E", 97.87896,
  3, "E", 97.91697,
  3, "E", 98.73682,
  3, "E", 96.20074,
  3, "E", 99.22644,
  3, "C", 102.06662,
  3, "C", 102.04738,
  3, "C", 96.40601,
  3, "C", 99.34420,
  3, "C", 99.20519,
  3, "C", 101.22471,
  3, "C", 99.60168,
  3, "C", 106.73832,
  3, "C", 110.09276
)

test_that("MNR within each batch and condition matches CMH17-STATS", {
  expect_warning(
    expect_warning(
      res_unmodified <- test_dat %>%
        basis_pooled_sd(strength, cond, batch, modcv = FALSE),
      "outliers_within_batch"
    ),
    "between_group_variability"
  )

  expect_true("outliers_within_batch" %in%
                res_unmodified$diagnostic_failures)

  expect_warning(
    res_modcv <- test_dat %>%
      basis_pooled_sd(strength, cond, batch, modcv = TRUE),
    "outliers_within_batch"
  )

  expect_true("outliers_within_batch" %in%
                res_modcv$diagnostic_failures)

  expect_warning(
    expect_warning(
      res_unmodified <- test_dat %>%
        basis_pooled_cv(strength, cond, batch, modcv = FALSE),
      "outliers_within_batch"
    ),
    "between_group_variability"
  )

  expect_true("outliers_within_batch" %in%
                res_unmodified$diagnostic_failures)

  expect_warning(
    res_modcv <- test_dat %>%
      basis_pooled_cv(strength, cond, batch, modcv = TRUE),
    "outliers_within_batch"
  )

  expect_true("outliers_within_batch" %in%
                res_modcv$diagnostic_failures)

  conditions <- unique(test_dat$cond)
  batches <- unique(test_dat$batch)

  sapply(conditions, function(c) {
    sapply(batches, function(b) {
      dat <- test_dat %>%
        filter(cond == c & batch == b)
      mnr <- maximum_normed_residual(dat, strength)
      n_outliers <- glance(mnr)$n_outliers

      if (c == 1 & b == "C") {
        expect_gt(n_outliers, 0)
      } else {
        expect_equal(n_outliers, 0)
      }
    })
  })
})

test_that("ADK for between-batch var. within each cond matches CMH17-STATS", {
  expect_warning(
    expect_warning(
      res_unmodified <- test_dat %>%
        basis_pooled_cv(strength, cond, batch, modcv = FALSE),
      "outliers_within_batch"
    ),
    "between_group_variability"
  )

  expect_true("between_group_variability" %in%
                res_unmodified$diagnostic_failures)

  expect_warning(
    res_modcv <- test_dat %>%
      basis_pooled_cv(strength, cond, batch, modcv = TRUE),
    "outliers_within_batch"
  )

  expect_false("between_group_variability" %in%
                 res_modcv$diagnostic_failures)

  expect_warning(
    expect_warning(
      res_unmodified <- test_dat %>%
        basis_pooled_sd(strength, cond, batch, modcv = FALSE),
      "outliers_within_batch"
    ),
    "between_group_variability"
  )

  expect_true("between_group_variability" %in%
                res_unmodified$diagnostic_failures)

  expect_warning(
    res_modcv <- test_dat %>%
      basis_pooled_sd(strength, cond, batch, modcv = TRUE),
    "outliers_within_batch"
  )

  expect_false("between_group_variability" %in%
                 res_modcv$diagnostic_failures)

  conditions <- unique(test_dat$cond)

  lapply(conditions, function(c) {
    dat <- test_dat %>%
      filter(cond == c)
    adk <- ad_ksample(dat, strength, batch)
    ad <- glance(adk)$ad
    k <- glance(adk)$k

    expected <- case_when(c == 1 ~ 1.243,
                          c == 2 ~ 3.376,
                          c == 3 ~ 2.793)

    expect_equal(ad / (k - 1), expected, tolerance = 7e-3)
  })

  lapply(conditions, function(c) {
    dat <- test_dat %>%
      mutate(strength = transform_mod_cv_ad(strength, cond, batch)) %>%
      filter(cond == c)
    adk <- ad_ksample(dat, strength, batch)
    ad <- glance(adk)$ad
    k <- glance(adk)$k

    expected <- case_when(c == 1 ~ 0.504,
                          c == 2 ~ 1.106,
                          c == 3 ~ 1.508)

    expect_equal(ad / (k - 1), expected, tolerance = 2e-3)
  })
})

test_that("MNR within each condition matches CMH17-STATS", {
  expect_warning(
    expect_warning(
      res_unmodified <- test_dat %>%
        basis_pooled_sd(strength, cond, batch, modcv = FALSE),
      "outliers_within_batch"
    ),
    "between_group_variability"
  )

  expect_false("outliers_within_group" %in%
                res_unmodified$diagnostic_failures)

  expect_warning(
    res_modcv <- test_dat %>%
      basis_pooled_sd(strength, cond, batch, modcv = TRUE),
    "outliers_within_batch"
  )

  expect_false("outliers_within_group" %in%
                res_modcv$diagnostic_failures)

  expect_warning(
    expect_warning(
      res_unmodified <- test_dat %>%
        basis_pooled_cv(strength, cond, batch, modcv = FALSE),
      "outliers_within_batch"
    ),
    "between_group_variability"
  )

  expect_false("outliers_within_group" %in%
                res_unmodified$diagnostic_failures)

  expect_warning(
    res_modcv <- test_dat %>%
      basis_pooled_cv(strength, cond, batch, modcv = TRUE),
    "outliers_within_batch"
  )

  expect_false("outliers_within_group" %in%
                res_modcv$diagnostic_failures)

  conditions <- unique(test_dat$cond)

  lapply(conditions, function(c) {
    dat <- test_dat %>%
      filter(cond == c)
    mnr <- maximum_normed_residual(dat, strength)
    n_outliers <- glance(mnr)$n_outliers
    expect_equal(n_outliers, 0)
  })
})

test_that("OSL of data from each batch CMH17-STATS", {
  conditions <- unique(test_dat$cond)

  lapply(conditions, function(c) {
    dat <- test_dat %>%
      filter(cond == c)
    osl <- anderson_darling_normal(dat, strength)
    osl <- glance(osl)$osl
    expected <- case_when(c == 1 ~ 0.485,
                          c == 2 ~ 0.306,
                          c == 3 ~ 0.287)
    expect_equal(osl, expected, tolerance = 1e-3)
  })

  lapply(conditions, function(c) {
    dat <- test_dat %>%
      mutate(strength = transform_mod_cv_ad(strength, cond, batch)) %>%
      filter(cond == c)
    osl <- anderson_darling_normal(dat, strength)
    osl <- glance(osl)$osl

    expected <- case_when(c == 1 ~ 0.494,
                          c == 2 ~ 0.467,
                          c == 3 ~ 0.718)

    expect_equal(osl, expected, tolerance = 1e-3)
  })
})

test_that("OSL of pooled data after norm to group mean matches CMH17-STATS", {
  expect_warning(
    expect_warning(
      res_unmodified <- test_dat %>%
        basis_pooled_sd(strength, cond, batch, modcv = FALSE),
      "outliers_within_batch"
    ),
    "between_group_variability"
  )

  expect_false("pooled_data_normal" %in%
                 res_unmodified$diagnostic_failures)

  expect_warning(
    res_modcv <- test_dat %>%
      basis_pooled_sd(strength, cond, batch, modcv = TRUE),
    "outliers_within_batch"
  )

  expect_false("pooled_data_normal" %in%
                 res_modcv$diagnostic_failures)

  expect_warning(
    expect_warning(
      res_unmodified <- test_dat %>%
        basis_pooled_cv(strength, cond, batch, modcv = FALSE),
      "outliers_within_batch"
    ),
    "between_group_variability"
  )

  expect_false("pooled_data_normal" %in%
                 res_unmodified$diagnostic_failures)

  expect_warning(
    res_modcv <- test_dat %>%
      basis_pooled_cv(strength, cond, batch, modcv = TRUE),
    "outliers_within_batch"
  )

  expect_false("pooled_data_normal" %in%
                 res_modcv$diagnostic_failures)

  res <- test_dat %>%
    mutate(normalized_strength = normalize_group_mean(strength, cond)) %>%
    anderson_darling_normal(normalized_strength)
  expect_equal(res$osl, 0.0798, tolerance = 1e-3)

  res <- test_dat %>%
    mutate(strength = transform_mod_cv_ad(strength, cond, batch)) %>%
    mutate(normalized_strength = normalize_group_mean(strength, cond)) %>%
    anderson_darling_normal(normalized_strength)
  expect_equal(res$osl, 0.7045, tolerance = 1e-4)
})

test_that("Levene's test among condition groups matches CMH17-STATS", {
  expect_warning(
    expect_warning(
      res_unmodified <- test_dat %>%
        basis_pooled_sd(strength, cond, batch, modcv = FALSE),
      "outliers_within_batch"
    ),
    "between_group_variability"
  )

  expect_false("pooled_variance_equal" %in%
                 res_unmodified$diagnostic_failures)

  lev <- test_dat %>%
    levene_test(strength, groups = cond)
  expect_equal(lev$f, 1.051, tolerance = 1e-3)

  expect_warning(
    res_modcv <- test_dat %>%
      basis_pooled_sd(strength, cond, batch, modcv = TRUE),
    "outliers_within_batch"
  )

  expect_false("pooled_variance_equal" %in%
                 res_modcv$diagnostic_failures)

  lev <- test_dat %>%
    group_by(cond) %>%
    mutate(strength = transform_mod_cv(strength)) %>%
    ungroup() %>%
    levene_test(strength, groups = cond)
  expect_equal(lev$f, 0.005, tolerance = 0.05)

  expect_warning(
    expect_warning(
      res_unmodified <- test_dat %>%
        basis_pooled_cv(strength, cond, batch, modcv = FALSE),
      "outliers_within_batch"
    ),
    "between_group_variability"
  )

  lev <- test_dat %>%
    mutate(strength = normalize_group_mean(strength, cond)) %>%
    levene_test(strength, groups = cond)
  expect_equal(lev$f, 1.631, tolerance = 0.05)

  expect_warning(
    res_modcv <- test_dat %>%
      basis_pooled_cv(strength, cond, batch, modcv = TRUE),
    "outliers_within_batch"
  )

  lev <- test_dat %>%
    group_by(cond) %>%
    mutate(strength = transform_mod_cv(strength)) %>%
    ungroup() %>%
    mutate(strength = normalize_group_mean(strength, cond)) %>%
    levene_test(strength, groups = cond)
  expect_equal(lev$f, 0.064, tolerance = 0.05 / 0.064)
})

test_that("Basis values match CMH17-STATS", {
  expect_warning(
    expect_warning(
      res <- test_dat %>%
        basis_pooled_cv(strength, cond, batch, modcv = FALSE),
      "outliers_within_batch"
    ),
    "between_group_variability"
  )

  res$basis %>%
    mutate(expected = case_when(group == 1 ~ 98.21,
                                group == 2 ~ 99.27,
                                group == 3 ~ 92.22)) %>%
    mutate(expect_equal(value, expected, tolerance = 1e-2))

  expect_warning(
    res <- test_dat %>%
      basis_pooled_cv(strength, cond, batch, modcv = TRUE),
    "outliers_within_batch"
  )
  res$basis %>%
    mutate(expected = case_when(group == 1 ~ 94.49,
                                group == 2 ~ 95.51,
                                group == 3 ~ 88.76)) %>%
    mutate(expect_equal(value, expected, tolerance = 1e-2))

  expect_warning(
    expect_warning(
      res <- test_dat %>%
        basis_pooled_sd(strength, cond, batch, modcv = FALSE),
      "outliers_within_batch"
    ),
    "between_group_variability"
  )
  res$basis %>%
    mutate(expected = case_when(group == 1 ~ 98.42,
                                group == 2 ~ 99.55,
                                group == 3 ~ 92.00)) %>%
    mutate(expect_equal(value, expected, tolerance = 1e-2))

  expect_warning(
    res <- test_dat %>%
      basis_pooled_sd(strength, cond, batch, modcv = TRUE),
    "outliers_within_batch"
  )
  res$basis %>%
    mutate(expected = case_when(group == 1 ~ 94.71,
                                group == 2 ~ 95.84,
                                group == 3 ~ 88.32)) %>%
    mutate(expect_equal(value, expected, tolerance = 1e-2))
})

context("normalize")

suppressMessages(library(dplyr))

test_that("normalization to ply thickness takes correct argument lengths", {
  expect_error(normalize_ply_thickness(c(1, 2), 1, 1))
  expect_error(normalize_ply_thickness(1, c(1, 2), 1))
  expect_error(normalize_ply_thickness(c(1, 2), c(1, 2), c(1, 2)))
})

test_that("normalization to ply thickness produces expected numeric results", {
  expect_equal(normalize_ply_thickness(1, 1, 0.25), 4)
  expect_equal(normalize_ply_thickness(c(1, 2),
                                       c(0.5, 0.25),
                                       0.25),
               c(2, 2)
  )
})

test_that("normalize_group_mean correctly handles errors and edge cases", {
  expect_error(normalize_group_mean(x = 1:10, group = 1:8), "length")
  expect_equal(normalize_group_mean(x = numeric(0L), group = numeric(0L)),
               numeric(0L))
})

test_that("normalize to group mean produces means of one for all groups", {
  # since each group is normalized to its own mean, the mean of the normalized
  # values within each group should be equal to one.
  res <- carbon.fabric %>%
    filter(test == "WT") %>%
    mutate(norm_str = normalize_group_mean(strength, condition)) %>%
    group_by(condition) %>%
    summarise(mean_norm_str = mean(norm_str)) %>%
    rowwise() %>%
    mutate(test = expect_equal(mean_norm_str, 1))
})

# data from CMH-17-1G Section 8.3.11.1.2
poolable_data <- tribble(
  ~batch, ~strength, ~condition,
  1, 79.04517, "CTD",
  1, 102.6014, "CTD",
  1, 97.79372, "CTD",
  1, 92.86423, "CTD",
  1, 117.218,  "CTD",
  1, 108.7168, "CTD",
  1, 112.2773, "CTD",
  1, 114.0129, "CTD",
  2, 106.8452, "CTD",
  2, 112.3911, "CTD",
  2, 115.5658, "CTD",
  2, 87.40657, "CTD",
  2, 102.2785, "CTD",
  2, 110.6073, "CTD",
  3, 105.2762, "CTD",
  3, 110.8924, "CTD",
  3, 108.7638, "CTD",
  3, 110.9833, "CTD",
  3, 101.3417, "CTD",
  3, 100.0251, "CTD",
  1, 103.2006, "RTD",
  1, 105.1034, "RTD",
  1, 105.1893, "RTD",
  1, 100.4189, "RTD",
  2, 85.32319, "RTD",
  2, 92.69923, "RTD",
  2, 98.45242, "RTD",
  2, 104.1014, "RTD",
  2, 91.51841, "RTD",
  2, 101.3746, "RTD",
  2, 101.5828, "RTD",
  2, 99.57384, "RTD",
  2, 88.84826, "RTD",
  3, 92.18703, "RTD",
  3, 101.8234, "RTD",
  3, 97.68909, "RTD",
  3, 101.5172, "RTD",
  3, 100.0481, "RTD",
  3, 102.0544, "RTD",
  1, 63.22764, "ETW",
  1, 70.84454, "ETW",
  1, 66.43223, "ETW",
  1, 75.37771, "ETW",
  1, 72.43773, "ETW",
  1, 68.43073, "ETW",
  1, 69.72524, "ETW",
  2, 66.20343, "ETW",
  2, 60.51251, "ETW",
  2, 65.69334, "ETW",
  2, 62.73595, "ETW",
  2, 59.00798, "ETW",
  2, 62.37761, "ETW",
  3, 64.3947,  "ETW",
  3, 72.8491,  "ETW",
  3, 66.56226, "ETW",
  3, 66.56779, "ETW",
  3, 66.00123, "ETW",
  3, 59.62108, "ETW",
  3, 60.61167, "ETW",
  3, 57.65487, "ETW",
  3, 66.51241, "ETW",
  3, 64.89347, "ETW",
  3, 57.73054, "ETW",
  3, 68.94086, "ETW",
  3, 61.63177, "ETW",
  1, 54.09806, "ETW2",
  1, 58.87615, "ETW2",
  1, 61.60167, "ETW2",
  1, 60.23973, "ETW2",
  1, 61.4808,  "ETW2",
  1, 64.55832, "ETW2",
  2, 57.76131, "ETW2",
  2, 49.91463, "ETW2",
  2, 61.49271, "ETW2",
  2, 57.7281,  "ETW2",
  2, 62.11653, "ETW2",
  2, 62.69353, "ETW2",
  3, 61.38523, "ETW2",
  3, 60.39053, "ETW2",
  3, 59.17616, "ETW2",
  3, 60.17616, "ETW2",
  3, 46.47396, "ETW2",
  3, 51.16616, "ETW2"
)

test_that("Modified CV transform produces values that match CMH17-STATS", {
  res <- poolable_data %>%
    group_by(condition) %>%
    mutate(trans_strength = transform_mod_cv(strength)) %>%
    summarise(cv = sd(strength) / mean(strength),
              mod_cv = sd(trans_strength) / mean(trans_strength))

  expect_equal(res$cv[res$condition == "CTD"], 0.0933, tolerance = 1e-4)
  expect_equal(res$mod_cv[res$condition == "CTD"], 0.0933, tolerance = 1e-4)
  expect_equal(res$cv[res$condition == "RTD"], 0.0580, tolerance = 1e-4)
  expect_equal(res$mod_cv[res$condition == "RTD"], 0.0690, tolerance = 1e-4)
  expect_equal(res$cv[res$condition == "ETW"], 0.0723, tolerance = 1e-4)
  expect_equal(res$mod_cv[res$condition == "ETW"], 0.0761, tolerance = 1e-4)
  expect_equal(res$cv[res$condition == "ETW2"], 0.0836, tolerance = 1e-4)
  expect_equal(res$mod_cv[res$condition == "ETW2"], 0.0836, tolerance = 1e-4)
})

test_that("mod CV transforms handles errors and edge cases correctly", {
  expect_error(transform_mod_cv_2_within_condition(x = 1:9, batch = 1:4, 0.5),
               "length")

  expect_error(transform_mod_cv_ad(1:10, 1:9, 1:10), "length")
  expect_error(transform_mod_cv_ad(1:10, 1:10, 1:9), "length")

  expect_equal(
    transform_mod_cv_ad(1:10, rep("A", 10), rep("A", 10)),
    transform_mod_cv_ad(1:10, NULL, NULL)
  )

  expect_error(transform_mod_cv_grouped(1:10, 1:9), "length")
})

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

# If any of the required packages are unavailable,
# don't re-run the code
required <- c("dplyr", "ggplot2", "tidyr", "cmstatr", "purrr")
if (!all(unlist(lapply(required, function(pkg) {
    requireNamespace(pkg, quietly = TRUE)}
  )))) {
  knitr::opts_chunk$set(eval = FALSE)
}

## ----message=FALSE------------------------------------------------------------
library(cmstatr)
library(dplyr)
library(ggplot2)
library(tidyr)
library(purrr)

## -----------------------------------------------------------------------------
carbon.fabric.2 %>%
  head(10)

## -----------------------------------------------------------------------------
norm_data <- carbon.fabric.2 %>%
  filter(test == "WT" | test == "FC") %>%
  mutate(strength.norm = normalize_ply_thickness(strength,
                                                 thickness / nplies,
                                                 0.0079))

norm_data %>%
  head(10)

## -----------------------------------------------------------------------------
norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  anderson_darling_normal(strength.norm)

## ----include=FALSE------------------------------------------------------------
# Verify that the AD test always provides the same conclusion
# If this assertion fails, the Vignette needs to be re-written
if (0.05 >= (norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  anderson_darling_normal(strength.norm))$osl) {
  stop("Unexpected vale for Anderson-Darling test")
  }

## -----------------------------------------------------------------------------
norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  basis_normal(strength.norm)

## -----------------------------------------------------------------------------
norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  basis_normal(strength.norm, 
               override = c("outliers_within_batch",
                            "between_batch_variability"))

## -----------------------------------------------------------------------------
norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  basis_normal(strength.norm, batch)

## -----------------------------------------------------------------------------
norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  ad_ksample(strength.norm, batch)

## -----------------------------------------------------------------------------
norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  group_by(batch) %>%
  ggplot(aes(x = strength.norm, color = batch)) +
  stat_normal_surv_func() +
  stat_esf() +
  ggtitle("Distribution of Data For Each Batch")

## -----------------------------------------------------------------------------
norm_data %>%
  filter(test == "FC") %>%
  group_by(condition, batch) %>%
  nest() %>%
  mutate(mnr = map(data,
                   ~maximum_normed_residual(data = .x, x = strength.norm)),
         tidied = map(mnr, glance)) %>%
  select(-c(mnr, data)) %>%  # remove unneeded columns
  unnest(tidied)

## ----include=FALSE------------------------------------------------------------
if ((norm_data %>%
  filter(test == "FC") %>%
  group_by(condition, batch) %>%
  summarise(
    n_outliers = maximum_normed_residual(x = strength.norm)$n_outliers
    ) %>%
  ungroup() %>%
  summarise(n_outliers = sum(n_outliers)))[[1]] != 0) {
  stop("Unexpected number of outliers")
  }

## -----------------------------------------------------------------------------
norm_data %>%
  filter(test == "FC") %>%
  group_by(condition) %>%
  nest() %>%
  mutate(adk = map(data, ~ad_ksample(data = .x,
                                     x = strength.norm,
                                     groups = batch)),
         tidied = map(adk, glance)) %>%
  select(-c(data, adk)) %>%  # remove unneeded columns
  unnest(tidied)

## ----include=FALSE------------------------------------------------------------
if (!all(!(norm_data %>%
  filter(test == "FC") %>%
  group_by(condition) %>%
  summarise(different_dist =
           ad_ksample(x = strength.norm, groups = batch)$reject_same_dist
  ))$different_dist)) {
  stop("Unexpected ADK result")
  }

## -----------------------------------------------------------------------------
norm_data %>%
  filter(test == "FC") %>%
  group_by(condition) %>%
  nest() %>%
  mutate(mnr = map(data, ~maximum_normed_residual(data = .x,
                                                  x = strength.norm)),
         tidied = map(mnr, glance)) %>%
  select(-c(mnr, data)) %>%  # remove unneeded columns
  unnest(tidied)

## ----include=FALSE------------------------------------------------------------
if ((norm_data %>%
  filter(test == "FC") %>%
  group_by(condition) %>%
  summarise(
    n_outliers = maximum_normed_residual(x = strength.norm)$n_outliers
    ) %>%
  ungroup() %>%
  summarise(n_outliers = sum(n_outliers)))[[1]] != 0) {
  stop("Unexpected number of outliers")
  }

## -----------------------------------------------------------------------------
norm_data %>%
  filter(test == "FC") %>%
  levene_test(strength.norm, condition)

## ----include=FALSE------------------------------------------------------------
if (!(norm_data %>%
  filter(test == "FC") %>%
  levene_test(strength.norm, condition))$reject_equal_variance) {
  stop("Unexpected result from Levene's test")
  }

## -----------------------------------------------------------------------------
norm_data %>%
  filter(test == "FC") %>%
  mutate(
    strength_norm_group = normalize_group_mean(strength.norm, condition)) %>%
  levene_test(strength_norm_group, condition)

## ----include=FALSE------------------------------------------------------------
if ((norm_data %>%
  filter(test == "FC") %>%
  mutate(
    strength_norm_group = normalize_group_mean(strength.norm, condition)) %>%
  levene_test(strength_norm_group, condition))$reject_equal_variance) {
  stop("Unexpected value from Levene's test")
  }

## -----------------------------------------------------------------------------
norm_data %>%
  filter(test == "FC") %>%
  mutate(
    strength_norm_group = normalize_group_mean(strength.norm, condition)) %>%
  anderson_darling_normal(strength_norm_group)

## ----include=FALSE------------------------------------------------------------
if ((norm_data %>%
  filter(test == "FC") %>%
  mutate(
    strength_norm_group = normalize_group_mean(strength.norm, condition)) %>%
  anderson_darling_normal(strength_norm_group))$osl <= 0.05) {
  stop("Unexpected value from AD test")
  }

## -----------------------------------------------------------------------------
norm_data %>%
  filter(test == "FC") %>%
  basis_pooled_cv(strength.norm, condition, batch)

## -----------------------------------------------------------------------------
norm_data %>%
  mutate(condition = ordered(condition,
                             c("CTD", "RTD", "ETD", "ETW", "ETW2"))) %>%
  filter(test == "FC") %>%
  basis_pooled_cv(strength.norm, condition, batch)

## -----------------------------------------------------------------------------
carbon.fabric.2 %>%
  filter(test == "FC" & condition == "RTD") %>%
  equiv_mean_extremum(strength, n_sample = 5, alpha = 0.01)

#' Produce basis summary statistics for each (environmental) condition
#'
#' @description
#' Produces a `data.frame` containing the sample size and mean for each
#' condition. If a reference condition (e.g. "RTD") is specified, the ratio
#' of each condition mean value to the mean value for the reference condition
#' is also calculated. If a `basis` object returned by one of the
#' `basis_pooled` functions is given as an argument, this table also contains
#' the basis value for each condition.
#'
#' @param data a `data.frame` or a `basis` object
#' @param x the column name of the variable of interest (usually strength)
#' @param condition the column name for the condition
#' @param ref_condition a `character` representing the reference condition
#' @param ... not used
#'
#' @return a `data.frame`
#'
#' @examples
#' library(dplyr)
#' carbon.fabric.2 %>%
#'   filter(test == "WT") %>%
#'   condition_summary(strength, condition, "RTD")
#'
#' ##   condition  n     mean mean_fraction
#' ## 1       CTD 19 135.4719     0.9702503
#' ## 2       RTD 28 139.6257     1.0000000
#' ## 3       ETW 18 134.1009     0.9604312
#' ## 4      ETW2 21 130.1545     0.9321673
#'
#' carbon.fabric %>%
#'   filter(test == "FT") %>%
#'   basis_pooled_sd(strength, condition, batch) %>%
#'   condition_summary("RTD")
#'
#' ##   condition  n     mean mean_fraction    basis basis_fraction
#' ## 1       RTD 18 127.6211     1.0000000 116.8894      1.0000000
#' ## 2       ETW 18 117.8080     0.9231072 107.0762      0.9160476
#' ## 3       CTD 18 125.9629     0.9870063 115.2311      0.9858133
#'
#' @name condition_summary
NULL

condition_summary_base <- function(x, groups, ref_condition, basis) {
  if (!is.null(ref_condition)  && !ref_condition %in% groups) {
    stop("Specified condition not found.")
  }

  ref_cond_mean <- mean(x[groups == ref_condition])

  map_dfr(
    unique(groups),
    function(grp) {
      res <- data.frame(
        condition = grp,
        n = sum(groups == grp),
        mean = mean(x[groups == grp])
      )
      if (!is.null(ref_condition)) {
        res[["mean_fraction"]] <- res[["mean"]] / ref_cond_mean
      }
      if (!is.null(basis)) {
        res[["basis"]] <- basis$value[basis$group == grp]
      }
      if (!is.null(ref_condition) && !is.null(basis)) {
        res[["basis_fraction"]] <- res[["basis"]] /
          basis$value[basis$group == ref_condition]
      }
      res
    }
  )
}


#' @rdname condition_summary
#' @export
condition_summary <- function(data, ...) {
  UseMethod("condition_summary")
}

#' @rdname condition_summary
#' @export
condition_summary.data.frame <- function(data, x, condition,
                                         ref_condition = NULL, ...) {
  x <- eval_tidy(enquo(x), data)
  groups <- eval_tidy(enquo(condition), data)

  condition_summary_base(x, groups, ref_condition, basis = NULL)
}

#' @rdname condition_summary
#' @export
condition_summary.basis <- function(data, ref_condition = NULL, ...) {
  if (is.null(data$groups)) {
    stop("`groups` is empty, probably because a pooling method wasn't used.")
  }
  if (!is.data.frame(data$basis)) {
    stop("basis is not data.frame, likely because pooling method not used")
  }

  condition_summary_base(data$data, data$groups, ref_condition, data$basis)
}

#' Calculate the coefficient of variation
#'
#' @description
#' The coefficient of variation (CV) is the ratio of the standard
#' deviation to the mean of a sample. This function takes a vector
#' of data and calculates the CV.
#'
#' @param x a vector
#'
#' @return
#' The calculated CV
#'
#' @examples
#' set.seed(15)  # make this example reproducible
#' x <- rnorm(100, mean = 100, sd = 5)
#' cv(x)
#' ## [1] 0.04944505
#'
#' # the cv function can also be used within a call to dplyr::summarise
#' library(dplyr)
#' carbon.fabric %>%
#' filter(test == "WT") %>%
#'   group_by(condition) %>%
#'   summarise(mean = mean(strength), cv = cv(strength))
#'
#' ## # A tibble: 3 x 3
#' ##   condition  mean     cv
#' ##   <chr>     <dbl>  <dbl>
#' ## 1 CTD        137. 0.0417
#' ## 2 ETW        135. 0.0310
#' ## 3 RTD        142. 0.0451
#'
#'
#' @export
cv <- function(x) {
  sd(x) / mean(x)
}

# To add extra release questions that devtools::release asks
# Ref. https://devtools.r-lib.org/reference/release.html
release_questions <- function() {
  c(
    "Did you re-build the hk_ext.Rmd using `rebuild-long-running-vignette.R`?"
  )
}

# To allow the vignette to build correctly
#' @importFrom stats dnorm
NULL

#' Test for decrease in mean or minimum individual
#'
#' @description
#' This test is used when determining if a new process or
#' manufacturing location produces material properties that are
#' "equivalent" to an existing dataset, and hence the existing
#' basis values are applicable to the new dataset. This test is also
#' sometimes used for determining if a new batch of material is acceptable.
#' This function determines thresholds based on both minimum
#' individual and mean, and optionally evaluates a sample against those
#' thresholds. The joint distribution between the sample mean
#' and sample minimum is used to generate these thresholds.
#' When there is no true difference between the existing ("qualification")
#' and the new population from which the sample is obtained, there is a
#' probability of \eqn{\alpha} of falsely concluding that there is a
#' difference in mean or variance. It is assumed that both the original
#' and new populations are normally distributed.
#' According to Vangel (2002), this test provides improved power compared
#' with a test of mean and standard deviation.
#'
#' @param df_qual (optional) a data.frame containing the qualification data.
#' Defaults to NULL.
#' @param alpha the acceptable probability of a type I error
#' @param data_sample (optional) a vector of observations from the sample for
#' which equivalency is being tested. Defaults to NULL
#' @param n_sample (optional) the number of observations in the sample for
#' which equivalency will be tested. Defaults to NULL
#' @param data_qual (optional) a vector of observations from the
#' "qualification" data to which equivalency is being tested. Or the column of
#' \code{df_qual} that contains this data. Defaults to NULL
#' @param mean_qual (optional) the mean from the "qualification" data to which
#' equivalency is being tested. Defaults to NULL
#' @param sd_qual (optional) the standard deviation from the "qualification"
#' data to which equivalency is being tested. Defaults to NULL
#' @param modcv (optional) a boolean value indicating whether a modified CV
#' should be used. Defaults to FALSE, in which case the standard deviation
#' supplied (or calculated from \code{data_qual}) will be used directly.
#'
#' @return
#' Returns an object of class \code{equiv_mean_extremum}. This object is a list
#' with the following named elements:
#'
#' \item{\code{call}}{the expression used to call this function}
#' \item{\code{alpha}}{the value of alpha passed to this function}
#' \item{\code{n_sample}}{the number of observations in the sample for which
#'   equivalency is being checked. This is either the value \code{n_sample}
#'   passed to this function or the length of the vector \code{data_sample}.}
#' \item{\code{k1}}{the factor used to calculate the minimum individual
#'   threshold. The minimum individual threshold is calculated as
#'   \eqn{W_{min} = qual\,mean - k_1 \cdot qual\,sd}{
#'   Wmin = qual_mean - k1 * qual_sd}}
#' \item{\code{k2}}{the factor used to calculate the threshold for mean. The
#'   threshold for mean is calculated as
#'   \eqn{W_{mean} = qual\,mean - k_2 \cdot qual\,sd}{
#'   Wmean = qual_mean - k2 * qual_sd}}
#' \item{\code{modcv}}{logical value indicating whether the acceptance
#'   thresholds are calculated using the modified CV approach}
#' \item{\code{cv}}{the coefficient of variation of the qualification data.
#'   This value is not modified, even if \code{modcv=TRUE}}
#' \item{\code{cv_star}}{The modified coefficient of variation. If
#'   \code{modcv=FALSE}, this will be \code{NULL}}
#' \item{\code{threshold_min_indiv}}{The calculated threshold value for
#'   minimum individual}
#' \item{\code{threshold_mean}}{The calculated threshold value for mean}
#' \item{\code{result_min_indiv}}{a character vector of either "PASS" or
#'   "FAIL" indicating whether the data from \code{data_sample} passes the
#'   test for minimum individual. If \code{data_sample} was not supplied,
#'   this value will be \code{NULL}}
#' \item{\code{result_mean}}{a character vector of either "PASS" or
#'   "FAIL" indicating whether the data from \code{data_sample} passes the
#'   test for mean. If \code{data_sample} was not supplied, this value will
#'   be  \code{NULL}}
#' \item{\code{min_sample}}{The minimum value from the vector
#'   \code{data_sample}. if \code{data_sample} was not supplied, this will
#'   have a value of \code{NULL}}
#' \item{\code{mean_sample}}{The mean value from the vector
#'   \code{data_sample}. If \code{data_sample} was not supplied, this will
#'   have a value of \code{NULL}}
#'
#' @details
#' This function is used to
#' determine acceptance limits for a sample mean and sample minimum.
#' These acceptance limits are often used to set acceptance limits for
#' material strength for each lot of material, or each new manufacturing
#' site. When a sample meets the criteria that its mean and its minimum are
#' both greater than these limits, then one may accept the lot of material
#' or the new manufacturing site.
#'
#' This procedure is used to ensure that the strength of material processed
#' at a second site, or made with a new batch of material are not degraded
#' relative to the data originally used to determine basis values for the
#' material. For more information about the use of this procedure, see
#' CMH-17-1G or PS-ACE 100-2002-006.
#'
#' There are several optional arguments to this function. However, you can't
#' omit all of the optional arguments. You must supply either
#' \code{data_sample} or \code{n_sample}, but not both. You must also supply
#' either \code{data_qual} (and \code{df_qual} if \code{data_qual} is a
#' variable name and not a vector) or both \code{mean_qual} and \code{sd_qual},
#' but if you supply \code{data_qual} (and possibly \code{df_qual}) you should
#' not supply either \code{mean_qual} or \code{sd_qual} (and visa-versa). This
#' function will issue a warning or error if you violate any of these rules.
#'
#' If \code{modcv} is TRUE, the standard deviation used to calculate the
#' thresholds will be replaced with a standard deviation calculated
#' using the Modified Coefficient of Variation (CV) approach.
#' The Modified CV approach is a way of adding extra variance to the
#' qualification data in the case that the qualification data has less
#' variance than expected, which sometimes occurs when qualification testing
#' is performed in a short period of time.
#' Using the Modified CV approach, the standard deviation is calculated by
#' multiplying \code{CV_star * mean_qual} where \code{mean_qual} is either the
#' value supplied or the value calculated by \code{mean(data_qual)} and
#' \eqn{CV*} is the value computed by \code{\link{calc_cv_star}}.
#'
#' @examples
#' equiv_mean_extremum(alpha = 0.01, n_sample = 6,
#'                     mean_qual = 100, sd_qual = 5.5, modcv = TRUE)
#' ##
#' ## Call:
#' ## equiv_mean_extremum(mean_qual = 100, sd_qual = 5.5, n_sample = 6,
#' ##     alpha = 0.01, modcv = TRUE)
#' ##
#' ## Modified CV used: CV* = 0.0675 ( CV = 0.055 )
#' ##
#' ## For alpha = 0.01 and n = 6
#' ## ( k1 = 3.128346 and k2 = 1.044342 )
#' ##                   Min Individual   Sample Mean
#' ##      Thresholds:    78.88367        92.95069
#'
#' @seealso \code{\link{k_equiv}}
#' @seealso \code{\link{calc_cv_star}}
#'
#' @references
#' M. G. Vangel. Lot Acceptance and Compliance Testing Using the Sample Mean
#' and an Extremum, Technometrics, vol. 44, no. 3. pp. 242–249. 2002.
#'
#' “Composite Materials Handbook, Volume 1. Polymer Matrix Composites
#' Guideline for Characterization of Structural Materials,” SAE International,
#' CMH-17-1G, Mar. 2012.
#'
#' Federal Aviation Administration, “Material Qualification and Equivalency
#' for Polymer Matrix Composite Material Systems,” PS-ACE 100-2002-006,
#' Sep. 2003.
#'
#' @importFrom rlang enquo eval_tidy
#'
#' @export
equiv_mean_extremum <- function(df_qual = NULL, data_qual = NULL,
                                mean_qual = NULL, sd_qual = NULL,
                                data_sample = NULL, n_sample = NULL,
                                alpha, modcv = FALSE) {
  res <- list()
  class(res) <- "equiv_mean_extremum"

  res$call <- match.call()

  if (alpha <= 0) {
    stop("alpha must be positive")
  }
  if (alpha >= 1) {
    stop("alpha must be less than 1")
  }

  if (!is.null(df_qual)) {
    data_qual_enq <- enquo(data_qual)
    data_qual <- eval_tidy(data_qual_enq, df_qual)
  }

  if (!is.null(data_sample)) {
    if (!is.null(n_sample)) {
      warning("Both data_sample and n_sample were supplied. n_sample ignored.")
    }
    n_sample <- length(data_sample)
  }

  if (!is.null(data_qual)) {
    if (!is.null(mean_qual)) {
      warning("Both data_qual and mean_qual were supplied. mean_qual ignored.")
    }
    mean_qual <- mean(data_qual)

    if (!is.null(sd_qual)) {
      warning("Both data_qual and sd_qual were supplied. sd_qual ignored.")
    }
    sd_qual <- stats::sd(data_qual)
  }

  if (is.null(n_sample)) {
    stop(paste("Number of observations in sample not defined.",
               "You must provide either data_sample or n_sample"))
  }

  if (is.null(mean_qual)) {
    stop(paste("Qualification mean not defined.",
               "You must provide either data_qual or mean_qual"))
  }

  if (is.null(sd_qual)) {
    stop(paste("Qualification SD not defined.",
               "You must provide either data_qual or sd_qual"))
  }

  k <- k_equiv(alpha = alpha, n = n_sample)

  res$alpha <- alpha
  res$n_sample <- n_sample
  res$k1 <- k[1]
  res$k2 <- k[2]

  cv <- sd_qual / mean_qual
  res$modcv <- modcv
  res$cv <- cv
  res$cv_star <- NULL
  if (modcv) {
    cv <- calc_cv_star(cv)
    res$cv_star <- cv
    sd_qual <- cv * mean_qual
  }

  res$threshold_min_indiv <- mean_qual - sd_qual * res$k1
  res$threshold_mean <- mean_qual - sd_qual * res$k2

  res$result_min_indiv <- NULL
  res$result_mean <- NULL
  res$min_sample <- NULL
  res$mean_sample <- NULL
  if (!is.null(data_sample)) {
    res$min_sample <- min(data_sample)
    res$mean_sample <- mean(data_sample)
    res$result_min_indiv <- ifelse(res$min_sample >= res$threshold_min_indiv,
                                   "PASS", "FAIL")
    res$result_mean <- ifelse(res$mean_sample >= res$threshold_mean,
                              "PASS", "FAIL")
  }

  return(res)
}

#' Glance at an \code{equiv_mean_extremum} object
#'
#' @description
#' Glance accepts an object of type \code{equiv_mean_extremum} and returns a
#' \code{\link[tibble:tibble]{tibble::tibble}} with
#' one row of summaries.
#'
#' Glance does not do any calculations: it just gathers the results in a
#' tibble.
#'
#' @param x an equiv_mean_extremum object returned from
#'          \code{\link{equiv_mean_extremum}}
#' @param ... Additional arguments. Not used. Included only to match generic
#'            signature.
#'
#'
#' @return
#' A one-row \code{\link[tibble:tibble]{tibble::tibble}} with the following
#' columns:
#'
#' \item{\code{alpha}}{the value of alpha passed to this function}
#' \item{\code{n_sample}}{the number of observations in the sample for which
#'   equivalency is being checked. This is either the value \code{n_sample}
#'   passed to this function or the length of the vector \code{data_sample}.}
#' \item{\code{modcv}}{logical value indicating whether the acceptance
#'   thresholds are calculated using the modified CV approach}
#' \item{\code{threshold_min_indiv}}{The calculated threshold value for
#'   minimum individual}
#' \item{\code{threshold_mean}}{The calculated threshold value for mean}
#' \item{\code{result_min_indiv}}{a character vector of either "PASS" or
#'   "FAIL" indicating whether the data from \code{data_sample} passes the
#'   test for minimum individual. If \code{data_sample} was not supplied,
#'   this value will be \code{NULL}}
#' \item{\code{result_mean}}{a character vector of either "PASS" or
#'   "FAIL" indicating whether the data from \code{data_sample} passes the
#'   test for mean. If \code{data_sample} was not supplied, this value will
#'   be  \code{NULL}}
#' \item{\code{min_sample}}{The minimum value from the vector
#'   \code{data_sample}. if \code{data_sample} was not supplied, this will
#'   have a value of \code{NULL}}
#' \item{\code{mean_sample}}{The mean value from the vector
#'   \code{data_sample}. If \code{data_sample} was not supplied, this will
#'   have a value of \code{NULL}}
#'
#'
#' @seealso
#' \code{\link{equiv_mean_extremum}}
#'
#' @examples
#' x0 <- rnorm(30, 100, 4)
#' x1 <- rnorm(5, 91, 7)
#' eq <- equiv_mean_extremum(data_qual = x0, data_sample = x1, alpha = 0.01)
#' glance(eq)
#'
#' ## # A tibble: 1 x 9
#' ##   alpha n_sample modcv threshold_min_indiv threshold_mean
#' ##   <dbl>    <int> <lgl>               <dbl>          <dbl>
#' ## 1  0.01        5 FALSE                86.2           94.9
#' ## # ... with 4 more variables: result_min_indiv <chr>, result_mean <chr>,
#' ## #   min_sample <dbl>, mean_sample <dbl>
#'
#' @method glance equiv_mean_extremum
#' @importFrom tibble tibble
#'
#' @export
glance.equiv_mean_extremum <- function(x, ...) {  # nolint
  res <- tibble::tibble(
    alpha = x[["alpha"]],
    n_sample = x[["n_sample"]],
    modcv = x[["modcv"]],
    threshold_min_indiv = x[["threshold_min_indiv"]],
    threshold_mean = x[["threshold_mean"]]
  )

  if (!is.null(x[["result_min_indiv"]])) {
    res[["result_min_indiv"]] <- x[["result_min_indiv"]]
  }
  if (!is.null(x[["result_mean"]])) {
    res[["result_mean"]] <- x[["result_mean"]]
  }
  if (!is.null(x[["min_sample"]])) {
    res[["min_sample"]] <- x[["min_sample"]]
  }
  if (!is.null(x[["mean_sample"]])) {
    res[["mean_sample"]] <- x[["mean_sample"]]
  }
  res
}


#' @export
print.equiv_mean_extremum <- function(x, ...) {
  cat("\nCall:\n",
      paste(deparse(x$call), sep = "\n", collapse = "\n"), "\n\n", sep = "")

  justify <- c("right", "centre", "centre")
  col_width <- c(16L, 16L, 16L)

  if (x$modcv) {
    cat("Modified CV used: CV* =", format(x$cv_star, ...),
        "( CV =", format(x$cv, ...),
        ")\n\n")
  }

  cat("For alpha =", format(x$alpha, ...),
      "and n =", format(x$n_sample, ...),
      "\n( k1 =", format(x$k1, ...),
      "and k2 =", format(x$k2, ...), ")\n")

  cat(format_row(list("", "Min Individual", "Sample Mean"),
                 justify, col_width, ...))

  if (!is.null(x$min_sample)) {
    cat(format_row(list("Sample:", x$min_sample, x$mean_sample),
                   justify, col_width, ...))
  }
  cat(format_row(list("Thresholds:", x$threshold_min_indiv, x$threshold_mean),
                 justify, col_width, ...))
  if (!is.null(x$result_min_indiv)) {
    cat(format_row(list("Equivalency:", x$result_min_indiv, x$result_mean),
                   justify, col_width, ...))
  }
}


#' k-factors for determining acceptance based on sample mean and an extremum
#'
#' @param alpha the acceptable probability of a type I error
#' @param n the number of observations in the sample to test
#' @return a vector with elements c(k1, k2). k1 is for testing the sample
#'   extremum. k2 is for testing the sample mean
#' @details
#'   The k-factors returned by this function are used for determining
#'   whether to accept a new dataset.
#'
#'   This function is used as part of the procedure for
#'   determining acceptance limits for a sample mean and sample minimum.
#'   These acceptance limits are often used to set acceptance limits for
#'   material strength for each lot of material, or each new manufacturing
#'   site. When a sample meets the criteria that its mean and its minimum are
#'   both greater than these limits, then one may accept the lot of material
#'   or the new manufacturing site.
#'
#'   This procedure is used to ensure that the strength of material processed
#'   at a second site, or made with a new batch of material are not degraded
#'   relative to the data originally used to determine basis values for the
#'   material. For more information about the use of this procedure, see
#'   CMH-17-1G or PS-ACE 100-2002-006.
#'
#'   According to Vangel (2002), the use of mean and extremum for this purpose
#'   is more powerful than the use of mean and standard deviation.
#'
#'   The results of this function match those published by Vangel within
#'   0.05\% for \eqn{n > 2} and \eqn{\alpha \le 0.25}. Those results published
#'   by Vangel are identical to those published in CMH-17-1G.
#'
#'   This function uses numerical integration and numerical optimization to
#'   find values of the factors \eqn{k_1} and \eqn{k_2} based on Vangel's
#'   saddle point approximation.
#'
#'   The value \eqn{n} refers to the number of observations in the sample
#'   being compared with the original population (the qualification sample is
#'   usually assumed to be equal to the population statistics).
#'
#'   The value of \eqn{alpha} is the acceptable probability of a type I error.
#'   Normally, this is set to 0.05 for material or process equivalency and 0.01
#'   when setting lot acceptance limits. Though, in principle, this parameter
#'   can be set to any number between 0 and 1. This function, however, has only
#'   been validated in the range of \eqn{1e-5 \le alpha \le 0.5}.
#'
#' @references
#' M. G. Vangel. Lot Acceptance and Compliance Testing Using the Sample Mean
#' and an Extremum, Technometrics, vol. 44, no. 3. pp. 242–249. 2002.
#'
#' “Composite Materials Handbook, Volume 1. Polymer Matrix Composites
#' Guideline for Characterization of Structural Materials,” SAE International,
#' CMH-17-1G, Mar. 2012.
#'
#' Federal Aviation Administration, “Material Qualification and Equivalency
#' for Polymer Matrix Composite Material Systems,” PS-ACE 100-2002-006,
#' Sep. 2003.
#'
#' @seealso
#' \code{\link{equiv_mean_extremum}}
#'
#' @examples
#' qual_mean <- 100
#' qual_sd <- 3.5
#' k <- k_equiv(0.01, 5)
#' print("Minimum Individual Acceptance Limit:")
#' print(qual_mean - qual_sd * k[1])
#' print("Minimum Average Acceptance Limit:")
#' print(qual_mean - qual_sd * k[2])
#'
#' ## [1] "Minimum Individual Acceptance Limit:"
#' ## [1] 89.24981
#' ## [1] "Minimum Average Acceptance Limit:"
#' ## [1] 96.00123
#'
#' @export
k_equiv <- function(alpha, n) {
  # If you are making changes to this function, you should run the full
  # validation test in test/test-equiv.R by setting full_test = TRUE.
  # The full validation test is not run by default as it takes several
  # minutes to run, which is impractical for normal test-driven-development
  # workflows.

  if (alpha <= 0) {
    stop("alpha must be positive")
  }
  if (alpha >= 1) {
    stop("alpha must be less than 1")
  }
  if (alpha < 1e-5 || alpha > 0.5) {
    warning(paste("k-factor solution has only been validated",
                  "for 1e-5 <= alpha <= 0.5"))
  }
  if (n < 2) {
    stop("n must be greater than 2")
  }

  # In order to speed up convergence, a linear model was fit to estimate the
  # values of k1 and k2. These estimated values are used as a starting point
  # for the numerical optimization
  theta <- c(0.318 * log(n) - 0.285 * log(alpha) + 1.114,
             -0.16751 * log(n)
             - 0.29011 * log(alpha)
             + 0.08414 * log(n) * log(alpha)
             + 0.65848)

  # The function f returns a penalty value for use in numerical optimization.
  # The first parameter, k, should be a vector of c(k1, k2).
  f <- function(k, n, alpha) {
    fx1 <- 1 - (stats::pnorm(-k[1], lower.tail = FALSE)) ** n
    fxbar <- stats::pnorm(sqrt(n) * (-k[2]))

    logh <- function(t) {
      stats::dnorm(t, log = TRUE) - stats::pnorm(t, lower.tail = FALSE,
                                                 log.p = TRUE)
    }
    h <- function(t) {
      exp(stats::dnorm(t, log = TRUE) - stats::pnorm(t, lower.tail = FALSE,
                                                     log.p = TRUE))
    }
    h_minus_t <- function(t) {
      ifelse(t > 60, t ** -1, h(t) - t)
    }

    lambda_hat <- stats::uniroot(
      function(lambda) {
        ((n - 1) / n) * h_minus_t(lambda) - k[1] + k[2]
      },
      interval = c(-1000, 1000),
      extendInt = "yes"
    )$root

    a_fcn <- function(t, n) {
      hmt <- h_minus_t(t)
      exp(
        - (n - 1) * logh(t) +
          (n - 1) ** 2 / (2 * n) * hmt ** 2 +
          (n - 1) * t * hmt
      ) * sqrt(ifelse(t > 60, t ** -2, 1 - h(t) * hmt))
    }

    fx1xbar_numerator <- stats::pnorm(sqrt(n) * (-k[2])) * stats::integrate(
      function(t) {
        a_fcn(t, n)
      },
      lower = -Inf,
      upper = lambda_hat,
      subdivisions = 1000L
    )$value + stats::integrate(
      function(t) {
        stats::pnorm(
          sqrt(n) * (-k[1] + (n - 1) / n * h_minus_t(t))
        ) * a_fcn(t, n)
      },
      lower = lambda_hat,
      upper = Inf,
      subdivisions = 1000L,
      rel.tol = 1e-8
    )$value

    fx1xbar_denominator <- stats::integrate(function(t) a_fcn(t, n),
                                            lower = -Inf, upper = Inf)$value
    fx1xbar <- fx1xbar_numerator / fx1xbar_denominator

    # Use the sum of the absolute values of the two functions being solved as
    # the penalty function. However, it was found empirically that the first
    # function listed is more sensitive, so give it a higher weight to aid in
    # finding hte correct solution
    return(
      abs(fx1 + fxbar - fx1xbar - alpha) * 100 + abs(fx1 - fxbar)
    )
  }

  # Perform the constrained numeric optimization to find the values of k1 and
  # k2. The constrait enforces k1 > k2, which is required such that all values
  # are real
  res <- stats::constrOptim(
    theta,
    f,
    grad = NULL,
    ui = c(1, -1),
    ci = 0,
    n = n,
    alpha = alpha
  )
  # nocov start
  if (res$convergence != 0) {
    warning(
      "k-factor search did not converge. The results are unreliable."
      )
  }
  # nocov end

  return(res$par)
}


#' Equivalency based on change in mean value
#'
#' @description
#' Checks for change in the mean value between a qualification data set and
#' a sample. This is normally used to check for properties such as modulus.
#' This function is a wrapper for a two-sample t--test.
#'
#' @param df_qual (optional) a data.frame containing the qualification data.
#' Defaults to NULL.
#' @param alpha the acceptable probability of a Type I error
#' @param data_sample a vector of observations from the sample being compared
#'   for equivalency
#' @param n_sample the number of observations in the sample being compared for
#'   equivalency
#' @param mean_sample the mean of the sample being compared for equivalency
#' @param sd_sample the standard deviation of the sample being compared for
#'   equivalency
#' @param data_qual (optional) a vector of observations from the
#'   "qualification" data to which equivalency is being tested. Or the column of
#'   \code{df_qual} that contains this data. Defaults to NULL
#' @param n_qual the number of observations in the qualification data to which
#'   the sample is being compared for equivalency
#' @param mean_qual the mean from the qualification data to which the sample
#'   is being compared for equivalency
#' @param sd_qual the standard deviation from the qualification data to which
#'   the sample is being compared for equivalency
#' @param modcv a logical value indicating whether the modified CV approach
#'   should be used. Defaults to \code{FALSE}
#'
#' @return
#' \item{\code{call}}{the expression used to call this function}
#' \item{\code{alpha}}{the value of alpha passed to this function}
#' \item{\code{n_sample}}{the number of observations in the sample for which
#'   equivalency is being checked. This is either the value \code{n_sample}
#'   passed to this function or the length of the vector \code{data_sample}.}
#' \item{\code{mean_sample}}{the mean of the observations in the sample for
#'   which equivalency is being checked. This is either the value
#'   \code{mean_sample} passed to this function or the mean of the vector
#'   \code{data-sample}.}
#' \item{\code{sd_sample}}{the standard deviation of the observations in the
#'   sample for which equivalency is being checked. This is either the value
#'   \code{mean_sample} passed to this function or the standard deviation of
#'   the vector \code{data-sample}.}
#' \item{\code{n_qual}}{the number of observations in the qualification data
#'   to which the sample is being compared for equivalency. This is either
#'   the value \code{n_qual} passed to this function or the length of the
#'   vector \code{data_qual}.}
#' \item{\code{mean_qual}}{the mean of the qualification data to which the
#'   sample is being compared for equivalency. This is either the value
#'   \code{mean_qual} passed to this function or the mean of the vector
#'   \code{data_qual}.}
#' \item{\code{sd_qual}}{the standard deviation of the qualification data to
#'   which the sample is being compared for equivalency. This is either the
#'   value \code{mean_qual} passed to this function or the standard deviation
#'   of the vector \code{data_qual}.}
#' \item{\code{modcv}}{logical value indicating whether the equivalency
#'   calculations were performed using the modified CV approach}
#' \item{\code{sp}}{the value of the pooled standard deviation. If
#'   \code{modecv = TRUE}, this pooled standard deviation includes the
#'   modification to the qualification CV.}
#' \item{\code{t0}}{the test statistic}
#' \item{\code{t_req}}{the t-value for \eqn{\alpha / 2} and
#'   \eqn{df = n1 + n2 -2}}
#' \item{\code{threshold}}{a vector with two elements corresponding to the
#'   minimum and maximum values of the sample mean that would result in a
#'   pass}
#' \item{\code{result}}{a character vector of either "PASS" or "FAIL"
#'   indicating the result of the test for change in mean}
#'
#' @details
#' There are several optional arguments to this function. Either (but not both)
#' \code{data_sample} or all of \code{n_sample}, \code{mean_sample} and
#' \code{sd_sample} must be supplied. And, either (but not both)
#' \code{data_qual}
#' (and also \code{df_qual} if \code{data_qual} is a column name and not a
#' vector) or all of \code{n_qual}, \code{mean_qual} and \code{sd_qual} must
#' be supplied. If these requirements are violated, warning(s) or error(s) will
#' be issued.
#'
#' This function uses a two-sample t-test to determine if there is a difference
#' in the mean value of the qualification data and the sample. A pooled
#' standard deviation is used in the t-test. The procedure is per CMH-17-1G.
#'
#' If \code{modcv} is TRUE, the standard deviation used to calculate the
#' thresholds will be replaced with a standard deviation calculated
#' using the Modified Coefficient of Variation (CV) approach.
#' The Modified CV approach is a way of adding extra variance to the
#' qualification data in the case that the qualification data has less
#' variance than expected, which sometimes occurs when qualification testing
#' is performed in a short period of time.
#' Using the Modified CV approach, the standard deviation is calculated by
#' multiplying \code{CV_star * mean_qual} where \code{mean_qual} is either the
#' value supplied or the value calculated by \code{mean(data_qual)} and
#' \eqn{CV*} is determined using \code{\link{calc_cv_star}}.
#'
#' Note that the modified CV option should only be used if that data passes the
#' Anderson--Darling test.
#'
#' @examples
#' equiv_change_mean(alpha = 0.05, n_sample = 9, mean_sample = 9.02,
#'                   sd_sample = 0.15785, n_qual = 28, mean_qual = 9.24,
#'                   sd_qual = 0.162, modcv = TRUE)
#'
#' ## Call:
#' ## equiv_change_mean(n_qual = 28, mean_qual = 9.24, sd_qual = 0.162,
#' ##                   n_sample = 9, mean_sample = 9.02, sd_sample = 0.15785,
#' ##                   alpha = 0.05,modcv = TRUE)
#' ##
#' ## For alpha = 0.05
#' ## Modified CV used
#' ##                   Qualification        Sample
#' ##           Number        28               9
#' ##             Mean       9.24             9.02
#' ##               SD      0.162           0.15785
#' ##           Result               PASS
#' ##    Passing Range       8.856695 to 9.623305
#'
#' @references
#' “Composite Materials Handbook, Volume 1. Polymer Matrix Composites
#' Guideline for Characterization of Structural Materials,” SAE International,
#' CMH-17-1G, Mar. 2012.
#'
#' @seealso \code{\link{calc_cv_star}}
#' @seealso \code{\link[stats]{t.test}}
#'
#' @export
equiv_change_mean <- function(df_qual = NULL, data_qual = NULL,
                              n_qual = NULL, mean_qual = NULL,
                              sd_qual = NULL, data_sample = NULL,
                              n_sample = NULL, mean_sample = NULL,
                              sd_sample = NULL, alpha, modcv = FALSE) {
  if (alpha <= 0 | alpha >= 1) {
    stop("alpha must be positive and less than 1")
  }

  if (!is.null(df_qual)) {
    data_qual_enq <- enquo(data_qual)
    data_qual <- eval_tidy(data_qual_enq, df_qual)
  }

  if (!is.null(data_sample)) {
    if (!is.null(n_sample)) {
      warning("Both data_sample and n_sample supplied. n_sample ignored.")
    }
    n_sample <- length(data_sample)

    if (!is.null(mean_sample)) {
      warning("Both data_sample and mean_sample supplied. mean_sample ignored")
    }
    mean_sample <- mean(data_sample)

    if (!is.null(sd_sample)) {
      warning("Both data_sample and sd_sample supplied. sd_sample ignored")
    }
    sd_sample <- stats::sd(data_sample)
  }

  if (!is.null(data_qual)) {
    if (!is.null(n_qual)) {
      warning("Both data_qual and n_qual supplied. n_qual ignored.")
    }
    n_qual <- length(data_qual)

    if (!is.null(mean_qual)) {
      warning("Both data_qual and mean_qual supplied. mean_qual ignored")
    }
    mean_qual <- mean(data_qual)

    if (!is.null(sd_qual)) {
      warning("Both data_qual and sd_qual supplied. sd_qual ignored")
    }
    sd_qual <- stats::sd(data_qual)
  }

  verify_equiv_change_mean_var(n_sample, mean_sample, sd_sample,
                               n_qual, mean_qual, sd_qual)

  res <- list()
  class(res) <- "equiv_change_mean"

  res$call <- match.call()
  res$alpha <- alpha
  res$n_sample <- n_sample
  res$mean_sample <- mean_sample
  res$sd_sample <- sd_sample
  res$n_qual <- n_qual
  res$mean_qual <- mean_qual
  res$sd_qual <- sd_qual
  res$modcv <- modcv

  cv <- sd_qual / mean_qual
  res$modcv <- modcv
  if (modcv) {
    cv <- calc_cv_star(cv)
    sd_qual <- cv * mean_qual
  }

  sp <- sqrt(
    ((n_qual - 1) * sd_qual ** 2 + (n_sample - 1) * sd_sample ** 2) /
      (n_qual + n_sample - 2)
  )
  res$sp <- sp

  t0 <- (mean_sample - mean_qual) /
    (sp * sqrt(1 / n_qual + 1 / n_sample))
  res$t0 <- t0

  t_req <- stats::qt(alpha / 2, n_qual + n_sample - 2, lower.tail = FALSE)
  res$t_req <- t_req

  res$threshold <- c(
    mean_qual - t_req * sp * sqrt(1 / n_qual + 1 / n_sample),
    mean_qual + t_req * sp * sqrt(1 / n_qual + 1 / n_sample)
  )

  res$result <- ifelse(-t_req <= t0 & t0 <= t_req, "PASS", "FAIL")

  return(res)
}


verify_equiv_change_mean_var <- function(n_sample, mean_sample, sd_sample,
                                         n_qual, mean_qual, sd_qual) {
  if (is.null(n_sample)) {
    stop("n_sample not set")
  }
  if (is.null(mean_sample)) {
    stop("mean_sample not set")
  }
  if (is.null(sd_sample)) {
    stop("sd_sample not set")
  }
  if (is.null(n_qual)) {
    stop("n_qual not set")
  }
  if (is.null(mean_qual)) {
    stop("mean_qual not set")
  }
  if (is.null(sd_qual)) {
    stop("sd_qual not set")
  }
}


#' Glance at a \code{equiv_change_mean} object
#'
#' @description
#' Glance accepts an object of type \code{equiv_change_mean}
#'  and returns a \code{\link[tibble:tibble]{tibble::tibble}} with
#' one row of summaries.
#'
#' Glance does not do any calculations: it just gathers the results in a
#' tibble.
#'
#' @param x a \code{equiv_change_mean} object returned from
#'          \code{\link{equiv_change_mean}}
#' @param ... Additional arguments. Not used. Included only to match generic
#'            signature.
#'
#'
#' @return
#' A one-row \code{\link[tibble:tibble]{tibble::tibble}} with the following
#' columns:
#'
#' \item{\code{alpha}}{the value of alpha passed to this function}
#' \item{\code{n_sample}}{the number of observations in the sample for which
#'   equivalency is being checked. This is either the value \code{n_sample}
#'   passed to this function or the length of the vector \code{data_sample}.}
#' \item{\code{mean_sample}}{the mean of the observations in the sample for
#'   which equivalency is being checked. This is either the value
#'   \code{mean_sample} passed to this function or the mean of the vector
#'   \code{data-sample}.}
#' \item{\code{sd_sample}}{the standard deviation of the observations in the
#'   sample for which equivalency is being checked. This is either the value
#'   \code{mean_sample} passed to this function or the standard deviation of
#'   the vector \code{data-sample}.}
#' \item{\code{n_qual}}{the number of observations in the qualification data
#'   to which the sample is being compared for equivalency. This is either
#'   the value \code{n_qual} passed to this function or the length of the
#'   vector \code{data_qual}.}
#' \item{\code{mean_qual}}{the mean of the qualification data to which the
#'   sample is being compared for equivalency. This is either the value
#'   \code{mean_qual} passed to this function or the mean of the vector
#'   \code{data_qual}.}
#' \item{\code{sd_qual}}{the standard deviation of the qualification data to
#'   which the sample is being compared for equivalency. This is either the
#'   value \code{mean_qual} passed to this function or the standard deviation
#'   of the vector \code{data_qual}.}
#' \item{\code{modcv}}{logical value indicating whether the equivalency
#'   calculations were performed using the modified CV approach}
#' \item{\code{sp}}{the value of the pooled standard deviation. If
#'   \code{modecv = TRUE}, this pooled standard deviation includes the
#'   modification to the qualification CV.}
#' \item{\code{t0}}{the test statistic}
#' \item{\code{t_req}}{the t-value for \eqn{\alpha / 2} and
#'   \eqn{df = n1 + n2 -2}}
#' \item{\code{threshold_min}}{the minimum value of the sample mean that would
#'   result in a pass}
#' \item{\code{threshold_max}}{the maximum value of the sample mean that would
#'   result in a pass}
#' \item{\code{result}}{a character vector of either "PASS" or "FAIL"
#'   indicating the result of the test for change in mean}
#'
#'
#' @seealso
#' \code{\link{equiv_change_mean}}
#'
#' @examples
#' x0 <- rnorm(30, 100, 4)
#' x1 <- rnorm(5, 91, 7)
#' eq <- equiv_change_mean(data_qual = x0, data_sample = x1, alpha = 0.01)
#' glance(eq)
#'
#' ## # A tibble: 1 x 14
#' ##   alpha n_sample mean_sample sd_sample n_qual mean_qual sd_qual modcv
#' ##   <dbl>    <int>       <dbl>     <dbl>  <int>     <dbl>   <dbl> <lgl>
#' ## 1  0.01        5        85.8      9.93     30      100.    3.90 FALSE
#' ## # ... with 6 more variables: sp <dbl>, t0 <dbl>, t_req <dbl>,
#' ## #   threshold_min <dbl>, threshold_max <dbl>, result <chr>
#'
#' @method glance equiv_change_mean
#' @importFrom tibble tibble
#'
#' @export
glance.equiv_change_mean <- function(x, ...) {  # nolint
  with(
    x,
    tibble::tibble(
      alpha = alpha,
      n_sample = n_sample,
      mean_sample = mean_sample,
      sd_sample = sd_sample,
      n_qual = n_qual,
      mean_qual = mean_qual,
      sd_qual = sd_qual,
      modcv = modcv,
      sp = sp,
      t0 = t0,
      t_req = t_req,
      threshold_min = threshold[1],
      threshold_max = threshold[2],
      result = result
    )
  )
}


#' @export
print.equiv_change_mean <- function(x, ...) {
  cat("\nCall:\n",
      paste(deparse(x$call), sep = "\n", collapse = "\n"), "\n\n", sep = "")

  cat("For alpha =", format(x$alpha, ...), "\n")

  if (x$modcv) {
    cat("Modified CV used\n")
  }

  justify3 <- c("right", "centre", "centre")
  width3 <- c(16L, 16L, 16L)

  justify2 <- c("right", "centre")
  width2 <- c(16L, 32L)

  cat(format_row(list("", "Qualification", "Sample"),
                 justify3, width3, ...))
  cat(format_row(list("Number", x$n_qual, x$n_sample),
                 justify3, width3, ...))
  cat(format_row(list("Mean", x$mean_qual, x$mean_sample),
                 justify3, width3, ...))
  cat(format_row(list("SD", x$sd_qual, x$sd_sample),
                 justify3, width3, ...))
  cat(format_row(list("Result", x$result),
                 justify2, width2, ...))
  cat(format_row(
    list("Passing Range", paste0(format(x$threshold[1], ...),
                                 " to ",
                                 format(x$threshold[2], ...))),
    justify2, width2, ...)
  )
}

context("equiv")

suppressMessages(library(dplyr))

test_that("k-factor warnings and errors are raised", {
  expect_error(k_equiv(-0.01, 3))
  expect_error(k_equiv(1.01, 3))
  expect_warning(k_equiv(1e-9, 3))
  expect_warning(k_equiv(0.75, 3))
  expect_error(k_equiv(0.05, 1))
})

test_that("k-factors match those published in literature", {

  if (requireNamespace("tidyr", quietly = TRUE)) {

    # the files k1.vangel.csv and k2.vangel.csv contain the k factors
    # published in Vangel's 2002 paper.

    k1_vangel <- read.csv(system.file("extdata", "k1.vangel.csv",
                                      package = "cmstatr")) %>%
      tidyr::gather(n, k1, X2:X10) %>%
      mutate(n = as.numeric(substring(n, 2)))

    k2_vangel <- read.csv(system.file("extdata", "k2.vangel.csv",
                                      package = "cmstatr")) %>%
      tidyr::gather(n, k2, X2:X10) %>%
      mutate(n = as.numeric(substring(n, 2)))

    # This test is super slow, so we will only run a sampling of the values
    # that
    # would normally be used in the validation, unless the flag full_test is
    # set to TRUE
    full_test <- FALSE
    diff_df <- inner_join(k1_vangel, k2_vangel, by = c("alpha", "n")) %>%
      mutate(error_threshold = case_when(n == 2 & alpha > 0.25 ~ 0.10,
                                         n == 2 & alpha <= 0.25 ~ 0.02,
                                         n > 2 & alpha > 0.25 ~ 0.005,
                                         TRUE ~ 5e-4))

    diff_df <- diff_df %>%
      sample_n(ifelse(full_test, length(diff_df$k1), 5)) %>%
      tidyr::gather(fct, vangel, k1:k2) %>%
      group_by(alpha, n) %>%
      mutate(calc = k_equiv(first(alpha), first(n))) %>%
      mutate(diff = (vangel - calc) / vangel) %>%
      ungroup() %>%
      rowwise() %>%
      mutate(check = expect_lte(abs(diff), error_threshold, label =
                                  paste0("Validation failure for ",
                                         "alpha=", alpha,
                                         ", n=", n,
                                         " computed ", fct, "=", calc,
                                         " but validation ", fct, "=", vangel,
                                         ".\n")))
  }
})

test_that("check equiv_mean_extremum against HYTEQ using some example data", {
  data_sample <- c(145.055, 148.329, 142.667, 141.795, 144.139,
                   135.923, 136.177, 133.523, 134.350)
  res <- equiv_mean_extremum(alpha = 0.05, data_sample = data_sample,
                             mean_qual = 141.310, sd_qual = 6.415)

  expect_equal(res$alpha, 0.05)
  expect_equal(res$n_sample, 9)
  expect_equal(res$modcv, FALSE)
  expect_equal(res$cv, 6.415 / 141.310, tolerance = 1e-8)
  expect_equal(res$threshold_min_indiv, 123.725, tolerance = 1e-2)
  expect_equal(res$threshold_mean, 137.197, tolerance = 1e-2)
  expect_equal(res$result_min_indiv, "PASS")
  expect_equal(res$result_mean, "PASS")

  expect_output(print(res), "alpha\\W*=\\W*0.05")
  expect_output(print(res), "n\\W*=\\W*9")
  expect_output(print(res), "Sample[s:]*\\W*133.5\\d*\\W*140.2\\d*")
  expect_output(print(res), "Threshold[s:]*\\W*123.7\\d*\\W*137.[12]")
  expect_output(print(res), "Equiv\\w*\\W*PASS\\W*PASS")
})

test_that("check equiv_mean_extremum against HYTEQ using an example (modCV)", {
  data_sample <- c(145.055, 148.329, 142.667, 141.795, 144.139,
                   135.923, 136.177, 133.523, 134.350)
  res <- equiv_mean_extremum(alpha = 0.05, data_sample = data_sample,
                             mean_qual = 141.310, sd_qual = 6.415,
                             modcv = TRUE)

  expect_equal(res$alpha, 0.05)
  expect_equal(res$n_sample, 9)
  expect_equal(res$modcv, TRUE)
  expect_equal(res$cv, 6.415 / 141.310, tolerance = 1e-8)
  expect_equal(res$cv_star, 0.0627, tolerance = 1e-3)
  expect_equal(res$threshold_min_indiv, 117.024, tolerance = 1e-2)
  expect_equal(res$threshold_mean, 135.630, tolerance = 1e-2)
  expect_equal(res$result_min_indiv, "PASS")
  expect_equal(res$result_mean, "PASS")

  #ensure print indicates it's modCV
  expect_output(print(res), "[Mm]od\\w*\\WCV")

  expect_output(print(res), "alpha\\W*=\\W*0.05")
  expect_output(print(res), "n\\W*=\\W*9")
  expect_output(print(res), "Sample[s:]*\\W*133.5\\d*\\W*140.2\\d*")
  expect_output(print(res), "Threshold[s:]*\\W*117.0\\d*\\W*135.6")
  expect_output(print(res), "Equiv\\w*\\W*PASS\\W*PASS")
})

test_that("check three ways of specifying qual data are same (mean_ext)", {
  data_qual <- c(145.055, 148.329, 142.667, 141.795, 144.139,
                   135.923, 136.177, 133.523, 134.350)
  data_qual_df <- data.frame(strength = data_qual)

  res1 <- equiv_mean_extremum(alpha = 0.05, data_qual = data_qual,
                              n_sample = 5)
  res2 <- equiv_mean_extremum(alpha = 0.05, df_qual = data_qual_df,
                              data_qual = strength, n_sample = 5)
  res3 <- equiv_mean_extremum(alpha = 0.05, mean_qual = mean(data_qual),
                              sd_qual = sd(data_qual), n_sample = 5)

  # the calls will be different, and that's okay
  res1$call <- NULL
  res2$call <- NULL
  res3$call <- NULL

  expect_equal(res1, res2)
  expect_equal(res1, res3)
})

test_that("check that glance.equiv_mean_extremum produces expected results", {
  data_sample <- c(145.055, 148.329, 142.667, 141.795, 144.139,
                   135.923, 136.177, 133.523, 134.350)
  res <- equiv_mean_extremum(alpha = 0.05, data_sample = data_sample,
                             mean_qual = 141.310, sd_qual = 6.415,
                             modcv = TRUE)
  res <- glance(res)

  expect_equal(res$alpha[1], 0.05)
  expect_equal(res$n_sample[1], 9)
  expect_equal(res$modcv[1], TRUE)
  expect_equal(res$threshold_min_indiv[1], 117.024, tolerance = 1e-2)
  expect_equal(res$threshold_mean[1], 135.630, tolerance = 1e-2)
  expect_equal(res$result_min_indiv[1], "PASS")
  expect_equal(res$result_mean[1], "PASS")
  expect_equal(res$min_sample[1], min(data_sample))
  expect_equal(res$mean_sample[1], mean(data_sample))

  res <- equiv_mean_extremum(alpha = 0.05, mean_qual = 141.310,
                             sd_qual = 6.415, n_sample = 9, modcv = TRUE)
  res <- glance(res)

  expect_equal(ncol(res), 5)
  expect_equal(res$alpha[1], 0.05)
  expect_equal(res$n_sample[1], 9)
  expect_equal(res$modcv[1], TRUE)
  expect_equal(res$threshold_min_indiv[1], 117.024, tolerance = 1e-2)
  expect_equal(res$threshold_mean[1], 135.630, tolerance = 1e-2)
})

test_that("check equiv_change_mean against HYTEQ using some example data", {
  res <- equiv_change_mean(alpha = 0.05, n_sample = 9, mean_sample = 9.02,
                           sd_sample = 0.15785, n_qual = 28, mean_qual = 9.24,
                           sd_qual = 0.162)

  expect_equal(res$alpha, 0.05)
  expect_equal(res$n_sample, 9)
  expect_equal(res$mean_sample, 9.02)
  expect_equal(res$sd_sample, 0.15785)
  expect_equal(res$n_qual, 28)
  expect_equal(res$mean_qual, 9.24)
  expect_equal(res$sd_qual, 0.162)
  expect_equal(res$sp, 0.1608, tolerance = 5e-4)
  expect_equal(res$t0, -3.570, tolerance = 5e-3)
  expect_equal(res$t_req, 2.030, tolerance = 5e-3)
  expect_equal(res$threshold, c(9.115, 9.365), tolerance = 5e-3)
  expect_equal(res$modcv, FALSE)
  expect_equal(res$result, "FAIL")

  expect_output(print(res), "alpha\\W*=\\W*0.05")
  expect_output(print(res), "Number\\W*28\\W*9")
  expect_output(print(res), "Mean\\W*9.24\\d*\\W*9.02\\d*")
  expect_output(print(res), "Result\\W*FAIL")
  expect_output(print(res), "Range\\W*9.11[45]\\d*\\W*\\w*\\W*9.365\\d*")
})

test_that("check equiv_change_mean against HYTEQ using an example (modCV)", {
  res <- equiv_change_mean(alpha = 0.05, n_sample = 9, mean_sample = 9.02,
                           sd_sample = 0.15785, n_qual = 28, mean_qual = 9.24,
                           sd_qual = 0.162, modcv = TRUE)

  expect_equal(res$alpha, 0.05)
  expect_equal(res$n_sample, 9)
  expect_equal(res$mean_sample, 9.02)
  expect_equal(res$sd_sample, 0.15785)
  expect_equal(res$n_qual, 28)
  expect_equal(res$mean_qual, 9.24)
  expect_equal(res$sd_qual, 0.162)
  expect_equal(res$sp, 0.4927, tolerance = 5e-4)
  expect_equal(res$t0, -1.165, tolerance = 5e-3)
  expect_equal(res$t_req, 2.03, tolerance = 5e-3)
  expect_equal(res$threshold, c(8.857, 9.623), tolerance = 5e-3)
  expect_equal(res$modcv, TRUE)
  expect_equal(res$result, "PASS")

  #ensure print indicates it's modCV
  expect_output(print(res), "[Mm]od\\w*\\WCV")

  expect_output(print(res), "alpha\\W*=\\W*0.05")
  expect_output(print(res), "Number\\W*28\\W*9")
  expect_output(print(res), "Mean\\W*9.24\\d*\\W*9.02\\d*")
  expect_output(print(res), "Result\\W*PASS")
  expect_output(print(res), "Range\\W*8.85[67]\\d*\\W*\\w*\\W*9.623\\d*")
})

test_that("check four ways of specifying qual data are same (chg in mean)", {
  data_qual <- c(145.055, 148.329, 142.667, 141.795, 144.139,
                 135.923, 136.177, 133.523, 134.350)
  data_qual_df <- data.frame(strength = data_qual)
  data_sample <- c(145.055, 148.329, 142.667, 141.795, 144.139)

  res1 <- equiv_change_mean(alpha = 0.05, data_qual = data_qual,
                            n_sample = length(data_sample),
                            mean_sample = mean(data_sample),
                            sd_sample = sd(data_sample))
  res2 <- equiv_change_mean(alpha = 0.05, df_qual = data_qual_df,
                            data_qual = strength,
                            n_sample = length(data_sample),
                            mean_sample = mean(data_sample),
                            sd_sample = sd(data_sample))
  res3 <- equiv_change_mean(alpha = 0.05, mean_qual = mean(data_qual),
                            sd_qual = sd(data_qual),
                            n_qual = length(data_qual),
                            n_sample = length(data_sample),
                            mean_sample = mean(data_sample),
                            sd_sample = sd(data_sample))
  res4 <- equiv_change_mean(alpha = 0.05, data_qual = data_qual,
                            data_sample = data_sample)

  # the calls will be different, and that's okay
  res1$call <- NULL
  res2$call <- NULL
  res3$call <- NULL
  res4$call <- NULL

  expect_equal(res1, res2)
  expect_equal(res1, res3)
  expect_equal(res1, res4)
})

test_that("glance.equiv_change_mean produces expected results", {
  res <- equiv_change_mean(alpha = 0.05, n_sample = 9, mean_sample = 9.02,
                           sd_sample = 0.15785, n_qual = 28, mean_qual = 9.24,
                           sd_qual = 0.162)

  res <- glance(res)

  expect_equal(res$alpha[1], 0.05)
  expect_equal(res$n_sample[1], 9)
  expect_equal(res$mean_sample[1], 9.02)
  expect_equal(res$sd_sample[1], 0.15785)
  expect_equal(res$n_qual[1], 28)
  expect_equal(res$mean_qual[1], 9.24)
  expect_equal(res$sd_qual[1], 0.162)
  expect_equal(res$sp[1], 0.1608, tolerance = 5e-4)
  expect_equal(res$t0[1], -3.570, tolerance = 5e-3)
  expect_equal(res$t_req[1], 2.030, tolerance = 5e-3)
  expect_equal(res$threshold_min[1], 9.115, tolerance = 5e-3)
  expect_equal(res$threshold_max[1], 9.365, tolerance = 5e-3)
  expect_equal(res$modcv[1], FALSE)
  expect_equal(res$result[1], "FAIL")
})

test_that("equiv_mean_extremum produces expected errors and warnings", {
  expect_error(
    equiv_mean_extremum(
      alpha = -0.05, n_sample = 9,
      mean_qual = 9.24,
      sd_qual = 0.162),
    "alpha"
  )

  expect_error(
    equiv_mean_extremum(
      alpha = 1.05, n_sample = 9,
      mean_qual = 9.24,
      sd_qual = 0.162),
    "alpha"
  )

  expect_warning(
    equiv_mean_extremum(
      alpha = 0.05,
      data_sample = runif(9),
      n_sample = 9,
      mean_qual = 9.24,
      sd_qual = 0.162),
    "n_sample"
  )

  expect_warning(
    equiv_mean_extremum(
      alpha = 0.05,
      n_sample = 9,
      data_qual = runif(28),
      mean_qual = 9.24,
      sd_qual = 0.162),
    "mean_qual"
  )

  expect_warning(
    equiv_mean_extremum(
      alpha = 0.05,
      n_sample = 9,
      data_qual = runif(28),
      mean_qual = 9.24,
      sd_qual = 0.162),
    "sd_qual"
  )

  expect_error(
    equiv_mean_extremum(
      alpha = 0.05,
      mean_sample = 9.02,
      sd_sample = 0.15785,
      n_qual = 28, mean_qual = 9.24,
      sd_qual = 0.162),
    "sample"
  )

  expect_error(
    equiv_mean_extremum(
      alpha = 0.05,
      n_sample = 9,
      sd_qual = 0.162),
    "mean_qual"
  )

  expect_error(
    equiv_mean_extremum(
      alpha = 0.05,
      n_sample = 9,
      mean_qual = 9.24),
    "sd_qual"
  )

  expect_error(
    equiv_mean_extremum(
      alpha = 0.05,
      mean_qual = 9.24,
      sd_qual = 0.162),
    "n_sample"
  )
})

test_that("equiv_change_mean produces expected errors and warnings", {
  expect_error(
    equiv_change_mean(
      alpha = -0.05, n_sample = 9, mean_sample = 9.02,
      sd_sample = 0.15785, n_qual = 28, mean_qual = 9.24,
      sd_qual = 0.162),
    "alpha"
  )

  expect_error(
    equiv_change_mean(
      alpha = 1.05, n_sample = 9, mean_sample = 9.02,
      sd_sample = 0.15785, n_qual = 28, mean_qual = 9.24,
      sd_qual = 0.162),
    "alpha"
  )

  expect_warning(
    equiv_change_mean(
      alpha = 0.05,
      data_sample = runif(9),
      n_sample = 9, mean_sample = 9.02,
      sd_sample = 0.15785, n_qual = 28, mean_qual = 9.24,
      sd_qual = 0.162),
    "n_sample"
  )

  expect_error(
    equiv_change_mean(
      alpha = 0.05,
      n_sample = 9,
      sd_sample = 0.15785, n_qual = 28, mean_qual = 9.24,
      sd_qual = 0.162),
    "mean_sample"
  )

  expect_error(
    equiv_change_mean(
      alpha = 0.05,
      n_sample = 9, mean_sample = 9.02,
      n_qual = 28, mean_qual = 9.24,
      sd_qual = 0.162),
    "sd_sample"
  )

  expect_error(
    equiv_change_mean(
      alpha = 0.05,
      n_sample = 9, mean_sample = 9.02,
      sd_sample = 0.15785,
      mean_qual = 9.24,
      sd_qual = 0.162),
    "n_qual"
  )

  expect_warning(
    equiv_change_mean(
      alpha = 0.05,
      n_sample = 9, mean_sample = 9.02,
      sd_sample = 0.15785,
      data_qual = runif(28),
      n_qual = 28, mean_qual = 9.24,
      sd_qual = 0.162),
    "mean_qual"
  )

  expect_warning(
    equiv_change_mean(
      alpha = 0.05,
      n_sample = 9, mean_sample = 9.02,
      sd_sample = 0.15785,
      data_qual = runif(28),
      n_qual = 28, mean_qual = 9.24,
      sd_qual = 0.162),
    "sd_qual"
  )

  expect_error(
    equiv_change_mean(
      alpha = 0.05,
      mean_sample = 9.02,
      sd_sample = 0.15785,
      n_qual = 28, mean_qual = 9.24,
      sd_qual = 0.162),
    "sample"
  )

  expect_error(
    equiv_change_mean(
      alpha = 0.05,
      n_sample = 9, mean_sample = 9.02,
      sd_sample = 0.15785,
      n_qual = 28,
      sd_qual = 0.162),
    "mean_qual"
  )

  expect_error(
    equiv_change_mean(
      alpha = 0.05,
      n_sample = 9, mean_sample = 9.02,
      sd_sample = 0.15785,
      n_qual = 28, mean_qual = 9.24),
    "sd_qual"
  )
})

#' @importFrom ggplot2 ggproto
#' @importFrom ggplot2 Stat
StatNormalSurvFunc <- ggproto(  # nolint
  "stat_normal_surv_func",
  Stat,
  compute_group = function(data, scales, n) {
    x <- seq(from = min(data$x), to = max(data$x), length.out = n)
    y <- pnorm(x, mean(data$x), sd(data$x), lower.tail = FALSE)
    data.frame(x = x, y = y)
  },
  required_aes = c("x")
)

#' Normal Survival Function
#'
#' @description
#' The Normal survival function provides a visualization of a
#' distribution. A normal curve is fit based on the mean and standard
#' deviation of the data, and the survival function of this normal
#' curve is plotted. The survival function is simply one minus the
#' CDF.
#'
#' @param mapping Set of aesthetic mappings created by `aes()`.
#' @param data The data to be displayed in this layer. This has the
#'             same usage as a `ggplot2` `stat` function.
#' @param geom The geometric object to use to display the data.
#' @param position Position argument
#' @param ... Other arguments to pass on to `layer`.
#' @param n If `NULL`, do not interpolated. Otherwise, the
#'          number of points to interpolate.
#' @param pad If `TRUE`, pad the ESF with additional points
#'            `(-Inf, 0)` and `(0, Inf)`.
#' @param show.legend Should this layer be included in the legends?
#' @param inherit.aes If `FALSE`, overrides the default aesthetic,
#'                    rather than combining with them.
#'
#' @importFrom ggplot2 layer
#'
#' @export
stat_normal_surv_func <- function(mapping = NULL, data = NULL,
                                  geom = "smooth", position = "identity",
                                  show.legend = NA, inherit.aes = TRUE,  # nolint
                                  n = 100, pad = FALSE, ...) {
  layer(
    stat = StatNormalSurvFunc, data = data, geom = geom, position = position,
    mapping = mapping, show.legend = show.legend,  # nolint
    inherit.aes = inherit.aes, params = list(n = n, ...)
  )
}

#' @importFrom ggplot2 ggproto
#' @importFrom ggplot2 Stat
StatESF <- ggproto(  # nolint
  "stat_esf",
  Stat,
  compute_group = function(data, scales, n, pad) {
    if (is.null(n)) {
      x <- unique(data$x)
    } else {
      x <- seq(from = min(data$x), to = max(data$x), length.out = n)
    }
    if (pad) {
      x <- c(-Inf, x, Inf)
    }
    y <- 1 - ecdf(data$x)(x)
    data.frame(x = x, y = y)
  },
  required_aes = c("x")
)

#' Empirical Survival Function
#'
#' @description
#' The empirical survival function (ESF) provides a visualization of a
#' distribution. This is closely related to the empirical cumulative
#' distribution function (ECDF). The empirical survival function is
#' simply ESF = 1 - ECDF.
#'
#' @param mapping Set of aesthetic mappings created by `aes()`.
#' @param data The data to be displayed in this layer. This has the
#'             same usage as a `ggplot2` `stat` function.
#' @param geom The geometric object to use to display the data.
#' @param position Position argument
#' @param ... Other arguments to pass on to `layer`.
#' @param n If `NULL`, do not interpolated. Otherwise, the
#'          number of points to interpolate.
#' @param pad If `TRUE`, pad the ESF with additional points
#'            `(-Inf, 0)` and `(0, Inf)`.
#' @param show.legend Should this layer be included in the legends?
#' @param inherit.aes If `FALSE`, overrides the default aesthetic,
#'                    rather than combining with them.
#'
#' @importFrom ggplot2 layer
#'
#' @export
stat_esf <- function(mapping = NULL, data = NULL, geom = "point",
                     position = "identity", show.legend = NA,  # nolint
                     inherit.aes = TRUE, n = NULL,  # nolint
                     pad = FALSE, ...) {
  layer(
    stat = StatESF, data = data, geom = geom, position = position,
    mapping = mapping, show.legend = show.legend,  # nolint
    inherit.aes = inherit.aes, params = list(n = n, pad = pad, ...)  # nolint
  )
}

suppressMessages(library(dplyr))
suppressMessages(library(ggplot2))


test_that("stat_esf", {
  skip_if_not_installed("vdiffr")
  vdiffr::expect_doppelganger("stat_esf", {
    carbon.fabric.2 %>%
      filter(test == "WT" & condition == "RTD") %>%
      group_by(batch) %>%
      ggplot(aes(x = strength, color = batch)) +
      stat_esf(pad = TRUE) +
      ggtitle("Distribution of Data For Each Batch")
  })
})

test_that("stat_normal_surv_func", {
  skip_if_not_installed("vdiffr")
  vdiffr::expect_doppelganger("stat_normal_surv_func", {
    carbon.fabric.2 %>%
      filter(test == "WT" & condition == "RTD") %>%
      group_by(batch) %>%
      ggplot(aes(x = strength, color = batch)) +
      stat_normal_surv_func() +
      ggtitle("Distribution of Data For Each Batch")
  })
})

test_that("stat_normal_surv_func and stat_esf", {
  skip_if_not_installed("vdiffr")
  vdiffr::expect_doppelganger("stat_normal_surv_func and stat_esf", {
    set.seed(100)
    data.frame(
      strength = c(rnorm(400, 100, 10),
                   rnorm(400, 120, 10),
                   rnorm(400, 140, 10)),
      batch = c(rep("A", 400), rep("B", 400), rep("C", 400))
    ) %>%
      group_by(batch) %>%
      ggplot(aes(x = strength, color = batch)) +
      stat_esf(pad = TRUE) +
      stat_normal_surv_func() +
      ggtitle("Distribution of Data For Each Batch")
  })
})

#' Detect outliers using the maximum normed residual method
#'
#' @description
#' This function detects outliers using the maximum normed residual
#' method described in CMH-17-1G. This method identifies a value
#' as an outlier if the absolute difference between the value and
#' the sample mean divided by the sample standard deviation
#' exceeds a critical value.
#'
#' @param data a data.frame
#' @param x the variable in the data.frame for which to find the MNR
#'          or a vector if \code{data=NULL}
#' @param alpha the significance level for the test. Defaults to 0.05
#'
#' @details
#' \code{data} is an optional argument. If \code{data} is given, it
#' should be a
#' \code{data.frame} (or similar object). When \code{data} is specified, the
#' value of \code{x} is expected to be a variable within \code{data}. If
#' \code{data} is not specified, \code{x} must be a vector.
#'
#' The maximum normed residual test is a test for outliers. The test statistic
#' is given in CMH-17-1G. Outliers are identified in the returned object.
#'
#' The maximum normed residual test statistic is defined as:
#'
#' \deqn{MNR = max \frac{\left| x_i - \bar{x} \right|}{s} }{
#'   MNR = max | x_i- x_bar | / s }
#'
#' When the value of the MNR test statistic exceeds the critical value
#' defined in Section 8.3.3.1 of CMH-17-1G, the corresponding value
#' is identified as an outlier. It is then removed from the sample, and
#' the test statistic is computed again and compared with the critical
#' value corresponding with the new sample. This process is repeated until
#' no values are identified as outliers.
#'
#' @return an object of class \code{mnr}
#' This object has the following fields:
#'  \item{\code{call}}{the expression used to call this function}
#'  \item{\code{data}}{the original data used to compute the MNR}
#'  \item{\code{alpha}}{the value of alpha given by the user}
#'  \item{\code{mnr}}{the computed MNR test statistic}
#'  \item{\code{crit}}{the critical value given the sample size and the
#'                     significance level}
#'  \item{\code{outliers}}{a data.frame containing the \code{index} and
#'                         \code{value} of each of the identified outliers}
#'  \item{\code{n_outliers}}{the number of outliers found}
#'
#' @examples
#' library(dplyr)
#'
#' carbon.fabric.2 %>%
#'   filter(test=="FC" & condition=="ETW2" & batch=="A") %>%
#'   maximum_normed_residual(strength)
#'
#' ## Call:
#' ## maximum_normed_residual(data = ., x = strength)
#' ##
#' ## MNR =  1.958797  ( critical value = 1.887145 )
#' ##
#' ## Outliers ( alpha = 0.05 ):
#' ##   Index  Value
#' ##       6  44.26
#'
#' carbon.fabric.2 %>%
#'   filter(test=="FC" & condition=="ETW2" & batch=="B") %>%
#'   maximum_normed_residual(strength)
#'
#' ## Call:
#' ## maximum_normed_residual(data = ., x = strength)
#' ##
#' ## MNR =  1.469517  ( critical value = 1.887145 )
#' ##
#' ## No outliers detected ( alpha = 0.05 )
#'
#' @references
#' “Composite Materials Handbook, Volume 1. Polymer Matrix Composites
#' Guideline for Characterization of Structural Materials,” SAE International,
#' CMH-17-1G, Mar. 2012.
#'
#' @importFrom rlang eval_tidy enquo
#'
#' @export
maximum_normed_residual <- function(data = NULL, x, alpha = 0.05) {
  res <- list()
  class(res) <- "mnr"

  res$call <- match.call()

  verify_tidy_input(
    df = data,
    x = x,
    c = match.call(),
    arg_name = "x")
  cur_data <- eval_tidy(enquo(x), data)
  res$data <- cur_data

  res$alpha <- alpha

  indicies_cur <- seq_along(res$data)

  cur_mnr <- max(abs(res$data - mean(res$data)) / sd(res$data))
  res$mnr <- cur_mnr
  cur_crit <- maximum_normed_residual_crit(length(res$data), alpha)
  res$crit <- cur_crit

  res$outliers <- data.frame(index = c(), value = c())

  res$n_outliers <- 0

  if (is.na(cur_mnr)) {
    return(res)
  }

  for (i in 1:(length(res$data) - 1)) {
    if (cur_mnr >= cur_crit) {
      worst_index_cur <- which.max(abs(cur_data - mean(cur_data)))
      res$outliers <- rbind(
        res$outliers,
        data.frame(
          index = indicies_cur[worst_index_cur],
          value = cur_data[worst_index_cur]
        )
      )
      res$n_outliers <- res$n_outliers + 1
      cur_data <- cur_data[-worst_index_cur]
      indicies_cur <- indicies_cur[-worst_index_cur]
      cur_mnr <- max(abs(cur_data - mean(cur_data)) / sd(cur_data))
      cur_crit <- maximum_normed_residual_crit(length(cur_data), alpha)
    }
  }

  return(res)
}

maximum_normed_residual_crit <- function(n, alpha) {
  t <- qt(p = 1 - alpha / (2 * n), df = n - 2, lower.tail = TRUE, log.p = FALSE)
  return((n - 1) / sqrt(n) * sqrt(t ^ 2 / (n - 2 + t ^ 2)))
}

#' Glance at a \code{mnr} (maximum normed residual) object
#'
#' @description
#' Glance accepts an object of type \code{mnr} and returns a
#' \code{\link[tibble:tibble]{tibble::tibble}} with
#' one row of summaries.
#'
#' Glance does not do any calculations: it just gathers the results in a
#' tibble.
#'
#' @param x An \code{mnr} object
#' @param ... Additional arguments. Not used. Included only to match generic
#'            signature.
#'
#'
#' @return
#' A one-row \code{\link[tibble:tibble]{tibble::tibble}} with the following
#' columns:
#'
#' \item{\code{mnr}}{the computed MNR test statistic}
#' \item{\code{alpha}}{the value of alpha used for the test}
#' \item{\code{crit}}{the critical value given the sample size and the
#'                    significance level}
#' \item{\code{n_outliers}}{the number of outliers found}
#'
#'
#' @seealso
#' \code{\link{maximum_normed_residual}}
#'
#' @examples
#' x <- c(rnorm(20, 100, 5), 10)
#' m <- maximum_normed_residual(x = x)
#' glance(m)
#'
#' ## # A tibble: 1 x 4
#' ##     mnr alpha  crit n_outliers
#' ##   <dbl> <dbl> <dbl>      <dbl>
#' ## 1  4.23  0.05  2.73          1
#'
#' @method glance mnr
#' @importFrom tibble tibble
#'
#' @export
glance.mnr <- function(x, ...) {  # nolint
  with(
    x,
    tibble::tibble(
      mnr = mnr,
      alpha = alpha,
      crit = crit,
      n_outliers = n_outliers
    )
  )
}

#' Augment data with information from an \code{mnr} object
#'
#' @description
#' Augment accepts an \code{mnr} object (returned from the function
#' \code{\link{maximum_normed_residual}}) and a dataset and adds the column
#' \code{.outlier} to the dataset. The column \code{.outlier} is a logical
#' vector indicating whether each observation is an outlier.
#'
#' When passing data into \code{augment} using the \code{data} argument,
#' the data must be exactly the data that was passed to
#' \code{maximum_normed_residual}.
#'
#' @param x an \code{mnr} object created by
#'          \code{\link{maximum_normed_residual}}
#' @param data a \code{data.frame} or
#'             \code{\link[tibble:tibble]{tibble::tibble}}
#'             containing the original data that was passed to
#'             \code{maximum_normed_residual}
#' @param ... Additional arguments. Not used. Included only to match generic
#'            signature.
#'
#' @return
#' When \code{data} is supplied, \code{augment} returns \code{data}, but with
#' one column appended. When \code{data} is not supplied, \code{augment}
#' returns a new \code{\link[tibble:tibble]{tibble::tibble}} with the column
#' \code{values} containing the original values used by
#' \code{maximum_normed_residaul} plus one additional column. The additional
#' column is:
#'
#' \item{\code{.outler}}{a logical value indicating whether the observation
#'                       is an outlier}
#'
#' @examples
#' data <- data.frame(strength = c(80, 98, 96, 97, 98, 120))
#' m <- maximum_normed_residual(data, strength)
#'
#' # augment can be called with the original data
#' augment(m, data)
#'
#' ##   strength .outlier
#' ## 1       80    FALSE
#' ## 2       98    FALSE
#' ## 3       96    FALSE
#' ## 4       97    FALSE
#' ## 5       98    FALSE
#' ## 6      120    FALSE
#'
#' # or augment can be called without the orignal data and it will be
#' # reconstructed
#' augment(m)
#'
#' ## # A tibble: 6 x 2
#' ##   values .outlier
#' ##    <dbl> <lgl>
#' ## 1     80 FALSE
#' ## 2     98 FALSE
#' ## 3     96 FALSE
#' ## 4     97 FALSE
#' ## 5     98 FALSE
#' ## 6    120 FALSE
#'
#' @seealso
#' \code{\link{maximum_normed_residual}}
#'
#' @method augment mnr
#' @importFrom tibble tibble
#'
#' @export
augment.mnr <- function(x, data = x$data, ...) {  # nolint
  if (is.data.frame(data)) {
    df <- data
  } else {
    df <- tibble::tibble(values = data)
  }

  res <- df
  res[[".outlier"]] <- FALSE
  res$.outlier[x$outliers$index] <- TRUE
  res
}

#' @export
print.mnr <- function(x, ...) {
  cat("\nCall:\n",
      paste(deparse(x$call), sep = "\n", collapse = "\n"), "\n\n", sep = "")
  cat("MNR =", x$mnr, " ( critical value =", x$crit, ")\n\n")
  if (nrow(x$outliers) == 0) {
    cat("No outliers detected ( alpha =", x$alpha, ")\n\n")
  } else {
    cat("Outliers ( alpha =", x$alpha, "):\n")

    justify <- c("right", "left", "left")
    width <- c(8L, 2L, 16L)

    cat(format_row(list("Index", " ", "Value"), justify, width, ...))

    for (j in seq(along.with = x$outliers$index)) {
      cat(format_row(
        list(x$outliers[["index"]][j], " ", x$outliers[["value"]][j]),
        justify, width, ...)
      )
    }

  }
}

#' @importFrom rlang call2
build_nesting <- function(x, group_df, i_group, label, stat, type, extras) {
  if (i_group > ncol(group_df)) {
    if (type == "Points") {
      nodes <- NULL
    } else {
      nodes <- list(build_nesting(x, group_df, i_group, label, stat, "Points",
                                  extras))
    }
  } else {
    group_levels <- levels(as.factor(group_df[[i_group]]))

    nodes <- lapply(group_levels, function(gl) {
      mask <- group_df[[i_group]] == gl

      node_x <- subset(x, mask)
      node_group_df <- subset(group_df, mask)
      extras_subset <- lapply(extras, function(e) {
        subset(e, mask)
      })
      n <- build_nesting(node_x, node_group_df, i_group + 1, gl, stat, "Label",
                         extras_subset)
      n
    })
  }

  list(
    type = type,
    x = x,
    stat = eval(call2(.fn = stat, x)),
    label = label,
    nodes = nodes,
    extras = extras
  )
}

build_drawing_element_list <- function(nesting, group_df, y_gap) {
  elm_list <- list()

  # We'll need a queue for a level order tree traversal
  # We'll track the heights of each grouping too
  level_counts <- c()
  queue <- list()
  push_queue <- function(obj, parent, level) {
    # the root (not drawn) is level=1
    parent_level_order <- ifelse(is.null(parent), 1, parent$level_order)
    # level_heights is an accumulator that keeps increasing (by 1) as
    # subsequent items within a level are drawn
    level_counts[level] <<- sum(level_counts[level], 1, na.rm = TRUE)
    level_order <- level_counts[level]
    level_heights <- c(1, level_counts)
    ancestor_level_heights <- ifelse(level <= 2,
                                     0,
                                     sum(level_heights[2:(level - 1)]))
    y_obj <-
      ancestor_level_heights +
      y_gap * max(0, level - 2) +
      (parent_level_order - 1)
    queue <<- c(queue, list(list(
      obj = obj,
      parent = parent,
      level = level,
      level_order = level_order,
      y = -y_obj
    )))
  }
  pop_queue <- function() {
    obj <- queue[[1]]
    queue <<- queue[-1]
    obj
  }

  # We'll traverse the "nesting" tree using level-order traversal
  push_queue(nesting, NULL, 1)
  while (length(queue) > 0) {
    cur_item <- pop_queue()
    for (n in cur_item$obj$nodes) {
      push_queue(n, cur_item, cur_item$level + 1)
    }

    if (cur_item$obj$type != "Root") {
      elm_list <- c(elm_list, list(cur_item))
    }
  }

  minor_breaks <- sapply(seq_along(level_counts) + 1, function(level) {
    level_heights <- c(1, level_counts)
    ancestor_level_heights <- ifelse(level <= 2,
                                     0,
                                     sum(level_heights[2:(level - 1)]))
    y_obj <- # nolint
      ancestor_level_heights +
      y_gap * max(0, level - 2) -
      y_gap / 2 - 0.5
  })
  labels <- c(names(group_df), "")
  breaks <- minor_breaks[-1] - diff(minor_breaks) / 2

  axis_info <- list(
    breaks = rev(-breaks),
    labels = rev(labels),
    minor_breaks = rev(-minor_breaks)
  )

  list(
    elm_list = (elm_list),
    axis_info = (axis_info)
  )
}

#' @importFrom ggplot2 xlab ylab scale_y_continuous expansion theme
#' @importFrom ggplot2 element_blank element_line geom_hline
set_axes <- function(g, xlabel, axis_info, divider_color) {
  breaks <- axis_info$breaks
  labels <- axis_info$labels
  minor_breaks <- axis_info$minor_breaks
  g <- g +
    xlab(xlabel) +
    scale_y_continuous(
      breaks = breaks,
      labels = labels,
      minor_breaks = minor_breaks,
      limits = c(minor_breaks[1], minor_breaks[length(minor_breaks)]),
      expand = expansion()
    ) +
    theme(panel.grid.major.y = element_blank(),
          panel.grid.minor.y = element_line(linewidth = 0.1, color = "black")) +
    ylab("")

  if (!is.null(divider_color)) {
    lapply(minor_breaks, function(b) {
      g <<- g + geom_hline(yintercept = b, color = divider_color)
    })
  }

  g
}

#' @importFrom purrr map_dfr
#' @importFrom rlang exec
#' @importFrom ggplot2 aes
#' @importFrom rlang .data
draw_connectors <- function(g, elm_list, connector_args) {
  points <- map_dfr(
    elm_list,
    function(cur_item) {
      if (!is.null(cur_item$parent) & !is.na(cur_item$parent$y) &
          cur_item$parent$obj$type == "Label") {
        data.frame(
          x = cur_item$parent$obj$stat,
          y = cur_item$y
        )
      } else {
        NULL
      }
    }
  )

  if (nrow(points) == 0) {
    g
  } else {
    g +
      exec(
        ggplot2::geom_point,
        data = points,
        mapping = aes(x = .data$x, y = .data$y),
        !!!connector_args
      )
  }
}

#' @importFrom purrr map_dfr
#' @importFrom rlang exec
#' @importFrom ggplot2 aes
#' @importFrom rlang .data
draw_vert_lines_to_labels <- function(g, elm_list, vline_args) {
  line_segments <- map_dfr(
    elm_list,
    function(cur_item) {
      if (!is.null(cur_item$parent) & !is.na(cur_item$parent$y) &
          cur_item$parent$obj$type == "Label") {
        data.frame(
          x = cur_item$parent$obj$stat,
          xend = cur_item$parent$obj$stat,
          y = cur_item$y,
          yend = cur_item$parent$y
        )
      } else {
        NULL
      }
    }
  )

  if (nrow(line_segments) == 0) {
    g
  } else {
    g +
      exec(
        ggplot2::geom_segment,
        data = line_segments,
        mapping = aes(x = .data$x, xend = .data$xend,
                      y = .data$y, yend = .data$yend),
        !!!vline_args
      )
  }
}

#' @importFrom purrr map_dfr
#' @importFrom rlang exec
#' @importFrom ggplot2 aes
#' @importFrom rlang .data
draw_horiz_lines_to_labels <- function(g, elm_list, hline_args) {
  line_segments <- map_dfr(
    elm_list,
    function(cur_item) {
      if (!is.null(cur_item$parent) & !is.na(cur_item$parent$y) &
          cur_item$parent$obj$type == "Label") {
        data.frame(
          x = cur_item$obj$stat,
          xend = cur_item$parent$obj$stat,
          y = cur_item$y,
          yend = cur_item$y
        )
      } else {
        NULL
      }
    }
  )

  if (nrow(line_segments) == 0) {
    g
  } else {
    g +
      exec(
        ggplot2::geom_segment,
        data = line_segments,
        mapping = aes(x = .data$x, xend = .data$xend,
                      y = .data$y, yend = .data$yend),
        !!!hline_args
      )
  }
}

add_extras_data <- function(df, extras, name, must_be_equal) {
  if (name %in% names(extras)) {
    if (length(levels(as.factor(extras[[name]]))) > 1 && must_be_equal) {
      # Not all values are the same
      return(df)
    } else {
      # All values are the same, so we can just take the first
      # This will work regardless of how many rows are in df
      df[[name]] <- extras[[name]][1]
      return(df)
    }
  } else {
    return(df)
  }
}


#' @importFrom purrr map_dfr
#' @importFrom rlang exec
#' @importFrom ggplot2 aes labs
#' @importFrom rlang .data
draw_points <- function(g, elm_list, point_args, dline_args, extras_names) {
  points <- map_dfr(
    elm_list,
    function(cur_item) {
      if (cur_item$obj$type == "Points") {
        add_extras_data(
          data.frame(
            x = cur_item$obj$x,
            y = cur_item$y
          ),
          cur_item$obj$extras, "color", FALSE
        )
      } else {
        return(NULL)
      }
    }
  )

  line_segments <- map_dfr(
    elm_list,
    function(cur_item) {
      if (cur_item$obj$type == "Points") {
        data.frame(
          x = min(cur_item$obj$x),
          xend = max(cur_item$obj$x),
          y = cur_item$y,
          yend = cur_item$y
        )
      } else {
        return(NULL)
      }
    }
  )

  if ("color" %in% names(extras_names)) {
    g <- g +
      exec(
        ggplot2::geom_point,
        data = points,
        mapping = aes(x = .data$x, y = .data$y, color = .data$color),
        !!!point_args
      ) +
      labs(
        color = extras_names[["color"]]
      )
  } else {
    # No color variable
    g <- g +
      exec(
        ggplot2::geom_point,
        data = points,
        mapping = aes(x = .data$x, y = .data$y),
        !!!point_args
      )
  }

  g +
    exec(
      ggplot2::geom_segment,
      data = line_segments,
      mapping = aes(
        x = .data$x,
        xend = .data$xend,
        y = .data$y,
        yend = .data$yend
      ),
      !!!dline_args
    )
}

#' @importFrom purrr map_dfr
#' @importFrom rlang exec
#' @importFrom ggplot2 aes labs
#' @importFrom rlang .data
draw_labels <- function(g, elm_list, label_args, extras_names) {
  labels <- map_dfr(
    elm_list,
    function(cur_item) {
      if (cur_item$obj$type == "Label") {
        add_extras_data(
          data.frame(
            x = cur_item$obj$stat,
            y = cur_item$y,
            label = cur_item$obj$label
          ),
          cur_item$obj$extras, "fill", TRUE
        )
      } else {
        NULL
      }
    }
  )

  if (nrow(labels) == 0) {
    g
  } else {
    if ("fill" %in% names(extras_names)) {
      labels_fill <- subset(labels, !is.na(labels$fill))
      g <- g +
        exec(
          ggplot2::geom_label,
          data = labels_fill,
          mapping = aes(x = .data$x, y = .data$y, label = .data$label,
                        fill = .data$fill),
          !!!label_args
        ) +
        labs(
          fill = extras_names[["fill"]]
        )
      labels <- subset(labels, is.na(labels$fill))
    }

    # Data with no fill variable
    g +
      exec(
        ggplot2::geom_label,
        data = labels,
        mapping = aes(x = .data$x, y = .data$y, label = .data$label),
        !!!label_args
      )
  }
}

#' Create a plot of nested sources of variation
#'
#' @description
#' Creates a plot showing the breakdown of variation within a sample. This
#' function uses `ggplot2` internally.
#'
#' @param dat a `data.frame` or similar object
#' @param x the variable within `dat` to plot. Most often this would be a
#'          strength or modulus variable.
#' @param groups a vector of variables to group the data by
#' @param stat a function for computing the central location for each group.
#'             This is normally "mean" but could be "median" or another
#'             function.
#' @param ... extra options. See Details.
#' @param y_gap the vertical gap between grouping variables
#' @param divider_color the color of the lines between grouping variables.
#'                      Or `NULL` to omit these lines.
#' @param point_args arguments to pass to [ggplot2::geom_point] when plotting
#'                   individual data points.
#' @param dline_args arguments to pass to [ggplot2::geom_segment] when plotting
#'                   the horizontal lines between data points.
#' @param vline_args arguments to pass to [ggplot2::geom_segment] when plotting
#'                   vertical lines
#' @param hline_args arguments to pass to [ggplot2::geom_segment] when plotting
#'                   horizontal lines connecting levels in groups
#' @param label_args arguments to pass to [ggplot2::geom_label] when plotting
#'                   labels
#' @param connector_args arguments to pass to [ggplot2::geom_point] when
#'                       plotting the connection between the vertical lines
#'                       and the horizontal lines connecting levels in groups
#'
#' @details
#' Extra options can be included to control aesthetic options. The following
#' options are supported. Any (or all) can be set to a single variable
#' in the data set.
#'
#' - `color`: Controls the color of the data points.
#' - `fill`: Controls the fill color of the labels. When a particular label
#'   is associated with data points with more than one level of the supplied
#'   variable, the fill is omitted.
#'
#' @examples
#' library(dplyr)
#' carbon.fabric.2 %>%
#'   filter(test == "WT" & condition == "RTD") %>%
#'   nested_data_plot(strength,
#'                    groups = c(batch, panel))
#'
#' # Labels can be filled too
#' carbon.fabric.2 %>%
#'   filter(test == "WT" & condition == "RTD") %>%
#'   nested_data_plot(strength,
#'                    groups = c(batch, panel),
#'                    fill = batch)
#'
#' @importFrom rlang ensym
#' @importFrom rlang quo_get_expr enquos
#' @importFrom ggplot2 ggplot
#' @importFrom dplyr ungroup
#' @export
nested_data_plot <- function(dat, x, groups = c(),
                             stat = "mean",
                             ...,
                             y_gap = 1,
                             divider_color = "grey50",
                             point_args = list(),
                             dline_args = list(),
                             vline_args = list(),
                             hline_args = list(),
                             label_args = list(),
                             connector_args = list()) {
  dat <- ungroup(dat)
  group_df <- select(dat, !!enquo(groups))
  xlabel <- ensym(x)
  x <- select(dat, !!enquo(x))[[1]]

  en_dots <- enquos(..., .ignore_empty = "all")
  extras <- lapply(en_dots, function(ed) select(dat, !!quo_get_expr(ed))[[1]])
  extras_names <- lapply(en_dots, function(ed) quo_get_expr(ed))

  nesting <- build_nesting(x, group_df, 1, "Root", stat, "Root", extras)
  el <- build_drawing_element_list(nesting, group_df, y_gap)
  elm_list <- el$elm_list
  axis_info <- el$axis_info
  g <- ggplot()
  g <- set_axes(g, xlabel, axis_info, divider_color)
  g <- draw_vert_lines_to_labels(g, elm_list, vline_args)
  g <- draw_horiz_lines_to_labels(g, elm_list, hline_args)
  g <- draw_connectors(g, elm_list, connector_args)
  g <- draw_points(g, elm_list, point_args, dline_args, extras_names)
  g <- draw_labels(g, elm_list, label_args, extras_names)
  g
}

suppressMessages(library(dplyr))
suppressMessages(library(ggplot2))


test_that("nested_data_plot-no_grouping", {
  skip_if_not_installed("vdiffr")
  vdiffr::expect_doppelganger("nested_data_plot-no_grouping", {
    carbon.fabric.2 %>%
      filter(test == "WT") %>%
      nested_data_plot(strength)
  })
})

test_that("nested_data_plot-single_grouping", {
  skip_if_not_installed("vdiffr")
  vdiffr::expect_doppelganger("nested_data_plot-single_grouping", {
    carbon.fabric.2 %>%
      filter(test == "WT" & condition == "RTD") %>%
      nested_data_plot(strength,
                       groups = c(batch))
  })
})

test_that("nested_data_plot-two_groupings", {
  skip_if_not_installed("vdiffr")
  vdiffr::expect_doppelganger("nested_data_plot-two_groupings", {
    carbon.fabric.2 %>%
      filter(test == "WT" & condition == "RTD") %>%
      nested_data_plot(strength,
                       groups = c(batch, panel))
  })
})

test_that("nested_data_plot-single_obs_per_group", {
  skip_if_not_installed("vdiffr")
  vdiffr::expect_doppelganger("nested_data_plot-single_obs_per_group", {
    carbon.fabric.2 %>%
      filter(test == "WT" & condition == "RTD") %>%
      nested_data_plot(strength,
                       groups = c(batch, panel, strength))
  })
})

test_that("nested_data_plot-color", {
  skip_if_not_installed("vdiffr")
  vdiffr::expect_doppelganger("nested_data_plot-color", {
    carbon.fabric.2 %>%
      filter(test == "WT" & condition == "RTD") %>%
      nested_data_plot(strength,
                       groups = c(batch, panel),
                       color = batch)
  })
})

test_that("nested_data_plot-color-and-fill", {
  skip_if_not_installed("vdiffr")
  vdiffr::expect_doppelganger("nested_data_plot-color-and-fill", {
    carbon.fabric.2 %>%
      filter(test == "WT") %>%
      nested_data_plot(strength,
                       groups = c(condition, batch, panel),
                       color = batch,
                       fill = condition)
  })
})

#' Anderson--Darling K-Sample Test
#'
#' @description
#' This function performs an Anderson--Darling k-sample test. This is used to
#' determine if several samples (groups) share a common (unspecified)
#' distribution.
#'
#' @param data a data.frame
#' @param x the variable in the data.frame on which to perform the
#'          Anderson--Darling k-Sample test (usually strength)
#' @param groups a variable in the data.frame that defines the groups
#' @param alpha the significance level (default 0.025)
#'
#' @return
#' Returns an object of class `adk`. This object has the following fields:
#' - `call` the expression used to call this function
#' - `data` the original data used to compute the ADK
#' - `groups` a vector of the groups used in the computation
#' - `alpha` the value of alpha specified
#' - `n` the total number of observations
#' - `k` the number of groups
#' - `sigma` the computed standard deviation of the test statistic
#' - `ad` the value of the Anderson--Darling k-Sample test statistic
#' - `p` the computed p-value
#' - `reject_same_dist` a boolean value indicating whether the null
#'   hypothesis that all samples come from the same distribution is rejected
#' - `raw` the original results returned from
#'   [ad.test][kSamples::ad.test]
#'
#'
#' @details
#' This function is a wrapper for the [ad.test][kSamples::ad.test] function from
#' the package `kSamples`. The method "exact" is specified in the call to
#' `ad.test`. Refer to that package's documentation for details.
#'
#' There is a minor difference in the formulation of the Anderson--Darling
#' k-Sample test in CMH-17-1G, compared with that in the Scholz and
#' Stephens (1987). This difference affects the test statistic and the
#' critical value in the same proportion, and therefore the conclusion of
#' the test is unaffected. When
#' comparing the test statistic generated by this function to that generated
#' by software that uses the CMH-17-1G formulation (such as ASAP, CMH17-STATS,
#' etc.), the test statistic reported by this function will be greater by
#' a factor of \eqn{(k - 1)}, with a corresponding change in the critical
#' value.
#'
#' For more information about the difference between this function and
#' the formulation in CMH-17-1G, see the vignette on the subject, which
#' can be accessed by running `vignette("adktest")`
#'
#' @references
#' F. W. Scholz and M. Stephens, “K-Sample Anderson--Darling Tests,” Journal
#' of the American Statistical Association, vol. 82, no. 399. pp. 918–924,
#' Sep-1987.
#'
#' “Composite Materials Handbook, Volume 1. Polymer Matrix Composites
#' Guideline for Characterization of Structural Materials,” SAE International,
#' CMH-17-1G, Mar. 2012.
#'
#' @examples
#' library(dplyr)
#'
#' carbon.fabric %>%
#'   filter(test == "WT") %>%
#'   filter(condition == "RTD") %>%
#'   ad_ksample(strength, batch)
#' ##
#' ## Call:
#' ## ad_ksample(data = ., x = strength, groups = batch)
#' ##
#' ## N = 18          k = 3
#' ## ADK = 0.912     p-value = 0.95989
#' ## Conclusion: Samples come from the same distribution ( alpha = 0.025 )
#'
#' @importFrom rlang enquo eval_tidy
#' @importFrom kSamples ad.test
#' @export
ad_ksample <- function(data = NULL, x, groups, alpha = 0.025) {
  res <- list()
  class(res) <- "adk"

  res$call <- match.call()

  verify_tidy_input(
    df = data,
    x = x,
    c = match.call(),
    arg_name = "x")
  res$data <- eval_tidy(enquo(x), data)

  verify_tidy_input(
    df = data,
    x = groups,
    c = match.call(),
    arg_name = "groups")
  res$groups <- eval_tidy(enquo(groups), data)

  if (length(res$data) != length(res$groups)) {
    stop("Error: `x` and `groups` must be of same length.")
  }

  res$alpha <- alpha

  td <- NULL

  res$transformed_data <- td

  grps <- lapply(levels(as.factor(res[["groups"]])),
                 function(l) {
                   res[["data"]][res[["groups"]] == l]
                 }
  )

  raw <- ad.test(grps, method = "exact")
  res$n <- raw$N
  res$k <- raw$k
  res$sigma <- raw$sig
  res$ad <- raw$ad[2, 1]
  res$p <- raw$ad[2, 3]
  res$reject_same_dist <- res$p < alpha

  res$raw <- raw

  return(res)
}

#' Glance at a `adk` (Anderson--Darling k-Sample) object
#'
#' @description
#' Glance accepts an object of type `adk` and returns a
#' [tibble::tibble()] with
#' one row of summaries.
#'
#' Glance does not do any calculations: it just gathers the results in a
#' tibble.
#'
#' @param x an `adk` object
#' @param ... Additional arguments. Not used. Included only to match generic
#'            signature.
#'
#'
#' @return
#' A one-row [tibble::tibble()] with the following
#' columns:
#'
#' - `alpha` the significance level for the test
#' - `n` the sample size for the test
#' - `k` the number of samples
#' - `sigma` the computed standard deviation of the test statistic
#' - `ad` the test statistic
#' - `p` the p-value of the test
#' - `reject_same_dist` whether the test concludes that the samples
#'   are drawn from different populations
#'
#'
#' @seealso
#' [ad_ksample()]
#'
#' @examples
#' x <- c(rnorm(20, 100, 5), rnorm(20, 105, 6))
#' k <- c(rep(1, 20), rep(2, 20))
#' a <- ad_ksample(x = x, groups = k)
#' glance(a)
#'
#' ## A tibble: 1 x 7
#' ##   alpha     n     k sigma    ad       p reject_same_dist
#' ##   <dbl> <int> <int> <dbl> <dbl>   <dbl> <lgl>
#' ## 1 0.025    40     2 0.727  4.37 0.00487 TRUE
#'
#' @method glance adk
#' @importFrom tibble tibble
#'
#' @export
glance.adk <- function(x, ...) {  # nolint
  # nolint start: object_usage_linter
  with(
    x,
    tibble::tibble(
      alpha = alpha,
      n = n,
      k = k,
      sigma = sigma,
      ad = ad,
      p = p,
      reject_same_dist = reject_same_dist
    )
  )
  # nolint end
}

#' @export
print.adk <- function(x, ...) {
  cat("\nCall:\n",
      paste(deparse(x$call), sep = "\n", collapse = "\n"), "\n\n", sep = "")

  justify <- c("left", "left")
  width <- c(16L, 16L)

  cat(format_row_equal(list("N", x$n, "k", x$k),
                       justify, width, ...))
  cat(format_row_equal(list("ADK", x$ad, "p-value", x$p),
                       justify, width, ...))
  if (x$reject_same_dist) {
    cat("Conclusion: Samples do not come from the same distribution (alpha =",
        x$alpha, ")\n\n")
  } else {
    cat("Conclusion: Samples come from the same distribution ( alpha =",
        x$alpha, ")\n\n")
  }
}

context("Anderson-Darling k-Sample")

suppressMessages(library(dplyr))
suppressMessages(library(kSamples))  # nolint

test_that("kSamples package gives results that match published example", {
  # Reproduce the example from:
  # F. W. Scholz and M. Stephens, “K-Sample Anderson-Darling Tests,” Journal
  #   of the American Statistical Association, vol. 82, no. 399.
  #   pp. 918–924, Sep-1987.

  df <- data.frame(
    smoothness = c(
      38.7, 41.5, 43.8, 44.5, 45.5, 46.0, 47.7, 58.0,
      39.2, 39.3, 39.7, 41.4, 41.8, 42.9, 43.3, 45.8,
      34.0, 35.0, 39.0, 40.0, 43.0, 43.0, 44.0, 45.0,
      34.0, 34.8, 34.8, 35.4, 37.2, 37.8, 41.2, 42.8
    ),
    lab = c(rep("A", 8), rep("B", 8), rep("C", 8), rep("D", 8))
  )

  res <- ad.test(smoothness ~ lab, data = df)

  expect_equal(res[["sig"]], 1.2038, tolerance = 1e-4)

  ad <- res[["ad"]]

  expect_equal(ad["version 1:", "AD"], 8.3559, tolerance = 1e-3)
  expect_equal(ad["version 1:", " asympt. P-value"], 0.0023, tolerance = 1e-3)
  expect_equal(ad["version 2:", "AD"], 8.3926, tolerance = 1e-3)
  expect_equal(ad["version 2:", " asympt. P-value"], 0.0022, tolerance = 1e-3)
})

test_that("ADK test match ASAP", {
  res <- carbon.fabric %>%
    filter(test == "WT") %>%
    filter(condition == "RTD") %>%
    ad_ksample(strength, batch)
  expect_equal(res$ad / (res$k - 1), 0.456, tolerance = 0.002)
  expect_false(res$reject_same_dist)

  res <- carbon.fabric %>%
    filter(test == "WT") %>%
    filter(condition == "ETW") %>%
    ad_ksample(strength, batch)
  expect_equal(res$ad / (res$k - 1), 1.604, tolerance = 0.002)
  expect_false(res$reject_same_dist)

  res <- carbon.fabric %>%
    filter(test == "WT") %>%
    filter(condition == "CTD") %>%
    ad_ksample(strength, batch)
  expect_equal(res$ad / (res$k - 1), 1.778, tolerance = 0.002)
  expect_false(res$reject_same_dist)

  res <- carbon.fabric %>%
    filter(test == "FT") %>%
    filter(condition == "RTD") %>%
    ad_ksample(strength, batch)
  expect_equal(res$ad / (res$k - 1), 0.777, tolerance = 0.003)
  expect_false(res$reject_same_dist)

  res <- carbon.fabric %>%
    filter(test == "FT") %>%
    filter(condition == "ETW") %>%
    ad_ksample(strength, batch)
  expect_equal(res$ad / (res$k - 1), 1.355, tolerance = 0.002)
  expect_false(res$reject_same_dist)

  res <- carbon.fabric %>%
    filter(test == "FT") %>%
    filter(condition == "CTD") %>%
    ad_ksample(strength, batch)
  expect_equal(res$ad / (res$k - 1), 1.432, tolerance = 0.002)
  expect_false(res$reject_same_dist)

  res <- carbon.fabric %>%
    filter(test == "WC") %>%
    filter(condition == "RTD") %>%
    ad_ksample(strength, batch)
  expect_equal(res$ad / (res$k - 1), 0.384, tolerance = 0.002)
  expect_false(res$reject_same_dist)

  res <- carbon.fabric %>%
    filter(test == "WC") %>%
    filter(condition == "ETW") %>%
    ad_ksample(strength, batch)
  expect_equal(res$ad / (res$k - 1), 0.723, tolerance = 0.003)
  expect_false(res$reject_same_dist)

  res <- carbon.fabric %>%
    filter(test == "WC") %>%
    filter(condition == "CTD") %>%
    ad_ksample(strength, batch)
  expect_equal(res$ad / (res$k - 1), 1.145, tolerance = 0.002)
  expect_false(res$reject_same_dist)

  res <- carbon.fabric %>%
    filter(test == "FC") %>%
    filter(condition == "RTD") %>%
    ad_ksample(strength, batch)
  expect_equal(res$ad / (res$k - 1), 0.865, tolerance = 0.002)
  expect_false(res$reject_same_dist)

  res <- carbon.fabric %>%
    filter(test == "FC") %>%
    filter(condition == "ETW") %>%
    ad_ksample(strength, batch)
  expect_equal(res$ad / (res$k - 1), 0.934, tolerance = 0.002)
  expect_false(res$reject_same_dist)

  res <- carbon.fabric %>%
    filter(test == "FC") %>%
    filter(condition == "CTD") %>%
    ad_ksample(strength, batch)
  expect_equal(res$ad / (res$k - 1), 0.501, tolerance = 0.002)
  expect_false(res$reject_same_dist)
})

test_that("ADK test matches example from CMH-17-1G", {
  # Reference the example in section 8.3.11.1.1
  etw <- tribble(
    ~batch, ~strength,
    1, 106.357525,
    1, 105.898733,
    1, 88.4640082,
    1, 103.901744,
    1, 80.2058219,
    1, 109.199597,
    1, 61.0139431,
    2, 99.3207107,
    2, 115.86177,
    2, 82.6133082,
    2, 85.3690411,
    2, 115.801622,
    2, 44.3217741,
    2, 117.328077,
    2, 88.6782903,
    3, 107.676986,
    3, 108.960241,
    3, 116.12264,
    3, 80.2334815,
    3, 106.14557,
    3, 104.667866,
    3, 104.234953
  )

  # ETW:
  #  ADK = 0.793 / same distribution
  res <- etw %>%
    ad_ksample(strength, batch)
  expect_equal(res$ad / (res$k - 1), 0.793, tolerance = 0.003)
  expect_false(res$reject_same_dist)
  expect_output(print(res), ".*N.*22")
  expect_output(print(res), ".*k.*3")
  expect_output(print(res), "Conclusion: Samples come")

  etw2 <- tribble(
    ~batch, ~strength,
    1, 99.0239966,
    1, 103.341238,
    1, 100.30213,
    1, 98.4634133,
    1, 92.264728,
    1, 103.487693,
    1, 113.734763,
    2, 108.172659,
    2, 108.426732,
    2, 116.260375,
    2, 121.04961,
    2, 111.223082,
    2, 104.574843,
    2, 103.222552,
    3, 99.3918538,
    3, 87.3421658,
    3, 102.730741,
    3, 96.3694916,
    3, 99.5946088,
    3, 97.0712407
  )

  # ETW2:
  #  ADK = 3.024 / different distribution
  res <- etw2 %>%
    ad_ksample(strength, batch)
  expect_equal(res$ad / (res$k - 1), 3.024, tolerance = 0.003)
  expect_true(res$reject_same_dist)
  expect_output(print(res), ".*N.*20")
  expect_output(print(res), ".*k.*3")
  expect_output(print(res), "Conclusion: Samples do not come")
})

test_that("glance.adk produces expected output", {
  res <- carbon.fabric %>%
    filter(test == "WT") %>%
    filter(condition == "RTD") %>%
    ad_ksample(strength, batch)

  glance_res <- glance(res)

  expect_equal(glance_res[["alpha"]][1], 0.025)
  expect_equal(glance_res[["n"]][1], 18)
  expect_equal(glance_res[["k"]][1], 3)
  expect_equal(glance_res[["sigma"]], 0.944, tolerance = 0.001)
  expect_equal(glance_res[["ad"]][1], 0.456 * (res$k - 1))
  expect_equal(glance_res[["p"]][1], 0.96, tolerance = 0.001)
  expect_equal(glance_res[["reject_same_dist"]], FALSE)

})

test_that("ad_ksample checks that input vectors are equal length", {
  expect_error({
    ad_ksample(x = c(1, 2, 3), groups = c("A", "B"))
  })
})

#' Separate multiple failure modes into multiple rows
#'
#' @param data a `data.frame`
#' @param failure_mode the column in `data` containing the failure modes
#' @param sep a regular expression with the character(s) separating individual
#'            failure modes. Default `"[/, ]+"`.
#'
#' @description
#' For a `data.frame` containing a column with (some) multiple failure modes,
#' this function expands the `data.frame` by repeating each row with multiple
#' failure modes so that each row contains only a single failure mode.
#'
#' @details
#' When multiple failure modes are reported, they are commonly reported in
#' the format "LGM/GIT" or "LGM,GIT". This function will separate these multiple
#' failure modes into multiple rows.
#'
#' This can be useful when counting the number of coupons exhibited each
#' failure mode.
#'
#' @examples
#' library(dplyr)
#' data.frame(strength = c(101, 102), fm = c("LGM/GIT", "LGM")) %>%
#'   separate_failure_modes(fm)
#' ##
#' ## # A tibble: 3 × 2
#' ##   strength fm
#' ##      <dbl> <chr>
#' ## 1      101 LGM
#' ## 2      101 GIT
#' ## 3      102 LGM
#'
#' @importFrom tidyr separate_rows
#'
#' @export
separate_failure_modes <- function(data, failure_mode, sep = "[/, ]+") {
  separate_rows(data, {{ failure_mode }}, sep = sep)
}



#' @importFrom ggplot2 GeomPoint
#nolint start
GeomPointFailureMode <- ggproto(
  "GeomPointFailureMode",
  GeomPoint,
#nolint end
  required_aes = c("x", "y", "colour|shape"),
  extra_params = c("sep"),
  setup_data = function(data, params) {
    if ("colour" %in% names(data)) {
      new_data <- separate_failure_modes(data, "colour", sep = params$sep)
      if ("shape" %in% names(data)) {
        # both colour and shape specified
        if (!identical(data$colour, data$shape)) {
          stop(paste0("If both `shape` and `colour` are specified, ",
                      "geom_jitter_failure_mode requires that both are equal"))
        }
        new_data$shape <- new_data$colour
      }
    } else if ("shape" %in% names(data)) {
      new_data <- separate_failure_modes(data, "shape", sep = params$sep)
    } else {
      # neither were specified
      new_data <- data
    }
    new_data
  }
)

#' Jittered points showing (possibly multiple) failure modes
#'
#' @description
#' The `geom_jitter_failure_mode` is very similar to
#' [ggplot2::geom_jitter()] except that a failure mode variable specified
#' as the color and/or shape aesthetic is parsed to separate multiple
#' failure modes and plot them separately. For example, if an observation
#' has the failure mode "LAT/LAB", two points will be plotted, one with the
#' failure mode "LAT" and the second with the failure mode "LAB".
#'
#'
#' @param mapping Set of aesthetic mapping created by `aes()`.
#'                See [ggplot2::geom_jitter()] for additional details.
#' @param data The data to be displayed by this layer.
#'             See [ggplot2::geom_jitter()] for additional details.
#' @param stat The statistical transformation to use on the data for this layer.
#'             See [ggplot2::geom_jitter()] for additional details.
#' @param position A position adjustment to use on the data for this layer.
#'                 See [ggplot2::geom_jitter()] for additional details.
#' @param ... Other arguments passed on to `layer()`'s `params` argument.
#'            See [ggplot2::geom_jitter()] for additional details.
#' @param width The amount of horizontal jitter.
#'              See [ggplot2::geom_jitter()] for additional details.
#' @param height The amount of vertical jitter.
#'               See [ggplot2::geom_jitter()] for additional details.
#' @param na.rm If FALSE, the default, missing values are removed with warning.
#'              See [ggplot2::geom_jitter()] for additional details.
#' @param show.legend `NA`, the default, indicates that nay aesthetics are
#'                    mapped.
#'                    See [ggplot2::geom_jitter()] for additional details.
#' @param inherit.aes if FALSE, overrides the default aesthetics.
#'                    See [ggplot2::geom_jitter()] for additional details.
#' @param sep A regular expression indicating the character(s) separating
#'            multiple failure modes. By default `"[/, ]+"`
#'
#' @details
#' The variable specified for the aesthetics `shape` and `color` are passed
#' to the function [separate_failure_modes()] to parse the failure modes and
#' separate multiple failure modes separated by character(s) specified in
#' the regular expression given in the parameter `sep`. By default, multiple
#' failure modes are expected to be separated by spaces, commas or forward
#' slashes, but this can be overridden.
#'
#' If both `shape` and `color` aesthetics are specified, both must be identical.
#'
#' @examples
#' library(dplyr)
#' library(ggplot2)
#' carbon.fabric.2 %>%
#'   filter(test == "WT") %>%
#'   ggplot(aes(x = condition, y = strength)) +
#'   geom_boxplot() +
#'   geom_jitter_failure_mode(aes(color = failure_mode, shape = failure_mode))
#'
#' @seealso [separate_failure_modes()]
#' @seealso [ggplot2::geom_jitter()]
#'
#' @importFrom ggplot2 position_jitter
#' @importFrom ggplot2 layer
#'
#' @export
geom_jitter_failure_mode <- function(mapping = NULL, data = NULL,
                                     stat = "identity", position = "jitter",
                                     ...,
                                     width = NULL,
                                     height = NULL,
                                     #nolint start
                                     na.rm = FALSE,
                                     show.legend = NA,
                                     inherit.aes = TRUE,
                                     #nolint end
                                     sep = "[/, ]+") {
  if (!missing(width) || !missing(height)) {
    if (!missing(position)) {
      stop(paste0(
        "Both `position` and `width`/`height` were supplied. ",
        "Choose a single approach to alter the position."
      ))
    }

    position <- position_jitter(width = width, height = height)
  }

  layer(
    data = data,
    mapping = mapping,
    stat = stat,
    geom = GeomPointFailureMode,
    position = position,
    show.legend = show.legend,
    inherit.aes = inherit.aes,
    params = list2(
      na.rm = na.rm,
      sep = sep,
      ...
    )
  )
}

suppressMessages(library(dplyr))
suppressMessages(library(ggplot2))

test_that("separate_failure_modes produces expected output", {
  df <- tibble(x = 1, fm = "FAT")
  expect_equal(df, separate_failure_modes(df, fm))

  df <- tibble(x = c(1, 2), fm = c("FGT/FGB/FAT", "FGT,FGB"))
  df1 <- tibble(x = c(1, 1, 1, 2, 2), fm = c("FGT", "FGB", "FAT", "FGT", "FGB"))
  expect_equal(separate_failure_modes(df, fm), df1)

  df <- tibble(x = c(1, 2), fm = c("FGT /FGB / FAT", "FGT, FGB"))
  expect_equal(separate_failure_modes(df, fm), df1)

  df <- tibble(x = c(1, 2), fm = c("FGT|FGB|FAT", "FGT_FGB"))
  expect_equal(separate_failure_modes(df, fm, sep = "[|_]+"), df1)
})

test_that("geom_jitter_failure_mode produces expected errors", {
  dat <- carbon.fabric.2 %>%
    filter(test == "WT") %>%
    mutate(condition = ordered(condition, c("CTD", "RTD", "ETW", "ETW2")))

  # at least one of color or shape
  gg <- ggplot(dat, aes(x = batch, y = strength)) +
    geom_jitter_failure_mode()
  expect_error(
    print(gg),
    regexp = ".*missing aesthetic.*"
  )
  gg <- dat %>%
    ggplot(aes(x = batch, y = strength)) +
    geom_jitter_failure_mode(
      aes(color = failure_mode))
  expect_no_error(print(gg))
  gg <- dat %>%
    ggplot(aes(x = batch, y = strength)) +
    geom_jitter_failure_mode(
      aes(shape = failure_mode))
  expect_no_error(print(gg))

  # if color and shape specified, both equal
  gg <- dat %>%
    ggplot(aes(x = batch, y = strength)) +
    geom_jitter_failure_mode(
      aes(color = failure_mode, shape = failure_mode))
  expect_no_error(print(gg))

  gg <- dat %>%
    ggplot(aes(x = batch, y = strength)) +
    geom_jitter_failure_mode(
      aes(colour = failure_mode, shape = batch))
  expect_error(
    print(gg),
    ".*both are equal.*"
  )

})

test_that("geom_jitter_failure_mode options work", {
  skip_if_not_installed("vdiffr")

  dat <- carbon.fabric.2 %>%
    filter(test == "WT") %>%
    mutate(condition = ordered(condition, c("CTD", "RTD", "ETW", "ETW2")))

  set.seed(123)  # since jitter is random, make sure this is reproducible
  vdiffr::expect_doppelganger("geom_jitter_failure_mode_sep1", {
    dat %>%
      mutate(failure_mode = sub("/", "|", failure_mode)) %>%
      ggplot(aes(x = batch, y = strength)) +
      geom_jitter_failure_mode(
        aes(color = failure_mode))
  })

  set.seed(123)  # since jitter is random, make sure this is reproducible
  vdiffr::expect_doppelganger("geom_jitter_failure_mode_sep2", {
    dat %>%
      mutate(failure_mode = sub("/", "|", failure_mode)) %>%
      ggplot(aes(x = batch, y = strength)) +
      geom_jitter_failure_mode(
        aes(color = failure_mode),
        sep = "[|, ]+")
  })

  # the result above should be identical to the one below. Test using
  # the same filename
  set.seed(123)  # since jitter is random, make sure this is reproducible
  vdiffr::expect_doppelganger("geom_jitter_failure_mode_sep2", {
    dat %>%
      ggplot(aes(x = batch, y = strength)) +
      geom_jitter_failure_mode(
        aes(color = failure_mode),
        sep = "[/, ]+")
  })
})

test_that("geom_jitter_failure_mode dopleganger", {
  skip_if_not_installed("vdiffr")

  dat <- carbon.fabric.2 %>%
    filter(test == "WT") %>%
    mutate(condition = ordered(condition, c("CTD", "RTD", "ETW", "ETW2")))

  set.seed(123)  # since jitter is random, make sure this is reproducible
  vdiffr::expect_doppelganger("geom_jitter_failure_mode_basic", {
    dat %>%
      ggplot(aes(x = batch, y = strength)) +
      geom_jitter_failure_mode(
        aes(color = failure_mode, shape = failure_mode))
  })
})

#' Sample data for a generic carbon fabric
#'
#' Datasets containing sample data that is typical of a generic carbon
#' fabric prepreg. This data is used in several examples within the
#' `cmstatr` package. This data is fictional and should
#' only be used for learning how to use this package.
#'
#' @name carbon.fabric

#' @rdname carbon.fabric
"carbon.fabric"

#' @rdname carbon.fabric
"carbon.fabric.2"

#' Anderson--Darling test for goodness of fit
#'
#' @description
#' Calculates the Anderson--Darling test statistic for a sample given
#' a particular distribution, and determines whether to reject the
#' hypothesis that a sample is drawn from that distribution.
#'
#' @param data a data.frame-like object (optional)
#' @param x a numeric vector or a variable in the data.frame
#' @param alpha the required significance level of the test.
#'              Defaults to 0.05.
#'
#' @return
#' an object of class `anderson_darling`. This object has the following
#' fields.
#'
#' - `call` the expression used to call this function
#' - `dist` the distribution used
#' - `data` a copy of the data analyzed
#' - `n` the number of observations in the sample
#' - `A` the Anderson--Darling test statistic
#' - `osl` the observed significance level (p-value),
#'   assuming the
#'   parameters of the distribution are estimated from the data
#' - `alpha` the required significance level for the test.
#'   This value is given by the user.
#' - `reject_distribution` a logical value indicating whether
#'   the hypothesis that the data is drawn from the specified distribution
#'   should be rejected
#'
#' @details
#' The Anderson--Darling test statistic is calculated for the distribution
#' given by the user.
#'
#' The observed significance level (OSL), or p-value, is calculated assuming
#' that the parameters
#' of the distribution are unknown; these parameters are estimate from the
#' data.
#'
#' The function `anderson_darling_normal` computes the Anderson--Darling
#' test statistic given a normal distribution with mean and standard deviation
#' equal to the sample mean and standard deviation.
#'
#' The function `anderson_darling_lognormal` is the same as
#' `anderson_darling_normal` except that the data is log transformed
#' first.
#'
#' The function `anderson_darling_weibull` computes the Anderson--Darling
#' test statistic given a Weibull distribution with shape and scale parameters
#' estimated from the data using a maximum likelihood estimate.
#'
#' The test statistic, `A`, is modified to account for
#' the fact that the parameters of the population are not known,
#' but are instead estimated from the sample. This modification is
#' a function of the sample size only, and is different for each
#' distribution (normal/lognormal or Weibull). Several such modifications
#' have been proposed. This function uses the modification published in
#' Stephens (1974), Lawless (1982) and CMH-17-1G. Some other implementations
#' of the Anderson-Darling test, such as the implementation in the
#' `nortest` package, use other modifications, such as the one
#' published in D'Agostino and Stephens (1986). As such, the p-value
#' reported by this function may differ from the p-value reported
#' by implementations of the Anderson--Darling test that use
#' different modifiers. Only the unmodified
#' test statistic is reported in the result of this function, but
#' the modified test statistic is used to compute the OSL (p-value).
#'
#' This function uses the formulae for observed significance
#' level (OSL) published in CMH-17-1G. These formulae depend on the particular
#' distribution used.
#'
#' The results of this function have been validated against
#' published values in Lawless (1982).
#'
#'
#' @references
#' J. F. Lawless, *Statistical models and methods for lifetime data*.
#' New York: Wiley, 1982.
#'
#' "Composite Materials Handbook, Volume 1. Polymer Matrix
#' Composites Guideline for Characterization of Structural
#' Materials," SAE International, CMH-17-1G, Mar. 2012.
#'
#' M. A. Stephens, “EDF Statistics for Goodness of Fit and Some
#' Comparisons,”
#' Journal of the American Statistical Association, vol. 69, no. 347.
#' pp. 730–737, 1974.
#'
#' R. D’Agostino and M. Stephens, Goodness-of-Fit Techniques.
#' New York: Marcel Dekker, 1986.
#'
#' @examples
#' library(dplyr)
#'
#' carbon.fabric %>%
#'   filter(test == "FC") %>%
#'   filter(condition == "RTD") %>%
#'   anderson_darling_normal(strength)
#' ## Call:
#' ## anderson_darling_normal(data = ., x = strength)
#' ##
#' ## Distribution:  Normal ( n = 18 )
#' ## Test statistic:  A = 0.9224776
#' ## OSL (p-value):  0.01212193  (assuming unknown parameters)
#' ## Conclusion: Sample is not drawn from a Normal distribution (alpha = 0.05)
#'
#' @importFrom rlang enquo eval_tidy
#'
#' @name anderson_darling
NULL

# A non-exported function for an Anderson--Darling goodness of fit test
# The function \code{dist} should be
# the cumulative distribution function. Additional parameters, such as
# the parameters for the distribution, can be passed through the argument
# to the function \code{...} to the function \code{dist}.
anderson_darling <- function(x0, call, ad_p_unknown_param_fcn,
                             dist_name, dist, alpha, ...) {
  f0 <- dist
  x0_sorted <- sort(x0)
  n <- length(x0_sorted)
  ii <- 1:n
  u <- f0(x0_sorted, ...)
  a <- -n - sum((2 * ii - 1) / n * (log(u) + log(1 - rev(u))))

  res <- list(
    call = call,
    dist = dist_name,
    data = x0,
    n = n,
    A = a,  # nolint
    osl = ad_p_unknown_param_fcn(a, n),
    alpha = alpha
  )
  res$reject_distribution <- res$osl <= res$alpha

  class(res) <- "anderson_darling"

  return(res)
}


#' Glance at an `anderson_darling` object
#'
#' @description
#' Glance accepts an object of type `anderson_darling` and
#' returns a [tibble::tibble()] with
#' one row of summaries.
#'
#' Glance does not do any calculations: it just gathers the results in a
#' tibble.
#'
#' @param x an `anderson_darling` object
#' @param ... Additional arguments. Not used. Included only to match generic
#'            signature.
#'
#'
#' @return
#' A one-row [tibble::tibble()] with the following
#' columns:
#'
#' - `dist` the distribution used
#' - `n` the number of observations in the sample
#' - `A` the Anderson--Darling test statistic
#' - `osl` the observed significance level (p-value),
#'     assuming the
#'     parameters of the distribution are estimated from the data
#'  - `alpha` the required significance level for the test.
#'    This value is given by the user.
#' - `reject_distribution` a logical value indicating whether
#'    the hypothesis that the data is drawn from the specified distribution
#'    should be rejected
#'
#'
#' @seealso
#' [anderson_darling()]
#'
#' @examples
#' x <- rnorm(100, 100, 4)
#' ad <- anderson_darling_weibull(x = x)
#' glance(ad)
#'
#' ## # A tibble: 1 x 6
#' ##   dist        n     A        osl alpha reject_distribution
#' ##   <chr>   <int> <dbl>      <dbl> <dbl> <lgl>
#' ## 1 Weibull   100  2.62 0.00000207  0.05 TRUE
#'
#' @method glance anderson_darling
#' @importFrom tibble tibble
#'
#' @export
glance.anderson_darling <- function(x, ...) {  # nolint
  # nolint start: object_usage_linter
  with(
    x,
    tibble::tibble(
      dist = dist,
      n = n,
      A = A,  # nolint
      osl = osl,
      alpha = alpha,
      reject_distribution = reject_distribution
    )
  )
  # nolint end
}

#' @export
print.anderson_darling <- function(x, ...) {
  cat("\nCall:\n",
      paste(deparse(x$call), sep = "\n", collapse = "\n"), "\n\n", sep = "")

  cat("Distribution: ", x$dist, "( n =", x$n, ")", "\n")

  cat("Test statistic:  A =", x$A, "\n")
  cat(
    "OSL (p-value): ",
    x$osl,
    " (assuming unknown parameters)\n"
  )
  if (x$reject_distribution) {
    cat("Conclusion: Sample is not drawn from a",
        x$dist,
        "distribution ( alpha =", x$alpha, ")")
  } else {
    cat("Conclusion: Sample is drawn from a",
        x$dist,
        "distribution ( alpha =", x$alpha, ")")
  }
}

#' @importFrom rlang enquo eval_tidy
#' @importFrom stats pnorm
#'
#' @rdname anderson_darling
#'
#' @export
anderson_darling_normal <- function(data = NULL, x, alpha = 0.05) {
  verify_tidy_input(
    df = data,
    x = x,
    c = match.call(),
    arg_name = "x")
  x0 <- eval_tidy(enquo(x), data)
  return(anderson_darling(x0, call = match.call(),
                          ad_p_unknown_param_fcn = ad_p_normal_unknown_param,
                          dist_name = "Normal",
                          alpha = alpha,
                          dist = pnorm, mean = mean(x0), sd = sd(x0)))
}

#' @importFrom rlang enquo eval_tidy
#' @importFrom stats pnorm
#'
#' @rdname anderson_darling
#'
#' @export
anderson_darling_lognormal <- function(data = NULL, x, alpha = 0.05) {
  verify_tidy_input(
    df = data,
    x = x,
    c = match.call(),
    arg_name = "x")
  x0 <- eval_tidy(enquo(x), data)
  x0 <- log(x0)
  return(anderson_darling(x0, call = match.call(),
                          ad_p_unknown_param_fcn = ad_p_normal_unknown_param,
                          dist_name = "Lognormal",
                          alpha = alpha,
                          dist = pnorm, mean = mean(x0), sd = sd(x0)))
}

#' @importFrom rlang enquo eval_tidy
#' @importFrom stats pweibull
#' @importFrom MASS fitdistr
#'
#' @rdname anderson_darling
#'
#' @export
anderson_darling_weibull <- function(data = NULL, x, alpha = 0.05) {
  verify_tidy_input(
    df = data,
    x = x,
    c = match.call(),
    arg_name = "x")
  x0 <- eval_tidy(enquo(x), data)
  dist <- fitdistr(x0, "weibull")
  return(anderson_darling(x0, call = match.call(),
                          dist = pweibull,
                          dist_name = "Weibull",
                          alpha = alpha,
                          ad_p_unknown_param_fcn = ad_p_weibull_unknown_param,
                          shape = dist$estimate[["shape"]],
                          scale = dist$estimate[["scale"]]))
}

ad_p_normal_unknown_param <- function(z, n) {
  ad_star <- (1 + 4 / n - 25 / n ^ 2) * z
  osl <- 1 / (1 + exp(-0.48 + 0.78 * log(ad_star) + 4.58 * ad_star))
  return(osl)
}

ad_p_weibull_unknown_param <- function(z, n) {
  ad_star <- (1 + 0.2 / sqrt(n)) * z
  osl <- 1 / (1 + exp(-0.1 + 1.24 * log(ad_star) + 4.48 * ad_star))
  return(osl)
}

context("Anderson-Darling test for goodness of fit")

test_that("AD test gives same results for a data frame and a vector", {
  data <- data.frame(
    strength = c(
      79.9109621761937,
      77.9447436346388,
      79.717168019752,
      87.3547460860547,
      76.2404769192413,
      75.7026911300246,
      79.5952709280298,
      76.7833784980155,
      77.5791472067831,
      78.4164523339268,
      79.2819398818745,
      77.6346481930964,
      81.2182937743241,
      81.1826431730731,
      86.0561762593461,
      82.1837784884495,
      80.7564920650884,
      79.3614980225488
    ))
  res_vec1 <- anderson_darling_normal(
    x = data$strength)

  # value from STAT17 (0.0840)
  expect_equal(res_vec1$osl, 0.0840, tolerance = 0.002)

  res_df <- anderson_darling_normal(
    data,
    strength)

  expect_equal(res_vec1$osl, res_df$osl)
})

test_that("AD test matches results from STAT17 (normal)", {
  data <- data.frame(
    strength = c(
      137.4438,
      139.5395,
      150.89,
      141.4474,
      141.8203,
      151.8821,
      143.9245,
      132.9732,
      136.6419,
      138.1723,
      148.7668,
      143.283,
      143.5429,
      141.7023,
      137.4732,
      152.338,
      144.1589,
      128.5218
    ))
  res_vec <- anderson_darling_normal(x = data$strength)
  expect_equal(res_vec$osl, 0.465, tolerance = 0.002)
  # Do it again but pass in a data.frame
  res_df <- anderson_darling_normal(data, strength)
  expect_equal(res_vec$osl, res_df$osl)
})

test_that("AD test matches results from STAT17 (lognormal)", {
  data <- data.frame(
    strength = c(
      137.4438,
      139.5395,
      150.89,
      141.4474,
      141.8203,
      151.8821,
      143.9245,
      132.9732,
      136.6419,
      138.1723,
      148.7668,
      143.283,
      143.5429,
      141.7023,
      137.4732,
      152.338,
      144.1589,
      128.5218
    ))
  res_vec <- anderson_darling_lognormal(x = data$strength)
  expect_equal(res_vec$osl, 0.480, tolerance = 0.002)
  # Do it again but pass in a data.frame
  res_df <- anderson_darling_lognormal(data, strength)
  expect_equal(res_vec$osl, res_df$osl)
})

test_that("AD test matches results from STAT17 (weibull)", {
  data <- data.frame(
    strength = c(
      137.4438,
      139.5395,
      150.89,
      141.4474,
      141.8203,
      151.8821,
      143.9245,
      132.9732,
      136.6419,
      138.1723,
      148.7668,
      143.283,
      143.5429,
      141.7023,
      137.4732,
      152.338,
      144.1589,
      128.5218
    )
  )
  # OSL: 0.179
  res_vec <- anderson_darling_weibull(x = data$strength)
  expect_equal(res_vec$osl, 0.179, tolerance = 0.002)
  # Do it again but pass in a data.frame
  res_df <- anderson_darling_weibull(data, strength)
  expect_equal(res_vec$osl, res_df$osl)
})

test_that("ad_p_unknown_parameters matches normal results from Lawless", {
  # Comparison with the results from:
  # J. F. Lawless, \emph{Statistical models and methods for lifetime data}.
  # New York: Wiley, 1982.
  # See page 458
  fcn <- function(a, n) {
    ad_p_normal_unknown_param(a / (1 + 4 / n - 25 / n ^ 2), n)
  }

  n <- 5
  expect_equal(fcn(0.576, n), 0.15, tolerance = 0.002)
  expect_equal(fcn(0.656, n), 0.10, tolerance = 0.002)
  expect_equal(fcn(0.787, n), 0.05, tolerance = 0.002)
  expect_equal(fcn(0.918, n), 0.025, tolerance = 0.002)
  expect_equal(fcn(1.092, n), 0.01, tolerance = 0.002)

  n <- 10
  expect_equal(fcn(0.576, n), 0.15, tolerance = 0.002)
  expect_equal(fcn(0.656, n), 0.10, tolerance = 0.002)
  expect_equal(fcn(0.787, n), 0.05, tolerance = 0.002)
  expect_equal(fcn(0.918, n), 0.025, tolerance = 0.002)
  expect_equal(fcn(1.092, n), 0.01, tolerance = 0.002)

  n <- 20
  expect_equal(fcn(0.576, n), 0.15, tolerance = 0.002)
  expect_equal(fcn(0.656, n), 0.10, tolerance = 0.002)
  expect_equal(fcn(0.787, n), 0.05, tolerance = 0.002)
  expect_equal(fcn(0.918, n), 0.025, tolerance = 0.002)
  expect_equal(fcn(1.092, n), 0.01, tolerance = 0.002)
})

test_that("ad_p_unknown_parameters matches weibull results from Lawless", {
  # Comparison with the results from:
  # J. F. Lawless, \emph{Statistical models and methods for lifetime data}.
  # New York: Wiley, 1982.
  # See p. 455
  fcn <- function(a, n) {
    ad_p_weibull_unknown_param(a / (1 + 0.2 / sqrt(n)), n)
  }

  n <- 5
  expect_equal(fcn(0.474, n), 0.25, tolerance = 0.002)
  expect_equal(fcn(0.637, n), 0.10, tolerance = 0.002)
  expect_equal(fcn(0.757, n), 0.05, tolerance = 0.002)
  expect_equal(fcn(0.877, n), 0.025, tolerance = 0.002)
  expect_equal(fcn(1.038, n), 0.01, tolerance = 0.002)

  n <- 10
  expect_equal(fcn(0.474, n), 0.25, tolerance = 0.002)
  expect_equal(fcn(0.637, n), 0.10, tolerance = 0.002)
  expect_equal(fcn(0.757, n), 0.05, tolerance = 0.002)
  expect_equal(fcn(0.877, n), 0.025, tolerance = 0.002)
  expect_equal(fcn(1.038, n), 0.01, tolerance = 0.002)

  n <- 20
  expect_equal(fcn(0.474, n), 0.25, tolerance = 0.002)
  expect_equal(fcn(0.637, n), 0.10, tolerance = 0.002)
  expect_equal(fcn(0.757, n), 0.05, tolerance = 0.002)
  expect_equal(fcn(0.877, n), 0.025, tolerance = 0.002)
  expect_equal(fcn(1.038, n), 0.01, tolerance = 0.002)
})

test_that("print.anderson_darling contains expected values", {
  data <- data.frame(
    strength = c(
      137.4438,
      139.5395,
      150.89,
      141.4474,
      141.8203,
      151.8821,
      143.9245,
      132.9732,
      136.6419,
      138.1723,
      148.7668,
      143.283,
      143.5429,
      141.7023,
      137.4732,
      152.338,
      144.1589,
      128.5218
    ))
  res_vec <- anderson_darling_normal(x = data$strength)
  # should include the distribution
  expect_output(print(res_vec), "distribution.*normal", ignore.case = TRUE)
  # should include the signficance for known parameters
  expect_output(print(res_vec), "OSL.*0.464.*unknown", ignore.case = TRUE)
  # conclusion should be printed
  expect_output(print(res_vec), "conclusion.*is drawn.*alpha.*0.05",
                ignore.case = TRUE)
  expect_false(res_vec$reject_distribution)

  # if alpha is adjusted to be above OSL, the conclusion should be reversed
  res_vec <- anderson_darling_normal(x = data$strength, alpha = 0.470)
  expect_output(print(res_vec), "conclusion.*is not drawn.*alpha.*0.47",
                ignore.case = TRUE)
  expect_true(res_vec$reject_distribution)
})

test_that("glance method produces expected results", {
  x <- c(
    137.4438,
    139.5395,
    150.89,
    141.4474,
    141.8203,
    151.8821,
    143.9245,
    132.9732,
    136.6419,
    138.1723,
    148.7668,
    143.283,
    143.5429,
    141.7023,
    137.4732,
    152.338,
    144.1589,
    128.5218
  )
  res <- anderson_darling_lognormal(x = x)
  glance_res <- glance(res)

  expect_equal(glance_res$osl[1], 0.480, tolerance = 0.002)
  expect_equal(glance_res$dist[1], "Lognormal")
  expect_equal(glance_res$n[1], 18)
  expect_equal(glance_res$A[1], 0.277, tolerance = 0.005)
  expect_equal(glance_res$alpha, 0.05)
  expect_equal(glance_res$reject_distribution, FALSE)
})

#' Performs a list of checks according to specified rules
#'
#' @description
#' Takes a named list of function (the rules). Each function should take
#' the arguments it requires plus ... Each function should return a list
#' with the following contents:
#'  - msg (character): a message indicating the check failure, or "" for a pass
#'  - check (any object): an object representing the check performed.
#'
#' This function will produce warning(s) for each of the rules that fail.
#'
#' @param rules a named list of rules
#' @param override a vector of names of rules to ignore
#' @param ... arguments to pass to the rules
#'
#' @return
#' a named vector of lists. Each list has the following contents:
#'   - msg: The message generated by the check. "" if it passed.
#'   - obj: The object representing the check
#'   - pfo: "P" if the test passed, "F" if the test failed,
#'          "O" if the test was overridden, and NA if the test was not run
#'          due to a missing argument.
#'
#' @noRd
#'
#' @importFrom rlang list2
#' @importFrom rlang fn_fmls_names
#' @importFrom rlang warn
#' @importFrom rlang inform
perform_checks <- function(rules, override = c(), ...) {
  args <- list2(...)

  lapply(names(rules), function(cur_rule_name) {
    if (!(cur_rule_name %in% override)) {
      cur_rule <- rules[[cur_rule_name]]
      all_formal_names <- fn_fmls_names(cur_rule)
      missing_formals <- vapply(all_formal_names, function(cur_formal_name) {
        is.null(args[[cur_formal_name]]) & cur_formal_name != "..."
      },
      FUN.VALUE = logical(1L)
      )
      if (!any(missing_formals)) {
        check_res <- tryCatch(
          do.call(cur_rule, args),
          error = function(e) {
            stop(paste0("During evaluation of `", cur_rule_name, "`: ", e))
          })
        if (check_res$msg != "") {
          warn(paste0("`", cur_rule_name, "` failed: ", check_res$msg))
          check_res[["pfo"]] <- "F"
          check_res[["rule_name"]] <- cur_rule_name
          return(check_res)
        }
        check_res[["pfo"]] <- "P"
        check_res[["rule_name"]] <- cur_rule_name
        check_res
      } else {
        inform(
          paste0("`", cur_rule_name, "` not run because parameter",
                 ifelse(sum(missing_formals) > 1, "s", ""),
                 " `",
                 paste(all_formal_names[missing_formals], collapse = "`, `"),
                 "` not specified")
        )
        return(list(
          msg = "Not run",
          obj = NULL,
          rule_name = cur_rule_name,
          pfo = NA_character_))
      }
    } else {
      # in override list
      return(list(
        msg = "Overridden",
        obj = NULL,
        rule_name = cur_rule_name,
        pfo = "O"))
    }
  })
}

#' Gets the pass/fail/overridden results for a set of checks
#'
#' @return
#' a named vector of characters. "P" if the test passed,
#' "F" if the test failed, "O" if the test was overridden,
#' and NA if the test was not run
#' due to a missing argument.
#'
#' @noRd
get_check_pfo <- function(check_results) {
  res <- vapply(
    check_results,
    function(cr) {
      cr[["pfo"]]
    },
    FUN.VALUE = character(1L)
  )
  names(res) <- sapply(
    check_results,
    function(cr) {
      cr[["rule_name"]]
    }
  )
  res
}

#' Gets the object representing results for a set of checks
#'
#' @return
#' a named vector of objects
#'
#' @noRd
get_check_obj <- function(check_results) {
  res <- sapply(
    check_results,
    function(cr) {
      cr[["obj"]]
    }
  )
  names(res) <- sapply(
    check_results,
    function(cr) {
      cr[["rule_name"]]
    }
  )
  res
}

#' Gets the names of the diagnostic tests that failed
#'
#' @param x a named character vector created by perform_checks
#'
#' @return
#' A character vector of the tests that failed (if any)
#'
#' @noRd
get_check_failure_names <- function(x) {
  names(x[x == "F" & !is.na(x)])
}

process_overrides <- function(override, rules) {
  if ("all" %in% override) {
    # Remove the "all" value
    override <- override[!override %in% "all"]
    # Add all of the rules to the override vector
    override <- c(override, names(rules))
  }
  # Keep only the unique values of override
  override <- unique(override)
  # Warn if there are invalid overrides
  for (ov in override) {
    if (!ov %in% names(rules)) {
      warn(paste0("`", ov, "` is not a valid diagnostic test to override"))
    }
  }

  override
}

test_that("perform_checks produces warnings unless overriden", {
  sample_rules <- list(
    positive = function(pos, ...) {
      list(
        msg = ifelse(pos > 0, "", "Not positive"),
        obj = pos > 0
      )
    },
    negative = function(neg, ...) {
      list(
        msg = ifelse(neg < 0, "", "Not negative"),
        obj = neg < 0
      )
    },
    zero = function(z, ...) {
      list(
        msg = ifelse(z == 0, "", "Not zero"),
        obj = z == 0
      )
    }
  )

  expect_warning(res <- perform_checks(sample_rules, pos = -1, neg = -1, z = 0),
                 regexp = "positive")
  res <- get_check_pfo(res)
  names(res) <- NULL
  expect_equal(res, c("F", "P", "P"))
  perform_checks(sample_rules, pos = -1, neg = -1, z = 0,
                 override = "positive")
  expect_warning(res <- perform_checks(sample_rules, pos = 1, neg = 1, z = 0),
                 regexp = "negative")
  res <- get_check_pfo(res)
  names(res) <- NULL
  expect_equal(res, c("P", "F", "P"))
  perform_checks(sample_rules, pos = 1, neg = 1, z = 0,
                 override = "negative")
  expect_warning(res <- perform_checks(sample_rules, pos = 1, neg = -1, z = 1),
                 regexp = "zero")
  res <- get_check_pfo(res)
  names(res) <- NULL
  expect_equal(res, c("P", "P", "F"))
  perform_checks(sample_rules, pos = 1, neg = -1, z = 1,
                 override = "zero")

  # should produce no warnings
  perform_checks(sample_rules, pos = 1, neg = -1, z = 0)
})

test_that("Messages are created for missing parameters", {
  sample_rules <- list(
    positive = function(pos, ...) {
      list(
        msg = ifelse(pos > 0, "", "Not positive"),
        obj = pos > 0
      )
    },
    negative = function(neg, ...) {
      list(
        msg = ifelse(neg < 0, "", "Not negative"),
        obj = neg < 0
      )
    },
    zero = function(z, ...) {
      list(
        msg = ifelse(z == 0, "", "Not zero"),
        obj = z == 0
      )
    },
    neg_or_zero = function(neg, z, ...) {
      list(
        msg = "",
        obj = TRUE
      )
    }
  )

  expect_message(perform_checks(sample_rules, pos = NULL, neg = -1, z = 0),
                 regexp = "`positive`.+`pos`",
                 all = TRUE)
  expect_message(
    expect_message(
      perform_checks(sample_rules, pos = 1, neg = NULL, z = 0),
      "negative"),
    "neg_or_zero"
  )

  # two parameters missing
  expect_message(
    expect_message(
      expect_message(
        perform_checks(sample_rules, pos = 1, neg = NULL, z = NULL),
        "negative"
      ),
      "zero"
    ),
    "neg_or_zero"
  )
})

test_that("User can specify `all` for overrides", {
  sample_rules <- list(
    positive = function(pos, ...) {
      ifelse(pos > 0, "", "Not positive")
    },
    negative = function(neg, ...) {
      ifelse(neg < 0, "", "Not negative")
    },
    zero = function(z, ...) {
      ifelse(z == 0, "", "Not zero")
    }
  )

  expect_equal(
    process_overrides("all", sample_rules),
    c("positive", "negative", "zero")
  )

  expect_equal(
    process_overrides(c("all"), sample_rules),
    c("positive", "negative", "zero")
  )

  expect_equal(
    process_overrides(c("all", "positive"), sample_rules),
    c("positive", "negative", "zero")
  )

  expect_equal(
    process_overrides(c("positive", "all"), sample_rules),
    c("positive", "negative", "zero")
  )
})

test_that("Invalid overrides produce warnings", {
  sample_rules <- list(
    positive = function(pos, ...) {
      ifelse(pos > 0, "", "Not positive")
    },
    negative = function(neg, ...) {
      ifelse(neg < 0, "", "Not negative")
    },
    zero = function(z, ...) {
      ifelse(z == 0, "", "Not zero")
    }
  )

  expect_warning(process_overrides("invalid", sample_rules), "diagnostic")
})

test_that("Errors raised are wrapped with source of warning", {
  sample_rules <- list(
    positive = function(pos, ...) {
      list(
        msg = ifelse(pos > 0, "", "Not positive"),
        obj = pos > 0
      )
    },
    negative = function(neg, ...) {
      list(
        msg = ifelse(neg < 0, "", "Not negative"),
        obj = neg < 0
      )
    },
    zero = function(z, ...) {
      list(
        msg = ifelse(z == 0, "", "Not zero"),
        obj = z == 0
      )
    },
    always_error = function(...) {
      stop("This is an error")
      list(
        msg = "",
        obj = FALSE
      )
    }
  )

  # The "always_error" diagnostic test should result in an error containing
  # the name of the diagnostic test
  expect_error(
    perform_checks(sample_rules, pos = +1, neg = -1, z = 0),
    "always_error"
  )

  # The behavior should be the same, even if some tests fail
  # In this case, the warnings should still be raised, at least
  # if they occur before the error
  expect_error(
    expect_warning(
      perform_checks(sample_rules, pos = +1, neg = +1, z = 0),
      "negative"),
    "always_error"
  )
})

#' @importFrom generics augment
#' @export
#' @seealso [generics::augment()]
generics::augment

#' @importFrom generics tidy
#' @export
#' @seealso [generics::tidy()]
generics::tidy

#' @importFrom generics glance
#' @export
#' @seealso [generics::glance()]
generics::glance

if (requireNamespace("lintr", quietly = TRUE)) {
  test_that("Package Style", {
    lintr::expect_lint_free(
      path = getwd(),
      relative_path = FALSE,
      exclusions = list(
        "R/RcppExports.R",
        # Ignore the auto-generated R files
        "vignettes/adktest.R",
        "vignettes/cmstatr_Graphing.R",
        "vignettes/cmstatr_Tutorial.R",
        "vignettes/cmstatr_Validation.R",
        "vignettes/hk_ext.R"
      ),
      linters = lintr::linters_with_defaults(
        line_length_linter = NULL,
        trailing_whitespace_linter = NULL,
        indentation_linter = NULL
      )
    )
  })
}

suppressMessages(library(dplyr))

test_that("CV produces expected results", {
  x <- rnorm(30, 100, 8)

  res <- cv(x)

  expect_equal(res, sd(x) / mean(x))
})

test_that("CV with na.rm=TRUE works", {
  x <- rnorm(30, 100, 8)
  x <- c(x, NA, NA, NA)

  res <- cv(x, na.rm = TRUE)

  expect_equal(res, sd(x[1:30]) / mean(x[1:30]))
})

test_that("cv works inside dplyr::summarize", {
  data.frame(
    x = rnorm(30, 100, 8)
  ) %>%
    summarize(cv = cv(x),
              test = expect_equal(cv, sd(x) / mean(x)))
})

test_that("cv works inside dplyr::summarize with na.rm=TRUE", {
  data.frame(
    x = rnorm(30, 100, 8)
  ) %>%
    bind_rows(
      data.frame(x = c(NA, NA, NA))
    ) %>%
    summarize(cv = cv(x, na.rm = TRUE),
              test = expect_equal(cv, sd(x, na.rm = TRUE) /
                                    mean(x, na.rm = TRUE)))
})

test_that("calc_cv_star works for single values and vectors", {
  expect_equal(calc_cv_star(0.01), 0.06)
  expect_equal(calc_cv_star(c(0.01, 0.09)), c(0.06, 0.09))
})

---
title: "Plotting Composite Material Data"
author: "Ally Fraser"
date: "2-May-2020"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Plotting Composite Material Data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 6
)

# If any of the required packages are unavailable,
# don't re-run the code
# nolint start
required <- c("dplyr", "ggplot2", "tidyr", "cmstatr")
if (!all(unlist(lapply(required, function(pkg) {
    requireNamespace(pkg, quietly = TRUE)}
  )))) {
  knitr::opts_chunk$set(eval = FALSE)
}
# nolint end
```

This vignette demonstrates how to create some of the graphs commonly used
when analyzing composite material data. Here, we rely on the
[`ggplot2`](https://ggplot2.tidyverse.org/) package for graphing. This package
can be loaded either on its own, or through the `tidyverse` meta-package, which
also includes packages such as `dplyr` that we will also use.

We'll need to load a few packages in order to proceed.

```{r message=FALSE}
library(dplyr)
library(ggplot2)
library(tidyr)
library(cmstatr)
```

Throughout this vignette, we'll use one of the example data sets that
comes with `cmstatr` and we'll focus on the warp-tension data as an
example. We'll load the example data in a variable as follows. By default
the condition will be in an arbitrary order, but throughout the visualization,
we'll want the conditions shown in a particular order (from coldest and driest
to hottest and wettest). We can define the order of the conditions using
the `ordered` function. For brevity, only the first few rows of the data set
are displayed below.

```{r}
dat <- carbon.fabric.2 %>%
  filter(test == "WT") %>%
  mutate(condition = ordered(condition, c("CTD", "RTD", "ETW", "ETW2")))

dat %>%
  head(10)
```

We'll then calculate the B-Basis value using the pooling by standard deviation
method. This data set happens to fail some of the diagnostic tests, but for the
purpose of this example, we'll ignore those failures using the
`override` argument.

```{r}
b_basis_pooled <- dat %>%
  basis_pooled_cv(strength, condition, batch,
                  override = c("between_group_variability",
                               "normalized_variance_equal"))

b_basis_pooled
```

The object returned from `basis_pooled_cv` contains a number of values.
One value is a `data.frame` containing the groups (i.e. conditions)
and the corresponding basis values. This looks like the following. We'll
use this in the visualizations.

```{r}
b_basis_pooled$basis
```


# Batch Plots
Batch plots are used to identify differences between batches.
Simple batch plots can be created using box plots and adding
horizontal lines for the basis values as follows. Note that
the heavy line in the box of the box plot is the *median*, not
the mean. The two hinges correspond with the first and third
quantiles and the whiskers extend to the most extreme data point,
or 1.5 times the inner quantile range.

In the code below, we use the function `rename` to rename the
column `group` to `condition`. The `data.frame` produced by
`basis_pooled_cv` uses the columns `value` and `group`, but
to match the data, we need the column with the conditions
to be named `condition`.

```{r}
dat %>%
  ggplot(aes(x = batch, y = strength)) +
  geom_boxplot() +
  geom_jitter(width = 0.25) +
  geom_hline(aes(yintercept = value),
             data = b_basis_pooled$basis %>% rename(condition = group),
             color = "blue") +
  facet_grid(. ~ condition) +
  theme_bw() +
  ggtitle("Batch Plot")
```

It's sometimes useful add failure modes to the plot so that you can easily
identify whether failure modes differ between conditions or batches.
This can be done with the `geom_jitter_failure_mode()` function. This
behaves very similarly to `geom_jitter()`, except that `color` or `shape`
must be specified, and if there are multiple failure modes (e.g. "LAB/LAT"),
both failure modes are plotted as separate points.

```{r}
dat %>%
  ggplot(aes(x = batch, y = strength)) +
  geom_boxplot() +
  geom_jitter_failure_mode(aes(color = failure_mode, shape = failure_mode), width = 0.25) +
  geom_hline(aes(yintercept = value),
             data = b_basis_pooled$basis %>% rename(condition = group),
             color = "blue") +
  facet_grid(. ~ condition) +
  theme_bw() +
  ggtitle("Batch Plot with Failure Modes")
```

# Quantile Plots
A quantile plot provides a graphical summary of sample values. 
This plot displays the sample values and the corresponding quantile.
A quantile  plot can  be used to examine the symmetry and tail sizes of the
underlying distribution. Sharp rises may indicate the presence of outliers. 

```{r}
dat %>%
  ggplot(aes(x = strength, color = condition)) +
  stat_ecdf(geom = "point") +
  coord_flip() +
  theme_bw() +
  ggtitle("Quantile Plot")
```

# Normal Survival Function Plots
An empirical survival function, and the corresponding normal survival
function can be plotted using two `ggplot` "stat" functions provided
by `cmstatr`. In the example below, the empirical survival function
is plotted for each condition, and the survival function for a normal
distribution with the mean and variance from the data is also plotted
(the survival function is 1 minus the cumulative distribution function).
This type of plot can be used to identify how closely the data follows
a normal distribution, and also to compare the distributions of the
various conditions.

```{r}
dat %>%
  ggplot(aes(x = strength, color = condition)) +
  stat_normal_surv_func() +
  stat_esf() +
  theme_bw() +
  ggtitle("Normal Survival Function Plot")
```


# Normal Score Plots
The normal scores plot calculates the normal score and plots it against
the normal score. Normal plots are useful to investigate distributions of
the data.

```{r}
dat %>%
  group_by(condition) %>%
  mutate(norm.score = scale(strength)) %>%
  ggplot(aes(x = norm.score, y = strength, colour = condition)) +
  geom_point() +
  ggtitle("Normal Scores Plot") +
  theme_bw()
```

# Q-Q Plots

A Q-Q plot compares the data against the
theoretical quantiles for a particular distribution.
A line is also plotted showing
the normal distribution with mean and variance from the data. If the data
exactly followed a normal distribution, all points would fall on this line.

```{r}
dat %>%
  ggplot(aes(sample = strength, colour = condition)) +
  geom_qq() +
  geom_qq_line() +
  ggtitle("Q-Q Plot") +
  theme_bw()
```


# Property Plots

Property plots allow for a variety of properties for a group to be 
compared to other properties within the same group, as well as to 
other group properties. The properties included in this plot are A-Basis, 
B-Basis, Pooled A- and B-Basis, Pooled Modified CV (Coefficient of Variation)
A- and B-Basis, Mean, and Min for each group.

The property plots will take a bit of work to construct.

First, the distribution of each group must be determined. Once the
distribution has been determined, the proper basis calculation based 
on that distribution should be filled in below. We also have a
column in the tables below for extra arguments to pass to the `basis`
function, such as overrides required or the method for the `basis_hk_ext`
function to use.

```{r}
b_basis_fcn <- tribble(
  ~condition, ~fcn, ~args,
  "CTD", "basis_normal", list(override = c("between_batch_variability")),
  "RTD", "basis_normal", list(override = c("between_batch_variability")),
  "ETW", "basis_hk_ext", NULL,
  "ETW2", "basis_normal", list(override = c("between_batch_variability"))
)

a_basis_fcn <- tribble(
  ~condition, ~fcn, ~args,
  "CTD", "basis_normal", list(override = c("between_batch_variability")),
  "RTD", "basis_normal", list(override = c("between_batch_variability")),
  "ETW", "basis_hk_ext", list(method = "woodward-frawley"),
  "ETW2", "basis_normal", list(override = c("between_batch_variability"))
)
```

We'll write a function that takes the data and information about
the distribution and computes the single-point
basis value. We'll use this
function for both A- and B-Basis, so we'll add a parameter for the
probability (0.90 or 0.99).

```{r}
single_point_fcn <- function(group_x, group_batch, cond, basis_fcn, p) {
  fcn <- basis_fcn$fcn[basis_fcn$condition == cond[1]]
  extra_args <- basis_fcn$args[basis_fcn$condition == cond[1]]

  args <- c(
    list(x = group_x, batch = group_batch, p = p),
    unlist(extra_args))
  basis <- do.call(fcn, args)
  basis$basis
}

single_point_results <- dat %>%
  group_by(condition) %>%
  summarise(single_point_b_basis = single_point_fcn(
              strength, batch, condition, b_basis_fcn, 0.90),
            single_point_a_basis = single_point_fcn(
              strength, batch, condition, a_basis_fcn, 0.99),
            minimum = min(strength),
            mean = mean(strength)) %>%
  mutate(condition = ordered(condition, c("CTD", "RTD", "ETW", "ETW2")))

single_point_results
```

In the above code, we also ensure that the condition column is still
in the order we expect.

We've already computed the B-Basis of the data using a pooling method.
We'll do the same for A-Basis:

```{r}
a_basis_pooled <- dat %>%
  basis_pooled_cv(strength, condition, batch, p = 0.99,
                  override = c("between_group_variability",
                               "normalized_variance_equal"))

a_basis_pooled
```

As we saw before, the returned object has a property called `basis`,
which is a `data.frame` for the pooling methods.

```{r}
a_basis_pooled$basis
```

We can take this `data.frame` and change the column names
to suit our needs.

```{r}
a_basis_pooled$basis %>%
  rename(condition = group,
         b_basis_pooled = value)
```

We can combine all these steps into one statement. We'll also ensure
that the conditions are listed in the order we want.

```{r}
a_basis_pooled_results <- a_basis_pooled$basis %>%
  rename(condition = group,
         a_basis_pooled = value) %>%
  mutate(condition = ordered(condition, c("CTD", "RTD", "ETW", "ETW2")))

a_basis_pooled_results
```

And the same thing for B-Basis:

```{r}
b_basis_pooled_results <- b_basis_pooled$basis %>%
  rename(condition = group,
         b_basis_pooled = value) %>%
  mutate(condition = ordered(condition, c("CTD", "RTD", "ETW", "ETW2")))

b_basis_pooled_results
```

We can use the function `inner_join` from the `dplyr` package to combine
the three sets of computational results. Each row for each condition
will be concatenated.

```{r}
single_point_results %>%
  inner_join(b_basis_pooled_results, by = "condition") %>%
  inner_join(a_basis_pooled_results, by = "condition")
```

To use this table in the plot we're trying to construct, we want to
"lengthen" the table as follows.

```{r}
single_point_results %>%
  inner_join(b_basis_pooled_results, by = "condition") %>%
  inner_join(a_basis_pooled_results, by = "condition") %>%
  pivot_longer(cols = single_point_b_basis:a_basis_pooled)
```


We can now make a plot based on this:

```{r}
single_point_results %>%
  inner_join(b_basis_pooled_results, by = "condition") %>%
  inner_join(a_basis_pooled_results, by = "condition") %>%
  pivot_longer(cols = single_point_b_basis:a_basis_pooled) %>%
  ggplot(aes(x = condition, y = value)) +
  geom_boxplot(aes(y = strength), data = dat) +
  geom_point(aes(shape = name, color = name)) +
  ggtitle("Property Graph") +
  theme_bw()
```


# Nested Data Plots

`cmstatr` contains the function `nested_data_plot`. This function creates a
plot showing the sources of variation. In the following example, the data
is grouped according to the variables in the `group` argument. The data
is first grouped according to `batch`, then according to `panel`. The
labels located according to the data points that fall under them.
By default, the mean is used, but that `stat` argument can be used to
locate the labels according to `median` or some other statistic.


```{r}
carbon.fabric.2 %>%
  mutate(panel = as.character(panel)) %>%
  filter(test == "WT") %>%
  nested_data_plot(strength,
                   groups = c(batch, panel))
```

Optionally, `fill` or `color` can be set as follows:

```{r}
carbon.fabric.2 %>%
  mutate(panel = as.character(panel)) %>%
  filter(test == "WT" & condition == "RTD") %>%
  nested_data_plot(strength,
                   groups = c(batch, panel),
                   fill = batch,
                   color = panel)
```

---
title: "Anderson-Darling k-Sample Test"
author: "Stefan Kloppenborg, Jeffrey Borlik"
date: "20-Jan-2019"
output: rmarkdown::html_vignette
bibliography: bibliography.json
csl: ieee.csl
vignette: >
  %\VignetteIndexEntry{Anderson-Darling k-Sample Test}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

This vignette explores the Anderson--Darling k-Sample test.
CMH-17-1G [@CMH-17-1G] provides a formulation for this test that appears different than the formulation given by Scholz and Stephens in their 1987 paper [@Stephens1987].

Both references use different nomenclature, which is summarized as follows:

Term                                               | CMH-17-1G             | Scholz and Stephens
---------------------------------------------------|-----------------------|---------------------
A sample                                           | $i$                   | $i$
The number of samples                              | $k$                   | $k$
An observation within a sample                     | $j$                   | $j$
The number of observations within the sample $i$   | $n_i$                 | $n_i$
The total number of observations within all samples| $n$                   | $N$
Distinct values in combined data, ordered          | $z_{(1)}$...$z_{(L)}$ | $Z_1^*$...$Z_L^*$
The number of distinct values in the combined data | $L$                   | $L$


Given the possibility of ties in the data, the discrete version of the test must be used
Scholz and Stephens (1987) give the test statistic as:

$$
A_{a k N}^2 = \frac{N - 1}{N}\sum_{i=1}^k \frac{1}{n_i}\sum_{j=1}^{L}\frac{l_j}{N}\frac{\left(N M_{a i j} - n_i B_{a j}\right)^2}{B_{a j}\left(N - B_{a j}\right) - N l_j / 4}
$$


CMH-17-1G gives the test statistic as:

$$
ADK = \frac{n - 1}{n^2\left(k - 1\right)}\sum_{i=1}^k\frac{1}{n_i}\sum_{j=1}^L h_j \frac{\left(n F_{i j} - n_i H_j\right)^2}{H_j \left(n - H_j\right) - n h_j / 4}
$$

By inspection, the CMH-17-1G version of this test statistic contains an extra factor of $\frac{1}{\left(k - 1\right)}$.

Scholz and Stephens indicate that one rejects $H_0$ at a significance level of $\alpha$ when:

$$
\frac{A_{a k N}^2 - \left(k - 1\right)}{\sigma_N} \ge t_{k - 1}\left(\alpha\right)
$$

This can be rearranged to give a critical value:

$$
A_{c r i t}^2 = \left(k - 1\right) + \sigma_N t_{k - 1}\left(\alpha\right)
$$

CHM-17-1G gives the critical value for $ADK$ for $\alpha=0.025$ as:

$$
ADC = 1 + \sigma_n \left(1.96 + \frac{1.149}{\sqrt{k - 1}} - \frac{0.391}{k - 1}\right)
$$

The definition of $\sigma_n$ from the two sources differs by a factor of $\left(k - 1\right)$.

The value in parentheses in the CMH-17-1G critical value corresponds to the interpolation formula for $t_m\left(\alpha\right)$ given in Scholz and Stephen's paper.
It should be noted that this is *not* the student's t-distribution, but rather a distribution referred to as the $T_m$ distribution.

The `cmstatr` package use the package `kSamples` to perform the k-sample Anderson--Darling tests.
This package uses the original formulation from Scholz and Stephens, so the test statistic will differ from that given software based on the CMH-17-1G formulation by a factor of $\left(k-1\right)$.

For comparison, [SciPy's implementation](https://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.anderson_ksamp.html) also
uses the original Scholz and Stephens formulation.  The statistic that it returns, however, is the normalized statistic, 
$\left[A_{a k N}^2 - \left(k - 1\right)\right] / \sigma_N$, rather than `kSamples`'s $A_{a k N}^2$ value.  To be consistent, SciPy also returns the critical values $t_{k-1}(\alpha)$ directly.  (Currently, SciPy also floors/caps the returned p-value at 0.1% / 25%.)  The values of $k$ and $\sigma_N$ are available in `cmstatr`'s `ad_ksample` return value, if an exact comparison to Python SciPy is necessary.

The conclusions about the null hypothesis drawn, however, will be the same, whether R or CMH-17-1G or SciPy.

# References

---
title: "cmstatr Tutorial"
author: "Stefan Kloppenborg"
date: "1-Apr-2020"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{cmstatr Tutorial}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

# If any of the required packages are unavailable,
# don't re-run the code
# nolint start
required <- c("dplyr", "ggplot2", "tidyr", "cmstatr", "purrr")
if (!all(unlist(lapply(required, function(pkg) {
    requireNamespace(pkg, quietly = TRUE)}
  )))) {
  knitr::opts_chunk$set(eval = FALSE)
}
#nolint end
```

`cmstatr` is an R package for analyzing composite material data for use in the
aerospace industry. The statistical methods are based on those published in
[CMH-17-1G](https://www.cmh17.org/). This package is intended to facilitate
reproducible statistical analysis of composite materials. In this tutorial,
we'll explore the basic functionality of `cmstatr`.

Before we can actually use the package, we'll need to load it. We'll also load
the `dplyr` package, which we'll talk about shortly. There are also a few other
packages that we'll load. These could all be loaded by loading the
`tidyverse` package instead.

```{r message=FALSE}
library(cmstatr)
library(dplyr)
library(ggplot2)
library(tidyr)
library(purrr)
```

# Input Data
`cmstatr` is built with the assumption that the data is in (so called)
[tidy data](http://vita.had.co.nz/papers/tidy-data.html) format. This means
that the data is in a data frame and that each observation (i.e. test result)
has its own row and that each variable has its own column. Included in this
package is a sample composite material data set (this data set is fictional:
don't use it for anything other than learning this package). The data set
`carbon.fabric.2` has the expected format. We'll just show the first 10 rows
of the data for now.

```{r}
carbon.fabric.2 %>%
  head(10)
```

If your data set is not yet in this type of format (note: that the column
names *do not* need to match the column names in the example), there are
many ways to get it into this format. One of the easier ways of doing so
is to use the [`tidyr`](https://tidyr.tidyverse.org/) package. The use of this
package is outside the scope of this vignette.

# Working With Data
Throughout this vignette, we will be using some of the `tidyverse` tools for
working with data. There are several ways to work with data in R, but in the
opinion of the author of this vignette, the `tidyverse` provides the easiest
way to do so. As such, this is the approach used in this vignette. Feel free
to use whichever approach works best for you.

# Normalizing Data to Cured Ply Thickness
Very often, you'll want to normalize as-measured strength data to a nominal
cured ply thickness for fiber-dominated properties. Very often, this will
reduce the apparent variance in the data. The `normalize_ply_thickness`
function can be used to normalize strength or modulus data to a certain
cured ply thickness. This function takes three arguments: the value to
normalize (i.e.. strength or modulus), the measured thickness and the
nominal thickness. In our case, the nominal cured ply thickness of the
material is $0.0079$. We can then normalize the warp-tension and
fill-compression data as follows:

```{r}
norm_data <- carbon.fabric.2 %>%
  filter(test == "WT" | test == "FC") %>%
  mutate(strength.norm = normalize_ply_thickness(strength,
                                                 thickness / nplies,
                                                 0.0079))

norm_data %>%
  head(10)
```

# Calculating Single-Point Basis Value
The simplest thing that you will likely do is to calculate a basis value based
of a set of numbers that you consider as unstructured data. An example of this
would be calculating the B-Basis of the `RTD` warp tension (`WT`) data.

There are a number of diagnostic tests that we should run before actually
calculating a B-Basis value. We'll talk about those later, but for now, let's
just get right to checking how the data are distributed and calculating the
B-Basis.

We'll use an Anderson--Darling test to check if the data are normally
distributed. The `cmstatr` package provides the function
`anderson_darling_normal` and related functions for other distributions.
We can run an Anderson--Darling test for normality on the warp tension RTD
data as follows. We'll perform this test on the normalized strength.

```{r}
norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  anderson_darling_normal(strength.norm)
```

```{r include=FALSE}
# Verify that the AD test always provides the same conclusion
# If this assertion fails, the Vignette needs to be re-written
if (0.05 >= (norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  anderson_darling_normal(strength.norm))$osl) {
  stop("Unexpected vale for Anderson-Darling test")
  }
```

Now that we know that this data follows a normal distribution (since the
observed significance level (OSL) of the Anderson--Darling test is
greater than $0.05$), we can
proceed to calculate a basis value based based on the assumption of normally
distributed data. The `cmstatr` package provides the function `basis_normal`
as well as related functions for other distributions. By default, the B-Basis
value is calculated, but other population proportions and confidence bounds
can be specified (for example, specify `p = 0.99, conf = 0.99` for A-Basis).

```{r}
norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  basis_normal(strength.norm)
```

We see that the calculated B-Basis is $129.96$. We also see two messages
issued by the `cmstatr` package. These messages relate to the automated
diagnostic tests performed by the basis calculation functions. In this case
we see messages that two of the diagnostic tests were not performed because
we didn't specify the batch of each observation. The batch is not required
for calculating single-point basis values, but it is required for performing
batch-to-batch variability and within-batch outlier diagnostic tests.

The `basis_normal` function performs the following diagnostic tests by default:

- Within batch outliers using `maximum_normed_residual()`
- Between batch variability using `ad_ksample()`
- Outliers using `maximum_normed_residual()`
- Normality of data using `anderson_darling_normal()`

There are two ways that we can deal with the two messages that we see. We can
pass in a column that specifies the batch for each observation, or we can
override those two diagnostic tests so that `cmstatr` doesn't run them.

To override the two diagnostic tests, we set the argument `override` to a list
of the names of the diagnostic tests that we want to skip. The names of the
diagnostic tests that were not run are shown between back-ticks (\`) in the
message. Our call to `basis_normal()` would be updated as follows:

```{r}
norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  basis_normal(strength.norm, 
               override = c("outliers_within_batch",
                            "between_batch_variability"))
```

Obviously, you should be cautious about overriding the diagnostic tests.
There are certainly times when it is appropriate to do so, but sound
engineering judgment is required.

The better approach would be to specify the batch. This can be done as
follows. We'll store the result in the variable `b_basis_wt_rtd` for reasons
that will become clear later.

```{r}
b_basis_wt_rtd <- norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  basis_normal(strength.norm, batch)
```

Now that batch is specified, we see that one of the diagnostic tests
actually fails: the Anderson--Darling k-Sample test shows that the batches
are not drawn from the same (unspecified) distribution. We can interrogate
the failing test by accessing the `diagnostic_obj` element of the return
value from `basis_normal()`. This contains elements for each of the diagnostic
tests. We can access the `between_batch_variability` result as follows:

```{r}
b_basis_wt_rtd$diagnostic_obj$between_batch_variability
```

We could have also run the failing diagnostic test directly as follows:

```{r}
norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  ad_ksample(strength.norm, batch)
```

For the Anderson--Darling k-Sample test, $\alpha=0.025$ is normally used.
In this case the p-value is $p=0.0026$, so it is no where near $\alpha$
(note the number of decimal places).

We can plot the distribution of this data and make a judgment call about
whether to continue.


```{r}
norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  group_by(batch) %>%
  ggplot(aes(x = strength.norm, color = batch)) +
  stat_normal_surv_func() +
  stat_esf() +
  ggtitle("Distribution of Data For Each Batch")
```

We can also run the other diagnostic test by themselves. These are described
in more detail in the following sections.

# Calculating Basis Values by Pooling Across Environments
In this section, we'll use the fill-compression data from the `carbon.fabric.2`
data set.

## Checking for Outliers
After checking that there are a sufficient number of conditions, batches and
specimens and that the failure modes are consistent, we would normally
check if there are outliers within each batch and condition. The maximum
normed residual test can be used for this. The `cmstatr` package provides the
function `maximum_normed_residual` to do this. First, we'll group the data
by condition and batch, then run the test on each group. The
`maximum_normed_residual` function returns an object that contains a number
of values. We'll create a `data.frame` that contains those values.

In order to do this, we need to use the `nest` function from the `tidyr`
package. This is explained in detail
[here](https://tidyr.tidyverse.org/articles/nest.html). Basically,
`nest` allows a column of `list`s or a column of `data.frame`s to be
added to a `data.frame`. Once nested, we can use the `glance` method
to unpack the values returned by `maximum_normed_residual` into a
one-row `data.frame`, and then use `unnest` to flatten this into a
single `data.frame`.


```{r}
norm_data %>%
  filter(test == "FC") %>%
  group_by(condition, batch) %>%
  nest() %>%
  mutate(mnr = map(data,
                   ~maximum_normed_residual(data = .x, x = strength.norm)),
         tidied = map(mnr, glance)) %>%
  select(-c(mnr, data)) %>%  # remove unneeded columns
  unnest(tidied)
```

```{r include=FALSE}
if ((norm_data %>%
  filter(test == "FC") %>%
  group_by(condition, batch) %>%
  summarise(
    n_outliers = maximum_normed_residual(x = strength.norm)$n_outliers
    ) %>%
  ungroup() %>%
  summarise(n_outliers = sum(n_outliers)))[[1]] != 0) {
  stop("Unexpected number of outliers")
  }
```

None of the groups have outliers, so we can continue.

# Batch-to-Batch Distribution
Next, we will use the Anderson--Darling k-Sample test to check that each batch
comes from the same distribution within each condition. We can use the
`ad_ksample` function from `cmstatr` to do so. Once again, we'll use
`nest`/`unnest` and `glance` to do so.

```{r}
norm_data %>%
  filter(test == "FC") %>%
  group_by(condition) %>%
  nest() %>%
  mutate(adk = map(data, ~ad_ksample(data = .x,
                                     x = strength.norm,
                                     groups = batch)),
         tidied = map(adk, glance)) %>%
  select(-c(data, adk)) %>%  # remove unneeded columns
  unnest(tidied)
```

```{r include=FALSE}
if (!all(!(norm_data %>%
  filter(test == "FC") %>%
  group_by(condition) %>%
  summarise(different_dist =
           ad_ksample(x = strength.norm, groups = batch)$reject_same_dist
  ))$different_dist)) {
  stop("Unexpected ADK result")
  }
```

For all conditions, the Anderson--Darling k-Sample test fails to reject the
hypothesis that each batch comes from the same (unspecified) distribution.
We can thus proceed to pooling the data.

## Checking for Outliers Within Each Condition
Just as we did when checking for outlier within each condition and each
batch, we can pool all the batches (within each condition) and check
for outliers within each condition.

```{r}
norm_data %>%
  filter(test == "FC") %>%
  group_by(condition) %>%
  nest() %>%
  mutate(mnr = map(data, ~maximum_normed_residual(data = .x,
                                                  x = strength.norm)),
         tidied = map(mnr, glance)) %>%
  select(-c(mnr, data)) %>%  # remove unneeded columns
  unnest(tidied)
```

```{r include=FALSE}
if ((norm_data %>%
  filter(test == "FC") %>%
  group_by(condition) %>%
  summarise(
    n_outliers = maximum_normed_residual(x = strength.norm)$n_outliers
    ) %>%
  ungroup() %>%
  summarise(n_outliers = sum(n_outliers)))[[1]] != 0) {
  stop("Unexpected number of outliers")
  }
```

We find no outliers, so we can continue.

## Investigation Conditions
When multiple conditions were tested, it's usually useful to view some basic
summary statistics for each condition before proceeding. The `condition_summary`
function can be used for this. You can pass a `data.frame` with the data and
the name of the condition variable to generate such summary statistics.

```{r}
norm_data %>%
  filter(test == "FC") %>%
  condition_summary(strength.norm, condition, "RTD")
```

## Pooling Across Environments
Often it is desirable to pool data across several environments. There
are two methods for doing so: "pooled standard deviation" and
"pooled CV" (CV is an abbreviation for Coefficient of Variation).

First, we will check for
equality of variance among the conditions. We will do so using Levene's test.
The `cmstatr` package provides the function `levene_test` to do so.

```{r}
norm_data %>%
  filter(test == "FC") %>%
  levene_test(strength.norm, condition)
```

```{r include=FALSE}
if (!(norm_data %>%
  filter(test == "FC") %>%
  levene_test(strength.norm, condition))$reject_equal_variance) {
  stop("Unexpected result from Levene's test")
  }
```

The result from Levene's test indicates that the variance for each condition
is not equal. This indicates that the data cannot be pooled using the
"pooled standard deviation" method.

We can check if the data can be pooled using the "pooled CV" method.
We'll start by normalizing the data from each group to the group's mean.
The `cmstatr` package provides the function `normalize_group_mean` for
this purpose.

```{r}
norm_data %>%
  filter(test == "FC") %>%
  mutate(
    strength_norm_group = normalize_group_mean(strength.norm, condition)) %>%
  levene_test(strength_norm_group, condition)
```

```{r include=FALSE}
if ((norm_data %>%
  filter(test == "FC") %>%
  mutate(
    strength_norm_group = normalize_group_mean(strength.norm, condition)) %>%
  levene_test(strength_norm_group, condition))$reject_equal_variance) {
  stop("Unexpected value from Levene's test")
  }
```

The Levene's test thus shows the variances of the pooled data are equal.
We can move on to performing an Anderson--Darling test for normality on
the pooled data.

```{r}
norm_data %>%
  filter(test == "FC") %>%
  mutate(
    strength_norm_group = normalize_group_mean(strength.norm, condition)) %>%
  anderson_darling_normal(strength_norm_group)
```

```{r include=FALSE}
if ((norm_data %>%
  filter(test == "FC") %>%
  mutate(
    strength_norm_group = normalize_group_mean(strength.norm, condition)) %>%
  anderson_darling_normal(strength_norm_group))$osl <= 0.05) {
  stop("Unexpected value from AD test")
  }
```

The Anderson--Darling test indicates that the pooled data is drawn from
a normal distribution, so we can continue with calculating basis values
using the "pooled CV" method.

```{r}
norm_data %>%
  filter(test == "FC") %>%
  basis_pooled_cv(strength.norm, condition, batch)
```

The conditions listed in the output above are in alphabetical order. This
probably isn't what you want. Instead, you probably want the conditions
listed in a certain order. This can be done by ordering the data first as
demonstrated below. You're probably just do this one in at the start of
your analysis. In the example below, we'll store the result in the variable
`basis_res` before printing it.

```{r}
basis_res <- norm_data %>%
  mutate(condition = ordered(condition,
                             c("CTD", "RTD", "ETD", "ETW", "ETW2"))) %>%
  filter(test == "FC") %>%
  basis_pooled_cv(strength.norm, condition, batch)
basis_res
```

The summary statistics that we computed for each condition earlier can also
be generated using the `basis` object returned by `basis_pooled_cv` and related
functions.

```{r}
basis_res %>%
  condition_summary("RTD")
```



# Equivalency
Eventually, once you've finished calculating all your basis values,
you'll probably want to set specification requirements or evaluate
site/process equivalency. `cmstatr` has functionality to do both.

Let's say that you want to develop specification limits for fill compression
that you're going to put in your material specification. You can do this
as follows:

```{r}
carbon.fabric.2 %>%
  filter(test == "FC" & condition == "RTD") %>%
  equiv_mean_extremum(strength, n_sample = 5, alpha = 0.01)
```

If you're determining equivalency limits for modulus, a different
approach is generally used so that bilateral limits are set. `cmstatr`
can do this as well, using the function `equiv_change_mean`.

---
title: "cmstatr Validation"
author: "Stefan Kloppenborg"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 2
vignette: >
  %\VignetteIndexEntry{cmstatr Validation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
csl: ieee.csl
references:
- id: "ASAP2008"
  type: "report"
  number: "ASAP-2008"
  author:
  - given: K.S.
    family: Raju
  - given: J.S.
    family: Tomblin
  title: "AGATE Statistical Analysis Program"
  issued:
    year: 2008
  publisher: Wichita State University
- id: Stephens1987
  type: article
  author:
    - given: F.W.
      family: Scholz
    - given: M.A,
      family: Stephens
  title: K-Sample Anderson-Darling Tests
  container-title: Journal of the American Statistical Association
  volume: "82"
  issue: "399"
  page: 918-924
  issued:
    year: "1987"
    month: "09"
  DOI: 10.1080/01621459.1987.10478517
  URL: https://doi.org/10.1080/01621459.1987.10478517
- id: "CMH-17-1G"
  type: report
  number: "CMH-17-1G"
  title: Composite Materials Handbook, Volume 1. Polymer Matrix Composites
    Guideline for Characterization of Structural Materials
  publisher: SAE International
  issued:
    year: "2012"
    month: "03"
- id: "STAT-17"
  type: report
  number: "STAT-17 Rev 5"
  author:
    - literal: Materials Sciences Corporation
  title: CMH-17 Statistical Analysis for B-Basis and A-Basis Values
  publisher: Materials Sciences Corporation
  publisher-place: Horsham, PA
  issued:
    year: "2008"
    month: "01"
    day: "08"
- id: Vangel1994
  type: article
  author:
    - given: Mark
      family: Vangel
  title: One-Sided Nonparametric Tolerance Limits
  container-title: Communications in Statistics - Simulation and Computation
  volume: "23"
  issue: "4"
  page: 1137-1154
  issued:
    year: "1994"
  DOI: 10.1080/03610919408813222
- id: vangel_lot_2002
  type: article
  author:
    - given: Mark
      family: Vangel
  title: Lot Acceptance and Compliance Testing Using the Sample Mean and an Extremum
  container-title: Technometrics
  volume: "44"
  issue: "3"
  page: 242--249
  issued:
    year: 2002
    month: 8
  DOI: 10.1198/004017002188618428

---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# 1. Introduction

This vignette is intended to contain the same validation that is included
in the test suite within the `cmstatr` package, but in a format that is
easier for a human to read. The intent is that this vignette will include
only those validations that are included in the test suite, but that the
test suite may include more tests than are shown in this vignette.

The following packages will be used in this validation. The version of
each package used is listed at the end of this vignette.

```{r message=FALSE, warning=FALSE}
library(cmstatr)
library(dplyr)
library(purrr)
library(tidyr)
library(testthat)
```

Throughout this vignette, the `testthat` package will be used. Expressions
such as `expect_equal` are used to ensure that the two values are equal
(within some tolerance). If this expectation is not true, the vignette will
fail to build. The tolerance is a relative tolerance: a tolerance of 0.01
means that the two values must be within $1\%$ of each other.
As an example, the following expression checks that the value
`10` is equal to `10.1` within a tolerance of `0.01`. Such an expectation
should be satisfied.


```{r}
expect_equal(10, 10.1, tolerance = 0.01)
```

The `basis_...` functions automatically perform certain diagnostic tests.
When those diagnostic tests are not relevant to the validation, the
diagnostic tests are overridden by passing the argument `override = "all"`.

# 2. Validation Table
The following table provides a cross-reference between the various
functions of the `cmstatr` package and the tests shown within this
vignette. The sections in this vignette are organized by data set.
Not all checks are performed on all data sets.


Function                        | Tests
--------------------------------|---------------------------
`ad_ksample()`                  | [Section 3.1](#cf-ad), [Section 4.1.2](#c11-adk), [Section 6.1](#cl-adk)
`anderson_darling_normal()`     | [Section 4.1.3](#c11-ad), [Section 5.1](#ods-adn)
`anderson_darling_lognormal()`  | [Section 4.1.3](#c11-ad), [Section 5.2](#ods-adl)
`anderson_darling_weibull()`    | [Section 4.1.3](#c11-ad), [Section 5.3](#ods-adw)
`basis_normal()`                | [Section 5.4](#ods-nb)
`basis_lognormal()`             | [Section 5.5](#ods-lb)
`basis_weibull()`               | [Section 5.6](#ods-wb)
`basis_pooled_cv()`             | [Section 4.2.3](#c12-pcv), [Section 4.2.4](#c12-pcvmcv),
`basis_pooled_sd()`             | [Section 4.2.1](#c12-psd), [Section 4.2.2](#c12-psdmcv)
`basis_hk_ext()`                | [Section 4.1.6](#c11-hk-wf), [Section 5.7](#ods-hkb), [Section 5.8](#ods-hkb2)
`basis_nonpara_large_sample()`  | [Section 5.9](#ods-lsnb)
`basis_anova()`                 | [Section 4.1.7](#c11-anova)
`calc_cv_star()`                |
`cv()`                          |
`equiv_change_mean()`           | [Section 5.11](#ods-ecm)
`equiv_mean_extremum()`         | [Section 5.10](#ods-eme)
`hk_ext_z()`                    | [Section 7.3](#pf-hk), [Section 7.4](#pf-hk2)
`hk_ext_z_j_opt()`              | [Section 7.5](#pf-hk-opt)
`k_equiv()`                     | [Section 7.8](#pf-equiv)
`k_factor_normal()`             | [Section 7.1](#pf-kb), [Section 7.2](#pf-ka)
`levene_test()`                 | [Section 4.1.4](#c11-lbc), [Section 4.1.5](#c11-lbb)
`maximum_normed_residual()`     | [Section 4.1.1](#c11-mnr)
`nonpara_binomial_rank()`       | [Section 7.6](#pf-npbinom), [Section 7.7](#pf-npbinom2)
`normalize_group_mean()`        |
`normalize_ply_thickness()`     |
`transform_mod_cv_ad()`         |
`transform_mod_cv()`            |


# 3. `carbon.fabric` Data Set

This data set is example data that is provided with `cmstatr`.
The first few rows of this data are shown below.

```{r}
head(carbon.fabric)
```


## 3.1. Anderson--Darling k-Sample Test {#cf-ad}

This data was entered into ASAP 2008 [@ASAP2008] and the reported
Anderson--Darling k--Sample test statistics were recorded, as were
the conclusions.

The value of the test statistic reported by `cmstatr` and that reported
by ASAP 2008 differ by a factor of $k - 1$, as do the critical values
used. As such, the conclusion of the tests are identical. This is
described in more detail in the
[Anderson--Darling k--Sample Vignette](adktest.html).

When the RTD warp-tension data from this data set is entered into ASAP 2008,
it reports a test statistic of 0.456 and fails to reject the null hypothesis
that the batches are drawn from the same distribution. Adjusting for the
different definition of the test statistic, the results given by `cmstatr`
are very similar.


```{r}
res <- carbon.fabric %>%
  filter(test == "WT") %>%
  filter(condition == "RTD") %>%
  ad_ksample(strength, batch)

expect_equal(res$ad / (res$k - 1), 0.456, tolerance = 0.002)
expect_false(res$reject_same_dist)

res
```

When the ETW warp-tension data from this data set are entered into ASAP 2008,
the reported test statistic is 1.604 and it fails to reject the null
hypothesis that the batches are drawn from the same distribution. Adjusting
for the different definition of the test statistic, `cmstatr` gives nearly
identical results.

```{r}
res <- carbon.fabric %>%
  filter(test == "WT") %>%
  filter(condition == "ETW") %>%
  ad_ksample(strength, batch)

expect_equal(res$ad / (res$k - 1), 1.604, tolerance = 0.002)
expect_false(res$reject_same_dist)

res
```


# 4. Comparison with Examples from CMH-17-1G
## 4.1 Dataset From Section 8.3.11.1.1

CMH-17-1G [@CMH-17-1G] provides an example data set and results from
ASAP [@ASAP2008] and STAT17 [@STAT-17]. This example data set is duplicated
below:

```{r}
dat_8_3_11_1_1 <- tribble(
  ~batch, ~strength, ~condition,
  1, 118.3774604, "CTD", 1, 84.9581364, "RTD", 1, 83.7436035, "ETD",
  1, 123.6035612, "CTD", 1, 92.4891822, "RTD", 1, 84.3831677, "ETD",
  1, 115.2238092, "CTD", 1, 96.8212659, "RTD", 1, 94.8030433, "ETD",
  1, 112.6379744, "CTD", 1, 109.030325, "RTD", 1, 94.3931537, "ETD",
  1, 116.5564277, "CTD", 1, 97.8212659, "RTD", 1, 101.702222, "ETD",
  1, 123.1649896, "CTD", 1, 100.921519, "RTD", 1, 86.5372121, "ETD",
  2, 128.5589027, "CTD", 1, 103.699444, "RTD", 1, 92.3772684, "ETD",
  2, 113.1462103, "CTD", 2, 93.7908212, "RTD", 2, 89.2084024, "ETD",
  2, 121.4248107, "CTD", 2, 107.526709, "RTD", 2, 100.686001, "ETD",
  2, 134.3241906, "CTD", 2, 94.5769704, "RTD", 2, 81.0444192, "ETD",
  2, 129.6405117, "CTD", 2, 93.8831373, "RTD", 2, 91.3398070, "ETD",
  2, 117.9818658, "CTD", 2, 98.2296605, "RTD", 2, 93.1441939, "ETD",
  3, 115.4505226, "CTD", 2, 111.346590, "RTD", 2, 85.8204168, "ETD",
  3, 120.0369467, "CTD", 2, 100.817538, "RTD", 3, 94.8966273, "ETD",
  3, 117.1631088, "CTD", 3, 100.382203, "RTD", 3, 95.8068520, "ETD",
  3, 112.9302797, "CTD", 3, 91.5037811, "RTD", 3, 86.7842252, "ETD",
  3, 117.9114501, "CTD", 3, 100.083233, "RTD", 3, 94.4011973, "ETD",
  3, 120.1900159, "CTD", 3, 95.6393615, "RTD", 3, 96.7231171, "ETD",
  3, 110.7295966, "CTD", 3, 109.304779, "RTD", 3, 89.9010384, "ETD",
  3, 100.078562, "RTD", 3, 99.1205847, "RTD", 3, 89.3672306, "ETD",
  1, 106.357525, "ETW", 1, 99.0239966, "ETW2",
  1, 105.898733, "ETW", 1, 103.341238, "ETW2",
  1, 88.4640082, "ETW", 1, 100.302130, "ETW2",
  1, 103.901744, "ETW", 1, 98.4634133, "ETW2",
  1, 80.2058219, "ETW", 1, 92.2647280, "ETW2",
  1, 109.199597, "ETW", 1, 103.487693, "ETW2",
  1, 61.0139431, "ETW", 1, 113.734763, "ETW2",
  2, 99.3207107, "ETW", 2, 108.172659, "ETW2",
  2, 115.861770, "ETW", 2, 108.426732, "ETW2",
  2, 82.6133082, "ETW", 2, 116.260375, "ETW2",
  2, 85.3690411, "ETW", 2, 121.049610, "ETW2",
  2, 115.801622, "ETW", 2, 111.223082, "ETW2",
  2, 44.3217741, "ETW", 2, 104.574843, "ETW2",
  2, 117.328077, "ETW", 2, 103.222552, "ETW2",
  2, 88.6782903, "ETW", 3, 99.3918538, "ETW2",
  3, 107.676986, "ETW", 3, 87.3421658, "ETW2",
  3, 108.960241, "ETW", 3, 102.730741, "ETW2",
  3, 116.122640, "ETW", 3, 96.3694916, "ETW2",
  3, 80.2334815, "ETW", 3, 99.5946088, "ETW2",
  3, 106.145570, "ETW", 3, 97.0712407, "ETW2",
  3, 104.667866, "ETW",
  3, 104.234953, "ETW"
)
dat_8_3_11_1_1
```

### 4.1.1 Maximum Normed Residual Test {#c11-mnr}
CMH-17-1G Table 8.3.11.1.1(a) provides results of the MNR test from
ASAP for this data set. Batches 2 and 3 of the ETW data is considered
here and the results of `cmstatr` are compared with those published in
CMH-17-1G.

For Batch 2 of the ETW data, the results match those published in the
handbook within a small tolerance. The published test statistic is 2.008.

```{r}
res <- dat_8_3_11_1_1 %>%
  filter(condition == "ETW" & batch == 2) %>%
  maximum_normed_residual(strength, alpha = 0.05)

expect_equal(res$mnr, 2.008, tolerance = 0.001)
expect_equal(res$crit, 2.127, tolerance = 0.001)
expect_equal(res$n_outliers, 0)

res
```

Similarly, for Batch 3 of the ETW data, the results of `cmstatr` match
the results published in the handbook within a small tolerance. The published
test statistic is 2.119

```{r}
res <- dat_8_3_11_1_1 %>%
  filter(condition == "ETW" & batch == 3) %>%
  maximum_normed_residual(strength, alpha = 0.05)

expect_equal(res$mnr, 2.119, tolerance = 0.001)
expect_equal(res$crit, 2.020, tolerance = 0.001)
expect_equal(res$n_outliers, 1)

res
```


### 4.1.2 Anderson--Darling k--Sample Test {#c11-adk}

For the ETW condition, the ADK test statistic given in [@CMH-17-1G] is
$ADK = 0.793$ and the test concludes that the samples come from the
same distribution. Noting that `cmstatr` uses the definition of the
test statistic given in [@Stephens1987], so the test statistic given
by `cmstatr` differs from that given by ASAP by a factor of $k - 1$,
as described in the [Anderson--Darling k--Sample Vignette](adktest.html).

```{r}
res <- dat_8_3_11_1_1 %>%
  filter(condition == "ETW") %>%
  ad_ksample(strength, batch)

expect_equal(res$ad / (res$k - 1), 0.793, tolerance = 0.003)
expect_false(res$reject_same_dist)

res
```

Similarly, for the ETW2 condition, the test statistic given in [@CMH-17-1G]
is $ADK = 3.024$ and the test concludes that the samples come from different
distributions. This matches `cmstatr`

```{r}
res <- dat_8_3_11_1_1 %>%
  filter(condition == "ETW2") %>%
  ad_ksample(strength, batch)

expect_equal(res$ad / (res$k - 1), 3.024, tolerance = 0.001)
expect_true(res$reject_same_dist)

res
```

### 4.1.3 Anderson--Darling Tests for Distribution {#c11-ad}

CMH-17-1G Section 8.3.11.2.1 contains results from STAT17 for
the "observed significance level" from the Anderson--Darling test
for various distributions. In this section, the ETW condition from the
present data set is used. The published results are given in the 
following table. The results from `cmstatr` are below and are very
similar to those from STAT17.

Distribution | OSL
-------------|------------
Normal       | 0.006051
Lognormal    | 0.000307
Weibull      | 0.219

```{r}
res <- dat_8_3_11_1_1 %>%
  filter(condition == "ETW") %>%
  anderson_darling_normal(strength)
expect_equal(res$osl, 0.006051, tolerance = 0.001)
res
```
```{r}
res <- dat_8_3_11_1_1 %>%
  filter(condition == "ETW") %>%
  anderson_darling_lognormal(strength)
expect_equal(res$osl, 0.000307, tolerance = 0.001)
res
```

```{r}
res <- dat_8_3_11_1_1 %>%
  filter(condition == "ETW") %>%
  anderson_darling_weibull(strength)
expect_equal(res$osl, 0.0219, tolerance = 0.002)
res
```


### 4.1.4 Levene's Test (Between Conditions) {#c11-lbc}

CMH-17-1G Section 8.3.11.1.1 provides results from ASAP for Levene's test
for equality of variance between conditions after the ETW and ETW2 conditions
are removed. The handbook shows an F statistic of 0.58, however if this
data is entered into ASAP directly, ASAP gives an F statistic of 0.058,
which matches the result of `cmstatr`.

```{r}
res <- dat_8_3_11_1_1 %>%
  filter(condition != "ETW" & condition != "ETW2") %>%
  levene_test(strength, condition)
expect_equal(res$f, 0.058, tolerance = 0.01)
res
```

### 4.1.5 Levene's Test (Between Batches) {#c11-lbb}

CMH-17-1G Section 8.3.11.2.2 provides output from STAT17. The
ETW2 condition from the present data set was analyzed by STAT17
and that software reported an F statistic of 0.123 from Levene's
test when comparing the variance of the batches within this condition.
The result from `cmstatr` is similar.

```{r}
res <- dat_8_3_11_1_1 %>%
  filter(condition == "ETW2") %>%
  levene_test(strength, batch)
expect_equal(res$f, 0.123, tolerance = 0.005)
res
```

Similarly, the published value of the F statistic for the CTD condition is 
$3.850$. `cmstatr` produces very similar results.

```{r}
res <- dat_8_3_11_1_1 %>%
  filter(condition == "CTD") %>%
  levene_test(strength, batch)
expect_equal(res$f, 3.850, tolerance = 0.005)
res
```


### 4.1.6 Nonparametric Basis Values {#c11-hk-wf}
CMH-17-1G Section 8.3.11.2.1 provides STAT17 outputs for the ETW
condition of the present data set. The nonparametric Basis values are listed.
In this case, the Hanson--Koopmans method is used. The published A-Basis
value is 13.0 and the B-Basis is 37.9.


```{r}
res <- dat_8_3_11_1_1 %>%
  filter(condition == "ETW") %>%
  basis_hk_ext(strength, method = "woodward-frawley", p = 0.99, conf = 0.95,
               override = "all")

expect_equal(res$basis, 13.0, tolerance = 0.001)

res
```


```{r}
res <- dat_8_3_11_1_1 %>%
  filter(condition == "ETW") %>%
  basis_hk_ext(strength, method = "optimum-order", p = 0.90, conf = 0.95,
               override = "all")

expect_equal(res$basis, 37.9, tolerance = 0.001)

res
```


### 4.1.7 Single-Point ANOVA Basis Value {#c11-anova}
CMH-17-1G Section 8.3.11.2.2 provides output from STAT17 for
the ETW2 condition from the present data set. STAT17 reports
A- and B-Basis values based on the ANOVA method of 34.6 and
63.2, respectively. The results from `cmstatr` are similar.

```{r}
res <- dat_8_3_11_1_1 %>%
  filter(condition == "ETW2") %>%
  basis_anova(strength, batch, override = "number_of_groups",
              p = 0.99, conf = 0.95)
expect_equal(res$basis, 34.6, tolerance = 0.001)
res
```

```{r}
res <- dat_8_3_11_1_1 %>%
  filter(condition == "ETW2") %>%
  basis_anova(strength, batch, override = "number_of_groups")
expect_equal(res$basis, 63.2, tolerance = 0.001)
res
```

## 4.2 Dataset From Section 8.3.11.1.2

[@CMH-17-1G] provides an example data set and results from
ASAP [@ASAP2008]. This example data set is duplicated
below:

```{r}
dat_8_3_11_1_2 <- tribble(
  ~batch, ~strength, ~condition,
  1, 79.04517, "CTD", 1, 103.2006, "RTD", 1, 63.22764, "ETW", 1, 54.09806, "ETW2",
  1, 102.6014, "CTD", 1, 105.1034, "RTD", 1, 70.84454, "ETW", 1, 58.87615, "ETW2",
  1, 97.79372, "CTD", 1, 105.1893, "RTD", 1, 66.43223, "ETW", 1, 61.60167, "ETW2",
  1, 92.86423, "CTD", 1, 100.4189, "RTD", 1, 75.37771, "ETW", 1, 60.23973, "ETW2",
  1, 117.218,  "CTD", 2, 85.32319, "RTD", 1, 72.43773, "ETW", 1, 61.4808,  "ETW2",
  1, 108.7168, "CTD", 2, 92.69923, "RTD", 1, 68.43073, "ETW", 1, 64.55832, "ETW2",
  1, 112.2773, "CTD", 2, 98.45242, "RTD", 1, 69.72524, "ETW", 2, 57.76131, "ETW2",
  1, 114.0129, "CTD", 2, 104.1014, "RTD", 2, 66.20343, "ETW", 2, 49.91463, "ETW2",
  2, 106.8452, "CTD", 2, 91.51841, "RTD", 2, 60.51251, "ETW", 2, 61.49271, "ETW2",
  2, 112.3911, "CTD", 2, 101.3746, "RTD", 2, 65.69334, "ETW", 2, 57.7281,  "ETW2",
  2, 115.5658, "CTD", 2, 101.5828, "RTD", 2, 62.73595, "ETW", 2, 62.11653, "ETW2",
  2, 87.40657, "CTD", 2, 99.57384, "RTD", 2, 59.00798, "ETW", 2, 62.69353, "ETW2",
  2, 102.2785, "CTD", 2, 88.84826, "RTD", 2, 62.37761, "ETW", 3, 61.38523, "ETW2",
  2, 110.6073, "CTD", 3, 92.18703, "RTD", 3, 64.3947,  "ETW", 3, 60.39053, "ETW2",
  3, 105.2762, "CTD", 3, 101.8234, "RTD", 3, 72.8491,  "ETW", 3, 59.17616, "ETW2",
  3, 110.8924, "CTD", 3, 97.68909, "RTD", 3, 66.56226, "ETW", 3, 60.17616, "ETW2",
  3, 108.7638, "CTD", 3, 101.5172, "RTD", 3, 66.56779, "ETW", 3, 46.47396, "ETW2",
  3, 110.9833, "CTD", 3, 100.0481, "RTD", 3, 66.00123, "ETW", 3, 51.16616, "ETW2",
  3, 101.3417, "CTD", 3, 102.0544, "RTD", 3, 59.62108, "ETW",
  3, 100.0251, "CTD",                     3, 60.61167, "ETW",
                                          3, 57.65487, "ETW",
                                          3, 66.51241, "ETW",
                                          3, 64.89347, "ETW",
                                          3, 57.73054, "ETW",
                                          3, 68.94086, "ETW",
                                          3, 61.63177, "ETW"
)
```


### 4.2.1 Pooled SD A- and B-Basis {#c12-psd}
CMH-17-1G Table 8.3.11.2(k) provides outputs from ASAP for the data set
above. ASAP uses the pooled SD method. ASAP produces the following
results, which are quite similar to those produced by `cmstatr`.

Condition | CTD   | RTD   | ETW   | ETW2
----------|-------|-------|-------|------
B-Basis   | 93.64 | 87.30 | 54.33 | 47.12
A-Basis   | 89.19 | 79.86 | 46.84 | 39.69

```{r}

res <- basis_pooled_sd(dat_8_3_11_1_2, strength, condition,
                         override = "all")

expect_equal(res$basis$value[res$basis$group == "CTD"],
           93.64, tolerance = 0.001)
expect_equal(res$basis$value[res$basis$group == "RTD"],
           87.30, tolerance = 0.001)
expect_equal(res$basis$value[res$basis$group == "ETW"],
           54.33, tolerance = 0.001)
expect_equal(res$basis$value[res$basis$group == "ETW2"],
           47.12, tolerance = 0.001)
res
```


```{r}
res <- basis_pooled_sd(dat_8_3_11_1_2, strength, condition,
                       p = 0.99, conf = 0.95,
                       override = "all")
expect_equal(res$basis$value[res$basis$group == "CTD"],
           86.19, tolerance = 0.001)
expect_equal(res$basis$value[res$basis$group == "RTD"],
           79.86, tolerance = 0.001)
expect_equal(res$basis$value[res$basis$group == "ETW"],
           46.84, tolerance = 0.001)
expect_equal(res$basis$value[res$basis$group == "ETW2"],
           39.69, tolerance = 0.001)
res
```

### 4.2.2 Pooled SD A- and B-Basis (Mod CV) {#c12-psdmcv}

After removal of the ETW2 condition, CMH17-STATS reports the pooled A- and 
B-Basis (mod CV) shown in the following table.
`cmstatr` computes very similar values.

Condition | CTD   | RTD   | ETW
----------|-------|-------|------
B-Basis   | 92.25 | 85.91 | 52.97
A-Basis   | 83.81 | 77.48 | 44.47


```{r}
res <- dat_8_3_11_1_2 %>%
  filter(condition != "ETW2") %>%
  basis_pooled_sd(strength, condition, modcv = TRUE, override = "all")
expect_equal(res$basis$value[res$basis$group == "CTD"],
             92.25, tolerance = 0.001)
expect_equal(res$basis$value[res$basis$group == "RTD"],
             85.91, tolerance = 0.001)
expect_equal(res$basis$value[res$basis$group == "ETW"],
             52.97, tolerance = 0.001)
res
```


```{r}
res <- dat_8_3_11_1_2 %>%
  filter(condition != "ETW2") %>%
  basis_pooled_sd(strength, condition,
                  p = 0.99, conf = 0.95, modcv = TRUE, override = "all")
expect_equal(res$basis$value[res$basis$group == "CTD"],
             83.81, tolerance = 0.001)
expect_equal(res$basis$value[res$basis$group == "RTD"],
             77.48, tolerance = 0.001)
expect_equal(res$basis$value[res$basis$group == "ETW"],
             44.47, tolerance = 0.001)
res
```



### 4.2.3 Pooled CV A- and B-Basis {#c12-pcv}
This data set was input into CMH17-STATS and the Pooled CV method was selected.
The results from CMH17-STATS were as follows. `cmstatr` produces very similar
results.

Condition | CTD   | RTD   | ETW   | ETW2
----------|-------|-------|-------|------
B-Basis   | 90.89 | 85.37 | 56.79 | 50.55
A-Basis   | 81.62 | 76.67 | 50.98 | 45.40

```{r}
res <- basis_pooled_cv(dat_8_3_11_1_2, strength, condition, override = "all")
expect_equal(res$basis$value[res$basis$group == "CTD"],
             90.89, tolerance = 0.001)
expect_equal(res$basis$value[res$basis$group == "RTD"],
             85.37, tolerance = 0.001)
expect_equal(res$basis$value[res$basis$group == "ETW"],
             56.79, tolerance = 0.001)
expect_equal(res$basis$value[res$basis$group == "ETW2"],
             50.55, tolerance = 0.001)
res
```

```{r}
res <- basis_pooled_cv(dat_8_3_11_1_2, strength, condition,
                       p = 0.99, conf = 0.95, override = "all")
expect_equal(res$basis$value[res$basis$group == "CTD"],
             81.62, tolerance = 0.001)
expect_equal(res$basis$value[res$basis$group == "RTD"],
             76.67, tolerance = 0.001)
expect_equal(res$basis$value[res$basis$group == "ETW"],
             50.98, tolerance = 0.001)
expect_equal(res$basis$value[res$basis$group == "ETW2"],
             45.40, tolerance = 0.001)
res
```


### 4.2.4 Pooled CV A- and B-Basis (Mod CV) {#c12-pcvmcv}
This data set was input into CMH17-STATS and the Pooled CV method was selected
with the modified CV transform. Additionally, the ETW2 condition was removed.
The results from CMH17-STATS were as follows. `cmstatr` produces very similar
results.

Condition | CTD   | RTD   | ETW   
----------|-------|-------|-------
B-Basis   | 90.31 | 84.83 | 56.43 
A-Basis   | 80.57 | 75.69 | 50.33 


```{r}
res <- dat_8_3_11_1_2 %>%
  filter(condition != "ETW2") %>%
  basis_pooled_cv(strength, condition, modcv = TRUE, override = "all")
expect_equal(res$basis$value[res$basis$group == "CTD"],
             90.31, tolerance = 0.001)
expect_equal(res$basis$value[res$basis$group == "RTD"],
             84.83, tolerance = 0.001)
expect_equal(res$basis$value[res$basis$group == "ETW"],
             56.43, tolerance = 0.001)
res
```

```{r}
res <- dat_8_3_11_1_2 %>%
  filter(condition != "ETW2") %>%
  basis_pooled_cv(strength, condition, modcv = TRUE,
                  p = 0.99, conf = 0.95, override = "all")
expect_equal(res$basis$value[res$basis$group == "CTD"],
             80.57, tolerance = 0.001)
expect_equal(res$basis$value[res$basis$group == "RTD"],
             75.69, tolerance = 0.001)
expect_equal(res$basis$value[res$basis$group == "ETW"],
             50.33, tolerance = 0.001)
res
```


# 5. Other Data Sets
This section contains various small data sets. In most cases, these data sets
were generated randomly for the purpose of comparing `cmstatr` to other
software.

## 5.1 Anderson--Darling Test (Normal) {#ods-adn}
The following data set was randomly generated. When this is
entered into STAT17 [@STAT-17], that software gives the value
$OSL = 0.465$, which matches the result of `cmstatr`
within a small margin.

```{r}
dat <- data.frame(
  strength = c(
    137.4438, 139.5395, 150.89, 141.4474, 141.8203, 151.8821, 143.9245,
    132.9732, 136.6419, 138.1723, 148.7668, 143.283, 143.5429,
    141.7023, 137.4732, 152.338, 144.1589, 128.5218
  )
)
res <- anderson_darling_normal(dat, strength)

expect_equal(res$osl, 0.465, tolerance = 0.001)

res
```

## 5.2 Anderson--Darling Test (Lognormal) {#ods-adl}
The following data set was randomly generated. When this is
entered into STAT17 [@STAT-17], that software gives the value
$OSL = 0.480$, which matches the result of `cmstatr`
within a small margin.

```{r}
dat <- data.frame(
  strength = c(
    137.4438, 139.5395, 150.89, 141.4474, 141.8203, 151.8821, 143.9245,
    132.9732, 136.6419, 138.1723, 148.7668, 143.283, 143.5429,
    141.7023, 137.4732, 152.338, 144.1589, 128.5218
  )
)

res <- anderson_darling_lognormal(dat, strength)

expect_equal(res$osl, 0.480, tolerance = 0.001)

res
```

## 5.3 Anderson--Darling Test (Weibull) {#ods-adw}
The following data set was randomly generated. When this is
entered into STAT17 [@STAT-17], that software gives the value
$OSL = 0.179$, which matches the result of `cmstatr`
within a small margin.

```{r}
dat <- data.frame(
  strength = c(
    137.4438, 139.5395, 150.89, 141.4474, 141.8203, 151.8821, 143.9245,
    132.9732, 136.6419, 138.1723, 148.7668, 143.283, 143.5429,
    141.7023, 137.4732, 152.338, 144.1589, 128.5218
  )
)

res <- anderson_darling_weibull(dat, strength)

expect_equal(res$osl, 0.179, tolerance = 0.002)

res
```

## 5.4 Normal A- and B-Basis {#ods-nb}

The following data was input into STAT17 and the A- and B-Basis values
were computed assuming normally distributed data. The results were 120.336
and 129.287, respectively. `cmstatr` reports very similar values.


```{r}
dat <- c(
  137.4438, 139.5395, 150.8900, 141.4474, 141.8203, 151.8821, 143.9245,
  132.9732, 136.6419, 138.1723, 148.7668, 143.2830, 143.5429, 141.7023,
  137.4732, 152.3380, 144.1589, 128.5218
)

res <- basis_normal(x = dat, p = 0.99, conf = 0.95, override = "all")
expect_equal(res$basis, 120.336, tolerance = 0.0005)
res
```


```{r}
res <- basis_normal(x = dat, p = 0.9, conf = 0.95, override = "all")
expect_equal(res$basis, 129.287, tolerance = 0.0005)
res
```


## 5.5 Lognormal A- and B-Basis {#ods-lb}

The following data was input into STAT17 and the A- and B-Basis values
were computed assuming distributed according to a lognormal distribution.
The results were 121.710
and 129.664, respectively. `cmstatr` reports very similar values.


```{r}
dat <- c(
  137.4438, 139.5395, 150.8900, 141.4474, 141.8203, 151.8821, 143.9245,
  132.9732, 136.6419, 138.1723, 148.7668, 143.2830, 143.5429, 141.7023,
  137.4732, 152.3380, 144.1589, 128.5218
)

res <- basis_lognormal(x = dat, p = 0.99, conf = 0.95, override = "all")
expect_equal(res$basis, 121.710, tolerance = 0.0005)
res
```

```{r}
res <- basis_lognormal(x = dat, p = 0.9, conf = 0.95, override = "all")
expect_equal(res$basis, 129.664, tolerance = 0.0005)
res
```


## 5.6 Weibull A- and B-Basis {#ods-wb}

The following data was input into STAT17 and the A- and B-Basis values
were computed assuming data following the Weibull distribution.
The results were 109.150
and 125.441, respectively. `cmstatr` reports very similar values.


```{r}
dat <- c(
  137.4438, 139.5395, 150.8900, 141.4474, 141.8203, 151.8821, 143.9245,
  132.9732, 136.6419, 138.1723, 148.7668, 143.2830, 143.5429, 141.7023,
  137.4732, 152.3380, 144.1589, 128.5218
)

res <- basis_weibull(x = dat, p = 0.99, conf = 0.95, override = "all")
expect_equal(res$basis, 109.150, tolerance = 0.005)
res
```

```{r}
res <- basis_weibull(x = dat, p = 0.9, conf = 0.95, override = "all")
expect_equal(res$basis, 125.441, tolerance = 0.005)
res
```

## 5.7 Extended Hanson--Koopmans A- and B-Basis {#ods-hkb}

The following data was input into STAT17 and the A- and B-Basis values
were computed using the nonparametric (small sample) method.
The results were 99.651
and 124.156, respectively. `cmstatr` reports very similar values.

```{r}
dat <- c(
  137.4438, 139.5395, 150.8900, 141.4474, 141.8203, 151.8821, 143.9245,
  132.9732, 136.6419, 138.1723, 148.7668, 143.2830, 143.5429, 141.7023,
  137.4732, 152.3380, 144.1589, 128.5218
)

res <- basis_hk_ext(x = dat, p = 0.99, conf = 0.95,
                    method = "woodward-frawley", override = "all")
expect_equal(res$basis, 99.651, tolerance = 0.005)
res
```


```{r}
res <- basis_hk_ext(x = dat, p = 0.9, conf = 0.95,
                    method = "optimum-order", override = "all")
expect_equal(res$basis, 124.156, tolerance = 0.005)
res
```


## 5.8 Extended Hanson--Koopmans B-Basis {#ods-hkb2}
The following random numbers were generated.

```{r}
dat <- c(
  139.6734, 143.0032, 130.4757, 144.8327, 138.7818, 136.7693, 148.636,
  131.0095, 131.4933, 142.8856, 158.0198, 145.2271, 137.5991, 139.8298,
  140.8557, 137.6148, 131.3614, 152.7795, 145.8792, 152.9207, 160.0989,
  145.1920, 128.6383, 141.5992, 122.5297, 159.8209, 151.6720, 159.0156
)
```

All of the numbers above were input into STAT17 and the reported B-Basis
value using the Optimum Order nonparametric method was 122.36798. This
result matches the results of `cmstatr` within a small margin.

```{r}
res <- basis_hk_ext(x = dat, p = 0.9, conf = 0.95,
                    method = "optimum-order", override = "all")
expect_equal(res$basis, 122.36798, tolerance = 0.001)
res
```

The last two observations from the above data set were discarded, leaving
26 observations. This smaller data set was input into STAT17 and that
software calculated a B-Basis value of 121.57073 using the Optimum
Order nonparametric method. `cmstatr` reports a very similar number.

```{r}
res <- basis_hk_ext(x = head(dat, 26), p = 0.9, conf = 0.95,
                    method = "optimum-order", override = "all")
expect_equal(res$basis, 121.57073, tolerance = 0.001)
res
```

The same data set was further reduced such that only the first 22
observations were included. This smaller data set was input into STAT17
and that
software calculated a B-Basis value of 128.82397 using the Optimum
Order nonparametric method. `cmstatr` reports a very similar number.

```{r}
res <- basis_hk_ext(x = head(dat, 22), p = 0.9, conf = 0.95,
                    method = "optimum-order", override = "all")
expect_equal(res$basis, 128.82397, tolerance = 0.001)
res
```

## 5.9 Large Sample Nonparametric B-Basis {#ods-lsnb}

The following data was input into STAT17 and the B-Basis value
was computed using the nonparametric (large sample) method.
The results was 122.738297. `cmstatr` reports very similar values.

```{r}
dat <- c(
  137.3603, 135.6665, 136.6914, 154.7919, 159.2037, 137.3277, 128.821,
  138.6304, 138.9004, 147.4598, 148.6622, 144.4948, 131.0851, 149.0203,
  131.8232, 146.4471, 123.8124, 126.3105, 140.7609, 134.4875, 128.7508,
  117.1854, 129.3088, 141.6789, 138.4073, 136.0295, 128.4164, 141.7733,
  134.455,  122.7383, 136.9171, 136.9232, 138.8402, 152.8294, 135.0633,
  121.052,  131.035,  138.3248, 131.1379, 147.3771, 130.0681, 132.7467,
  137.1444, 141.662,  146.9363, 160.7448, 138.5511, 129.1628, 140.2939,
  144.8167, 156.5918, 132.0099, 129.3551, 136.6066, 134.5095, 128.2081,
  144.0896, 141.8029, 130.0149, 140.8813, 137.7864
)

res <- basis_nonpara_large_sample(x = dat, p = 0.9, conf = 0.95,
                                  override = "all")
expect_equal(res$basis, 122.738297, tolerance = 0.005)
res
```

## 5.10 Acceptance Limits Based on Mean and Extremum {#ods-eme}

Results from `cmstatr`'s `equiv_mean_extremum` function were compared with
results from HYTEQ. The summary statistics for the qualification data
were set as `mean = 141.310` and `sd=6.415`. For a value of `alpha=0.05` and
`n = 9`,
HYTEQ reported thresholds of 123.725 and 137.197 for minimum individual
and mean, respectively. `cmstatr` produces very similar results.

```{r}
res <- equiv_mean_extremum(alpha = 0.05, mean_qual = 141.310, sd_qual = 6.415,
                           n_sample = 9)
expect_equal(res$threshold_min_indiv, 123.725, tolerance = 0.001)
expect_equal(res$threshold_mean, 137.197, tolerance = 0.001)
res
```

Using the same parameters, but using the modified CV method,
HYTEQ produces thresholds of 117.024 and 135.630 for minimum individual
and mean, respectively. `cmstatr` produces very similar results.

```{r}
res <- equiv_mean_extremum(alpha = 0.05, mean_qual = 141.310, sd_qual = 6.415,
                           n_sample = 9, modcv = TRUE)
expect_equal(res$threshold_min_indiv, 117.024, tolerance = 0.001)
expect_equal(res$threshold_mean, 135.630, tolerance = 0.001)
res
```


## 5.11 Acceptance Based on Change in Mean {#ods-ecm}

Results from `cmstatr`'s `equiv_change_mean` function were compared with
results from HYTEQ. The following parameters were used. A value of
`alpha = 0.05` was selected.

Parameter | Qualification | Sample
----------|---------------|---------
Mean      | 9.24          | 9.02
SD        | 0.162         | 0.15785
n         | 28            | 9

HYTEQ gives an acceptance range of 9.115 to 9.365. `cmstatr` produces
similar results.

```{r}
res <- equiv_change_mean(alpha = 0.05, n_sample = 9, mean_sample = 9.02,
                         sd_sample = 0.15785, n_qual = 28, mean_qual = 9.24,
                         sd_qual = 0.162)
expect_equal(res$threshold, c(9.115, 9.365), tolerance = 0.001)
res
```

After selecting the modified CV method, HYTEQ gives an acceptance
range of 8.857 to 9.623. `cmstatr` produces similar results.

```{r}
res <- equiv_change_mean(alpha = 0.05, n_sample = 9, mean_sample = 9.02,
                         sd_sample = 0.15785, n_qual = 28, mean_qual = 9.24,
                         sd_qual = 0.162, modcv = TRUE)
expect_equal(res$threshold, c(8.857, 9.623), tolerance = 0.001)
res
```

# 6. Comparison with Literature
In this section, results from `cmstatr` are compared with values published
in literature.

## 6.1 Anderson--Darling K--Sample Test {#cl-adk}
[@Stephens1987] provides example data that compares measurements obtained
in four labs. Their paper gives values of the ADK test statistic as well as
p-values.

The data in [@Stephens1987] is as follows:

```{r}
dat_ss1987 <- data.frame(
  smoothness = c(
    38.7, 41.5, 43.8, 44.5, 45.5, 46.0, 47.7, 58.0,
    39.2, 39.3, 39.7, 41.4, 41.8, 42.9, 43.3, 45.8,
    34.0, 35.0, 39.0, 40.0, 43.0, 43.0, 44.0, 45.0,
    34.0, 34.8, 34.8, 35.4, 37.2, 37.8, 41.2, 42.8
  ),
  lab = c(rep("A", 8), rep("B", 8), rep("C", 8), rep("D", 8))
)
dat_ss1987
```

[@Stephens1987] lists the corresponding test statistics
$A_{akN}^2 = 8.3926$ and $\sigma_N = 1.2038$ with the p-value
$p = 0.0022$. These match the result of `cmstatr` within a small margin.

```{r}
res <- ad_ksample(dat_ss1987, smoothness, lab)

expect_equal(res$ad, 8.3926, tolerance = 0.001)
expect_equal(res$sigma, 1.2038, tolerance = 0.001)
expect_equal(res$p, 0.00226, tolerance = 0.01)

res
```


# 7. Comparison with Published Factors
Various factors, such as tolerance limit factors, are published in various
publications. This section compares those published factors with those
computed by `cmstatr`.

## 7.1 Normal kB Factors {#pf-kb}
B-Basis tolerance limit factors assuming a normal distribution are published in
CMH-17-1G. Those factors are reproduced below and are compared with the
results of `cmstatr`. The published factors and those computed by `cmstatr`
are quite similar.

```{r}
tribble(
  ~n, ~kB_published,
  2, 20.581, 36, 1.725, 70, 1.582, 104, 1.522,
  3, 6.157, 37, 1.718, 71, 1.579, 105, 1.521,
  4, 4.163, 38, 1.711, 72, 1.577, 106, 1.519,
  5, 3.408, 39, 1.704, 73, 1.575, 107, 1.518,
  6, 3.007, 40, 1.698, 74, 1.572, 108, 1.517,
  7, 2.756, 41, 1.692, 75, 1.570, 109, 1.516,
  8, 2.583, 42, 1.686, 76, 1.568, 110, 1.515,
  9, 2.454, 43, 1.680, 77, 1.566, 111, 1.513,
  10, 2.355, 44, 1.675, 78, 1.564, 112, 1.512,
  11, 2.276, 45, 1.669, 79, 1.562, 113, 1.511,
  12, 2.211, 46, 1.664, 80, 1.560, 114, 1.510,
  13, 2.156, 47, 1.660, 81, 1.558, 115, 1.509,
  14, 2.109, 48, 1.655, 82, 1.556, 116, 1.508,
  15, 2.069, 49, 1.650, 83, 1.554, 117, 1.507,
  16, 2.034, 50, 1.646, 84, 1.552, 118, 1.506,
  17, 2.002, 51, 1.642, 85, 1.551, 119, 1.505,
  18, 1.974, 52, 1.638, 86, 1.549, 120, 1.504,
  19, 1.949, 53, 1.634, 87, 1.547, 121, 1.503,
  20, 1.927, 54, 1.630, 88, 1.545, 122, 1.502,
  21, 1.906, 55, 1.626, 89, 1.544, 123, 1.501,
  22, 1.887, 56, 1.623, 90, 1.542, 124, 1.500,
  23, 1.870, 57, 1.619, 91, 1.540, 125, 1.499,
  24, 1.854, 58, 1.616, 92, 1.539, 126, 1.498,
  25, 1.839, 59, 1.613, 93, 1.537, 127, 1.497,
  26, 1.825, 60, 1.609, 94, 1.536, 128, 1.496,
  27, 1.812, 61, 1.606, 95, 1.534, 129, 1.495,
  28, 1.800, 62, 1.603, 96, 1.533, 130, 1.494,
  29, 1.789, 63, 1.600, 97, 1.531, 131, 1.493,
  30, 1.778, 64, 1.597, 98, 1.530, 132, 1.492,
  31, 1.768, 65, 1.595, 99, 1.529, 133, 1.492,
  32, 1.758, 66, 1.592, 100, 1.527, 134, 1.491,
  33, 1.749, 67, 1.589, 101, 1.526, 135, 1.490,
  34, 1.741, 68, 1.587, 102, 1.525, 136, 1.489,
  35, 1.733, 69, 1.584, 103, 1.523, 137, 1.488
) %>%
  arrange(n) %>%
  mutate(kB_cmstatr = k_factor_normal(n, p = 0.9, conf = 0.95)) %>%
  rowwise() %>%
  mutate(diff = expect_equal(kB_published, kB_cmstatr, tolerance = 0.001)) %>%
  select(-c(diff))
```

## 7.2 Normal kA Factors {#pf-ka}
A-Basis tolerance limit factors assuming a normal distribution are published in
CMH-17-1G. Those factors are reproduced below and are compared with the
results of `cmstatr`. The published factors and those computed by `cmstatr`
are quite similar.

```{r}
tribble(
  ~n, ~kA_published,
  2, 37.094, 36, 2.983, 70, 2.765, 104, 2.676,
  3, 10.553, 37, 2.972, 71, 2.762, 105, 2.674,
  4, 7.042, 38, 2.961, 72, 2.758, 106, 2.672,
  5, 5.741, 39, 2.951, 73, 2.755, 107, 2.671,
  6, 5.062, 40, 2.941, 74, 2.751, 108, 2.669,
  7, 4.642, 41, 2.932, 75, 2.748, 109, 2.667,
  8, 4.354, 42, 2.923, 76, 2.745, 110, 2.665,
  9, 4.143, 43, 2.914, 77, 2.742, 111, 2.663,
  10, 3.981, 44, 2.906, 78, 2.739, 112, 2.662,
  11, 3.852, 45, 2.898, 79, 2.736, 113, 2.660,
  12, 3.747, 46, 2.890, 80, 2.733, 114, 2.658,
  13, 3.659, 47, 2.883, 81, 2.730, 115, 2.657,
  14, 3.585, 48, 2.876, 82, 2.727, 116, 2.655,
  15, 3.520, 49, 2.869, 83, 2.724, 117, 2.654,
  16, 3.464, 50, 2.862, 84, 2.721, 118, 2.652,
  17, 3.414, 51, 2.856, 85, 2.719, 119, 2.651,
  18, 3.370, 52, 2.850, 86, 2.716, 120, 2.649,
  19, 3.331, 53, 2.844, 87, 2.714, 121, 2.648,
  20, 3.295, 54, 2.838, 88, 2.711, 122, 2.646,
  21, 3.263, 55, 2.833, 89, 2.709, 123, 2.645,
  22, 3.233, 56, 2.827, 90, 2.706, 124, 2.643,
  23, 3.206, 57, 2.822, 91, 2.704, 125, 2.642,
  24, 3.181, 58, 2.817, 92, 2.701, 126, 2.640,
  25, 3.158, 59, 2.812, 93, 2.699, 127, 2.639,
  26, 3.136, 60, 2.807, 94, 2.697, 128, 2.638,
  27, 3.116, 61, 2.802, 95, 2.695, 129, 2.636,
  28, 3.098, 62, 2.798, 96, 2.692, 130, 2.635,
  29, 3.080, 63, 2.793, 97, 2.690, 131, 2.634,
  30, 3.064, 64, 2.789, 98, 2.688, 132, 2.632,
  31, 3.048, 65, 2.785, 99, 2.686, 133, 2.631,
  32, 3.034, 66, 2.781, 100, 2.684, 134, 2.630,
  33, 3.020, 67, 2.777, 101, 2.682, 135, 2.628,
  34, 3.007, 68, 2.773, 102, 2.680, 136, 2.627,
  35, 2.995, 69, 2.769, 103, 2.678, 137, 2.626
) %>%
  arrange(n) %>%
  mutate(kA_cmstatr = k_factor_normal(n, p = 0.99, conf = 0.95)) %>%
  rowwise() %>%
  mutate(diff = expect_equal(kA_published, kA_cmstatr, tolerance = 0.001)) %>%
  select(-c(diff))
```

## 7.3 Nonparametric B-Basis Extended Hanson--Koopmans {#pf-hk}
Vangel [@Vangel1994] provides extensive tables of $z$ for the case where
$i=1$ and $j$ is the median observation. This section checks the results of
`cmstatr`'s function against those tables. Only the odd values of $n$
are checked so that the median is a single observation. The unit tests for
the `cmstatr` package include checks of a variety of values of $p$ and
confidence, but only the factors for B-Basis are checked here.

```{r}
tribble(
  ~n, ~z,
  3,  28.820048,
  5,  6.1981307,
  7,  3.4780112,
  9,  2.5168762,
  11, 2.0312134,
  13, 1.7377374,
  15, 1.5403989,
  17, 1.3979806,
  19, 1.2899172,
  21, 1.2048089,
  23, 1.1358259,
  25, 1.0786237,
  27, 1.0303046,
) %>%
  rowwise() %>%
  mutate(
    z_calc = hk_ext_z(n, 1, ceiling(n / 2), p = 0.90, conf = 0.95)
  ) %>%
  mutate(diff = expect_equal(z, z_calc, tolerance = 0.0001)) %>% 
  select(-c(diff))
```


## 7.4 Nonparametric A-Basis Extended Hanson--Koopmans {#pf-hk2}
CMH-17-1G provides Table 8.5.15, which contains factors for calculating
A-Basis values using the Extended Hanson--Koopmans nonparametric method.
That table is reproduced in part here and the factors are compared with
those computed by `cmstatr`. More extensive checks are performed in the
unit test of the `cmstatr` package. The factors computed by `cmstatr` are
very similar to those published in CMH-17-1G.

```{r}
tribble(
  ~n, ~k,
  2, 80.0038,
  4, 9.49579,
  6, 5.57681,
  8, 4.25011,
  10, 3.57267,
  12, 3.1554,
  14, 2.86924,
  16, 2.65889,
  18, 2.4966,
  20, 2.36683,
  25, 2.131,
  30, 1.96975,
  35, 1.85088,
  40, 1.75868,
  45, 1.68449,
  50, 1.62313,
  60, 1.5267,
  70, 1.45352,
  80, 1.39549,
  90, 1.34796,
  100, 1.30806,
  120, 1.24425,
  140, 1.19491,
  160, 1.15519,
  180, 1.12226,
  200, 1.09434,
  225, 1.06471,
  250, 1.03952,
  275, 1.01773
) %>%
  rowwise() %>%
  mutate(z_calc = hk_ext_z(n, 1, n, 0.99, 0.95)) %>%
  mutate(diff = expect_lt(abs(k - z_calc), 0.0001)) %>% 
  select(-c(diff))
```


## 7.5 Factors for Small Sample Nonparametric B-Basis {#pf-hk-opt}
CMH-17-1G Table 8.5.14 provides ranks orders and factors for computing
nonparametric B-Basis values. This table is reproduced below and
compared with the results of `cmstatr`. The results are similar. In some
cases, the rank order ($r$ in CMH-17-1G or $j$ in `cmstatr`) *and* the 
the factor ($k$) are different. These differences are discussed in detail in
the vignette
[Extended Hanson-Koopmans](hk_ext.html).

```{r}
tribble(
  ~n, ~r, ~k,
  2, 2, 35.177,
  3, 3, 7.859,
  4, 4, 4.505,
  5, 4, 4.101,
  6, 5, 3.064,
  7, 5, 2.858,
  8, 6, 2.382,
  9, 6, 2.253,
  10, 6, 2.137,
  11, 7, 1.897,
  12, 7, 1.814,
  13, 7, 1.738,
  14, 8, 1.599,
  15, 8, 1.540,
  16, 8, 1.485,
  17, 8, 1.434,
  18, 9, 1.354,
  19, 9, 1.311,
  20, 10, 1.253,
  21, 10, 1.218,
  22, 10, 1.184,
  23, 11, 1.143,
  24, 11, 1.114,
  25, 11, 1.087,
  26, 11, 1.060,
  27, 11, 1.035,
  28, 12, 1.010
) %>% 
  rowwise() %>%
  mutate(r_calc = hk_ext_z_j_opt(n, 0.90, 0.95)$j) %>%
  mutate(k_calc = hk_ext_z_j_opt(n, 0.90, 0.95)$z)
```



## 7.6 Nonparametric B-Basis Binomial Rank {#pf-npbinom}
CMH-17-1G Table 8.5.12 provides factors for computing B-Basis values
using the nonparametric binomial rank method. Part of that table is
reproduced below and compared with the results of `cmstatr`.
The results of `cmstatr` are similar to the published values.
A more complete comparison is performed in the units tests of the
`cmstatr` package.

```{r}
tribble(
  ~n, ~rb,
  29, 1,
  46, 2,
  61, 3,
  76, 4,
  89, 5,
  103, 6,
  116, 7,
  129, 8,
  142, 9,
  154, 10,
  167, 11,
  179, 12,
  191, 13,
  203, 14
) %>%
  rowwise() %>%
  mutate(r_calc = nonpara_binomial_rank(n, 0.9, 0.95)) %>%
  mutate(test = expect_equal(rb, r_calc)) %>%
  select(-c(test))
```

## 7.7 Nonparametric A-Basis Binomial Rank {#pf-npbinom2}
CMH-17-1G Table 8.5.13 provides factors for computing B-Basis values
using the nonparametric binomial rank method. Part of that table is
reproduced below and compared with the results of `cmstatr`.
The results of `cmstatr` are similar to the published values.
A more complete comparison is performed in the units tests of the
`cmstatr` package.

```{r}
tribble(
  ~n, ~ra,
  299, 1,
  473, 2,
  628, 3,
  773, 4,
  913, 5
) %>%
  rowwise() %>%
  mutate(r_calc = nonpara_binomial_rank(n, 0.99, 0.95)) %>%
  mutate(test = expect_equal(ra, r_calc)) %>%
  select(-c(test))
```


## 7.8 Factors for Equivalency {#pf-equiv}

Vangel's 2002 paper provides factors for calculating limits for sample
mean and sample extremum for various values of $\alpha$ and sample size ($n$).
A subset of those factors are reproduced below and compared with results
from `cmstatr`. The results are very similar for values of $\alpha$ and $n$
that are common for composite materials.


```{r}
read.csv(system.file("extdata", "k1.vangel.csv",
                     package = "cmstatr")) %>%
  gather(n, k1, X2:X10) %>%
  mutate(n = as.numeric(substring(n, 2))) %>%
  inner_join(
    read.csv(system.file("extdata", "k2.vangel.csv",
                         package = "cmstatr")) %>%
      gather(n, k2, X2:X10) %>%
      mutate(n = as.numeric(substring(n, 2))),
    by = c("n" = "n", "alpha" = "alpha")
  ) %>%
  filter(n >= 5 & (alpha == 0.01 | alpha == 0.05)) %>% 
  group_by(n, alpha) %>%
  nest() %>%
  mutate(equiv = map2(alpha, n, ~k_equiv(.x, .y))) %>%
  mutate(k1_calc = map(equiv, function(e) e[1]),
         k2_calc = map(equiv, function(e) e[2])) %>%
  select(-c(equiv)) %>% 
  unnest(cols = c(data, k1_calc, k2_calc)) %>% 
  mutate(check = expect_equal(k1, k1_calc, tolerance = 0.0001)) %>%
  select(-c(check)) %>% 
  mutate(check = expect_equal(k2, k2_calc, tolerance = 0.0001)) %>%
  select(-c(check))
```


# 8. Session Info
This copy of this vignette was build on the following system.

```{r}
sessionInfo()
```

# 9. References

---
title: "Extended Hanson-Koopmans"
author: "Stefan Kloppenborg"
date: "2021-09-30"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Extended Hanson-Koopmans}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
csl: ieee.csl
references:
- id: Hanson1964
  type: article
  author:
    - given: D. L.
      family: Hanson
    - given: L. H.
      family: Koopmans
  title: Tolerance Limits for the Class of Distributions with Increasing Hazard Rates
  container-title: The Annals of Mathematical Statistics
  volume: "35"
  issue: "4"
  page: 1561-1570
  issued:
    year: "1964"
  DOI: 10.1214/aoms/1177700380
- id: Vangel1994
  type: article
  author:
    - given: Mark
      family: Vangel
  title: One-Sided Nonparametric Tolerance Limits
  container-title: Communications in Statistics - Simulation and Computation
  volume: "23"
  issue: "4"
  page: 1137-1154
  issued:
    year: "1994"
  DOI: 10.1080/03610919408813222
- id: Harter1961
  type: article
  author:
    - given: H. Leon
      family: Harter
  title: Expected values of normal order statistics
  container-title: Biometrika
  volume: "48"
  issue: 1/2
  page: 151-165
  issued:
    year: "1961"
  DOI: https://doi.org/10.2307/2333139
- id: CMH-17-1G
  type: report
  number: CMH-17-1G
  title: Composite Materials Handbook, Volume 1. Polymer Matrix Composites
    Guideline for Characterization of Structural Materials
  publisher: SAE International
  issued:
    year: "2012"
    month: "03"
---



In this vignette, we'll use the following packages:


```r
library(dplyr)
library(ggplot2)
library(purrr)
library(tidyr)
```


The Extended Hanson--Koopmans method is a nonparametric method of
determining tolerance limits (such as A- or B-Basis values).
This method does not assume
any particular distribution, but does require that
$-\log\left(F\right)$ is convex, where $F$ is the cumulative distribution
function (CDF) of the distribution.

The functions `kh_ext_z`, `hk_ext_z_j_opt` and `basis_kh_ext` in `cmstatr`
are based on the Extended Hanson--Koopmans method, developed by
Vangel [@Vangel1994]. This is an extension of the method published in
[@Hanson1964].

Tolerance limits (Basis values) calculated using the Extended
Hanson--Koopmans method are calculated based on two order statistics [^1], $i$
and $j$, and a factor, $z$.
The function `hk_ext_z_j_opt` and the function
`basis_kh_ext` (with `method = "optimum-order"`) set the first of
these order statistics to the first (lowest) order statistic,
and a second order statistic determined by minimizing the following
function:

$$
\left| z E\left(X_{\left(1\right)}\right)
+ \left(1 - z\right) E\left(X_{\left(j\right)}\right)
- \Phi\left(p\right)\right|
$$

Where $E\left(X_{(i)}\right)$ is the expected value of the
$i$`th` order
statistic for a sample drawn from the standard normal distribution,
and $\Phi\left(p\right)$
is the CDF of a standard normal distribution for the content of the tolerance
limit (i.e. $p=0.9$ for B-Basis).

[^1]: The $i$`th` order statistic is the $i$`th`
lowest value in the sample.

The value of $z$ is calculated based on the sample size, $n$, the two order
statistics $i$ and $j$, the content $p$ and the confidence. The calculation
is performed using the method in [@Vangel1994] and
implemented in `kh_ext_z`. The value of $j$ is very sensitive to the way that
the expected value of the order statistics is calculated, and may be sensitive
to numerical precision.

In version 0.8.0 of `cmstatr` and prior, the expected value of an order
statistic for a sample drawn from a standard normal distribution was
determined in a crude way. After version 0.8.0, the method in
[@Harter1961] is used. These method produce different values of $j$ for certain
sample sizes. Additionally, a table of $j$ and $z$ values for various
sample sizes is published in CMH-17-1G[^2] [@CMH-17-1G].
This table gives slightly different
values of $j$ for some sample sizes.

[^2]: Note that CMH-17-1G uses the symbols $r$ and $k$ instead of
      $j$ and $z$.

The values of $j$ and $z$ produced by `cmstatr` in version 0.8.0 and before,
the values produced after version 0.8.0 and the value published in CMH-17-1G
are shown below. All of these values are for B-Basis (90% content,
95% confidence).



```r
factors <- tribble(
  ~n, ~j_pre_080, ~z_pre_080, ~j_post_080, ~z_post_080, ~j_cmh, ~z_cmh,
  2, 2, 35.1768141883907, 2, 35.1768141883907, 2, 35.177,
  3, 3, 7.85866787768029, 3, 7.85866787768029, 3, 7.859,
  4, 4, 4.50522447199018, 4, 4.50522447199018, 4, 4.505,
  5, 4, 4.10074820079326, 4, 4.10074820079326, 4, 4.101,
  6, 5, 3.06444416024793, 5, 3.06444416024793, 5, 3.064,
  7, 5, 2.85751000593839, 5, 2.85751000593839, 5, 2.858,
  8, 6, 2.38240998122575, 6, 2.38240998122575, 6, 2.382,
  9, 6, 2.25292053841772, 6, 2.25292053841772, 6, 2.253,
  10, 7, 1.98762060673102, 6, 2.13665759924781, 6, 2.137,
  11, 7, 1.89699586212496, 7, 1.89699586212496, 7, 1.897,
  12, 7, 1.81410756892749, 7, 1.81410756892749, 7, 1.814,
  13, 8, 1.66223343216608, 7, 1.73773765993598, 7, 1.738,
  14, 8, 1.59916281901889, 8, 1.59916281901889, 8, 1.599,
  15, 8, 1.54040000806181, 8, 1.54040000806181, 8, 1.54,
  16, 9, 1.44512878109546, 8, 1.48539432060546, 8, 1.485,
  17, 9, 1.39799975474842, 9, 1.39799975474842, 8, 1.434,
  18, 9, 1.35353033609361, 9, 1.35353033609361, 9, 1.354,
  19, 10, 1.28991705486727, 9, 1.31146980117942, 9, 1.311,
  20, 10, 1.25290765871981, 9, 1.27163203813793, 10, 1.253,
  21, 10, 1.21771654027026, 10, 1.21771654027026, 10, 1.218,
  22, 11, 1.17330587650406, 10, 1.18418267046374, 10, 1.184,
  23, 11, 1.14324511741536, 10, 1.15218647199938, 11, 1.143,
  24, 11, 1.11442082880151, 10, 1.12153586685854, 11, 1.114,
  25, 11, 1.08682185727661, 11, 1.08682185727661, 11, 1.087,
  26, 11, 1.06032912052507, 11, 1.06032912052507, 11, 1.06,
  27, 12, 1.03307994274081, 11, 1.03485308510789, 11, 1.035,
  28, 12, 1.00982188136729, 11, 1.01034609051393, 12, 1.01
)
```

For the sample sizes where $j$ is the same for each approach, the values
of $z$ are also equal within a small tolerance.







```r
factors %>%
  filter(j_pre_080 == j_post_080 & j_pre_080 == j_cmh)
#> # A tibble: 16 × 7
#>        n j_pre_080 z_pre_080 j_post_080 z_post_080 j_cmh z_cmh
#>    <dbl>     <dbl>     <dbl>      <dbl>      <dbl> <dbl> <dbl>
#>  1     2         2     35.2           2      35.2      2 35.2 
#>  2     3         3      7.86          3       7.86     3  7.86
#>  3     4         4      4.51          4       4.51     4  4.50
#>  4     5         4      4.10          4       4.10     4  4.10
#>  5     6         5      3.06          5       3.06     5  3.06
#>  6     7         5      2.86          5       2.86     5  2.86
#>  7     8         6      2.38          6       2.38     6  2.38
#>  8     9         6      2.25          6       2.25     6  2.25
#>  9    11         7      1.90          7       1.90     7  1.90
#> 10    12         7      1.81          7       1.81     7  1.81
#> 11    14         8      1.60          8       1.60     8  1.60
#> 12    15         8      1.54          8       1.54     8  1.54
#> 13    18         9      1.35          9       1.35     9  1.35
#> 14    21        10      1.22         10       1.22    10  1.22
#> 15    25        11      1.09         11       1.09    11  1.09
#> 16    26        11      1.06         11       1.06    11  1.06
```

The sample sizes where the value of $j$ differs are as follows:


```r
factor_diff <- factors %>%
  filter(j_pre_080 != j_post_080 | j_pre_080 != j_cmh | j_post_080 != j_cmh)
factor_diff
#> # A tibble: 11 × 7
#>        n j_pre_080 z_pre_080 j_post_080 z_post_080 j_cmh z_cmh
#>    <dbl>     <dbl>     <dbl>      <dbl>      <dbl> <dbl> <dbl>
#>  1    10         7      1.99          6       2.14     6  2.14
#>  2    13         8      1.66          7       1.74     7  1.74
#>  3    16         9      1.45          8       1.49     8  1.48
#>  4    17         9      1.40          9       1.40     8  1.43
#>  5    19        10      1.29          9       1.31     9  1.31
#>  6    20        10      1.25          9       1.27    10  1.25
#>  7    22        11      1.17         10       1.18    10  1.18
#>  8    23        11      1.14         10       1.15    11  1.14
#>  9    24        11      1.11         10       1.12    11  1.11
#> 10    27        12      1.03         11       1.03    11  1.03
#> 11    28        12      1.01         11       1.01    12  1.01
```

While there are differences in the three implementations, it's not clear
how much these differences will matter in terms of the tolerance limits
calculated. This can be investigated through simulation.

# Simulation with Normally Distributed Data
First, we'll generate a large number (10,000) of samples of sample size
$n$ from a normal distribution. Since we're generating the samples, we
know the true population parameters, so can calculate the true population
quantiles. We'll use the three sets of $j$ and $z$ values to compute
tolerance limits and compared those tolerance limits to the population
quantiles. The proportion of the calculated tolerance limits below the
population quantiles should be equal to the selected confidence. We'll
restrict the simulation study to the sample sizes where the values of
$j$ and $z$ differ in the three implementations of this method, and we'll
consider B-Basis (90% content, 95% confidence).


```r
mu_normal <- 100
sd_normal <- 6

set.seed(1234567)  # make this reproducible

tolerance_limit <- function(x, j, z) {
  x[j] * (x[1] / x[j]) ^ z
}

sim_normal <- pmap_dfr(factor_diff, function(n, j_pre_080, z_pre_080,
                                             j_post_080, z_post_080,
                                             j_cmh, z_cmh) {
  map_dfr(1:10000, function(i_sim) {
    x <- sort(rnorm(n, mu_normal, sd_normal))
    tibble(
      n = n,
      b_pre_080 = tolerance_limit(x, j_pre_080, z_pre_080),
      b_post_080 = tolerance_limit(x, j_post_080, z_post_080),
      b_cmh = tolerance_limit(x, j_cmh, z_cmh),
      x = list(x)
    )
  }
  )
})
sim_normal
#> # A tibble: 110,000 × 5
#>        n b_pre_080 b_post_080 b_cmh x         
#>    <dbl>     <dbl>      <dbl> <dbl> <list>    
#>  1    10      78.4       77.7  77.7 <dbl [10]>
#>  2    10      82.8       82.0  82.0 <dbl [10]>
#>  3    10      83.3       83.0  83.0 <dbl [10]>
#>  4    10      78.4       77.2  77.2 <dbl [10]>
#>  5    10      87.3       86.6  86.6 <dbl [10]>
#>  6    10      92.3       93.2  93.2 <dbl [10]>
#>  7    10      75.2       77.9  77.9 <dbl [10]>
#>  8    10      75.4       73.9  73.9 <dbl [10]>
#>  9    10      75.5       75.1  75.1 <dbl [10]>
#> 10    10      76.4       78.4  78.4 <dbl [10]>
#> # … with 109,990 more rows
```

One can see that the tolerance limits calculated with each set of
factors for (most) data sets is different. However, this does not necessarily
mean that any set of factors is more or less correct.

The distribution of the tolerance limits for each sample size is as follows:


```r
sim_normal %>%
  pivot_longer(cols = b_pre_080:b_cmh, names_to = "Factors") %>%
  ggplot(aes(x = value, color = Factors)) +
  geom_density() +
  facet_wrap(n ~ .) +
  theme_bw() +
  ggtitle("Distribution of Tolerance Limits for Various Values of n")
```

![plot of chunk distribution-normal](distribution-normal-1.png)

For all samples sizes, the distribution of tolerance limits is actually
very similar between all three sets of factors.

The true population quantile can be calculated as follows:


```r
x_p_normal <- qnorm(0.9, mu_normal, sd_normal, lower.tail = FALSE)
x_p_normal
#> [1] 92.31069
```

The proportion of calculated tolerance limit values that are below the
population quantile can be calculated as follows. We see that the in all
cases the tolerance limits are all conservative, and also that each
set of factors produce similar levels of conservatism.


```r
sim_normal %>%
  mutate(below_pre_080 = b_pre_080 < x_p_normal,
         below_post_080 = b_post_080 < x_p_normal,
         below_cmh = b_cmh < x_p_normal) %>%
  group_by(n) %>%
  summarise(
    prop_below_pre_080 = sum(below_pre_080) / n(),
    prop_below_post_080 = sum(below_post_080) / n(),
    prop_below_cmh = sum(below_cmh) / n()
  )
#> # A tibble: 11 × 4
#>        n prop_below_pre_080 prop_below_post_080 prop_below_cmh
#>    <dbl>              <dbl>               <dbl>          <dbl>
#>  1    10              0.984               0.980          0.980
#>  2    13              0.979               0.975          0.975
#>  3    16              0.969               0.967          0.967
#>  4    17              0.973               0.973          0.971
#>  5    19              0.962               0.961          0.961
#>  6    20              0.964               0.962          0.964
#>  7    22              0.961               0.960          0.960
#>  8    23              0.960               0.959          0.960
#>  9    24              0.962               0.961          0.962
#> 10    27              0.954               0.953          0.954
#> 11    28              0.952               0.952          0.952
```

# Simulation with Weibull Data

Next, we'll do a similar simulation using data drawn from a Weibull
distribution. Again, we'll generate 10,000 samples for each sample size.


```r
shape_weibull <- 60
scale_weibull <- 100

set.seed(234568)  # make this reproducible

sim_weibull <- pmap_dfr(factor_diff, function(n, j_pre_080, z_pre_080,
                                              j_post_080, z_post_080,
                                              j_cmh, z_cmh) {
  map_dfr(1:10000, function(i_sim) {
    x <- sort(rweibull(n, shape_weibull, scale_weibull))
    tibble(
      n = n,
      b_pre_080 = tolerance_limit(x, j_pre_080, z_pre_080),
      b_post_080 = tolerance_limit(x, j_post_080, z_post_080),
      b_cmh = tolerance_limit(x, j_cmh, z_cmh),
      x = list(x)
    )
  }
  )
})
sim_weibull
#> # A tibble: 110,000 × 5
#>        n b_pre_080 b_post_080 b_cmh x         
#>    <dbl>     <dbl>      <dbl> <dbl> <list>    
#>  1    10      95.3       95.1  95.1 <dbl [10]>
#>  2    10      88.5       88.3  88.3 <dbl [10]>
#>  3    10      89.7       89.3  89.3 <dbl [10]>
#>  4    10      94.7       94.4  94.4 <dbl [10]>
#>  5    10      96.9       96.9  96.9 <dbl [10]>
#>  6    10      93.6       93.2  93.2 <dbl [10]>
#>  7    10      86.1       85.5  85.5 <dbl [10]>
#>  8    10      91.9       91.9  91.9 <dbl [10]>
#>  9    10      93.7       93.4  93.4 <dbl [10]>
#> 10    10      90.9       90.4  90.4 <dbl [10]>
#> # … with 109,990 more rows
```


The distribution of the tolerance limits for each sample size is as follows.
Once again, we see that the distribution of tolerance limits is nearly
identical when each of the three sets of factors are used.


```r
sim_weibull %>%
  pivot_longer(cols = b_pre_080:b_cmh, names_to = "Factors") %>%
  ggplot(aes(x = value, color = Factors)) +
  geom_density() +
  facet_wrap(n ~ .) +
  theme_bw() +
  ggtitle("Distribution of Tolerance Limits for Various Values of n")
```

![plot of chunk distribution-Weibull](distribution-Weibull-1.png)

The true population quantile can be calculated as follows:


```r
x_p_weibull <- qweibull(0.9, shape_weibull, scale_weibull, lower.tail = FALSE)
x_p_weibull
#> [1] 96.31885
```

The proportion of calculated tolerance limit values that are below the
population quantile can be calculated as follows. We see that the in all
roughly 95% or more of the tolerance limits calculated for each sample
is below the population quantile. We also see very similar proportions
for each of the three sets of factors considered.


```r
sim_weibull %>%
  mutate(below_pre_080 = b_pre_080 < x_p_weibull,
         below_post_080 = b_post_080 < x_p_weibull,
         below_cmh = b_cmh < x_p_weibull) %>%
  group_by(n) %>%
  summarise(
    prop_below_pre_080 = sum(below_pre_080) / n(),
    prop_below_post_080 = sum(below_post_080) / n(),
    prop_below_cmh = sum(below_cmh) / n()
  )
#> # A tibble: 11 × 4
#>        n prop_below_pre_080 prop_below_post_080 prop_below_cmh
#>    <dbl>              <dbl>               <dbl>          <dbl>
#>  1    10              0.97                0.965          0.965
#>  2    13              0.966               0.964          0.964
#>  3    16              0.959               0.959          0.959
#>  4    17              0.961               0.961          0.96 
#>  5    19              0.957               0.956          0.956
#>  6    20              0.955               0.954          0.955
#>  7    22              0.953               0.952          0.952
#>  8    23              0.950               0.950          0.950
#>  9    24              0.953               0.953          0.953
#> 10    27              0.952               0.951          0.951
#> 11    28              0.950               0.950          0.950
```


# Conclusion
The values of $j$ and $z$ computed by the `kh_Ext_z_j_opt` function differs
for certain samples sizes ($n$) before and after version 0.8.0. Furthermore,
for certain sample sizes, these values differ from those published in
CMH-17-1G. The simulation study presented in this vignette shows that
the tolerance limit (Basis value) might differ for any individual sample
based on which set of $j$ and $z$ are used. However, each set of factors
produces tolerance limit factors that are either correct or conservative.
These three methods have very similar performance, and tolerance limits
produced with any of these three methods are equally valid.

# Session Info
This vignette is computed in advance. A system with the following configuration
was used:


```r
sessionInfo()
#> R version 4.1.1 (2021-08-10)
#> Platform: x86_64-pc-linux-gnu (64-bit)
#> Running under: Ubuntu 20.04.3 LTS
#> 
#> Matrix products: default
#> BLAS:   /usr/lib/x86_64-linux-gnu/blas/libblas.so.3.9.0
#> LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.9.0
#> 
#> locale:
#>  [1] LC_CTYPE=en_CA.UTF-8       LC_NUMERIC=C               LC_TIME=en_CA.UTF-8        LC_COLLATE=en_CA.UTF-8    
#>  [5] LC_MONETARY=en_CA.UTF-8    LC_MESSAGES=en_CA.UTF-8    LC_PAPER=en_CA.UTF-8       LC_NAME=C                 
#>  [9] LC_ADDRESS=C               LC_TELEPHONE=C             LC_MEASUREMENT=en_CA.UTF-8 LC_IDENTIFICATION=C       
#> 
#> attached base packages:
#> [1] stats     graphics  grDevices utils     datasets  methods   base     
#> 
#> other attached packages:
#> [1] tidyr_1.1.4   purrr_0.3.4   ggplot2_3.3.5 dplyr_1.0.7  
#> 
#> loaded via a namespace (and not attached):
#>  [1] tidyselect_1.1.1  xfun_0.26         remotes_2.4.0     colorspace_2.0-2  vctrs_0.3.8       generics_0.1.0   
#>  [7] testthat_3.0.4    htmltools_0.5.2   usethis_2.0.1     yaml_2.2.1        utf8_1.2.2        rlang_0.4.11.9001
#> [13] pkgbuild_1.2.0    pillar_1.6.3      glue_1.4.2        withr_2.4.2       DBI_1.1.1         waldo_0.3.1      
#> [19] cmstatr_0.9.0     sessioninfo_1.1.1 lifecycle_1.0.1   stringr_1.4.0     munsell_0.5.0     gtable_0.3.0     
#> [25] devtools_2.4.2    memoise_2.0.0     evaluate_0.14     labeling_0.4.2    knitr_1.35        callr_3.7.0      
#> [31] fastmap_1.1.0     ps_1.6.0          curl_4.3.1        fansi_0.5.0       highr_0.9         scales_1.1.1     
#> [37] kSamples_1.2-9    cachem_1.0.6      desc_1.4.0        pkgload_1.2.2     farver_2.1.0      fs_1.5.0         
#> [43] digest_0.6.28     stringi_1.7.4     processx_3.5.2    SuppDists_1.1-9.5 rprojroot_2.0.2   grid_4.1.1       
#> [49] cli_3.0.1         tools_4.1.1       magrittr_2.0.1    tibble_3.1.4      crayon_1.4.1      pkgconfig_2.0.3  
#> [55] MASS_7.3-54       ellipsis_0.3.2    prettyunits_1.1.1 assertthat_0.2.1  rmarkdown_2.11    rstudioapi_0.13  
#> [61] R6_2.5.1          compiler_4.1.1
```


# References

---
title: "cmstatr Tutorial"
author: "Stefan Kloppenborg"
date: "1-Apr-2020"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{cmstatr Tutorial}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

# If any of the required packages are unavailable,
# don't re-run the code
required <- c("dplyr", "ggplot2", "tidyr", "cmstatr", "purrr")
if (!all(unlist(lapply(required, function(pkg) {
    requireNamespace(pkg, quietly = TRUE)}
  )))) {
  knitr::opts_chunk$set(eval = FALSE)
}
```

`cmstatr` is an R package for analyzing composite material data for use in the
aerospace industry. The statistical methods are based on those published in
[CMH-17-1G](https://www.cmh17.org/). This package is intended to facilitate
reproducible statistical analysis of composite materials. In this tutorial,
we'll explore the basic functionality of `cmstatr`.

Before we can actually use the package, we'll need to load it. We'll also load
the `dplyr` package, which we'll talk about shortly. There are also a few other
packages that we'll load. These could all be loaded by loading the
`tidyverse` package instead.

```{r message=FALSE}
library(cmstatr)
library(dplyr)
library(ggplot2)
library(tidyr)
library(purrr)
```

# Input Data
`cmstatr` is built with the assumption that the data is in (so called)
[tidy data](http://vita.had.co.nz/papers/tidy-data.html) format. This means
that the data is in a data frame and that each observation (i.e. test result)
has its own row and that each variable has its own column. Included in this
package is a sample composite material data set (this data set is fictional:
don't use it for anything other than learning this package). The data set
`carbon.fabric.2` has the expected format. We'll just show the first 10 rows
of the data for now.

```{r}
carbon.fabric.2 %>%
  head(10)
```

If your data set is not yet in this type of format (note: that the column
names *do not* need to match the column names in the example), there are
many ways to get it into this format. One of the easier ways of doing so
is to use the [`tidyr`](https://tidyr.tidyverse.org/) package. The use of this
package is outside the scope of this vignette.

# Working With Data
Throughout this vignette, we will be using some of the `tidyverse` tools for
working with data. There are several ways to work with data in R, but in the
opinion of the author of this vignette, the `tidyverse` provides the easiest
way to do so. As such, this is the approach used in this vignette. Feel free
to use whichever approach works best for you.

# Normalizing Data to Cured Ply Thickness
Very often, you'll want to normalize as-measured strength data to a nominal
cured ply thickness for fiber-dominated properties. Very often, this will
reduce the apparent variance in the data. The `normalize_ply_thickness`
function can be used to normalize strength or modulus data to a certain
cured ply thickness. This function takes three arguments: the value to
normalize (i.e.. strength or modulus), the measured thickness and the
nominal thickness. In our case, the nominal cured ply thickness of the
material is $0.0079$. We can then normalize the warp-tension and
fill-compression data as follows:

```{r}
norm_data <- carbon.fabric.2 %>%
  filter(test == "WT" | test == "FC") %>%
  mutate(strength.norm = normalize_ply_thickness(strength,
                                                 thickness / nplies,
                                                 0.0079))

norm_data %>%
  head(10)
```

# Calculating Single-Point Basis Value
The simplest thing that you will likely do is to calculate a basis value based
of a set of numbers that you consider as unstructured data. An example of this
would be calculating the B-Basis of the `RTD` warp tension (`WT`) data.

There are a number of diagnostic tests that we should run before actually
calculating a B-Basis value. We'll talk about those later, but for now, let's
just get right to checking how the data are distributed and calculating the
B-Basis.

We'll use an Anderson--Darling test to check if the data are normally
distributed. The `cmstatr` package provides the function
`anderson_darling_normal` and related functions for other distributions.
We can run an Anderson--Darling test for normality on the warp tension RTD
data as follows. We'll perform this test on the normalized strength.

```{r}
norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  anderson_darling_normal(strength.norm)
```

```{r include=FALSE}
# Verify that the AD test always provides the same conclusion
# If this assertion fails, the Vignette needs to be re-written
if (0.05 >= (norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  anderson_darling_normal(strength.norm))$osl) {
  stop("Unexpected vale for Anderson-Darling test")
  }
```

Now that we know that this data follows a normal distribution (since the
observed significance level (OSL) of the Anderson--Darling test is
greater than $0.05$), we can
proceed to calculate a basis value based based on the assumption of normally
distributed data. The `cmstatr` package provides the function `basis_normal`
as well as related functions for other distributions. By default, the B-Basis
value is calculated, but other population proportions and confidence bounds
can be specified (for example, specify `p = 0.99, conf = 0.99` for A-Basis).

```{r}
norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  basis_normal(strength.norm)
```

We see that the calculated B-Basis is $129.96$. We also see two messages
issued by the `cmstatr` package. These messages relate to the automated
diagnostic tests performed by the basis calculation functions. In this case
we see messages that two of the diagnostic tests were not performed because
we didn't specify the batch of each observation. The batch is not required
for calculating single-point basis values, but it is required for performing
batch-to-batch variability and within-batch outlier diagnostic tests.

The `basis_normal` function performs the following diagnostic tests by default:

- Within batch outliers using `maximum_normed_residual()`
- Between batch variability using `ad_ksample()`
- Outliers using `maximum_normed_residual()`
- Normality of data using `anderson_darling_normal()`

There are two ways that we can deal with the two messages that we see. We can
pass in a column that specifies the batch for each observation, or we can
override those two diagnostic tests so that `cmstatr` doesn't run them.

To override the two diagnostic tests, we set the argument `override` to a list
of the names of the diagnostic tests that we want to skip. The names of the
diagnostic tests that were not run are shown between back-ticks (\`) in the
message. Our call to `basis_normal()` would be updated as follows:

```{r}
norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  basis_normal(strength.norm, 
               override = c("outliers_within_batch",
                            "between_batch_variability"))
```

Obviously, you should be cautious about overriding the diagnostic tests.
There are certainly times when it is appropriate to do so, but sound
engineering judgment is required.

The better approach would be to specify the batch. This can be done as
follows:

```{r}
norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  basis_normal(strength.norm, batch)
```

Now that batch is specified, we see that one of the diagnostic tests
actually fails: the Anderson--Darling k-Sample test shows that the batches
are not drawn from the same (unspecified) distribution. We can run this
diagnostic test directly to investigate further:

```{r}
norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  ad_ksample(strength.norm, batch)
```

For the Anderson--Darling k-Sample test, $\alpha=0.025$ is normally used.
In this case the p-value is $p=0.0026$, so it is no where near $\alpha$
(note the number of decimal places).

We can plot the distribution of this data and make a judgment call about
whether to continue.


```{r}
norm_data %>%
  filter(test == "WT" & condition == "RTD") %>%
  group_by(batch) %>%
  ggplot(aes(x = strength.norm, color = batch)) +
  stat_normal_surv_func() +
  stat_esf() +
  ggtitle("Distribution of Data For Each Batch")
```

We can also run the other diagnostic test by themselves. These are described
in more detail in the following sections.

# Calculating Basis Values by Pooling Across Environments
In this section, we'll use the fill-compression data from the `carbon.fabric.2`
data set.

## Checking for Outliers
After checking that there are a sufficient number of conditions, batches and
specimens and that the failure modes are consistent, we would normally
check if there are outliers within each batch and condition. The maximum
normed residual test can be used for this. The `cmstatr` package provides the
function `maximum_normed_residual` to do this. First, we'll group the data
by condition and batch, then run the test on each group. The
`maximum_normed_residual` function returns an object that contains a number
of values. We'll create a `data.frame` that contains those values.

In order to do this, we need to use the `nest` function from the `tidyr`
package. This is explained in detail
[here](https://tidyr.tidyverse.org/articles/nest.html). Basically,
`nest` allows a column of `list`s or a column of `data.frame`s to be
added to a `data.frame`. Once nested, we can use the `glance` method
to unpack the values returned by `maximum_normed_residual` into a
one-row `data.frame`, and then use `unnest` to flatten this into a
single `data.frame`.


```{r}
norm_data %>%
  filter(test == "FC") %>%
  group_by(condition, batch) %>%
  nest() %>%
  mutate(mnr = map(data,
                   ~maximum_normed_residual(data = .x, x = strength.norm)),
         tidied = map(mnr, glance)) %>%
  select(-c(mnr, data)) %>%  # remove unneeded columns
  unnest(tidied)
```

```{r include=FALSE}
if ((norm_data %>%
  filter(test == "FC") %>%
  group_by(condition, batch) %>%
  summarise(
    n_outliers = maximum_normed_residual(x = strength.norm)$n_outliers
    ) %>%
  ungroup() %>%
  summarise(n_outliers = sum(n_outliers)))[[1]] != 0) {
  stop("Unexpected number of outliers")
  }
```

None of the groups have outliers, so we can continue.

# Batch-to-Batch Distribution
Next, we will use the Anderson--Darling k-Sample test to check that each batch
comes from the same distribution within each condition. We can use the
`ad_ksample` function from `cmstatr` to do so. Once again, we'll use
`nest`/`unnest` and `glance` to do so.

```{r}
norm_data %>%
  filter(test == "FC") %>%
  group_by(condition) %>%
  nest() %>%
  mutate(adk = map(data, ~ad_ksample(data = .x,
                                     x = strength.norm,
                                     groups = batch)),
         tidied = map(adk, glance)) %>%
  select(-c(data, adk)) %>%  # remove unneeded columns
  unnest(tidied)
```

```{r include=FALSE}
if (!all(!(norm_data %>%
  filter(test == "FC") %>%
  group_by(condition) %>%
  summarise(different_dist =
           ad_ksample(x = strength.norm, groups = batch)$reject_same_dist
  ))$different_dist)) {
  stop("Unexpected ADK result")
  }
```

For all conditions, the Anderson--Darling k-Sample test fails to reject the
hypothesis that each batch comes from the same (unspecified) distribution.
We can thus proceed to pooling the data.

## Checking for Outliers Within Each Condition
Just as we did when checking for outlier within each condition and each
batch, we can pool all the batches (within each condition) and check
for outliers within each condition.

```{r}
norm_data %>%
  filter(test == "FC") %>%
  group_by(condition) %>%
  nest() %>%
  mutate(mnr = map(data, ~maximum_normed_residual(data = .x,
                                                  x = strength.norm)),
         tidied = map(mnr, glance)) %>%
  select(-c(mnr, data)) %>%  # remove unneeded columns
  unnest(tidied)
```

```{r include=FALSE}
if ((norm_data %>%
  filter(test == "FC") %>%
  group_by(condition) %>%
  summarise(
    n_outliers = maximum_normed_residual(x = strength.norm)$n_outliers
    ) %>%
  ungroup() %>%
  summarise(n_outliers = sum(n_outliers)))[[1]] != 0) {
  stop("Unexpected number of outliers")
  }
```

We find no outliers, so we can continue.

## Pooling Across Environments
Often it is desirable to pool data across several environments. There
are two methods for doing so: "pooled standard deviation" and
"pooled CV" (CV is an abbreviation for Coefficient of Variation)

First, we will check for
equality of variance among the conditions. We will do so using Levene's test.
The `cmstatr` package provides the function `levene_test` to do so.

```{r}
norm_data %>%
  filter(test == "FC") %>%
  levene_test(strength.norm, condition)
```

```{r include=FALSE}
if (!(norm_data %>%
  filter(test == "FC") %>%
  levene_test(strength.norm, condition))$reject_equal_variance) {
  stop("Unexpected result from Levene's test")
  }
```

The result from Levene's test indicates that the variance for each condition
is not equal. This indicates that the data cannot be pooled using the
"pooled standard deviation" method.

We can check if the data can be pooled using the "pooled CV" method.
We'll start by normalizing the data from each group to the group's mean.
The `cmstatr` package provides the function `normalize_group_mean` for
this purpose.

```{r}
norm_data %>%
  filter(test == "FC") %>%
  mutate(
    strength_norm_group = normalize_group_mean(strength.norm, condition)) %>%
  levene_test(strength_norm_group, condition)
```

```{r include=FALSE}
if ((norm_data %>%
  filter(test == "FC") %>%
  mutate(
    strength_norm_group = normalize_group_mean(strength.norm, condition)) %>%
  levene_test(strength_norm_group, condition))$reject_equal_variance) {
  stop("Unexpected value from Levene's test")
  }
```

The Levene's test thus shows the variances of the pooled data are equal.
We can move on to performing an Anderson--Darling test for normality on
the pooled data.

```{r}
norm_data %>%
  filter(test == "FC") %>%
  mutate(
    strength_norm_group = normalize_group_mean(strength.norm, condition)) %>%
  anderson_darling_normal(strength_norm_group)
```

```{r include=FALSE}
if ((norm_data %>%
  filter(test == "FC") %>%
  mutate(
    strength_norm_group = normalize_group_mean(strength.norm, condition)) %>%
  anderson_darling_normal(strength_norm_group))$osl <= 0.05) {
  stop("Unexpected value from AD test")
  }
```

The Anderson--Darling test indicates that the pooled data is drawn from
a normal distribution, so we can continue with calculating basis values
using the "pooled CV" method.

```{r}
norm_data %>%
  filter(test == "FC") %>%
  basis_pooled_cv(strength.norm, condition, batch)
```

The conditions listed in the output above are in alphabetical order. This
probably isn't what you want. Instead, you probably want the conditions
listed in a certain order. This can be done by ordering the data first as
demonstrated below. You're probably just do this one in at the start of
your analysis.

```{r}
norm_data %>%
  mutate(condition = ordered(condition,
                             c("CTD", "RTD", "ETD", "ETW", "ETW2"))) %>%
  filter(test == "FC") %>%
  basis_pooled_cv(strength.norm, condition, batch)
```

# Equivalency
Eventually, once you've finished calculating all your basis values,
you'll probably want to set specification requirements or evaluate
site/process equivalency. `cmstatr` has functionality to do both.

Let's say that you want to develop specification limits for fill compression
that you're going to put in your material specification. You can do this
as follows:

```{r}
carbon.fabric.2 %>%
  filter(test == "FC" & condition == "RTD") %>%
  equiv_mean_extremum(strength, n_sample = 5, alpha = 0.01)
```

If you're determining equivalency limits for modulus, a different
approach is generally used so that bilateral limits are set. `cmstatr`
can do this as well, using the function `equiv_change_mean`.

---
title: "Anderson-Darling k-Sample Test"
author: "Stefan Kloppenborg"
date: "20-Jan-2019"
output: rmarkdown::html_vignette
bibliography: bibliography.json
csl: ieee.csl
vignette: >
  %\VignetteIndexEntry{Anderson-Darling k-Sample Test}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

This vignette explores the Anderson--Darling k-Sample test.
CMH-17-1G [@CMH-17-1G] provides a formulation for this test that appears different than the formulation given by Scholz and Stephens in their 1987 paper [@Stephens1987].

Both references use different nomenclature, which is summarized as follows:

Term                                               | CMH-17-1G             | Scholz and Stephens
---------------------------------------------------|-----------------------|---------------------
A sample                                           | $i$                   | $i$
The number of samples                              | $k$                   | $k$
An observation within a sample                     | $j$                   | $j$
The number of observations within the sample $i$   | $n_i$                 | $n_i$
The total number of observations within all samples| $n$                   | $N$
Distinct values in combined data, ordered          | $z_{(1)}$...$z_{(L)}$ | $Z_1^*$...$Z_L^*$
The number of distinct values in the combined data | $L$                   | $L$


Given the possibility of ties in the data, the discrete version of the test must be used
Scholz and Stephens (1987) give the test statistic as:

$$
A_{a k N}^2 = \frac{N - 1}{N}\sum_{i=1}^k \frac{1}{n_i}\sum_{j=1}^{L}\frac{l_j}{N}\frac{\left(N M_{a i j} - n_i B_{a j}\right)^2}{B_{a j}\left(N - B_{a j}\right) - N l_j / 4}
$$


CMH-17-1G gives the test statistic as:

$$
ADK = \frac{n - 1}{n^2\left(k - 1\right)}\sum_{i=1}^k\frac{1}{n_i}\sum_{j=1}^L h_j \frac{\left(n F_{i j} - n_i H_j\right)^2}{H_j \left(n - H_j\right) - n h_j / 4}
$$

By inspection, the CMH-17-1G version of this test statistic contains an extra factor of $\frac{1}{\left(k - 1\right)}$.

Scholz and Stephens indicate that one rejects $H_0$ at a significance level of $\alpha$ when:

$$
\frac{A_{a k N}^2 - \left(k - 1\right)}{\sigma_N} \ge t_{k - 1}\left(\alpha\right)
$$

This can be rearranged to give a critical value:

$$
A_{c r i t}^2 = \left(k - 1\right) + \sigma_N t_{k - 1}\left(\alpha\right)
$$

CHM-17-1G gives the critical value for $ADK$ for $\alpha=0.025$ as:

$$
ADC = 1 + \sigma_n \left(1.96 + \frac{1.149}{\sqrt{k - 1}} - \frac{0.391}{k - 1}\right)
$$

The definition of $\sigma_n$ from the two sources differs by a factor of $\left(k - 1\right)$.

The value in parentheses in the CMH-17-1G critical value corresponds to the interpolation formula for $t_m\left(\alpha\right)$ given in Scholz and Stephen's paper.
It should be noted that this is *not* the student's t-distribution, but rather a distribution referred to as the $T_m$ distribution.

The `cmstatr` package use the package `kSamples` to perform the k-sample Anderson--Darling tests.
This package uses the original formulation from Scholz and Stephens, so the test statistic will differ from that given software based on the CMH-17-1G formulation by a factor of $\left(k-1\right)$.
The conclusions about the null hypothesis drawn, however, will be the same.

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/basis.R
\name{k_factor_normal}
\alias{k_factor_normal}
\title{Calculate k factor for basis values (\eqn{kB}, \eqn{kA}) with normal
distribution}
\usage{
k_factor_normal(n, p = 0.9, conf = 0.95)
}
\arguments{
\item{n}{the number of observations (i.e. coupons)}

\item{p}{the desired content of the tolerance bound.
Should be 0.90 for B-Basis and 0.99 for A-Basis}

\item{conf}{confidence level. Should be 0.95 for both A- and B-Basis}
}
\value{
the calculated factor
}
\description{
The factors returned by this function are used when calculating basis
values (one-sided confidence bounds) when the data are normally
distributed. The basis value will
be equal to \eqn{\bar{x} - k s}{x_bar - k s},
where \eqn{\bar{x}}{x_bar} is the sample mean,
\eqn{s} is the sample standard deviation and \eqn{k} is the result
of this function.
This function is internally used by \code{\link{basis_normal}} when
computing basis values.
}
\details{
This function calculates the k factors used when determining A- and
B-Basis values for normally distributed data. To get \eqn{kB}, set
the content of the tolerance bound to \code{p = 0.90} and
the confidence level to \code{conf = 0.95}. To get \eqn{kA}, set
\code{p = 0.99} and \code{conf = 0.95}. While other tolerance bound
contents and confidence levels may be computed, they are infrequently
needed in practice.

The k-factor is calculated using equation 2.2.3 of
Krishnamoorthy and Mathew (2008).

This function has been validated against the \eqn{kB} tables in
CMH-17-1G for each value of \eqn{n} from \eqn{n = 2} to \eqn{n = 95}.
It has been validated against the \eqn{kA} tables in CMH-17-1G for each
value of \eqn{n} from \eqn{n = 2} to \eqn{n = 75}. Larger values of \eqn{n}
also match the tables in CMH-17-1G, but R
emits warnings that "full precision may not have been achieved." When
validating the results of this function against the tables in CMH-17-1G,
the maximum allowable difference between the two is 0.002. The tables in
CMH-17-1G give values to three decimal places.

For more information about tolerance bounds in general, see
Meeker, et. al. (2017).
}
\examples{
kb <- k_factor_normal(n = 10, p = 0.9, conf = 0.95)
print(kb)

## [1] 2.35464

# This can be used to caclulate the B-Basis if
# the sample mean and sample standard deviation
# is known, and data is assumed to be normally
# distributed

sample_mean <- 90
sample_sd <- 5.2
print("B-Basis:")
print(sample_mean - sample_sd * kb)

## [1] B-Basis:
## [1] 77.75587

}
\references{
K. Krishnamoorthy and T. Mathew, Statistical Tolerance Regions: Theory,
Applications, and Computation. Hoboken: John Wiley & Sons, 2008.

W. Meeker, G. Hahn, and L. Escobar, Statistical Intervals: A Guide
for Practitioners and Researchers, Second Edition.
Hoboken: John Wiley & Sons, 2017.

“Composite Materials Handbook, Volume 1. Polymer Matrix Composites
Guideline for Characterization of Structural Materials,” SAE International,
CMH-17-1G, Mar. 2012.
}
\seealso{
\code{\link{basis_normal}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{stat_esf}
\alias{stat_esf}
\title{Empirical Survival Function}
\usage{
stat_esf(
  mapping = NULL,
  data = NULL,
  geom = "point",
  position = "identity",
  show.legend = NA,
  inherit.aes = TRUE,
  n = NULL,
  pad = FALSE,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{aes()}.}

\item{data}{The data to be displayed in this layer. This has the
same usage as a \code{ggplot2} \code{stat} function.}

\item{geom}{The geometric object to use to display the data.}

\item{position}{Position argument}

\item{show.legend}{Should this layer be included in the legends?}

\item{inherit.aes}{If \code{FALSE}, overrides the default aesthetic,
rather than combining with them.}

\item{n}{If \code{NULL}, do not interpolated. Otherwise, the
number of points to interpolate.}

\item{pad}{If \code{TRUE}, pad the ESF with additional points
\verb{(-Inf, 0)} and \verb{(0, Inf)}.}

\item{...}{Other arguments to pass on to \code{layer}.}
}
\description{
The empirical survival function (ESF) provides a visualization of a
distribution. This is closely related to the empirical cumulative
distribution function (ECDF). The empirical survival function is
simply ESF = 1 - ECDF.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cmstatr.R
\docType{package}
\name{cmstatr-package}
\alias{cmstatr}
\alias{cmstatr-package}
\title{cmstatr: Statistical Methods for Composite Material Data}
\description{
To learn more about \code{cmstatr}, start with the vignettes:
\code{browseVignettes(package = "cmstatr")}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://www.cmstatr.net/}
  \item \url{https://github.com/cmstatr/cmstatr}
  \item Report bugs at \url{https://github.com/cmstatr/cmstatr/issues}
}

}
\author{
\strong{Maintainer}: Stefan Kloppenborg \email{stefan@kloppenborg.ca} (\href{https://orcid.org/0000-0002-1908-5214}{ORCID})

Other contributors:
\itemize{
  \item Billy Cheng \email{bcheng@comtekadvanced.com} [contributor]
  \item Ally Fraser \email{ally.fraser25@gmail.com} [contributor]
  \item Jeffrey Borlik [contributor]
  \item Brice Langston \email{tblangst@gmail.com} [contributor]
  \item Comtek Advanced Structures, Ltd. [funder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/equiv.R
\name{glance.equiv_mean_extremum}
\alias{glance.equiv_mean_extremum}
\title{Glance at an \code{equiv_mean_extremum} object}
\usage{
\method{glance}{equiv_mean_extremum}(x, ...)
}
\arguments{
\item{x}{an equiv_mean_extremum object returned from
\code{\link[=equiv_mean_extremum]{equiv_mean_extremum()}}}

\item{...}{Additional arguments. Not used. Included only to match generic
signature.}
}
\value{
A one-row \code{\link[tibble:tibble]{tibble::tibble()}} with the following
columns:
\itemize{
\item \code{alpha} the value of alpha passed to this function
\item \code{n_sample} the number of observations in the sample for which
equivalency is being checked. This is either the value \code{n_sample}
passed to this function or the length of the vector \code{data_sample}.
\item \code{modcv} logical value indicating whether the acceptance
thresholds are calculated using the modified CV approach
\item \code{threshold_min_indiv} The calculated threshold value for
minimum individual
\item \code{threshold_mean} The calculated threshold value for mean
\item \code{result_min_indiv} a character vector of either "PASS" or
"FAIL" indicating whether the data from \code{data_sample} passes the
test for minimum individual. If \code{data_sample} was not supplied,
this value will be \code{NULL}
\item \code{result_mean} a character vector of either "PASS" or
"FAIL" indicating whether the data from \code{data_sample} passes the
test for mean. If \code{data_sample} was not supplied, this value will
be  \code{NULL}
\item \code{min_sample} The minimum value from the vector
\code{data_sample}. if \code{data_sample} was not supplied, this will
have a value of \code{NULL}
\item \code{mean_sample} The mean value from the vector
\code{data_sample}. If \code{data_sample} was not supplied, this will
have a value of \code{NULL}
}
}
\description{
Glance accepts an object of type \code{equiv_mean_extremum} and returns a
\code{\link[tibble:tibble]{tibble::tibble()}} with
one row of summaries.

Glance does not do any calculations: it just gathers the results in a
tibble.
}
\examples{
x0 <- rnorm(30, 100, 4)
x1 <- rnorm(5, 91, 7)
eq <- equiv_mean_extremum(data_qual = x0, data_sample = x1, alpha = 0.01)
glance(eq)

## # A tibble: 1 x 9
##   alpha n_sample modcv threshold_min_indiv threshold_mean
##   <dbl>    <int> <lgl>               <dbl>          <dbl>
## 1  0.01        5 FALSE                86.2           94.9
## # ... with 4 more variables: result_min_indiv <chr>, result_mean <chr>,
## #   min_sample <dbl>, mean_sample <dbl>

}
\seealso{
\code{\link[=equiv_mean_extremum]{equiv_mean_extremum()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/equiv.R
\name{k_equiv}
\alias{k_equiv}
\title{k-factors for determining acceptance based on sample mean and an extremum}
\usage{
k_equiv(alpha, n)
}
\arguments{
\item{alpha}{the acceptable probability of a type I error}

\item{n}{the number of observations in the sample to test}
}
\value{
a vector with elements c(k1, k2). k1 is for testing the sample
extremum. k2 is for testing the sample mean
}
\description{
k-factors for determining acceptance based on sample mean and an extremum
}
\details{
The k-factors returned by this function are used for determining
whether to accept a new dataset.

This function is used as part of the procedure for
determining acceptance limits for a sample mean and sample minimum.
These acceptance limits are often used to set acceptance limits for
material strength for each lot of material, or each new manufacturing
site. When a sample meets the criteria that its mean and its minimum are
both greater than these limits, then one may accept the lot of material
or the new manufacturing site.

This procedure is used to ensure that the strength of material processed
at a second site, or made with a new batch of material are not degraded
relative to the data originally used to determine basis values for the
material. For more information about the use of this procedure, see
CMH-17-1G or PS-ACE 100-2002-006.

According to Vangel (2002), the use of mean and extremum for this purpose
is more powerful than the use of mean and standard deviation.

The results of this function match those published by Vangel within
0.05\\% for \eqn{n > 2} and \eqn{\alpha \le 0.25}. Those results published
by Vangel are identical to those published in CMH-17-1G.

This function uses numerical integration and numerical optimization to
find values of the factors \eqn{k_1} and \eqn{k_2} based on Vangel's
saddle point approximation.

The value \eqn{n} refers to the number of observations in the sample
being compared with the original population (the qualification sample is
usually assumed to be equal to the population statistics).

The value of \eqn{alpha} is the acceptable probability of a type I error.
Normally, this is set to 0.05 for material or process equivalency and 0.01
when setting lot acceptance limits. Though, in principle, this parameter
can be set to any number between 0 and 1. This function, however, has only
been validated in the range of \eqn{1e-5 \le alpha \le 0.5}.
}
\examples{
qual_mean <- 100
qual_sd <- 3.5
k <- k_equiv(0.01, 5)
print("Minimum Individual Acceptance Limit:")
print(qual_mean - qual_sd * k[1])
print("Minimum Average Acceptance Limit:")
print(qual_mean - qual_sd * k[2])

## [1] "Minimum Individual Acceptance Limit:"
## [1] 89.24981
## [1] "Minimum Average Acceptance Limit:"
## [1] 96.00123

}
\references{
M. G. Vangel. Lot Acceptance and Compliance Testing Using the Sample Mean
and an Extremum, Technometrics, vol. 44, no. 3. pp. 242–249. 2002.

“Composite Materials Handbook, Volume 1. Polymer Matrix Composites
Guideline for Characterization of Structural Materials,” SAE International,
CMH-17-1G, Mar. 2012.

Federal Aviation Administration, “Material Qualification and Equivalency
for Polymer Matrix Composite Material Systems,” PS-ACE 100-2002-006,
Sep. 2003.
}
\seealso{
\code{\link[=equiv_mean_extremum]{equiv_mean_extremum()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/norm.R
\name{normalize_ply_thickness}
\alias{normalize_ply_thickness}
\title{Normalizes strength values to ply thickness}
\usage{
normalize_ply_thickness(strength, measured_thk, nom_thk)
}
\arguments{
\item{strength}{the strength to be normalized. Either a vector or a numeric}

\item{measured_thk}{the measured thickness of the samples. Must be the same
length as strength}

\item{nom_thk}{the nominal thickness. Must be a single numeric value.}
}
\value{
The normalized strength values
}
\description{
This function takes a vector of strength values and a
vector of measured thicknesses, and a nominal thickness
and returns the normalized strength.
}
\details{
It is often necessary to normalize strength values so that variation in
specimen thickness does not unnecessarily increase variation in strength.
See CMH-17-1G, or other references, for information about the cases where
normalization is appropriate.

Either cured ply thickness or laminate thickness may be used for
\code{measured_thk} and \code{nom_thk}, as long as the same decision
made for both values.

The formula applied is:
\deqn{normalized\,value = test\,value \frac{t_{measured}}{t_{nominal}}}{
normalized value = test value * t_measured / t_nominal}

If you need to normalize based on fiber volume fraction (or another method),
you will first need to calculate the nominal cured ply thickness (or laminate
thickness). Those calculations are outside the scope of this documentation.
}
\examples{
library(dplyr)

carbon.fabric.2 \%>\%
select(thickness, strength) \%>\%
  mutate(normalized_strength = normalize_ply_thickness(strength,
                                                       thickness,
                                                       0.105)) \%>\%
  head(10)

##    thickness strength normalized_strength
## 1      0.112  142.817            152.3381
## 2      0.113  135.901            146.2554
## 3      0.113  132.511            142.6071
## 4      0.112  135.586            144.6251
## 5      0.113  125.145            134.6799
## 6      0.113  135.203            145.5042
## 7      0.113  128.547            138.3411
## 8      0.113  127.709            137.4392
## 9      0.113  127.074            136.7558
## 10     0.114  126.879            137.7543

}
\references{
“Composite Materials Handbook, Volume 1. Polymer Matrix Composites
Guideline for Characterization of Structural Materials,” SAE International,
CMH-17-1G, Mar. 2012.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot-failure-modes.R
\name{separate_failure_modes}
\alias{separate_failure_modes}
\title{Separate multiple failure modes into multiple rows}
\usage{
separate_failure_modes(data, failure_mode, sep = "[/, ]+")
}
\arguments{
\item{data}{a \code{data.frame}}

\item{failure_mode}{the column in \code{data} containing the failure modes}

\item{sep}{a regular expression with the character(s) separating individual
failure modes. Default \code{"[/, ]+"}.}
}
\description{
For a \code{data.frame} containing a column with (some) multiple failure modes,
this function expands the \code{data.frame} by repeating each row with multiple
failure modes so that each row contains only a single failure mode.
}
\details{
When multiple failure modes are reported, they are commonly reported in
the format "LGM/GIT" or "LGM,GIT". This function will separate these multiple
failure modes into multiple rows.

This can be useful when counting the number of coupons exhibited each
failure mode.
}
\examples{
library(dplyr)
data.frame(strength = c(101, 102), fm = c("LGM/GIT", "LGM")) \%>\%
  separate_failure_modes(fm)
##
## # A tibble: 3 × 2
##   strength fm
##      <dbl> <chr>
## 1      101 LGM
## 2      101 GIT
## 3      102 LGM

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot-failure-modes.R
\name{geom_jitter_failure_mode}
\alias{geom_jitter_failure_mode}
\title{Jittered points showing (possibly multiple) failure modes}
\usage{
geom_jitter_failure_mode(
  mapping = NULL,
  data = NULL,
  stat = "identity",
  position = "jitter",
  ...,
  width = NULL,
  height = NULL,
  na.rm = FALSE,
  show.legend = NA,
  inherit.aes = TRUE,
  sep = "[/, ]+"
)
}
\arguments{
\item{mapping}{Set of aesthetic mapping created by \code{aes()}.
See \code{\link[ggplot2:geom_jitter]{ggplot2::geom_jitter()}} for additional details.}

\item{data}{The data to be displayed by this layer.
See \code{\link[ggplot2:geom_jitter]{ggplot2::geom_jitter()}} for additional details.}

\item{stat}{The statistical transformation to use on the data for this layer.
See \code{\link[ggplot2:geom_jitter]{ggplot2::geom_jitter()}} for additional details.}

\item{position}{A position adjustment to use on the data for this layer.
See \code{\link[ggplot2:geom_jitter]{ggplot2::geom_jitter()}} for additional details.}

\item{...}{Other arguments passed on to \code{layer()}'s \code{params} argument.
See \code{\link[ggplot2:geom_jitter]{ggplot2::geom_jitter()}} for additional details.}

\item{width}{The amount of horizontal jitter.
See \code{\link[ggplot2:geom_jitter]{ggplot2::geom_jitter()}} for additional details.}

\item{height}{The amount of vertical jitter.
See \code{\link[ggplot2:geom_jitter]{ggplot2::geom_jitter()}} for additional details.}

\item{na.rm}{If FALSE, the default, missing values are removed with warning.
See \code{\link[ggplot2:geom_jitter]{ggplot2::geom_jitter()}} for additional details.}

\item{show.legend}{\code{NA}, the default, indicates that nay aesthetics are
mapped.
See \code{\link[ggplot2:geom_jitter]{ggplot2::geom_jitter()}} for additional details.}

\item{inherit.aes}{if FALSE, overrides the default aesthetics.
See \code{\link[ggplot2:geom_jitter]{ggplot2::geom_jitter()}} for additional details.}

\item{sep}{A regular expression indicating the character(s) separating
multiple failure modes. By default \code{"[/, ]+"}}
}
\description{
The \code{geom_jitter_failure_mode} is very similar to
\code{\link[ggplot2:geom_jitter]{ggplot2::geom_jitter()}} except that a failure mode variable specified
as the color and/or shape aesthetic is parsed to separate multiple
failure modes and plot them separately. For example, if an observation
has the failure mode "LAT/LAB", two points will be plotted, one with the
failure mode "LAT" and the second with the failure mode "LAB".
}
\details{
The variable specified for the aesthetics \code{shape} and \code{color} are passed
to the function \code{\link[=separate_failure_modes]{separate_failure_modes()}} to parse the failure modes and
separate multiple failure modes separated by character(s) specified in
the regular expression given in the parameter \code{sep}. By default, multiple
failure modes are expected to be separated by spaces, commas or forward
slashes, but this can be overridden.

If both \code{shape} and \code{color} aesthetics are specified, both must be identical.
}
\examples{
library(dplyr)
library(ggplot2)
carbon.fabric.2 \%>\%
  filter(test == "WT") \%>\%
  ggplot(aes(x = condition, y = strength)) +
  geom_boxplot() +
  geom_jitter_failure_mode(aes(color = failure_mode, shape = failure_mode))

}
\seealso{
\code{\link[=separate_failure_modes]{separate_failure_modes()}}

\code{\link[ggplot2:geom_jitter]{ggplot2::geom_jitter()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/norm.R
\name{transform_mod_cv}
\alias{transform_mod_cv}
\alias{transform_mod_cv_ad}
\title{Transforms data according to the modified CV rule}
\usage{
transform_mod_cv_ad(x, condition, batch)

transform_mod_cv(x)
}
\arguments{
\item{x}{a vector of data to transform}

\item{condition}{a vector indicating the condition to which each
observation belongs}

\item{batch}{a vector indicating the batch to which each observation
belongs}
}
\value{
A vector of transformed data
}
\description{
Transforms data according to the modified coefficient of variation (CV)
rule. This is used to add additional variance to datasets with
unexpectedly low variance, which is sometimes encountered during
testing of new materials over short periods of time.

Two versions of this transformation are implemented. The first version,
\code{transform_mod_cv()}, transforms the data in a single group (with
no other structure) according to the modified CV rules.

The second
version, \code{transform_mod_cv_ad()}, transforms data that is structured
according to both condition and batch, as is commonly done for
the Anderson--Darling k-Sample and Anderson-Darling tests when pooling
across environments.
}
\details{
The modified CV transformation takes the general form:

\deqn{\frac{S_i^*}{S_i} (x_{ij} - \bar{x_i}) + \bar{x_i}}{
  Si*/Si (xij-x_bar_i) + x_bar_i
}

Where \eqn{S_i^*}{Si*} is the modified standard deviation
(mod CV times mean) for
the \eqn{ith} group; \eqn{S_i}{Si} is the standard deviation
for the \eqn{ith} group, \eqn{\bar{x_i}}{x_bar_i} is
the group mean and \eqn{x_{ij}}{xij} is the observation.

\code{transform_mod_cv()} takes a vector
containing the observations and transforms the data.
The equation above is used, and all observations
are considered to be from the same group.

\code{transform_mod_cv_ad()} takes a vector containing the observations
plus a vector containing the corresponding conditions and a vector
containing the batches. This function first calculates the modified
CV value from the data from each condition (independently). Then,
within each condition, the transformation
above is applied to produce the transformed data \eqn{x'}.
This transformed data is further transformed using the following
equation.

\deqn{x_{ij}'' = C (x'_{ij} - \bar{x_i}) + \bar{x_i}}{
  x_ij'' = C (x'_ij - x_bar_i) + x_bar_i}

Where:

\deqn{C = \sqrt{\frac{SSE^*}{SSE'}}}{C = sqrt(SSE* / SSE')}

\deqn{SSE^* = (n-1) (CV^* \bar{x})^2 - \sum(n_i(\bar{x_i}-\bar{x})^2)}{
  SSE* = (n-1) (CV* x_bar)^2 - sum(n_i(x_bar_i-x_bar)^2)}

\deqn{SSE' = \sum(x'_{ij} - \bar{x_i})^2}{SSE' = sum(x'_ij - x_bar_i)^2}
}
\examples{
# Transform data according to the modified CV transformation
# and report the original and modified CV for each condition

library(dplyr)
carbon.fabric \%>\%
filter(test == "FT") \%>\%
  group_by(condition) \%>\%
  mutate(trans_strength = transform_mod_cv(strength)) \%>\%
  head(10)

## # A tibble: 10 x 6
## # Groups:   condition [1]
##    id         test  condition batch strength trans_strength
##    <chr>      <chr> <chr>     <int>    <dbl>          <dbl>
##  1 FT-RTD-1-1 FT    RTD           1     126.           126.
##  2 FT-RTD-1-2 FT    RTD           1     139.           141.
##  3 FT-RTD-1-3 FT    RTD           1     116.           115.
##  4 FT-RTD-1-4 FT    RTD           1     132.           133.
##  5 FT-RTD-1-5 FT    RTD           1     129.           129.
##  6 FT-RTD-1-6 FT    RTD           1     130.           130.
##  7 FT-RTD-2-1 FT    RTD           2     131.           131.
##  8 FT-RTD-2-2 FT    RTD           2     124.           124.
##  9 FT-RTD-2-3 FT    RTD           2     125.           125.
## 10 FT-RTD-2-4 FT    RTD           2     120.           119.

# The CV of this transformed data can be computed to verify
# that the resulting CV follows the rules for modified CV

carbon.fabric \%>\%
  filter(test == "FT") \%>\%
  group_by(condition) \%>\%
  mutate(trans_strength = transform_mod_cv(strength)) \%>\%
  summarize(cv = sd(strength) / mean(strength),
            mod_cv = sd(trans_strength) / mean(trans_strength))

## # A tibble: 3 x 3
##   condition     cv mod_cv
##   <chr>      <dbl>  <dbl>
## 1 CTD       0.0423 0.0612
## 2 ETW       0.0369 0.0600
## 3 RTD       0.0621 0.0711

}
\seealso{
\code{\link{calc_cv_star}}

\code{\link{cv}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mnr.R
\name{augment.mnr}
\alias{augment.mnr}
\title{Augment data with information from an \code{mnr} object}
\usage{
\method{augment}{mnr}(x, data = x$data, ...)
}
\arguments{
\item{x}{an \code{mnr} object created by
\code{\link[=maximum_normed_residual]{maximum_normed_residual()}}}

\item{data}{a \code{data.frame} or
\code{\link[tibble:tibble]{tibble::tibble()}}
containing the original data that was passed to
\code{maximum_normed_residual}}

\item{...}{Additional arguments. Not used. Included only to match generic
signature.}
}
\value{
When \code{data} is supplied, \code{augment} returns \code{data}, but with
one column appended. When \code{data} is not supplied, \code{augment}
returns a new \code{\link[tibble:tibble]{tibble::tibble()}} with the column
\code{values} containing the original values used by
\code{maximum_normed_residaul} plus one additional column. The additional
column is:
\itemize{
\item \code{.outler} a logical value indicating whether the observation
is an outlier
}
}
\description{
Augment accepts an \code{mnr} object (returned from the function
\code{\link[=maximum_normed_residual]{maximum_normed_residual()}}) and a dataset and adds the column
\code{.outlier} to the dataset. The column \code{.outlier} is a logical
vector indicating whether each observation is an outlier.

When passing data into \code{augment} using the \code{data} argument,
the data must be exactly the data that was passed to
\code{maximum_normed_residual}.
}
\examples{
data <- data.frame(strength = c(80, 98, 96, 97, 98, 120))
m <- maximum_normed_residual(data, strength)

# augment can be called with the original data
augment(m, data)

##   strength .outlier
## 1       80    FALSE
## 2       98    FALSE
## 3       96    FALSE
## 4       97    FALSE
## 5       98    FALSE
## 6      120    FALSE

# or augment can be called without the orignal data and it will be
# reconstructed
augment(m)

## # A tibble: 6 x 2
##   values .outlier
##    <dbl> <lgl>
## 1     80 FALSE
## 2     98 FALSE
## 3     96 FALSE
## 4     97 FALSE
## 5     98 FALSE
## 6    120 FALSE

}
\seealso{
\code{\link[=maximum_normed_residual]{maximum_normed_residual()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mnr.R
\name{maximum_normed_residual}
\alias{maximum_normed_residual}
\title{Detect outliers using the maximum normed residual method}
\usage{
maximum_normed_residual(data = NULL, x, alpha = 0.05)
}
\arguments{
\item{data}{a data.frame}

\item{x}{the variable in the data.frame for which to find the MNR
or a vector if \code{data=NULL}}

\item{alpha}{the significance level for the test. Defaults to 0.05}
}
\value{
an object of class \code{mnr}
This object has the following fields:
 \item{\code{call}}{the expression used to call this function}
 \item{\code{data}}{the original data used to compute the MNR}
 \item{\code{alpha}}{the value of alpha given by the user}
 \item{\code{mnr}}{the computed MNR test statistic}
 \item{\code{crit}}{the critical value given the sample size and the
                    significance level}
 \item{\code{outliers}}{a data.frame containing the \code{index} and
                        \code{value} of each of the identified outliers}
 \item{\code{n_outliers}}{the number of outliers found}
}
\description{
This function detects outliers using the maximum normed residual
method described in CMH-17-1G. This method identifies a value
as an outlier if the absolute difference between the value and
the sample mean divided by the sample standard deviation
exceeds a critical value.
}
\details{
\code{data} is an optional argument. If \code{data} is given, it
should be a
\code{data.frame} (or similar object). When \code{data} is specified, the
value of \code{x} is expected to be a variable within \code{data}. If
\code{data} is not specified, \code{x} must be a vector.

The maximum normed residual test is a test for outliers. The test statistic
is given in CMH-17-1G. Outliers are identified in the returned object.

The maximum normed residual test statistic is defined as:

\deqn{MNR = max \frac{\left| x_i - \bar{x} \right|}{s} }{
  MNR = max | x_i- x_bar | / s }

When the value of the MNR test statistic exceeds the critical value
defined in Section 8.3.3.1 of CMH-17-1G, the corresponding value
is identified as an outlier. It is then removed from the sample, and
the test statistic is computed again and compared with the critical
value corresponding with the new sample. This process is repeated until
no values are identified as outliers.
}
\examples{
library(dplyr)

carbon.fabric.2 \%>\%
  filter(test=="FC" & condition=="ETW2" & batch=="A") \%>\%
  maximum_normed_residual(strength)

## Call:
## maximum_normed_residual(data = ., x = strength)
##
## MNR =  1.958797  ( critical value = 1.887145 )
##
## Outliers ( alpha = 0.05 ):
##   Index  Value
##       6  44.26

carbon.fabric.2 \%>\%
  filter(test=="FC" & condition=="ETW2" & batch=="B") \%>\%
  maximum_normed_residual(strength)

## Call:
## maximum_normed_residual(data = ., x = strength)
##
## MNR =  1.469517  ( critical value = 1.887145 )
##
## No outliers detected ( alpha = 0.05 )

}
\references{
“Composite Materials Handbook, Volume 1. Polymer Matrix Composites
Guideline for Characterization of Structural Materials,” SAE International,
CMH-17-1G, Mar. 2012.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/norm.R
\name{normalize_group_mean}
\alias{normalize_group_mean}
\title{Normalize values to group means}
\usage{
normalize_group_mean(x, group)
}
\arguments{
\item{x}{the variable containing the data to normalized}

\item{group}{the variable containing the groups}
}
\value{
Returns a vector of normalized values
}
\description{
This function computes the mean of each group, then divides each
observation by its corresponding group mean. This is commonly done
when pooling data across environments.
}
\details{
Computes the mean for each group, then divides each value by the mean for
the corresponding group.
}
\examples{
library(dplyr)
carbon.fabric.2 \%>\%
filter(test == "WT") \%>\%
  select(condition, strength) \%>\%
  mutate(condition_norm = normalize_group_mean(strength, condition)) \%>\%
  head(10)

##    condition strength condition_norm
## 1        CTD  142.817      1.0542187
## 2        CTD  135.901      1.0031675
## 3        CTD  132.511      0.9781438
## 4        CTD  135.586      1.0008423
## 5        CTD  125.145      0.9237709
## 6        CTD  135.203      0.9980151
## 7        CTD  128.547      0.9488832
## 8        CTD  127.709      0.9426974
## 9        CTD  127.074      0.9380101
## 10       CTD  126.879      0.9365706

}
\references{
“Composite Materials Handbook, Volume 1. Polymer Matrix Composites
Guideline for Characterization of Structural Materials,” SAE International,
CMH-17-1G, Mar. 2012.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mnr.R
\name{glance.mnr}
\alias{glance.mnr}
\title{Glance at a \code{mnr} (maximum normed residual) object}
\usage{
\method{glance}{mnr}(x, ...)
}
\arguments{
\item{x}{An \code{mnr} object}

\item{...}{Additional arguments. Not used. Included only to match generic
signature.}
}
\value{
A one-row \code{\link[tibble:tibble]{tibble::tibble}} with the following
columns:

\item{\code{mnr}}{the computed MNR test statistic}
\item{\code{alpha}}{the value of alpha used for the test}
\item{\code{crit}}{the critical value given the sample size and the
                   significance level}
\item{\code{n_outliers}}{the number of outliers found}
}
\description{
Glance accepts an object of type \code{mnr} and returns a
\code{\link[tibble:tibble]{tibble::tibble}} with
one row of summaries.

Glance does not do any calculations: it just gathers the results in a
tibble.
}
\examples{
x <- c(rnorm(20, 100, 5), 10)
m <- maximum_normed_residual(x = x)
glance(m)

## # A tibble: 1 x 4
##     mnr alpha  crit n_outliers
##   <dbl> <dbl> <dbl>      <dbl>
## 1  4.23  0.05  2.73          1

}
\seealso{
\code{\link{maximum_normed_residual}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adtest.R
\name{glance.anderson_darling}
\alias{glance.anderson_darling}
\title{Glance at an \code{anderson_darling} object}
\usage{
\method{glance}{anderson_darling}(x, ...)
}
\arguments{
\item{x}{an \code{anderson_darling} object}

\item{...}{Additional arguments. Not used. Included only to match generic
signature.}
}
\value{
A one-row \code{\link[tibble:tibble]{tibble::tibble}} with the following
columns:

\item{\code{dist}}{the distribution used}
\item{\code{n}}{the number of observations in the sample}
\item{\code{A}}{the Anderson--Darling test statistic}
\item{\code{osl}}{the observed significance level (p-value),
    assuming the
    parameters of the distribution are estimated from the data}
 \item{\code{alpha}}{the required significance level for the test.
   This value is given by the user.}
\item{\code{reject_distribution}}{a logical value indicating whether
   the hypothesis that the data is drawn from the specified distribution
   should be rejected}
}
\description{
Glance accepts an object of type \code{anderson_darling} and
returns a \code{\link[tibble:tibble]{tibble::tibble}} with
one row of summaries.

Glance does not do any calculations: it just gathers the results in a
tibble.
}
\examples{
x <- rnorm(100, 100, 4)
ad <- anderson_darling_weibull(x = x)
glance(ad)

## # A tibble: 1 x 6
##   dist        n     A        osl alpha reject_distribution
##   <chr>   <int> <dbl>      <dbl> <dbl> <lgl>
## 1 Weibull   100  2.62 0.00000207  0.05 TRUE

}
\seealso{
\code{\link{anderson_darling}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/equiv.R
\name{equiv_change_mean}
\alias{equiv_change_mean}
\title{Equivalency based on change in mean value}
\usage{
equiv_change_mean(
  df_qual = NULL,
  data_qual = NULL,
  n_qual = NULL,
  mean_qual = NULL,
  sd_qual = NULL,
  data_sample = NULL,
  n_sample = NULL,
  mean_sample = NULL,
  sd_sample = NULL,
  alpha,
  modcv = FALSE
)
}
\arguments{
\item{df_qual}{(optional) a data.frame containing the qualification data.
Defaults to NULL.}

\item{data_qual}{(optional) a vector of observations from the
"qualification" data to which equivalency is being tested. Or the column of
\code{df_qual} that contains this data. Defaults to NULL}

\item{n_qual}{the number of observations in the qualification data to which
the sample is being compared for equivalency}

\item{mean_qual}{the mean from the qualification data to which the sample
is being compared for equivalency}

\item{sd_qual}{the standard deviation from the qualification data to which
the sample is being compared for equivalency}

\item{data_sample}{a vector of observations from the sample being compared
for equivalency}

\item{n_sample}{the number of observations in the sample being compared for
equivalency}

\item{mean_sample}{the mean of the sample being compared for equivalency}

\item{sd_sample}{the standard deviation of the sample being compared for
equivalency}

\item{alpha}{the acceptable probability of a Type I error}

\item{modcv}{a logical value indicating whether the modified CV approach
should be used. Defaults to \code{FALSE}}
}
\value{
\itemize{
\item \code{call} the expression used to call this function
\item \code{alpha} the value of alpha passed to this function
\item \code{n_sample} the number of observations in the sample for which
equivalency is being checked. This is either the value \code{n_sample}
passed to this function or the length of the vector \code{data_sample}.
\item \code{mean_sample} the mean of the observations in the sample for
which equivalency is being checked. This is either the value
\code{mean_sample} passed to this function or the mean of the vector
\code{data-sample}.
\item \code{sd_sample} the standard deviation of the observations in the
sample for which equivalency is being checked. This is either the value
\code{mean_sample} passed to this function or the standard deviation of
the vector \code{data-sample}.
\item \code{n_qual} the number of observations in the qualification data
to which the sample is being compared for equivalency. This is either
the value \code{n_qual} passed to this function or the length of the
vector \code{data_qual}.
\item \code{mean_qual} the mean of the qualification data to which the
sample is being compared for equivalency. This is either the value
\code{mean_qual} passed to this function or the mean of the vector
\code{data_qual}.
\item \code{sd_qual} the standard deviation of the qualification data to
which the sample is being compared for equivalency. This is either the
value \code{mean_qual} passed to this function or the standard deviation
of the vector \code{data_qual}.
\item \code{modcv} logical value indicating whether the equivalency
calculations were performed using the modified CV approach
\item \code{sp} the value of the pooled standard deviation. If
\code{modecv = TRUE}, this pooled standard deviation includes the
modification to the qualification CV.
\item \code{t0} the test statistic
\item \code{t_req} the t-value for \eqn{\alpha / 2} and
\eqn{df = n1 + n2 -2}
\item \code{threshold} a vector with two elements corresponding to the
minimum and maximum values of the sample mean that would result in a
pass
\item \code{result} a character vector of either "PASS" or "FAIL"
indicating the result of the test for change in mean
}
}
\description{
Checks for change in the mean value between a qualification data set and
a sample. This is normally used to check for properties such as modulus.
This function is a wrapper for a two-sample t--test.
}
\details{
There are several optional arguments to this function. Either (but not both)
\code{data_sample} or all of \code{n_sample}, \code{mean_sample} and
\code{sd_sample} must be supplied. And, either (but not both)
\code{data_qual}
(and also \code{df_qual} if \code{data_qual} is a column name and not a
vector) or all of \code{n_qual}, \code{mean_qual} and \code{sd_qual} must
be supplied. If these requirements are violated, warning(s) or error(s) will
be issued.

This function uses a two-sample t-test to determine if there is a difference
in the mean value of the qualification data and the sample. A pooled
standard deviation is used in the t-test. The procedure is per CMH-17-1G.

If \code{modcv} is TRUE, the standard deviation used to calculate the
thresholds will be replaced with a standard deviation calculated
using the Modified Coefficient of Variation (CV) approach.
The Modified CV approach is a way of adding extra variance to the
qualification data in the case that the qualification data has less
variance than expected, which sometimes occurs when qualification testing
is performed in a short period of time.
Using the Modified CV approach, the standard deviation is calculated by
multiplying \code{CV_star * mean_qual} where \code{mean_qual} is either the
value supplied or the value calculated by \code{mean(data_qual)} and
\eqn{CV*} is determined using \code{\link[=calc_cv_star]{calc_cv_star()}}.

Note that the modified CV option should only be used if that data passes the
Anderson--Darling test.
}
\examples{
equiv_change_mean(alpha = 0.05, n_sample = 9, mean_sample = 9.02,
                  sd_sample = 0.15785, n_qual = 28, mean_qual = 9.24,
                  sd_qual = 0.162, modcv = TRUE)

## Call:
## equiv_change_mean(n_qual = 28, mean_qual = 9.24, sd_qual = 0.162,
##                   n_sample = 9, mean_sample = 9.02, sd_sample = 0.15785,
##                   alpha = 0.05,modcv = TRUE)
##
## For alpha = 0.05
## Modified CV used
##                   Qualification        Sample
##           Number        28               9
##             Mean       9.24             9.02
##               SD      0.162           0.15785
##           Result               PASS
##    Passing Range       8.856695 to 9.623305

}
\references{
“Composite Materials Handbook, Volume 1. Polymer Matrix Composites
Guideline for Characterization of Structural Materials,” SAE International,
CMH-17-1G, Mar. 2012.
}
\seealso{
\code{\link[=calc_cv_star]{calc_cv_star()}}

\code{\link[stats:t.test]{stats::t.test()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot-nested.R
\name{nested_data_plot}
\alias{nested_data_plot}
\title{Create a plot of nested sources of variation}
\usage{
nested_data_plot(
  dat,
  x,
  groups = c(),
  stat = "mean",
  ...,
  y_gap = 1,
  divider_color = "grey50",
  point_args = list(),
  dline_args = list(),
  vline_args = list(),
  hline_args = list(),
  label_args = list(),
  connector_args = list()
)
}
\arguments{
\item{dat}{a \code{data.frame} or similar object}

\item{x}{the variable within \code{dat} to plot. Most often this would be a
strength or modulus variable.}

\item{groups}{a vector of variables to group the data by}

\item{stat}{a function for computing the central location for each group.
This is normally "mean" but could be "median" or another
function.}

\item{...}{extra options. See Details.}

\item{y_gap}{the vertical gap between grouping variables}

\item{divider_color}{the color of the lines between grouping variables.
Or \code{NULL} to omit these lines.}

\item{point_args}{arguments to pass to \link[ggplot2:geom_point]{ggplot2::geom_point} when plotting
individual data points.}

\item{dline_args}{arguments to pass to \link[ggplot2:geom_segment]{ggplot2::geom_segment} when plotting
the horizontal lines between data points.}

\item{vline_args}{arguments to pass to \link[ggplot2:geom_segment]{ggplot2::geom_segment} when plotting
vertical lines}

\item{hline_args}{arguments to pass to \link[ggplot2:geom_segment]{ggplot2::geom_segment} when plotting
horizontal lines connecting levels in groups}

\item{label_args}{arguments to pass to \link[ggplot2:geom_text]{ggplot2::geom_label} when plotting
labels}

\item{connector_args}{arguments to pass to \link[ggplot2:geom_point]{ggplot2::geom_point} when
plotting the connection between the vertical lines
and the horizontal lines connecting levels in groups}
}
\description{
Creates a plot showing the breakdown of variation within a sample. This
function uses \code{ggplot2} internally.
}
\details{
Extra options can be included to control aesthetic options. The following
options are supported. Any (or all) can be set to a single variable
in the data set.
\itemize{
\item \code{color}: Controls the color of the data points.
\item \code{fill}: Controls the fill color of the labels. When a particular label
is associated with data points with more than one level of the supplied
variable, the fill is omitted.
}
}
\examples{
library(dplyr)
carbon.fabric.2 \%>\%
  filter(test == "WT" & condition == "RTD") \%>\%
  nested_data_plot(strength,
                   groups = c(batch, panel))

# Labels can be filled too
carbon.fabric.2 \%>\%
  filter(test == "WT" & condition == "RTD") \%>\%
  nested_data_plot(strength,
                   groups = c(batch, panel),
                   fill = batch)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adtest.R
\name{anderson_darling}
\alias{anderson_darling}
\alias{anderson_darling_normal}
\alias{anderson_darling_lognormal}
\alias{anderson_darling_weibull}
\title{Anderson--Darling test for goodness of fit}
\usage{
anderson_darling_normal(data = NULL, x, alpha = 0.05)

anderson_darling_lognormal(data = NULL, x, alpha = 0.05)

anderson_darling_weibull(data = NULL, x, alpha = 0.05)
}
\arguments{
\item{data}{a data.frame-like object (optional)}

\item{x}{a numeric vector or a variable in the data.frame}

\item{alpha}{the required significance level of the test.
Defaults to 0.05.}
}
\value{
an object of class \code{anderson_darling}. This object has the following
fields.

\item{\code{call}}{the expression used to call this function}
\item{\code{dist}}{the distribution used}
\item{\code{data}}{a copy of the data analyzed}
\item{\code{n}}{the number of observations in the sample}
\item{\code{A}}{the Anderson--Darling test statistic}
\item{\code{osl}}{the observed significance level (p-value),
  assuming the
  parameters of the distribution are estimated from the data}
\item{\code{alpha}}{the required significance level for the test.
 This value is given by the user.}
\item{\code{reject_distribution}}{a logical value indicating whether
 the hypothesis that the data is drawn from the specified distribution
 should be rejected}
}
\description{
Calculates the Anderson--Darling test statistic for a sample given
a particular distribution, and determines whether to reject the
hypothesis that a sample is drawn from that distribution.
}
\details{
The Anderson--Darling test statistic is calculated for the distribution
given by the user.

The observed significance level (OSL), or p-value, is calculated assuming
that the parameters
of the distribution are unknown; these parameters are estimate from the
data.

The function \code{anderson_darling_normal} computes the Anderson--Darling
test statistic given a normal distribution with mean and standard deviation
equal to the sample mean and standard deviation.

The function \code{anderson_darling_lognormal} is the same as
\code{anderson_darling_normal} except that the data is log transformed
first.

The function \code{anderson_darling_weibull} computes the Anderson--Darling
test statistic given a Weibull distribution with shape and scale parameters
estimated from the data using a maximum likelihood estimate.

The test statistic, \code{A}, is modified to account for
the fact that the parameters of the population are not known,
but are instead estimated from the sample. This modification is
a function of the sample size only, and is different for each
distribution (normal/lognormal or Weibull). Several such modifications
have been proposed. This function uses the modification published in
Stephens (1974), Lawless (1982) and CMH-17-1G. Some other implementations
of the Anderson-Darling test, such as the implementation in the
\code{nortest} package, use other modifications, such as the one
published in D'Agostino and Stephens (1986). As such, the p-value
reported by this function may differ from the p-value reported
by implementations of the Anderson--Darling test that use
different modifiers. Only the unmodified
test statistic is reported in the result of this function, but
the modified test statistic is used to compute the OSL (p-value).

This function uses the formulae for observed significance
level (OSL) published in CMH-17-1G. These formulae depend on the particular
distribution used.

The results of this function have been validated against
published values in Lawless (1982).
}
\examples{
library(dplyr)

carbon.fabric \%>\%
  filter(test == "FC") \%>\%
  filter(condition == "RTD") \%>\%
  anderson_darling_normal(strength)
## Call:
## anderson_darling_normal(data = ., x = strength)
##
## Distribution:  Normal ( n = 18 )
## Test statistic:  A = 0.9224776
## OSL (p-value):  0.01212193  (assuming unknown parameters)
## Conclusion: Sample is not drawn from a Normal distribution (alpha = 0.05)

}
\references{
J. F. Lawless, \emph{Statistical models and methods for lifetime data}.
New York: Wiley, 1982.

"Composite Materials Handbook, Volume 1. Polymer Matrix
Composites Guideline for Characterization of Structural
Materials," SAE International, CMH-17-1G, Mar. 2012.

M. A. Stephens, “EDF Statistics for Goodness of Fit and Some
Comparisons,”
Journal of the American Statistical Association, vol. 69, no. 347.
pp. 730–737, 1974.

R. D’Agostino and M. Stephens, Goodness-of-Fit Techniques.
New York: Marcel Dekker, 1986.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{stat_normal_surv_func}
\alias{stat_normal_surv_func}
\title{Normal Survival Function}
\usage{
stat_normal_surv_func(
  mapping = NULL,
  data = NULL,
  geom = "smooth",
  position = "identity",
  show.legend = NA,
  inherit.aes = TRUE,
  n = 100,
  pad = FALSE,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{aes()}.}

\item{data}{The data to be displayed in this layer. This has the
same usage as a \code{ggplot2} \code{stat} function.}

\item{geom}{The geometric object to use to display the data.}

\item{position}{Position argument}

\item{show.legend}{Should this layer be included in the legends?}

\item{inherit.aes}{If \code{FALSE}, overrides the default aesthetic,
rather than combining with them.}

\item{n}{If \code{NULL}, do not interpolated. Otherwise, the
number of points to interpolate.}

\item{pad}{If \code{TRUE}, pad the ESF with additional points
\verb{(-Inf, 0)} and \verb{(0, Inf)}.}

\item{...}{Other arguments to pass on to \code{layer}.}
}
\description{
The Normal survival function provides a visualization of a
distribution. A normal curve is fit based on the mean and standard
deviation of the data, and the survival function of this normal
curve is plotted. The survival function is simply one minus the
CDF.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/equiv.R
\name{glance.equiv_change_mean}
\alias{glance.equiv_change_mean}
\title{Glance at a \code{equiv_change_mean} object}
\usage{
\method{glance}{equiv_change_mean}(x, ...)
}
\arguments{
\item{x}{a \code{equiv_change_mean} object returned from
\code{\link{equiv_change_mean}}}

\item{...}{Additional arguments. Not used. Included only to match generic
signature.}
}
\value{
A one-row \code{\link[tibble:tibble]{tibble::tibble}} with the following
columns:

\item{\code{alpha}}{the value of alpha passed to this function}
\item{\code{n_sample}}{the number of observations in the sample for which
  equivalency is being checked. This is either the value \code{n_sample}
  passed to this function or the length of the vector \code{data_sample}.}
\item{\code{mean_sample}}{the mean of the observations in the sample for
  which equivalency is being checked. This is either the value
  \code{mean_sample} passed to this function or the mean of the vector
  \code{data-sample}.}
\item{\code{sd_sample}}{the standard deviation of the observations in the
  sample for which equivalency is being checked. This is either the value
  \code{mean_sample} passed to this function or the standard deviation of
  the vector \code{data-sample}.}
\item{\code{n_qual}}{the number of observations in the qualification data
  to which the sample is being compared for equivalency. This is either
  the value \code{n_qual} passed to this function or the length of the
  vector \code{data_qual}.}
\item{\code{mean_qual}}{the mean of the qualification data to which the
  sample is being compared for equivalency. This is either the value
  \code{mean_qual} passed to this function or the mean of the vector
  \code{data_qual}.}
\item{\code{sd_qual}}{the standard deviation of the qualification data to
  which the sample is being compared for equivalency. This is either the
  value \code{mean_qual} passed to this function or the standard deviation
  of the vector \code{data_qual}.}
\item{\code{modcv}}{logical value indicating whether the equivalency
  calculations were performed using the modified CV approach}
\item{\code{sp}}{the value of the pooled standard deviation. If
  \code{modecv = TRUE}, this pooled standard deviation includes the
  modification to the qualification CV.}
\item{\code{t0}}{the test statistic}
\item{\code{t_req}}{the t-value for \eqn{\alpha / 2} and
  \eqn{df = n1 + n2 -2}}
\item{\code{threshold_min}}{the minimum value of the sample mean that would
  result in a pass}
\item{\code{threshold_max}}{the maximum value of the sample mean that would
  result in a pass}
\item{\code{result}}{a character vector of either "PASS" or "FAIL"
  indicating the result of the test for change in mean}
}
\description{
Glance accepts an object of type \code{equiv_change_mean}
 and returns a \code{\link[tibble:tibble]{tibble::tibble}} with
one row of summaries.

Glance does not do any calculations: it just gathers the results in a
tibble.
}
\examples{
x0 <- rnorm(30, 100, 4)
x1 <- rnorm(5, 91, 7)
eq <- equiv_change_mean(data_qual = x0, data_sample = x1, alpha = 0.01)
glance(eq)

## # A tibble: 1 x 14
##   alpha n_sample mean_sample sd_sample n_qual mean_qual sd_qual modcv
##   <dbl>    <int>       <dbl>     <dbl>  <int>     <dbl>   <dbl> <lgl>
## 1  0.01        5        85.8      9.93     30      100.    3.90 FALSE
## # ... with 6 more variables: sp <dbl>, t0 <dbl>, t_req <dbl>,
## #   threshold_min <dbl>, threshold_max <dbl>, result <chr>

}
\seealso{
\code{\link{equiv_change_mean}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adk.R
\name{ad_ksample}
\alias{ad_ksample}
\title{Anderson--Darling K-Sample Test}
\usage{
ad_ksample(data = NULL, x, groups, alpha = 0.025)
}
\arguments{
\item{data}{a data.frame}

\item{x}{the variable in the data.frame on which to perform the
Anderson--Darling k-Sample test (usually strength)}

\item{groups}{a variable in the data.frame that defines the groups}

\item{alpha}{the significance level (default 0.025)}
}
\value{
Returns an object of class \code{adk}. This object has the following fields:
\item{\code{call}}{the expression used to call this function}
\item{\code{data}}{the original data used to compute the ADK}
\item{\code{groups}}{a vector of the groups used in the computation}
\item{\code{alpha}}{the value of alpha specified}
\item{\code{n}}{the total number of observations}
\item{\code{k}}{the number of groups}
\item{\code{sigma}}{the computed standard deviation of the test statistic}
\item{\code{ad}}{the value of the Anderson--Darling k-Sample test
  statistic}
\item{\code{p}}{the computed p-value}
\item{\code{reject_same_dist}}{a boolean value indicating whether the null
  hypothesis that all samples come from the same distribution is rejected}
\item{\code{raw}}{the original results returned from
  \link[kSamples]{ad.test}}
}
\description{
This function performs an Anderson--Darling k-sample test. This is used to
determine if several samples (groups) share a common (unspecified)
distribution.
}
\details{
This function is a wrapper for the \link[kSamples]{ad.test} function from
the package \code{kSamples}. The method "exact" is specified in the call to
\code{ad.test}. Refer to that package's documentation for details.

There is a minor difference in the formulation of the Anderson--Darling
k-Sample test in CMH-17-1G, compared with that in the Scholz and
Stephens (1987). This difference affects the test statistic and the
critical value in the same proportion, and therefore the conclusion of
the test is unaffected. When
comparing the test statistic generated by this function to that generated
by software that uses the CMH-17-1G formulation (such as ASAP, CMH17-STATS,
etc.), the test statistic reported by this function will be greater by
a factor of \eqn{(k - 1)}, with a corresponding change in the critical
value.

For more information about the difference between this function and
the formulation in CMH-17-1G, see the vignette on the subject, which
can be accessed by running \code{vignette("adktest")}
}
\examples{
library(dplyr)

carbon.fabric \%>\%
  filter(test == "WT") \%>\%
  filter(condition == "RTD") \%>\%
  ad_ksample(strength, batch)
##
## Call:
## ad_ksample(data = ., x = strength, groups = batch)
##
## N = 18          k = 3
## ADK = 0.912     p-value = 0.95989
## Conclusion: Samples come from the same distribution ( alpha = 0.025 )

}
\references{
F. W. Scholz and M. Stephens, “K-Sample Anderson--Darling Tests,” Journal
of the American Statistical Association, vol. 82, no. 399. pp. 918–924,
Sep-1987.

“Composite Materials Handbook, Volume 1. Polymer Matrix Composites
Guideline for Characterization of Structural Materials,” SAE International,
CMH-17-1G, Mar. 2012.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cv.R
\name{cv}
\alias{cv}
\title{Calculate the coefficient of variation}
\usage{
cv(x, na.rm = FALSE)
}
\arguments{
\item{x}{a vector}

\item{na.rm}{logical. Should missing values be removed?}
}
\value{
The calculated CV
}
\description{
The coefficient of variation (CV) is the ratio of the standard
deviation to the mean of a sample. This function takes a vector
of data and calculates the CV.
}
\examples{
set.seed(15)  # make this example reproducible
x <- rnorm(100, mean = 100, sd = 5)
cv(x)
## [1] 0.04944505

# the cv function can also be used within a call to dplyr::summarise
library(dplyr)
carbon.fabric \%>\%
filter(test == "WT") \%>\%
  group_by(condition) \%>\%
  summarise(mean = mean(strength), cv = cv(strength))

## # A tibble: 3 x 3
##   condition  mean     cv
##   <chr>     <dbl>  <dbl>
## 1 CTD        137. 0.0417
## 2 ETW        135. 0.0310
## 3 RTD        142. 0.0451


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/equiv.R
\name{equiv_mean_extremum}
\alias{equiv_mean_extremum}
\title{Test for decrease in mean or minimum individual}
\usage{
equiv_mean_extremum(
  df_qual = NULL,
  data_qual = NULL,
  mean_qual = NULL,
  sd_qual = NULL,
  data_sample = NULL,
  n_sample = NULL,
  alpha,
  modcv = FALSE
)
}
\arguments{
\item{df_qual}{(optional) a data.frame containing the qualification data.
Defaults to NULL.}

\item{data_qual}{(optional) a vector of observations from the
"qualification" data to which equivalency is being tested. Or the column of
\code{df_qual} that contains this data. Defaults to NULL}

\item{mean_qual}{(optional) the mean from the "qualification" data to which
equivalency is being tested. Defaults to NULL}

\item{sd_qual}{(optional) the standard deviation from the "qualification"
data to which equivalency is being tested. Defaults to NULL}

\item{data_sample}{(optional) a vector of observations from the sample for
which equivalency is being tested. Defaults to NULL}

\item{n_sample}{(optional) the number of observations in the sample for
which equivalency will be tested. Defaults to NULL}

\item{alpha}{the acceptable probability of a type I error}

\item{modcv}{(optional) a boolean value indicating whether a modified CV
should be used. Defaults to FALSE, in which case the standard deviation
supplied (or calculated from \code{data_qual}) will be used directly.}
}
\value{
Returns an object of class \code{equiv_mean_extremum}. This object is a list
with the following named elements:

\item{\code{call}}{the expression used to call this function}
\item{\code{alpha}}{the value of alpha passed to this function}
\item{\code{n_sample}}{the number of observations in the sample for which
  equivalency is being checked. This is either the value \code{n_sample}
  passed to this function or the length of the vector \code{data_sample}.}
\item{\code{k1}}{the factor used to calculate the minimum individual
  threshold. The minimum individual threshold is calculated as
  \eqn{W_{min} = qual\,mean - k_1 \cdot qual\,sd}{
  Wmin = qual_mean - k1 * qual_sd}}
\item{\code{k2}}{the factor used to calculate the threshold for mean. The
  threshold for mean is calculated as
  \eqn{W_{mean} = qual\,mean - k_2 \cdot qual\,sd}{
  Wmean = qual_mean - k2 * qual_sd}}
\item{\code{modcv}}{logical value indicating whether the acceptance
  thresholds are calculated using the modified CV approach}
\item{\code{cv}}{the coefficient of variation of the qualification data.
  This value is not modified, even if \code{modcv=TRUE}}
\item{\code{cv_star}}{The modified coefficient of variation. If
  \code{modcv=FALSE}, this will be \code{NULL}}
\item{\code{threshold_min_indiv}}{The calculated threshold value for
  minimum individual}
\item{\code{threshold_mean}}{The calculated threshold value for mean}
\item{\code{result_min_indiv}}{a character vector of either "PASS" or
  "FAIL" indicating whether the data from \code{data_sample} passes the
  test for minimum individual. If \code{data_sample} was not supplied,
  this value will be \code{NULL}}
\item{\code{result_mean}}{a character vector of either "PASS" or
  "FAIL" indicating whether the data from \code{data_sample} passes the
  test for mean. If \code{data_sample} was not supplied, this value will
  be  \code{NULL}}
\item{\code{min_sample}}{The minimum value from the vector
  \code{data_sample}. if \code{data_sample} was not supplied, this will
  have a value of \code{NULL}}
\item{\code{mean_sample}}{The mean value from the vector
  \code{data_sample}. If \code{data_sample} was not supplied, this will
  have a value of \code{NULL}}
}
\description{
This test is used when determining if a new process or
manufacturing location produces material properties that are
"equivalent" to an existing dataset, and hence the existing
basis values are applicable to the new dataset. This test is also
sometimes used for determining if a new batch of material is acceptable.
This function determines thresholds based on both minimum
individual and mean, and optionally evaluates a sample against those
thresholds. The joint distribution between the sample mean
and sample minimum is used to generate these thresholds.
When there is no true difference between the existing ("qualification")
and the new population from which the sample is obtained, there is a
probability of \eqn{\alpha} of falsely concluding that there is a
difference in mean or variance. It is assumed that both the original
and new populations are normally distributed.
According to Vangel (2002), this test provides improved power compared
with a test of mean and standard deviation.
}
\details{
This function is used to
determine acceptance limits for a sample mean and sample minimum.
These acceptance limits are often used to set acceptance limits for
material strength for each lot of material, or each new manufacturing
site. When a sample meets the criteria that its mean and its minimum are
both greater than these limits, then one may accept the lot of material
or the new manufacturing site.

This procedure is used to ensure that the strength of material processed
at a second site, or made with a new batch of material are not degraded
relative to the data originally used to determine basis values for the
material. For more information about the use of this procedure, see
CMH-17-1G or PS-ACE 100-2002-006.

There are several optional arguments to this function. However, you can't
omit all of the optional arguments. You must supply either
\code{data_sample} or \code{n_sample}, but not both. You must also supply
either \code{data_qual} (and \code{df_qual} if \code{data_qual} is a
variable name and not a vector) or both \code{mean_qual} and \code{sd_qual},
but if you supply \code{data_qual} (and possibly \code{df_qual}) you should
not supply either \code{mean_qual} or \code{sd_qual} (and visa-versa). This
function will issue a warning or error if you violate any of these rules.

If \code{modcv} is TRUE, the standard deviation used to calculate the
thresholds will be replaced with a standard deviation calculated
using the Modified Coefficient of Variation (CV) approach.
The Modified CV approach is a way of adding extra variance to the
qualification data in the case that the qualification data has less
variance than expected, which sometimes occurs when qualification testing
is performed in a short period of time.
Using the Modified CV approach, the standard deviation is calculated by
multiplying \code{CV_star * mean_qual} where \code{mean_qual} is either the
value supplied or the value calculated by \code{mean(data_qual)} and
\eqn{CV*} is the value computed by \code{\link{calc_cv_star}}.
}
\examples{
equiv_mean_extremum(alpha = 0.01, n_sample = 6,
                    mean_qual = 100, sd_qual = 5.5, modcv = TRUE)
##
## Call:
## equiv_mean_extremum(mean_qual = 100, sd_qual = 5.5, n_sample = 6,
##     alpha = 0.01, modcv = TRUE)
##
## Modified CV used: CV* = 0.0675 ( CV = 0.055 )
##
## For alpha = 0.01 and n = 6
## ( k1 = 3.128346 and k2 = 1.044342 )
##                   Min Individual   Sample Mean
##      Thresholds:    78.88367        92.95069

}
\references{
M. G. Vangel. Lot Acceptance and Compliance Testing Using the Sample Mean
and an Extremum, Technometrics, vol. 44, no. 3. pp. 242–249. 2002.

“Composite Materials Handbook, Volume 1. Polymer Matrix Composites
Guideline for Characterization of Structural Materials,” SAE International,
CMH-17-1G, Mar. 2012.

Federal Aviation Administration, “Material Qualification and Equivalency
for Polymer Matrix Composite Material Systems,” PS-ACE 100-2002-006,
Sep. 2003.
}
\seealso{
\code{\link{k_equiv}}

\code{\link{calc_cv_star}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/norm.R
\name{calc_cv_star}
\alias{calc_cv_star}
\title{Calculate the modified CV from the CV}
\usage{
calc_cv_star(cv)
}
\arguments{
\item{cv}{The CV to modify}
}
\value{
The value of the modified CV
}
\description{
This function calculates the modified coefficient of variation (CV)
based on a (unmodified) CV.
The modified CV is calculated based on the rules in CMH-17-1G. Those
rules are:
\itemize{
\item For CV < 4\\\%, CV* = 6\\\%
\item For 4\\\% <= CV < 8\\\%, CV* = CV / 2 + 4\\\%
\item For CV > 8\\\%, CV* = CV
}
}
\examples{
# The modified CV for values of CV smaller than 4\% is 6\%
calc_cv_star(0.01)
## [1] 0.06

# The modified CV for values of CV larger than 8\% is unchanged
calc_cv_star(0.09)
## [1] 0.09

}
\references{
"Composite Materials Handbook, Volume 1. Polymer Matrix Composites
Guideline for Characterization of Structural Materials,"
SAE International, CMH-17-1G, Mar. 2012.
}
\seealso{
\code{\link[=cv]{cv()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{carbon.fabric}
\alias{carbon.fabric}
\alias{carbon.fabric.2}
\title{Sample data for a generic carbon fabric}
\format{
An object of class \code{data.frame} with 216 rows and 5 columns.

An object of class \code{data.frame} with 177 rows and 8 columns.
}
\usage{
carbon.fabric

carbon.fabric.2
}
\description{
Datasets containing sample data that is typical of a generic carbon
fabric prepreg. This data is used in several examples within the
\code{cmstatr} package. This data is fictional and should
only be used for learning how to use this package.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{condition_summary}
\alias{condition_summary}
\alias{condition_summary.data.frame}
\alias{condition_summary.basis}
\title{Produce basis summary statistics for each (environmental) condition}
\usage{
condition_summary(data, ...)

\method{condition_summary}{data.frame}(data, x, condition, ref_condition = NULL, ...)

\method{condition_summary}{basis}(data, ref_condition = NULL, ...)
}
\arguments{
\item{data}{a \code{data.frame} or a \code{basis} object}

\item{...}{not used}

\item{x}{the column name of the variable of interest (usually strength)}

\item{condition}{the column name for the condition}

\item{ref_condition}{a \code{character} representing the reference condition}
}
\value{
a \code{data.frame}
}
\description{
Produces a \code{data.frame} containing the sample size and mean for each
condition. If a reference condition (e.g. "RTD") is specified, the ratio
of each condition mean value to the mean value for the reference condition
is also calculated. If a \code{basis} object returned by one of the
\code{basis_pooled} functions is given as an argument, this table also contains
the basis value for each condition.
}
\examples{
library(dplyr)
carbon.fabric.2 \%>\%
  filter(test == "WT") \%>\%
  condition_summary(strength, condition, "RTD")

##   condition  n     mean mean_fraction
## 1       CTD 19 135.4719     0.9702503
## 2       RTD 28 139.6257     1.0000000
## 3       ETW 18 134.1009     0.9604312
## 4      ETW2 21 130.1545     0.9321673

carbon.fabric \%>\%
  filter(test == "FT") \%>\%
  basis_pooled_sd(strength, condition, batch) \%>\%
  condition_summary("RTD")

##   condition  n     mean mean_fraction    basis basis_fraction
## 1       RTD 18 127.6211     1.0000000 116.8894      1.0000000
## 2       ETW 18 117.8080     0.9231072 107.0762      0.9160476
## 3       CTD 18 125.9629     0.9870063 115.2311      0.9858133

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adk.R
\name{glance.adk}
\alias{glance.adk}
\title{Glance at a \code{adk} (Anderson--Darling k-Sample) object}
\usage{
\method{glance}{adk}(x, ...)
}
\arguments{
\item{x}{an \code{adk} object}

\item{...}{Additional arguments. Not used. Included only to match generic
signature.}
}
\value{
A one-row \code{\link[tibble:tibble]{tibble::tibble()}} with the following
columns:
\itemize{
\item \code{alpha} the significance level for the test
\item \code{n} the sample size for the test
\item \code{k} the number of samples
\item \code{sigma} the computed standard deviation of the test statistic
\item \code{ad} the test statistic
\item \code{p} the p-value of the test
\item \code{reject_same_dist} whether the test concludes that the samples
are drawn from different populations
}
}
\description{
Glance accepts an object of type \code{adk} and returns a
\code{\link[tibble:tibble]{tibble::tibble()}} with
one row of summaries.

Glance does not do any calculations: it just gathers the results in a
tibble.
}
\examples{
x <- c(rnorm(20, 100, 5), rnorm(20, 105, 6))
k <- c(rep(1, 20), rep(2, 20))
a <- ad_ksample(x = x, groups = k)
glance(a)

## A tibble: 1 x 7
##   alpha     n     k sigma    ad       p reject_same_dist
##   <dbl> <int> <int> <dbl> <dbl>   <dbl> <lgl>
## 1 0.025    40     2 0.727  4.37 0.00487 TRUE

}
\seealso{
\code{\link[=ad_ksample]{ad_ksample()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/basis.R
\name{basis}
\alias{basis}
\alias{basis_normal}
\alias{basis_lognormal}
\alias{basis_weibull}
\alias{basis_pooled_cv}
\alias{basis_pooled_sd}
\alias{basis_hk_ext}
\alias{basis_nonpara_large_sample}
\alias{basis_anova}
\title{Calculate basis values}
\usage{
basis_normal(
  data = NULL,
  x,
  batch = NULL,
  p = 0.9,
  conf = 0.95,
  override = c()
)

basis_lognormal(
  data = NULL,
  x,
  batch = NULL,
  p = 0.9,
  conf = 0.95,
  override = c()
)

basis_weibull(
  data = NULL,
  x,
  batch = NULL,
  p = 0.9,
  conf = 0.95,
  override = c()
)

basis_pooled_cv(
  data = NULL,
  x,
  groups,
  batch = NULL,
  p = 0.9,
  conf = 0.95,
  modcv = FALSE,
  override = c()
)

basis_pooled_sd(
  data = NULL,
  x,
  groups,
  batch = NULL,
  p = 0.9,
  conf = 0.95,
  modcv = FALSE,
  override = c()
)

basis_hk_ext(
  data = NULL,
  x,
  batch = NULL,
  p = 0.9,
  conf = 0.95,
  method = c("optimum-order", "woodward-frawley"),
  override = c()
)

basis_nonpara_large_sample(
  data = NULL,
  x,
  batch = NULL,
  p = 0.9,
  conf = 0.95,
  override = c()
)

basis_anova(data = NULL, x, groups, p = 0.9, conf = 0.95, override = c())
}
\arguments{
\item{data}{a data.frame}

\item{x}{the variable in the data.frame for which to find the basis value}

\item{batch}{the variable in the data.frame that contains the batches.}

\item{p}{the content of the tolerance bound. Should be 0.90 for B-Basis
and 0.99 for A-Basis}

\item{conf}{confidence level Should be 0.95 for both A- and B-Basis}

\item{override}{a list of names of diagnostic tests to override,
if desired.}

\item{groups}{the variable in the data.frame representing the groups}

\item{modcv}{a logical value indicating whether the modified CV approach
should be used. Only applicable to pooling methods.}

\item{method}{the method for Hanson--Koopmans nonparametric basis values.
should be "optimum-order" for B-Basis and "woodward-frawley"
for A-Basis.}
}
\value{
an object of class \code{basis}
This object has the following fields:
\item{\code{call}}{the expression used to call this function}
\item{\code{distribution}}{the distribution used (normal, etc.)}
\item{\code{p}}{the value of \eqn{p} supplied}
\item{\code{conf}}{the value of \eqn{conf} supplied}
\item{\code{modcv}}{a logical value indicating whether the modified
                    CV approach was used. Only applicable to pooling
                    methods.}
\item{\code{data}}{a copy of the data used in the calculation}
\item{\code{groups}}{a copy of the groups variable.
                     Only used for pooling and ANOVA methods.}
\item{\code{batch}}{a copy of the batch data used for diagnostic tests}
\item{\code{modcv_transformed_data}}{the data after the modified CV
                                     transformation}
\item{\code{override}}{a vector of the names of diagnostic tests that
                       were overridden. \code{NULL} if none were
                       overridden}
\item{\code{diagnostic_results}}{a named character vector containing the
                                 results of all the diagnostic tests. See
                                 the Details section for additional
                                 information}
\item{\code{diagnostic_failures}}{a vector containing any diagnostic tests
                                  that produced failures}
\item{\code{n}}{the number of observations}
\item{\code{r}}{the number of groups, if a pooling method was used.
                Otherwise it is NULL.}
\item{\code{basis}}{the basis value computed. This is a number
                    except when pooling methods are used, in
                    which case it is a data.frame.}
}
\description{
Calculate the basis value for a given data set. There are various functions
to calculate the basis values for different distributions.
The basis value is the lower one-sided tolerance bound of a certain
proportion of the population. For more information on tolerance bounds,
see Meeker, et. al. (2017).
For B-Basis, set the content of tolerance bound to \eqn{p=0.90} and
the confidence level to \eqn{conf=0.95}; for A-Basis, set \eqn{p=0.99} and
\eqn{conf=0.95}. While other tolerance bound
contents and confidence levels may be computed, they are infrequently
needed in practice.

These functions also perform some automated diagnostic
tests of the data prior to calculating the basis values. These diagnostic
tests can be overridden if needed.
}
\details{
\code{data} is an optional argument. If \code{data} is given, it should
be a
\code{data.frame} (or similar object). When \code{data} is specified, the
value of \code{x} is expected to be a variable within \code{data}. If
\code{data} is not specified, \code{x} must be a vector.

When \code{modcv=TRUE} is set, which is only applicable to the
pooling methods,
the data is first modified according to the modified coefficient
of variation (CV)
rules. This modified data is then used when both calculating the
basis values and
also when performing the diagnostic tests. The modified CV approach
is a way of
adding extra variance to datasets with unexpectedly low variance.

\code{basis_normal} calculate the basis value by subtracting \eqn{k} times
the standard deviation from the mean. \eqn{k} is given by
the function \code{\link{k_factor_normal}}. The equations in
Krishnamoorthy and Mathew (2008) are used.
\code{basis_normal} also
performs a diagnostic test for outliers (using
\code{\link{maximum_normed_residual}})
and a diagnostic test for normality (using
\code{\link{anderson_darling_normal}}).
If the argument \code{batch} is given, this function also performs
a diagnostic test for outliers within
each batch (using \code{\link{maximum_normed_residual}})
and a diagnostic test for between batch variability (using
\code{\link{ad_ksample}}). The argument \code{batch} is only used
for these diagnostic tests.

\code{basis_lognormal} calculates the basis value in the same way
that \code{basis_normal} does, except that the natural logarithm of the
data is taken.

\code{basis_lognormal} function also performs
a diagnostic test for outliers (using
\code{\link{maximum_normed_residual}})
and a diagnostic test for normality (using
\code{\link{anderson_darling_lognormal}}).
If the argument \code{batch} is given, this function also performs
a diagnostic test for outliers within
each batch (using \code{\link{maximum_normed_residual}})
and a diagnostic test for between batch variability (using
\code{\link{ad_ksample}}). The argument \code{batch} is only used
for these diagnostic tests.

\code{basis_weibull} calculates the basis value for data distributed
according to a Weibull distribution. The confidence level for the
content requested is calculated using the conditional method, as
described in Lawless (1982) Section 4.1.2b. This has good agreement
with tables published in CMH-17-1G. Results differ between this function
and STAT17 by approximately 0.5\%.

\code{basis_weibull} function also performs
a diagnostic test for outliers (using
\code{\link{maximum_normed_residual}})
and a diagnostic test for normality (using
\code{\link{anderson_darling_weibull}}).
If the argument \code{batch} is given, this function also performs
a diagnostic test for outliers within
each batch (using \code{\link{maximum_normed_residual}})
and a diagnostic test for between batch variability (using
\code{\link{ad_ksample}}). The argument \code{batch} is only used
for these diagnostic tests.

\code{basis_hk_ext} calculates the basis value using the Extended
Hanson--Koopmans method, as described in CMH-17-1G and Vangel (1994).
For nonparametric distributions, this function should be used for samples
up to n=28 for B-Basis and up to \eqn{n=299} for A-Basis.
This method uses a pair of order statistics to determine the basis value.
CMH-17-1G suggests that for A-Basis, the first and last order statistic
is used: this is called the "woodward-frawley" method in this package,
after the paper in which this approach is described (as referenced
by Vangel (1994)). For B-Basis, another approach is used whereby the
first and \code{j-th} order statistic are used to calculate the basis value.
In this approach, the \code{j-th} order statistic is selected to minimize
the difference between the tolerance limit (assuming that the order
statistics are equal to the expected values from a standard normal
distribution) and the population quantile for a standard normal
distribution. This approach is described in Vangel (1994). This second
method (for use when calculating B-Basis values) is called
"optimum-order" in this package.
The results of \code{basis_hk_ext} have been
verified against example results from the program STAT-17. Agreement is
typically well within 0.2\%, however, for a few sample sizes, the
agreement can be as poor as 1\% with the result of this function
being more conservative than STAT-17.

\code{basis_hk_ext} also performs
a diagnostic test for outliers (using
\code{\link{maximum_normed_residual}})
and performs a pair of tests that the sample size and method selected
follow the guidance described above.
If the argument \code{batch} is given, this function also performs
a diagnostic test for outliers within
each batch (using \code{\link{maximum_normed_residual}})
and a diagnostic test for between batch variability (using
\code{\link{ad_ksample}}). The argument \code{batch} is only used
for these diagnostic tests.

\code{basis_nonpara_large_sample} calculates the basis value
using the large sample method described in CMH-17-1G. This method uses
a sum of binomials to determine the rank of the ordered statistic
corresponding with the desired tolerance limit (basis value). Results
of this function have been verified against results of the STAT-17
program.

\code{basis_nonpara_large_sample} also performs
a diagnostic test for outliers (using
\code{\link{maximum_normed_residual}})
and performs a test that the sample size is sufficiently large.
If the argument \code{batch} is given, this function also performs
a diagnostic test for outliers within
each batch (using \code{\link{maximum_normed_residual}})
and a diagnostic test for between batch variability (using
\code{\link{ad_ksample}}). The argument \code{batch} is only used
for these diagnostic tests.

\code{basis_anova} calculates basis values using the ANOVA method.
\code{x} specifies the data (normally strength) and \code{groups}
indicates the group corresponding to each observation. This method is
described in CMH-17-1G. This function automatically performs a diagnostic
test for outliers within each group
(using \code{\link{maximum_normed_residual}}) and a test for between
group variability (using \code{\link{ad_ksample}}) as well as checking
that the data contains at least 5 groups.
This function has been verified against the results of the STAT-17 program.

\code{basis_pooled_sd} calculates basis values by pooling the data from
several groups together. \code{x} specifies the data (normally strength)
and \code{group} indicates the group corresponding to each observation.
This method is described in CMH-17-1G and matches the pooling method
implemented in ASAP 2008.

\code{basis_pooled_cv} calculates basis values by pooling the data from
several groups together. \code{x} specifies the data (normally strength)
and \code{group} indicates the group corresponding to each observation.
This method is described in CMH-17-1G.

\code{basis_pooled_sd} and \code{basis_pooled_cv} both automatically
perform a number of diagnostic tests. Using
\code{\link{maximum_normed_residual}}, they check that there are no
outliers within each group and batch (provided that \code{batch} is
specified). They check the between batch variability using
\code{\link{ad_ksample}}. They check that there are no outliers within
each group (pooling all batches) using
\code{\link{maximum_normed_residual}}. They check for the normality
of the pooled data using \code{\link{anderson_darling_normal}}.
\code{basis_pooled_sd} checks for equality of variance of all
data using \code{\link{levene_test}} and \code{basis_pooled_cv}
checks for equality of variances of all data after transforming it
using \code{\link{normalize_group_mean}}
using \code{\link{levene_test}}.

The object returned by these functions includes the named vector
\code{diagnostic_results}. This contains all of the diagnostic tests
performed. The name of each element of the vector corresponds with the
name of the diagnostic test. The contents of each element will be
"P" if the diagnostic test passed, "F" if the diagnostic test failed,
"O" if the diagnostic test was overridden and \code{NA} if the
diagnostic test was skipped (typically because an optional
argument was not supplied).
}
\examples{
library(dplyr)

# A single-point basis value can be calculated as follows
# in this example, three failed diagnostic tests are
# overridden.

carbon.fabric \%>\%
  filter(test == "FC") \%>\%
  filter(condition == "RTD") \%>\%
  basis_normal(strength, batch,
               override = c("outliers",
                            "outliers_within_batch",
                            "anderson_darling_normal"))

## Call:
## basis_normal(data = ., x = strength, batch = batch,
##     override = c("outliers", "outliers_within_batch",
##    "anderson_darling_normal"))
##
## Distribution:  Normal 	( n = 18 )
## The following diagnostic tests were overridden:
##     `outliers`,
##     `outliers_within_batch`,
##     `anderson_darling_normal`
## B-Basis:   ( p = 0.9 , conf = 0.95 )
## 76.94656

# A set of pooled basis values can also be calculated
# using the pooled standard deviation method, as follows.
# In this example, one failed diagnostic test is overridden.
carbon.fabric \%>\%
  filter(test == "WT") \%>\%
  basis_pooled_sd(strength, condition, batch,
                  override = c("outliers_within_batch"))

## Call:
## basis_pooled_sd(data = ., x = strength, groups = condition,
##                 batch = batch, override = c("outliers_within_batch"))
##
## Distribution:  Normal - Pooled Standard Deviation 	( n = 54, r = 3 )
## The following diagnostic tests were overridden:
##     `outliers_within_batch`
## B-Basis:   ( p = 0.9 , conf = 0.95 )
## CTD  127.6914
## ETW  125.0698
## RTD  132.1457

}
\references{
J. F. Lawless, Statistical Models and Methods for Lifetime Data.
New York: John Wiley & Sons, 1982.

“Composite Materials Handbook, Volume 1. Polymer Matrix Composites
Guideline for Characterization of Structural Materials,” SAE International,
CMH-17-1G, Mar. 2012.

M. Vangel, “One-Sided Nonparametric Tolerance Limits,”
Communications in Statistics - Simulation and Computation,
vol. 23, no. 4. pp. 1137–1154, 1994.

K. Krishnamoorthy and T. Mathew, Statistical Tolerance Regions: Theory,
Applications, and Computation. Hoboken: John Wiley & Sons, 2008.

W. Meeker, G. Hahn, and L. Escobar, Statistical Intervals: A Guide
for Practitioners and Researchers, Second Edition.
Hoboken: John Wiley & Sons, 2017.
}
\seealso{
\code{\link{hk_ext_z_j_opt}}

\code{\link{k_factor_normal}}

\code{\link{transform_mod_cv}}

\code{\link{maximum_normed_residual}}

\code{\link{anderson_darling_normal}}

\code{\link{anderson_darling_lognormal}}

\code{\link{anderson_darling_weibull}}

\code{\link{ad_ksample}}

\code{\link{normalize_group_mean}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/basis.R
\name{nonpara_binomial_rank}
\alias{nonpara_binomial_rank}
\title{Rank for distribution-free tolerance bound}
\usage{
nonpara_binomial_rank(n, p, conf)
}
\arguments{
\item{n}{the sample size}

\item{p}{the desired content for the tolerance bound}

\item{conf}{the confidence level for the desired tolerance bound}
}
\value{
The rank corresponding with the desired tolerance bound
}
\description{
Calculates the rank order for finding distribution-free tolerance
bounds for large samples. This function should only be used for
computing B-Basis for samples larger than 28 or A-Basis for samples
larger than 298. This function is used by
\code{\link{basis_nonpara_large_sample}}.
}
\details{
This function uses the sum of binomial terms to determine the rank
of the ordered statistic that corresponds with the desired tolerance
limit. This approach does not assume any particular distribution. This
approach is described by Guenther (1969) and by CMH-17-1G.

The results of this function have been verified against the tables in
CMH-17-1G and agreement was found for all sample sizes published in
CMH-17-1G for both A- and B-Basis, as well as the sample sizes
\code{n+1} and \code{n-1}, where
\code{n} is the sample size published in CMH-17-1G.

The tables in CMH-17-1G purportedly list the smallest sample sizes
for which a particular rank can be used. That is, for a sample size
one less than the \code{n} published in the table, the next lowest rank
would be used. In some cases, the results of this function disagree by a
rank of one for sample sizes one less than the \code{n} published in the
table. This indicates a disagreement in that sample size at which
the rank should change. This is likely due to numerical
differences in this function and the procedure used to generate the tables.
However, the disagreement is limited to sample 6500 for A-Basis; no
discrepancies have been identified for B-Basis. Since these sample sizes are
uncommon for composite materials
testing, and the difference between subsequent order statistics will be
very small for samples this large, this difference will have no practical
effect on computed tolerance bounds.
}
\examples{
nonpara_binomial_rank(n = 1693, p = 0.99, conf = 0.95)
## [1] 11

# The above example indicates that for a sample of 1693 observations,
# the A-Basis is best approximated as the 11th ordered observation.
# In the example below, the same ordered observation would also be used
# for a sample of size 1702.

nonpara_binomial_rank(n = 1702, p = 0.99, conf = 0.95)
## [1] 11

}
\references{
W. Guenther, “Determination of Sample Size for Distribution-Free
Tolerance Limits,” Jan. 1969.
Available online: \url{https://www.duo.uio.no/handle/10852/48686}

“Composite Materials Handbook, Volume 1. Polymer Matrix Composites
Guideline for Characterization of Structural Materials,” SAE International,
CMH-17-1G, Mar. 2012.
}
\seealso{
\code{\link{basis_nonpara_large_sample}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/basis.R
\name{hk_ext}
\alias{hk_ext}
\alias{hk_ext_z}
\alias{hk_ext_z_j_opt}
\title{Calculate values related to Extended Hanson--Koopmans tolerance bounds}
\usage{
hk_ext_z(n, i, j, p, conf)

hk_ext_z_j_opt(n, p, conf)
}
\arguments{
\item{n}{the sample size}

\item{i}{the first order statistic (1 <= i < j)}

\item{j}{the second order statistic (i < j <= n)}

\item{p}{the content of the tolerance bound (normally 0.90 or 0.99)}

\item{conf}{the confidence level (normally 0.95)}
}
\value{
For \code{hk_ext_z}, the return value is a numeric value representing
the parameter z (denoted as k in CMH-17-1G).

For \code{hk_ext_z_j_opt}, the return value is named list containing
\code{z} and \code{k}. The former is the value of z, as defined by
Vangel (1994), and the latter is the corresponding order statistic.
}
\description{
Calculates values related to Extended Hanson--Koopmans tolerance bounds
as described by Vangel (1994).
}
\details{
Hanson (1964) presents a nonparametric method for determining
tolerance bounds based on consecutive order statistics.
Vangel (1994) extends this method using non-consecutive order statistics.

The extended Hanson--Koopmans method calculates a tolerance bound
(basis value) based on two order statistics and a weighting value
\code{z}. The value of \code{z} is based on the sample size, which
order statistics are selected, the desired content of the tolerance
bond and the desired confidence level.

The function \code{hk_ext_z} calculates the weighting variable \code{z}
based on selected order statistics \code{i} and \code{j}. Based on this
value \code{z}, the tolerance bound can be calculated as:

\deqn{S = z X_{(i)} + (1 - z) X_{(j)}}{S = z X(i) + (1 - z) X(j)}

Where \eqn{X_{(i)}}{X(i)} and \eqn{X_{(j)}}{X(j)} are the \code{i-th}
and \code{j-th} ordered observation.

The function \code{hk_ext_z_j_opt} determines the value of \code{j} and
the corresponding value of \code{z}, assuming \code{i=1}. The value
of \code{j} is selected such that the computed tolerance limit is
nearest to the desired population quantile for a standard normal
distribution when the order statistics are equal to the expected
value of the order statistics for the standard normal distribution.
}
\examples{
# The factors from Table 1 of Vangel (1994) can be recreated
# using the hk_ext_z function. For the sample size n=21,
# the median is the 11th ordered observation. The factor
# required for calculating the tolerance bound with a content
# of 0.9 and a confidence level of 0.95 based on the median
# and first ordered observation can be calculated as follows.
hk_ext_z(n = 21, i = 1, j = 11, p = 0.9, conf = 0.95)

## [1] 1.204806

# The hk_ext_z_j_opt function can be used to refine this value
# of z by finding an optimum value of j, rather than simply
# using the median. Here, we find that the optimal observation
# to use is the 10th, not the 11th (which is the median).
hk_ext_z_j_opt(n = 21, p = 0.9, conf = 0.95)

## $z
## [1] 1.217717
##
## $j
## [1] 10

}
\references{
M. Vangel, “One-Sided Nonparametric Tolerance Limits,”
Communications in Statistics - Simulation and Computation,
vol. 23, no. 4. pp. 1137–1154, 1994.

D. L. Hanson and L. H. Koopmans,
“Tolerance Limits for the Class of Distributions with Increasing
Hazard Rates,” The Annals of Mathematical Statistics,
vol. 35, no. 4. pp. 1561–1570, 1964.
}
\seealso{
\code{\link{basis_hk_ext}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/levene.R
\name{glance.levene}
\alias{glance.levene}
\title{Glance at a \code{levene} object}
\usage{
\method{glance}{levene}(x, ...)
}
\arguments{
\item{x}{a \code{levene} object returned from \code{\link[=levene_test]{levene_test()}}}

\item{...}{Additional arguments. Not used. Included only to match generic
signature.}
}
\value{
A one-row \code{\link[tibble:tibble]{tibble::tibble()}} with the following
columns:
\itemize{
\item \code{alpha} the value of alpha specified
\item \code{modcv} a logical value indicating whether the modified
CV approach was used.
\item \code{n} the total number of observations
\item \code{k} the number of groups
\item \code{f} the value of the F test statistic
\item \code{p} the computed p-value
\item \code{reject_equal_variance} a boolean value indicating whether the
null hypothesis that all samples have the same variance is rejected
}
}
\description{
Glance accepts an object of type \code{levene} and returns a
\code{\link[tibble:tibble]{tibble::tibble()}} with
one row of summaries.

Glance does not do any calculations: it just gathers the results in a
tibble.
}
\examples{
df <- data.frame(
  groups = c(rep("A", 5), rep("B", 6)),
  strength = c(rnorm(5, 100, 6), rnorm(6, 105, 7))
)
levene_result <- levene_test(df, strength, groups)
glance(levene_result)

## # A tibble: 1 x 7
##   alpha modcv     n     k      f     p reject_equal_variance
##   <dbl> <lgl> <int> <int>  <dbl> <dbl> <lgl>
## 1  0.05 FALSE    11     2 0.0191 0.893 FALSE

}
\seealso{
\code{\link[=levene_test]{levene_test()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/levene.R
\name{levene_test}
\alias{levene_test}
\title{Levene's Test (Median) for Equality of Variance}
\usage{
levene_test(data = NULL, x, groups, alpha = 0.05, modcv = FALSE)
}
\arguments{
\item{data}{a data.frame}

\item{x}{the variable in the data.frame or a vector on which to perform the
Levene's test (usually strength)}

\item{groups}{a variable in the data.frame that defines the groups}

\item{alpha}{the significance level (default 0.05)}

\item{modcv}{a logical value indicating whether the modified CV approach
should be used.}
}
\value{
Returns an object of class \code{levene}. This object has the following fields:
\itemize{
\item \code{call} the expression used to call this function
\item \code{data} the original data supplied by the user
\item \code{groups} a vector of the groups used in the computation
\item \code{alpha} the value of alpha specified
\item \code{modcv} a logical value indicating whether the modified
CV approach was used.
\item \code{n} the total number of observations
\item \code{k} the number of groups
\item \code{f} the value of the F test statistic
\item \code{p} the computed p-value
\item \code{reject_equal_variance} a boolean value indicating whether the
null hypothesis that all samples have the same variance is rejected
\item \code{modcv_transformed_data} the data after the modified CV transformation
}
}
\description{
This function performs the Levene's test for equality of variance using
the median. This is also known as the Brown-Forsythe test.
}
\details{
This function performs the Levene's test for equality of variance using
median (also known as the Brown-Forsythe test). The
data is transformed as follows:

\deqn{w_{ij} = \left| x_{ij} - m_i \right|}{wij = | xij - mi |}

Where \eqn{m_i}{mi} is median of the \eqn{ith} group. An F-Test is then
performed on the transformed data.

When \code{modcv=TRUE}, the data from each group is first transformed
according to the modified coefficient of variation (CV) rules before
performing Levene's test.
}
\examples{
library(dplyr)

carbon.fabric.2 \%>\%
  filter(test == "FC") \%>\%
  levene_test(strength, condition)
##
## Call:
## levene_test(data = ., x = strength, groups = condition)
##
## n = 91          k = 5
## F = 3.883818    p-value = 0.00600518
## Conclusion: Samples have unequal variance ( alpha = 0.05 )

}
\references{
“Composite Materials Handbook, Volume 1. Polymer Matrix Composites
Guideline for Characterization of Structural Materials,” SAE International,
CMH-17-1G, Mar. 2012.

\code{NIST/SEMATECH} e-Handbook of Statistical Methods,
\url{https://www.itl.nist.gov/div898/handbook/eda/section3/eda35a.htm}, 2024.

Brown, M. B. and Forsythe, A. B. (1974), Journal of the American
Statistical Association, 69, pp. 364-367.
}
\seealso{
\code{\link[=calc_cv_star]{calc_cv_star()}}

\code{\link[=transform_mod_cv]{transform_mod_cv()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/basis.R
\name{glance.basis}
\alias{glance.basis}
\title{Glance at a basis object}
\usage{
\method{glance}{basis}(x, include_diagnostics = FALSE, ...)
}
\arguments{
\item{x}{a basis object}

\item{include_diagnostics}{a logical value indicating whether to include
columns for diagnostic tests. Default FALSE.}

\item{...}{Additional arguments. Not used. Included only to match generic
signature.}
}
\value{
A \code{\link[tibble:tibble]{tibble::tibble()}} with the following
columns:
\itemize{
\item \code{p} the the content of the tolerance bound. Normally 0.90 or 0.99
\item \code{conf} the confidence level. Normally 0.95
\item \code{distribution} a string representing the distribution assumed
when calculating the basis value
\item \code{modcv} a logical value indicating whether the modified
CV approach was used. Only applicable to pooling methods.
\item \code{n} the sample size
\item \code{r} the number of groups used in the calculation. This will
be \code{NA} for single-point basis values
\item \code{basis} the basis value
}
}
\description{
Glance accepts an object of type basis and returns a
\code{\link[tibble:tibble]{tibble::tibble()}} with
one row of summaries for each basis value.

Glance does not do any calculations: it just gathers the results in a
tibble.
}
\details{
For the pooled basis methods (\code{basis_pooled_cv} and
\code{basis_pooled_sd}), the \code{\link[tibble:tibble]{tibble::tibble()}}
returned by \code{glance} will have one row for each group included in
the pooling. For all other basis methods, the resulting \code{tibble}
will have a single row.

If \code{include_diagnostics=TRUE}, there will be additional columns
corresponding with the diagnostic tests performed. These column(s) will
be of type character and will contain a "P" if the diagnostic test
passed, a "F" if the diagnostic test failed, an "O" if the diagnostic
test was overridden or \code{NA} if the test was not run (typically
because an optional argument was not passed to the function that
computed the basis value).
}
\examples{
set.seed(10)
x <- rnorm(20, 100, 5)
b <- basis_normal(x = x)
glance(b)

## # A tibble: 1 x 7
##       p  conf distribution modcv     n r     basis
##   <dbl> <dbl> <chr>        <lgl> <int> <lgl> <dbl>
## 1   0.9  0.95 Normal       FALSE    20 NA     92.0


glance(b, include_diagnostics = TRUE)

## # A tibble: 1 x 11
##        p  conf distribution modcv     n r     basis outliers_within…
##    <dbl> <dbl> <chr>        <lgl> <int> <lgl> <dbl> <chr>
##  1   0.9  0.95 Normal       FALSE    20 NA     92.0 NA
## # … with 3 more variables: between_batch_variability <chr>,
## #   outliers <chr>, anderson_darling_normal <chr>

}
\seealso{
\code{\link[=basis]{basis()}}
}
